@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CheatManager.cxx,v 1.16 2008/02/06 13:45:19 stephena Exp $
+// $Id: CheatManager.cxx 2001 2010-04-10 21:37:23Z stephena $
 //============================================================================
 
 #include <sstream>
@@ -55,7 +56,7 @@ const Cheat* CheatManager::add(const string& name, const string& code,
   // Delete duplicate entries
   for(unsigned int i = 0; i < myCheatList.size(); i++)
   {
-    if(myCheatList[i]->code() == code)
+    if(myCheatList[i]->name() == name || myCheatList[i]->code() == code)
     {
       myCheatList.remove_at(i);
       break;
