@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: FrameBufferSoft.cxx,v 1.78 2008/03/24 00:02:16 stephena Exp $
+// $Id: FrameBufferSoft.cxx 2029 2010-04-29 12:46:07Z stephena $
 //============================================================================
 
 #include <sstream>
@@ -23,20 +24,18 @@
 
 #include "Console.hxx"
 #include "Font.hxx"
-#include "MediaSrc.hxx"
 #include "OSystem.hxx"
 #include "RectList.hxx"
 #include "Settings.hxx"
-#include "Surface.hxx"
+#include "TIA.hxx"
 
 #include "FrameBufferSoft.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 FrameBufferSoft::FrameBufferSoft(OSystem* osystem)
   : FrameBuffer(osystem),
-    myZoomLevel(1),
     myRenderType(kSoftZoom_16),
-    myDirtyFlag(false),
+    myTiaDirty(false),
     myInUIMode(false),
     myRectList(NULL)
 {
@@ -49,7 +48,7 @@ FrameBufferSoft::~FrameBufferSoft()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool FrameBufferSoft::initSubsystem(VideoMode mode)
+bool FrameBufferSoft::initSubsystem(VideoMode& mode)
 {
   // Set up the rectangle list to be used in the dirty update
   delete myRectList;
@@ -57,7 +56,7 @@ bool FrameBufferSoft::initSubsystem(VideoMode mode)
 
   if(!myRectList)
   {
-    cerr << "ERROR: Unable to get memory for SDL rects" << endl;
+    myOSystem->logMessage("ERROR: Unable to get memory for SDL rects\n", 0);
     return false;
   }
 
@@ -86,55 +85,82 @@ string FrameBufferSoft::about() const
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool FrameBufferSoft::setVidMode(VideoMode mode)
+bool FrameBufferSoft::setVidMode(VideoMode& mode)
 {
-  myScreenDim.x = myScreenDim.y = 0;
-  myScreenDim.w = mode.screen_w;
-  myScreenDim.h = mode.screen_h;
-
-  myImageDim.x = mode.image_x;
-  myImageDim.y = mode.image_y;
-  myImageDim.w = mode.image_w;
-  myImageDim.h = mode.image_h;
-
-  myZoomLevel = mode.zoom;
-
   // Make sure to clear the screen
   if(myScreen)
   {
     SDL_FillRect(myScreen, NULL, 0);
     SDL_UpdateRect(myScreen, 0, 0, 0, 0);
   }
-  myScreen = SDL_SetVideoMode(myScreenDim.w, myScreenDim.h, 0, mySDLFlags);
+  myScreen = SDL_SetVideoMode(mode.screen_w, mode.screen_h, 0, mySDLFlags);
   if(myScreen == NULL)
   {
-    cerr << "ERROR: Unable to open SDL window: " << SDL_GetError() << endl;
+    ostringstream buf;
+    buf << "ERROR: Unable to open SDL window: " << SDL_GetError() << endl;
+    myOSystem->logMessage(buf.str(), 0);
     return false;
   }
   myFormat = myScreen->format;
   myBytesPerPixel = myFormat->BytesPerPixel;
 
-  // Make sure drawMediaSource() knows which renderer to use
-  stateChanged(myOSystem->eventHandler().state());
-  myBaseOffset = myImageDim.y * myPitch + myImageDim.x;
+  // Make sure the flags represent the current screen state
+  mySDLFlags = myScreen->flags;
+
+  // Make sure drawTIA() knows which renderer to use
+  switch(myBytesPerPixel)
+  {
+    case 2:  // 16-bit
+      myPitch = myScreen->pitch >> 1;
+      myRenderType = myUsePhosphor ? kPhosphor_16 : kSoftZoom_16;
+      break;
+    case 3:  // 24-bit
+      myPitch = myScreen->pitch;
+      myRenderType = myUsePhosphor ? kPhosphor_24 : kSoftZoom_24;
+      break;
+    case 4:  // 32-bit
+      myPitch = myScreen->pitch >> 2;
+      myRenderType = myUsePhosphor ? kPhosphor_32 : kSoftZoom_32;
+      break;
+  }
+  myBaseOffset = mode.image_y * myPitch + mode.image_x;
+
+  // If software mode can open the given screen, it will always be in the
+  // requested format, or not at all; we only update mode when the screen
+  // is successfully created
+  mode.screen_w = myScreen->w;
+  mode.screen_h = myScreen->h;
+  myZoomLevel = mode.gfxmode.zoom;
+// FIXME - look at gfxmode directly
 
   // Erase old rects, since they've probably been scaled for
   // a different sized screen
   myRectList->start();
 
+  // Any previously allocated surfaces have probably changed as well,
+  // so we should refresh them
+  resetSurfaces();
+
   return true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void FrameBufferSoft::drawMediaSource()
+void FrameBufferSoft::invalidate()
 {
-  MediaSource& mediasrc = myOSystem->console().mediaSource();
+  if(myScreen)
+    SDL_FillRect(myScreen, NULL, 0);
+}
 
-  uInt8* currentFrame   = mediasrc.currentFrameBuffer();
-  uInt8* previousFrame  = mediasrc.previousFrameBuffer();
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void FrameBufferSoft::drawTIA(bool fullRedraw)
+{
+  const TIA& tia = myOSystem->console().tia();
 
-  uInt32 width  = mediasrc.width();
-  uInt32 height = mediasrc.height();
+  uInt8* currentFrame   = tia.currentFrameBuffer();
+  uInt8* previousFrame  = tia.previousFrameBuffer();
+
+  uInt32 width  = tia.width();
+  uInt32 height = tia.height();
 
   switch(myRenderType)
   {
@@ -158,14 +184,14 @@ void FrameBufferSoft::drawMediaSource()
             uInt8 v = currentFrame[bufofs];
             uInt8 w = previousFrame[bufofs];
 
-            if(v != w || theRedrawTIAIndicator)
+            if(v != w || fullRedraw)
             {
               while(xstride--)
               {
                 buffer[pos++] = (uInt16) myDefPalette[v];
                 buffer[pos++] = (uInt16) myDefPalette[v];
               }
-              myDirtyFlag = true;
+              myTiaDirty = true;
             }
             else
               pos += xstride + xstride;
@@ -198,21 +224,20 @@ void FrameBufferSoft::drawMediaSource()
             uInt8 v = currentFrame[bufofs];
             uInt8 w = previousFrame[bufofs];
 
-            if(v != w || theRedrawTIAIndicator)
+            if(v != w || fullRedraw)
             {
-              uInt32 pixel = myDefPalette[v];
-              uInt8 r = (pixel & myFormat->Rmask) >> myFormat->Rshift;
-              uInt8 g = (pixel & myFormat->Gmask) >> myFormat->Gshift;
-              uInt8 b = (pixel & myFormat->Bmask) >> myFormat->Bshift;
+              uInt8 a = myDefPalette24[v][0],
+                    b = myDefPalette24[v][1],
+                    c = myDefPalette24[v][2];
 
               while(xstride--)
               {
-                buffer[pos++] = r;  buffer[pos++] = g;  buffer[pos++] = b;
-                buffer[pos++] = r;  buffer[pos++] = g;  buffer[pos++] = b;
+                buffer[pos++] = a;  buffer[pos++] = b;  buffer[pos++] = c;
+                buffer[pos++] = a;  buffer[pos++] = b;  buffer[pos++] = c;
               }
-              myDirtyFlag = true;
+              myTiaDirty = true;
             }
-            else  // try to eliminate multply whereever possible
+            else  // try to eliminate multiply whereever possible
               pos += xstride + xstride + xstride + xstride + xstride + xstride;
           }
           screenofsY += myPitch;
@@ -243,14 +268,14 @@ void FrameBufferSoft::drawMediaSource()
             uInt8 v = currentFrame[bufofs];
             uInt8 w = previousFrame[bufofs];
 
-            if(v != w || theRedrawTIAIndicator)
+            if(v != w || fullRedraw)
             {
               while(xstride--)
               {
                 buffer[pos++] = (uInt32) myDefPalette[v];
                 buffer[pos++] = (uInt32) myDefPalette[v];
               }
-              myDirtyFlag = true;
+              myTiaDirty = true;
             }
             else
               pos += xstride + xstride;
@@ -294,7 +319,7 @@ void FrameBufferSoft::drawMediaSource()
         bufofsY += width;
       }
       SDL_UnlockSurface(myScreen);
-      myDirtyFlag = true;
+      myTiaDirty = true;
       break;  // kPhosphor_16
     }
 
@@ -317,15 +342,25 @@ void FrameBufferSoft::drawMediaSource()
 
             uInt8 v = currentFrame[bufofs];
             uInt8 w = previousFrame[bufofs];
+            uInt8 a, b, c;
             uInt32 pixel = myAvgPalette[v][w];
-            uInt8 r = (pixel & myFormat->Rmask) >> myFormat->Rshift;
-            uInt8 g = (pixel & myFormat->Gmask) >> myFormat->Gshift;
-            uInt8 b = (pixel & myFormat->Bmask) >> myFormat->Bshift;
+            if(SDL_BYTEORDER == SDL_LIL_ENDIAN)
+            {
+              a = (pixel & myFormat->Bmask) >> myFormat->Bshift;
+              b = (pixel & myFormat->Gmask) >> myFormat->Gshift;
+              c = (pixel & myFormat->Rmask) >> myFormat->Rshift;
+            }
+            else
+            {
+              a = (pixel & myFormat->Rmask) >> myFormat->Rshift;
+              b = (pixel & myFormat->Gmask) >> myFormat->Gshift;
+              c = (pixel & myFormat->Bmask) >> myFormat->Bshift;
+            }
 
             while(xstride--)
             {
-              buffer[pos++] = r;  buffer[pos++] = g;  buffer[pos++] = b;
-              buffer[pos++] = r;  buffer[pos++] = g;  buffer[pos++] = b;
+              buffer[pos++] = a;  buffer[pos++] = b;  buffer[pos++] = c;
+              buffer[pos++] = a;  buffer[pos++] = b;  buffer[pos++] = c;
             }
           }
           screenofsY += myPitch;
@@ -333,7 +368,7 @@ void FrameBufferSoft::drawMediaSource()
         bufofsY += width;
       }
       SDL_UnlockSurface(myScreen);
-      myDirtyFlag = true;
+      myTiaDirty = true;
       break;  // kPhosphor_24
     }
 
@@ -368,37 +403,71 @@ void FrameBufferSoft::drawMediaSource()
         bufofsY += width;
       }
       SDL_UnlockSurface(myScreen);
-      myDirtyFlag = true;
+      myTiaDirty = true;
       break;  // kPhosphor_32
     }
   }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void FrameBufferSoft::preFrameUpdate()
+void FrameBufferSoft::postFrameUpdate()
 {
+  if(myTiaDirty && !myInUIMode)
+  {
+    SDL_UpdateRect(myScreen, 0, 0, 0, 0);
+    myTiaDirty = false;
+  }
+  else if(myRectList->numRects() > 0)
+  {
+//myRectList->print(myScreen->w, myScreen->h);
+    SDL_UpdateRects(myScreen, myRectList->numRects(), myRectList->rects());
+  }
+  myRectList->start();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void FrameBufferSoft::postFrameUpdate()
+void FrameBufferSoft::enablePhosphor(bool enable, int blend)
 {
-/*
-cerr << "FrameBufferSoft::postFrameUpdate()" << endl
-	<< "  myInUIMode:             " << myInUIMode << endl
-	<< "  myRectList->numRects(): " << myRectList->numRects() << endl
-	<< "  myDirtyFlag:            " << myDirtyFlag << endl
-	<< endl;
-*/
-  if(myInUIMode && myRectList->numRects() > 0)
+  myUsePhosphor   = enable;
+  myPhosphorBlend = blend;
+
+  // Make sure drawMediaSource() knows which renderer to use
+  switch(myBytesPerPixel)
   {
-    SDL_UpdateRects(myScreen, myRectList->numRects(), myRectList->rects());
+    case 2:  // 16-bit
+      myRenderType = myUsePhosphor ? kPhosphor_16 : kSoftZoom_16;
+      break;
+    case 3:  // 24-bit
+      myRenderType = myUsePhosphor ? kPhosphor_24 : kSoftZoom_24;
+      break;
+    case 4:  // 32-bit
+      myRenderType = myUsePhosphor ? kPhosphor_32 : kSoftZoom_32;
+      break;
   }
-  else if(myDirtyFlag || myRectList->numRects() > 0)
-  {
-    SDL_Flip(myScreen);
-    myDirtyFlag = false;
-  }
-  myRectList->start();
+  myRedrawEntireFrame = true;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+FBSurface* FrameBufferSoft::createSurface(int w, int h, bool isBase) const
+{
+  // For some unknown reason, OSX in software fullscreen mode doesn't like
+  // to use the underlying surface directly
+  // I suspect it's an SDL compatibility thing, since I get errors
+  // referencing Quartz vs. QuickDraw, and then a program crash
+  // For now, we'll just always use entire surfaces for OSX
+  // I don't think this will have much effect, since OpenGL mode is the
+  // preferred method in OSX (basically, all OSX installations have OpenGL
+  // support)
+#ifdef MAC_OSX
+  isBase = false;
+#endif
+
+  SDL_Surface* surface = isBase ? myScreen :
+      SDL_CreateRGBSurface(SDL_SWSURFACE, w, h, myFormat->BitsPerPixel,
+                           myFormat->Rmask, myFormat->Gmask, myFormat->Bmask,
+                           myFormat->Amask);
+
+  return new FBSurfaceSoft(*this, surface, w, h, isBase);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -411,12 +480,13 @@ void FrameBufferSoft::scanline(uInt32 row, uInt8* data) const
   uInt8 *p, r, g, b;
 
   // Row will be offset by the amount the actual image is shifted down
-  row += myImageDim.y;
+  const GUI::Rect& image = imageRect();
+  row += image.y();
   for(Int32 x = 0; x < myScreen->w; ++x)
   {
     p = (Uint8*) ((uInt8*)myScreen->pixels +              // Start at top of RAM
                  (row * myScreen->pitch) +                // Go down 'row' lines
-                 ((x + myImageDim.x) * myBytesPerPixel)); // Go in 'x' pixels
+                 ((x + image.x()) * myBytesPerPixel));    // Go in 'x' pixels
 
     switch(myBytesPerPixel)
     {
@@ -450,54 +520,75 @@ void FrameBufferSoft::scanline(uInt32 row, uInt8* data) const
   SDL_UnlockSurface(myScreen);
 }
 
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void FrameBufferSoft::toggleFilter()
+//  FBSurfaceSoft implementation follows ...
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+FBSurfaceSoft::FBSurfaceSoft(const FrameBufferSoft& buffer, SDL_Surface* surface,
+                             uInt32 w, uInt32 h, bool isBase)
+  : myFB(buffer),
+    mySurface(surface),
+    myWidth(w),
+    myHeight(h),
+    myIsBaseSurface(isBase),
+    mySurfaceIsDirty(false),
+    myPitch(0),
+    myXOrig(0),
+    myYOrig(0),
+    myXOffset(0),
+    myYOffset(0)
 {
-  // No filter added yet ...
+  reload();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void FrameBufferSoft::hLine(uInt32 x, uInt32 y, uInt32 x2, int color)
+FBSurfaceSoft::~FBSurfaceSoft()
 {
-  SDL_Rect tmp;
+  if(!myIsBaseSurface)
+    SDL_FreeSurface(mySurface);
+}
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void FBSurfaceSoft::hLine(uInt32 x, uInt32 y, uInt32 x2, uInt32 color)
+{
   // Horizontal line
-  tmp.x = myImageDim.x + x * myZoomLevel;
-  tmp.y = myImageDim.y + y * myZoomLevel;
-  tmp.w = (x2 - x + 1) * myZoomLevel;
-  tmp.h = myZoomLevel;
-  SDL_FillRect(myScreen, &tmp, myDefPalette[color]);
+  SDL_Rect tmp;
+  tmp.x = x + myXOffset;
+  tmp.y = y + myYOffset;
+  tmp.w = x2 - x + 1;
+  tmp.h = 1;
+  SDL_FillRect(mySurface, &tmp, myFB.myDefPalette[color]);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void FrameBufferSoft::vLine(uInt32 x, uInt32 y, uInt32 y2, int color)
+void FBSurfaceSoft::vLine(uInt32 x, uInt32 y, uInt32 y2, uInt32 color)
 {
-  SDL_Rect tmp;
-
   // Vertical line
-  tmp.x = myImageDim.x + x * myZoomLevel;
-  tmp.y = myImageDim.y + y * myZoomLevel;
-  tmp.w = myZoomLevel;
-  tmp.h = (y2 - y + 1) * myZoomLevel;
-  SDL_FillRect(myScreen, &tmp, myDefPalette[color]);
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void FrameBufferSoft::fillRect(uInt32 x, uInt32 y, uInt32 w, uInt32 h, int color)
-{
   SDL_Rect tmp;
-
-  // Fill the rectangle
-  tmp.x = myImageDim.x + x * myZoomLevel;
-  tmp.y = myImageDim.y + y * myZoomLevel;
-  tmp.w = w * myZoomLevel;
-  tmp.h = h * myZoomLevel;
-  SDL_FillRect(myScreen, &tmp, myDefPalette[color]);
+  tmp.x = x + myXOffset;
+  tmp.y = y + myYOffset;
+  tmp.w = 1;
+  tmp.h = y2 - y + 1;
+  SDL_FillRect(mySurface, &tmp, myFB.myDefPalette[color]);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void FrameBufferSoft::drawChar(const GUI::Font* font, uInt8 chr,
-                               uInt32 xorig, uInt32 yorig, int color)
+void FBSurfaceSoft::fillRect(uInt32 x, uInt32 y, uInt32 w, uInt32 h, uInt32 color)
+{
+  // Fill the rectangle
+  SDL_Rect tmp;
+  tmp.x = x + myXOffset;
+  tmp.y = y + myYOffset;
+  tmp.w = w;
+  tmp.h = h;
+  SDL_FillRect(mySurface, &tmp, myFB.myDefPalette[color]);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void FBSurfaceSoft::drawChar(const GUI::Font* font, uInt8 chr,
+                             uInt32 tx, uInt32 ty, uInt32 color)
 {
   const FontDesc& desc = font->desc();
 
@@ -507,84 +598,69 @@ void FrameBufferSoft::drawChar(const GUI::Font* font, uInt8 chr,
     if (chr == ' ') return;
     chr = desc.defaultchar;
   }
-
-  const Int32 w = font->getCharWidth(chr);
-  const Int32 h = font->getFontHeight();
   chr -= desc.firstchar;
-  const uInt32* tmp = desc.bits + (desc.offset ? desc.offset[chr] : (chr * h));
+ 
+  // Get the bounding box of the character
+  int bbw, bbh, bbx, bby;
+  if(!desc.bbx)
+  {
+    bbw = desc.fbbw;
+    bbh = desc.fbbh;
+    bbx = desc.fbbx;
+    bby = desc.fbby;
+  }
+  else
+  {
+    bbw = desc.bbx[chr].w;
+    bbh = desc.bbx[chr].h;
+    bbx = desc.bbx[chr].x;
+    bby = desc.bbx[chr].y;
+  }
 
-  // Scale the origins to the current zoom
-  xorig *= myZoomLevel;
-  yorig *= myZoomLevel;
-
-  SDL_LockSurface(myScreen);
-
-  int screenofsY = 0;
-  switch(myBytesPerPixel)
+  const uInt16* tmp = desc.bits + (desc.offset ? desc.offset[chr] : (chr * desc.fbbh));
+  switch(myFB.myBytesPerPixel)
   {
     case 2:
     {
       // Get buffer position where upper-left pixel of the character will be drawn
-      uInt16* buffer = (uInt16*) myScreen->pixels + myBaseOffset + yorig * myPitch + xorig;
-      for(int y = h; y; --y)
+      uInt16* buffer = (uInt16*)getBasePtr(tx + bbx, ty + desc.ascent - bby - bbh);
+
+      for(int y = 0; y < bbh; y++)
       {
-        const uInt32 fontbuf = *tmp++;
-        int ystride = myZoomLevel;
-        while(ystride--)
-        {
-          if(fontbuf)
-          {
-            uInt32 mask = 0x80000000;
-            int pos = screenofsY;
-            for(int x = 0; x < w; x++, mask >>= 1)
-            {
-              int xstride = myZoomLevel;
-              if((fontbuf & mask) != 0)
-                while(xstride--)
-                  buffer[pos++] = myDefPalette[color];
-              else
-                pos += xstride;
-            }
-          }
-          screenofsY += myPitch;
-        }
+        const uInt16 ptr = *tmp++;
+        uInt16 mask = 0x8000;
+ 
+        for(int x = 0; x < bbw; x++, mask >>= 1)
+          if(ptr & mask)
+            buffer[x] = (uInt16) myFB.myDefPalette[color];
+
+        buffer += myPitch;
       }
       break;
     }
     case 3:
     {
       // Get buffer position where upper-left pixel of the character will be drawn
-      uInt8* buffer = (uInt8*) myScreen->pixels + myBaseOffset + yorig * myPitch + xorig;
-      uInt32 pixel = myDefPalette[color];
-      uInt8 r = (pixel & myFormat->Rmask) >> myFormat->Rshift;
-      uInt8 g = (pixel & myFormat->Gmask) >> myFormat->Gshift;
-      uInt8 b = (pixel & myFormat->Bmask) >> myFormat->Bshift;
+      uInt8* buffer = (uInt8*)getBasePtr(tx + bbx, ty + desc.ascent - bby - bbh);
 
-      for(int y = h; y; --y)
+      uInt8 a = myFB.myDefPalette24[color][0],
+            b = myFB.myDefPalette24[color][1],
+            c = myFB.myDefPalette24[color][2];
+
+      for(int y = 0; y < bbh; y++, buffer += myPitch)
       {
-        const uInt32 fontbuf = *tmp++;
-        int ystride = myZoomLevel;
-        while(ystride--)
+        const uInt16 ptr = *tmp++;
+        uInt16 mask = 0x8000;
+
+        uInt8* buf_ptr = buffer;
+        for(int x = 0; x < bbw; x++, mask >>= 1)
         {
-          if(fontbuf)
+          if(ptr & mask)
           {
-            uInt32 mask = 0x80000000;
-            int pos = screenofsY;
-            for(int x = 0; x < w; x++, mask >>= 1)
-            {
-              int xstride = myZoomLevel;
-              if((fontbuf & mask) != 0)
-              {
-                while(xstride--)
-                {
-                  buffer[pos++] = r;  buffer[pos++] = g;  buffer[pos++] = b;
-                }
-              }
-              else
-                pos += xstride + xstride + xstride;
-            }
+            *buf_ptr++ = a;  *buf_ptr++ = b;  *buf_ptr++ = c;
           }
-          screenofsY += myPitch;
+          else
+            buf_ptr += 3;
         }
       }
       break;
@@ -592,181 +668,194 @@ void FrameBufferSoft::drawChar(const GUI::Font* font, uInt8 chr,
     case 4:
     {
       // Get buffer position where upper-left pixel of the character will be drawn
-      uInt32* buffer = (uInt32*) myScreen->pixels + myBaseOffset + yorig * myPitch + xorig;
-      for(int y = h; y; --y)
+      uInt32* buffer = (uInt32*)getBasePtr(tx + bbx, ty + desc.ascent - bby - bbh);
+      for(int y = 0; y < bbh; y++, buffer += myPitch)
       {
-        const uInt32 fontbuf = *tmp++;
-        int ystride = myZoomLevel;
-        while(ystride--)
-        {
-          if(fontbuf)
-          {
-            uInt32 mask = 0x80000000;
-            int pos = screenofsY;
-            for(int x = 0; x < w; x++, mask >>= 1)
-            {
-              int xstride = myZoomLevel;
-              if((fontbuf & mask) != 0)
-                while(xstride--)
-                  buffer[pos++] = myDefPalette[color];
-              else
-                pos += xstride;
-            }
-          }
-          screenofsY += myPitch;
-        }
+        const uInt16 ptr = *tmp++;
+        uInt16 mask = 0x8000;
+ 
+        for(int x = 0; x < bbw; x++, mask >>= 1)
+          if(ptr & mask)
+            buffer[x] = (uInt32) myFB.myDefPalette[color];
       }
       break;
     }
     default:
       break;
   }
-  SDL_UnlockSurface(myScreen);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void FrameBufferSoft::drawBitmap(uInt32* bitmap, Int32 xorig, Int32 yorig,
-                                 int color, Int32 h)
+void FBSurfaceSoft::drawBitmap(uInt32* bitmap, uInt32 tx, uInt32 ty,
+                               uInt32 color, uInt32 h)
 {
   SDL_Rect rect;
-  for(int y = 0; y < h; y++)
+  rect.y = ty + myYOffset;
+  rect.w = rect.h = 1;
+  for(uInt32 y = 0; y < h; y++)
   {
+    rect.x = tx + myXOffset;
     uInt32 mask = 0xF0000000;
-
-    for(int x = 0; x < 8; x++, mask >>= 4)
+    for(uInt32 x = 0; x < 8; x++, mask >>= 4)
     {
       if(bitmap[y] & mask)
-      {
-        rect.x = myImageDim.x + (x + xorig) * myZoomLevel;
-        rect.y = myImageDim.y + (y + yorig) * myZoomLevel;
-        rect.w = rect.h = myZoomLevel;
-        SDL_FillRect(myScreen, &rect, myDefPalette[color]);
-      }
+        SDL_FillRect(mySurface, &rect, myFB.myDefPalette[color]);
+
+      rect.x++;
     }
+    rect.y++;
   }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void FrameBufferSoft::drawSurface(const GUI::Surface* surface, Int32 x, Int32 y)
+void FBSurfaceSoft::drawPixels(uInt32* data, uInt32 tx, uInt32 ty,
+                               uInt32 numpixels)
 {
+  SDL_Rect rect;
+  rect.x = tx + myXOffset;
+  rect.y = ty + myYOffset;
+  rect.w = rect.h = 1;
+  for(uInt32 x = 0; x < numpixels; ++x)
+  {
+    SDL_FillRect(mySurface, &rect, data[x]);
+    rect.x++;
+  }
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void FBSurfaceSoft::drawSurface(const FBSurface* surface, uInt32 tx, uInt32 ty)
+{
+  const FBSurfaceSoft* s = (const FBSurfaceSoft*) surface;
+
   SDL_Rect dstrect;
-  dstrect.x = x * myZoomLevel + myImageDim.x;
-  dstrect.y = y * myZoomLevel + myImageDim.y;
+  dstrect.x = tx + myXOffset;
+  dstrect.y = ty + myYOffset;
   SDL_Rect srcrect;
   srcrect.x = 0;
   srcrect.y = 0;
-  srcrect.w = surface->myClipWidth * myZoomLevel;
-  srcrect.h = surface->myClipHeight * myZoomLevel;
+  srcrect.w = s->myWidth;
+  srcrect.h = s->myHeight;
 
-  SDL_BlitSurface(surface->myData, &srcrect, myScreen, &dstrect);
+  SDL_BlitSurface(s->mySurface, &srcrect, mySurface, &dstrect);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void FrameBufferSoft::bytesToSurface(GUI::Surface* surface, int row,
-                                     uInt8* data, int rowbytes) const
+void FBSurfaceSoft::addDirtyRect(uInt32 x, uInt32 y, uInt32 w, uInt32 h)
 {
-  // Calculate a scanline of zoomed surface data
-  SDL_Surface* s = surface->myData;
-  SDL_Rect rect;
-  rect.x = 0;
-  rect.y = row * myZoomLevel;
-  for(int c = 0; c < rowbytes; c += 3)
+//cerr << " -> addDirtyRect: x = " << x << ", y = " << y << ", w = " << w << ", h = " << h << endl;
+
+  // Base surfaces use dirty-rectangle updates, since they can be quite
+  // large, and updating the entire surface each frame would be too slow
+  // Non-base surfaces are usually smaller, and can be updated entirely
+  if(myIsBaseSurface)
   {
-    uInt32 pixel = SDL_MapRGB(s->format, data[c], data[c+1], data[c+2]);
-    rect.x += myZoomLevel;
-    rect.w = rect.h = myZoomLevel;
-    SDL_FillRect(surface->myData, &rect, pixel);
+    // Add a dirty rect to the UI rectangle list
+    // TODO - intelligent merging of rectangles, to avoid overlap
+    SDL_Rect temp;
+    temp.x = x + myXOrig;  temp.y = y + myYOrig;  temp.w = w;  temp.h = h;
+    myFB.myRectList->add(&temp);
+  }
+  else
+  {
+    SDL_Rect temp;
+    temp.x = myXOrig;  temp.y = myYOrig;  temp.w = myWidth;  temp.h = myHeight;
+    myFB.myRectList->add(&temp);
+
+    // Indicate that at least one dirty rect has been added
+    // This is an optimization for the update() method
+    mySurfaceIsDirty = true;
   }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void FrameBufferSoft::translateCoords(Int32& x, Int32& y) const
+void FBSurfaceSoft::getPos(uInt32& x, uInt32& y) const
 {
-  x = (x - myImageDim.x) / myZoomLevel;
-  y = (y - myImageDim.y) / myZoomLevel;
+  // Return the origin of the 'usable' area of a surface
+  if(myIsBaseSurface)
+  {
+    x = myXOffset;
+    y = myYOffset;
+  }
+  else
+  {
+    x = myXOrig;
+    y = myYOrig;
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void FrameBufferSoft::addDirtyRect(uInt32 x, uInt32 y, uInt32 w, uInt32 h)
+void FBSurfaceSoft::setPos(uInt32 x, uInt32 y)
 {
-  // Add a dirty rect to the UI rectangle list
-  // TODO - intelligent merging of rectangles, to avoid overlap
-  SDL_Rect temp;
-#if 1
-  temp.x = myImageDim.x + x * myZoomLevel;
-  temp.y = myImageDim.y + y * myZoomLevel;
-  temp.w = w * myZoomLevel;
-  temp.h = h * myZoomLevel;
-#else
-  temp.x = 0;
-  temp.y = 0;
-  temp.w = myScreenDim.w;
-  temp.h = myScreenDim.h;
-#endif
-  myRectList->add(&temp);
+  myXOrig = x;
+  myYOrig = y;
 
-//  cerr << "addDirtyRect():  "
-//       << "x=" << temp.x << ", y=" << temp.y << ", w=" << temp.w << ", h=" << temp.h << endl;
+  if(myIsBaseSurface)
+  {
+    myXOffset = myFB.imageRect().x();
+    myYOffset = myFB.imageRect().y();
+  }
+  else
+  {
+    myXOffset = myYOffset = 0;
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void FrameBufferSoft::enablePhosphor(bool enable, int blend)
+void FBSurfaceSoft::setWidth(uInt32 w)
 {
-  myUsePhosphor   = enable;
-  myPhosphorBlend = blend;
-
-  stateChanged(myOSystem->eventHandler().state());
+  myWidth = w;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void FrameBufferSoft::stateChanged(EventHandler::State state)
+void FBSurfaceSoft::setHeight(uInt32 h)
 {
-  if(!myScreen)
-    return;
+  myHeight = h;
+}
 
-  myInUIMode = (state == EventHandler::S_LAUNCHER ||
-                state == EventHandler::S_DEBUGGER);
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void FBSurfaceSoft::translateCoords(Int32& x, Int32& y) const
+{
+  x -= myXOrig;
+  y -= myYOrig;
+}
 
-  // Make sure drawMediaSource() knows which renderer to use
-  switch(myBytesPerPixel)
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void FBSurfaceSoft::update()
+{
+  // Since this method is called each frame, we only blit the surfaces when
+  // absolutely necessary
+  if(mySurfaceIsDirty /* && !myIsBaseSurface */)
+  {
+    SDL_Rect srcrect;
+    srcrect.x = 0;
+    srcrect.y = 0;
+    srcrect.w = myWidth;
+    srcrect.h = myHeight;
+
+    SDL_Rect dstrect;
+    dstrect.x = myXOrig;
+    dstrect.y = myYOrig;
+    dstrect.w = myWidth;
+    dstrect.h = myHeight;
+
+    SDL_BlitSurface(mySurface, &srcrect, myFB.myScreen, &dstrect);
+    mySurfaceIsDirty = false;
+  }
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void FBSurfaceSoft::reload()
+{
+  switch(mySurface->format->BytesPerPixel)
   {
     case 2:  // 16-bit
-      myPitch = myScreen->pitch/2;
-      if(myUsePhosphor)
-        myRenderType = kPhosphor_16;
-      else
-        myRenderType = kSoftZoom_16;
+      myPitch = mySurface->pitch >> 1;
       break;
     case 3:  // 24-bit
-      myPitch = myScreen->pitch;
-      if(myUsePhosphor)
-        myRenderType = kPhosphor_24;
-      else
-        myRenderType = kSoftZoom_24;
+      myPitch = mySurface->pitch;
       break;
     case 4:  // 32-bit
-      myPitch = myScreen->pitch/4;
-      if(myUsePhosphor)
-        myRenderType = kPhosphor_32;
-      else
-        myRenderType = kSoftZoom_32;
-      break;
-    default:
-      myRenderType = kSoftZoom_16; // What else should we do here?
+      myPitch = mySurface->pitch >> 2;
       break;
   }
-
-  // Have the changes take effect
-  myOSystem->eventHandler().refreshDisplay();
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-GUI::Surface* FrameBufferSoft::createSurface(int width, int height) const
-{
-  SDL_Surface* data =
-    SDL_CreateRGBSurface(SDL_SWSURFACE, width*myZoomLevel, height*myZoomLevel,
-                         myBytesPerPixel << 3, myFormat->Rmask, myFormat->Gmask,
-                         myFormat->Bmask, myFormat->Amask);
-
-  return data ? new GUI::Surface(width, height, data) : NULL;
 }
