@@ -8,17 +8,15 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: SoundNull.cxx,v 1.8 2008/02/06 13:45:19 stephena Exp $
+// $Id: SoundNull.cxx 2029 2010-04-29 12:46:07Z stephena $
 //============================================================================
 
-#include "Serializer.hxx"
-#include "Deserializer.hxx"
-
 #include "bspf.hxx"
 
 #include "OSystem.hxx"
@@ -27,11 +25,9 @@
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 SoundNull::SoundNull(OSystem* osystem)
-    : Sound(osystem)
+  : Sound(osystem)
 {
-  // Show some info
-  if(myOSystem->settings().getBool("showinfo"))
-    cout << "Sound disabled." << endl << endl;
+  myOSystem->logMessage("Sound disabled.\n\n", 1);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -40,9 +36,9 @@ SoundNull::~SoundNull()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool SoundNull::load(Deserializer& in)
+bool SoundNull::load(Serializer& in)
 {
-  string soundDevice = "TIASound";
+  const string& soundDevice = "TIASound";
   if(in.getString() != soundDevice)
     return false;
 
