@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: SoundSDL.cxx,v 1.44 2008/05/21 14:01:29 stephena Exp $
+// $Id: SoundSDL.cxx 2029 2010-04-29 12:46:07Z stephena $
 //============================================================================
 
 #ifdef SOUND_SUPPORT
@@ -25,18 +26,10 @@
 
 #include "TIASnd.hxx"
 #include "FrameBuffer.hxx"
-#include "Serializer.hxx"
-#include "Deserializer.hxx"
 #include "Settings.hxx"
 #include "System.hxx"
 #include "OSystem.hxx"
-
 #include "Console.hxx"
-#include "AtariVox.hxx"
-#ifdef SPEAKJET_EMULATION
-  #include "SpeakJet.hxx"
-#endif
-
 #include "SoundSDL.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -68,14 +61,13 @@ void SoundSDL::setEnabled(bool state)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void SoundSDL::initialize()
+void SoundSDL::open()
 {
   // Check whether to start the sound subsystem
   if(!myIsEnabled)
   {
     close();
-    if(myOSystem->settings().getBool("showinfo"))
-      cout << "Sound disabled." << endl << endl;
+    myOSystem->logMessage("Sound disabled.\n\n", 1);
     return;
   }
 
@@ -83,16 +75,18 @@ void SoundSDL::initialize()
   myRegWriteQueue.clear();
   myTIASound.reset();
 
-  if(!((SDL_WasInit(SDL_INIT_AUDIO) & SDL_INIT_AUDIO) > 0))
+  if(SDL_WasInit(SDL_INIT_AUDIO) == 0)
   {
     myIsInitializedFlag = false;
     myIsMuted = false;
     myLastRegisterSetCycle = 0;
 
+    ostringstream buf;
     if(SDL_InitSubSystem(SDL_INIT_AUDIO) < 0)
     {
-      cerr << "WARNING: Couldn't initialize SDL audio system! " << endl;
-      cerr << "         " << SDL_GetError() << endl;
+      buf << "WARNING: Couldn't initialize SDL audio system! " << endl
+          << "         " << SDL_GetError() << endl;
+      myOSystem->logMessage(buf.str(), 0);
       return;
     }
     else
@@ -115,8 +109,9 @@ void SoundSDL::initialize()
 
       if(SDL_OpenAudio(&desired, &myHardwareSpec) < 0)
       {
-        cerr << "WARNING: Couldn't open SDL audio system! " << endl;
-        cerr << "         " << SDL_GetError() << endl;
+        buf << "WARNING: Couldn't open SDL audio system! " << endl
+            << "         " << SDL_GetError() << endl;
+        myOSystem->logMessage(buf.str(), 0);
         return;
       }
 
@@ -124,9 +119,10 @@ void SoundSDL::initialize()
       // will not work so we'll need to disable the audio support)
       if(((float)myHardwareSpec.samples / (float)myHardwareSpec.freq) >= 0.25)
       {
-        cerr << "WARNING: Sound device doesn't support realtime audio! Make ";
-        cerr << "sure a sound" << endl;
-        cerr << "         server isn't running.  Audio is disabled." << endl;
+        buf << "WARNING: Sound device doesn't support realtime audio! Make "
+            << "sure a sound" << endl
+            << "         server isn't running.  Audio is disabled." << endl;
+        myOSystem->logMessage(buf.str(), 0);
 
         SDL_CloseAudio();
         return;
@@ -136,15 +132,6 @@ void SoundSDL::initialize()
       myIsMuted = false;
       myFragmentSizeLogBase2 = log((double)myHardwareSpec.samples) / log(2.0);
 
-		/*
-        cerr << "Freq: " << (int)myHardwareSpec.freq << endl;
-        cerr << "Format: " << (int)myHardwareSpec.format << endl;
-        cerr << "Channels: " << (int)myHardwareSpec.channels << endl;
-        cerr << "Silence: " << (int)myHardwareSpec.silence << endl;
-        cerr << "Samples: " << (int)myHardwareSpec.samples << endl;
-        cerr << "Size: " << (int)myHardwareSpec.size << endl;
-		  */
-
       // Now initialize the TIASound object which will actually generate sound
       myTIASound.outputFrequency(myHardwareSpec.freq);
       myTIASound.tiaFrequency(tiafreq);
@@ -158,15 +145,16 @@ void SoundSDL::initialize()
       setVolume(myVolume);
 
       // Show some info
-      if(myOSystem->settings().getBool("showinfo"))
-        cout << "Sound enabled:"  << endl
-             << "  Volume     : " << myVolume << endl
-             << "  Frag size  : " << fragsize << endl
-             << "  Frequency  : " << myHardwareSpec.freq << endl
-             << "  Format     : " << myHardwareSpec.format << endl
-             << "  TIA Freq.  : " << tiafreq << endl
-             << "  Channels   : " << myNumChannels << endl
-             << "  Clip volume: " << (int)clipvol << endl << endl;
+      buf << "Sound enabled:"  << endl
+          << "  Volume:      " << myVolume << endl
+          << "  Frag size:   " << fragsize << endl
+          << "  Frequency:   " << myHardwareSpec.freq << endl
+          << "  Format:      " << myHardwareSpec.format << endl
+          << "  TIA Freq:    " << tiafreq << endl
+          << "  Channels:    " << myNumChannels << endl
+          << "  Clip volume: " << (int)clipvol << endl
+          << endl;
+      myOSystem->logMessage(buf.str(), 1);
     }
   }
 
@@ -198,12 +186,6 @@ void SoundSDL::mute(bool state)
 {
   if(myIsInitializedFlag)
   {
-    // Ignore multiple calls to do the same thing
-    if(myIsMuted == state)
-    {
-      return;
-    }
-
     myIsMuted = state;
 
     SDL_PauseAudio(myIsMuted ? 1 : 0);
@@ -283,7 +265,7 @@ void SoundSDL::setChannels(uInt32 channels)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void SoundSDL::setFrameRate(float framerate)
 {
-  // FIXME, we should clear out the queue or adjust the values in it
+  // FIXME - should we clear out the queue or adjust the values in it?
   myDisplayFrameRate = framerate;
 }
 
@@ -301,8 +283,6 @@ void SoundSDL::set(uInt16 addr, uInt8 value, Int32 cycle)
   // the sound to "scale" correctly, we have to know the games real frame 
   // rate (e.g., 50 or 60) and the currently emulated frame rate. We use these
   // values to "scale" the time before the register change occurs.
-// FIXME - this always results in 1.0, so we don't really need it
-//  delta = delta * (myDisplayFrameRate / myOSystem->frameRate());
   RegWrite info;
   info.addr = addr;
   info.value = value;
@@ -336,7 +316,6 @@ void SoundSDL::processFragment(uInt8* stream, Int32 length)
       myTIASound.set(info.addr, info.value);
       myRegWriteQueue.dequeue();
     }
-//    cout << "Removed Items from RegWriteQueue!" << endl;
   }
 
   double position = 0.0;
@@ -411,33 +390,54 @@ void SoundSDL::callback(void* udata, uInt8* stream, int len)
 {
   SoundSDL* sound = (SoundSDL*)udata;
   sound->processFragment(stream, (Int32)len);
-
-#ifdef SPEAKJET_EMULATION
-//  cerr << "SoundSDL::callback(): len==" << len << endl;
-
-  // See if we need sound from the AtariVox
-  AtariVox *vox = sound->myOSystem->console().atariVox();
-  if(vox)
-  {
-    // If so, mix 'em together (this is a crappy way to mix audio streams...)
-    uInt8 *s = stream;
-    for(int i=0; i<len/OUTPUT_BUFFER_SIZE; i++)
-    {
-      int count;
-      uInt8 *voxSamples = vox->getSpeakJet()->getSamples(&count);
-      if(!count)
-        break;
-      SDL_MixAudio(s, voxSamples, OUTPUT_BUFFER_SIZE, SDL_MIX_MAXVOLUME);
-      s += OUTPUT_BUFFER_SIZE;
-    }
-  }
-#endif
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool SoundSDL::load(Deserializer& in)
+bool SoundSDL::save(Serializer& out) const
 {
-  string device = "TIASound";
+  const string& device = name();
+
+  try
+  {
+    out.putString(device);
+
+    uInt8 reg1 = 0, reg2 = 0, reg3 = 0, reg4 = 0, reg5 = 0, reg6 = 0;
+
+    // Only get the TIA sound registers if sound is enabled
+    if(myIsInitializedFlag)
+    {
+      reg1 = myTIASound.get(0x15);
+      reg2 = myTIASound.get(0x16);
+      reg3 = myTIASound.get(0x17);
+      reg4 = myTIASound.get(0x18);
+      reg5 = myTIASound.get(0x19);
+      reg6 = myTIASound.get(0x1a);
+    }
+
+    out.putByte((char)reg1);
+    out.putByte((char)reg2);
+    out.putByte((char)reg3);
+    out.putByte((char)reg4);
+    out.putByte((char)reg5);
+    out.putByte((char)reg6);
+
+    out.putInt(myLastRegisterSetCycle);
+  }
+  catch(const char* msg)
+  {
+    ostringstream buf;
+    buf << "ERROR: SoundSDL::save" << endl << "  " << msg << endl;
+    myOSystem->logMessage(buf.str(), 0);
+    return false;
+  }
+
+  return true;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool SoundSDL::load(Serializer& in)
+{
+  const string& device = name();
 
   try
   {
@@ -466,62 +466,14 @@ bool SoundSDL::load(Deserializer& in)
       myTIASound.set(0x18, reg4);
       myTIASound.set(0x19, reg5);
       myTIASound.set(0x1a, reg6);
-      SDL_PauseAudio(0);
+      if(!myIsMuted) SDL_PauseAudio(0);
     }
   }
-  catch(char *msg)
+  catch(const char* msg)
   {
-    cerr << msg << endl;
-    return false;
-  }
-  catch(...)
-  {
-    cerr << "Unknown error in load state for " << device << endl;
-    return false;
-  }
-
-  return true;
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool SoundSDL::save(Serializer& out)
-{
-  string device = "TIASound";
-
-  try
-  {
-    out.putString(device);
-
-    uInt8 reg1 = 0, reg2 = 0, reg3 = 0, reg4 = 0, reg5 = 0, reg6 = 0;
-
-    // Only get the TIA sound registers if sound is enabled
-    if(myIsInitializedFlag)
-    {
-      reg1 = myTIASound.get(0x15);
-      reg2 = myTIASound.get(0x16);
-      reg3 = myTIASound.get(0x17);
-      reg4 = myTIASound.get(0x18);
-      reg5 = myTIASound.get(0x19);
-      reg6 = myTIASound.get(0x1a);
-    }
-
-    out.putByte((char)reg1);
-    out.putByte((char)reg2);
-    out.putByte((char)reg3);
-    out.putByte((char)reg4);
-    out.putByte((char)reg5);
-    out.putByte((char)reg6);
-
-    out.putInt(myLastRegisterSetCycle);
-  }
-  catch(char *msg)
-  {
-    cerr << msg << endl;
-    return false;
-  }
-  catch(...)
-  {
-    cerr << "Unknown error in save state for " << device << endl;
+    ostringstream buf;
+    buf << "ERROR: SoundSDL::load" << endl << "  " << msg << endl;
+    myOSystem->logMessage(buf.str(), 0);
     return false;
   }
 
