@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: mainSDL.cxx,v 1.80 2008/02/06 13:45:19 stephena Exp $
+// $Id: mainSDL.cxx 2031 2010-05-03 12:01:43Z stephena $
 //============================================================================
 
 #include <SDL.h>
@@ -62,13 +63,17 @@
 OSystem* theOSystem = (OSystem*) NULL;
 
 // Does general Cleanup in case any operation failed (or at end of program)
-void Cleanup()
+int Cleanup()
 {
+  theOSystem->logMessage("Cleanup from mainSDL\n", 2);
+
   if(theOSystem)
     delete theOSystem;
 
   if(SDL_WasInit(SDL_INIT_VIDEO) & SDL_INIT_VIDEO)
     SDL_Quit();
+
+  return 0;
 }
 
 
@@ -97,42 +102,51 @@ int main(int argc, char* argv[])
 #endif
 
   theOSystem->settings().loadConfig();
+  theOSystem->logMessage("Loading config options ...\n", 2);
 
   // Take care of commandline arguments
+  theOSystem->logMessage("Loading commandline arguments ...\n", 2);
   string romfile = theOSystem->settings().loadCommandLine(argc, argv);
+  FilesystemNode romnode(romfile);
 
   // Finally, make sure the settings are valid
   // We do it once here, so the rest of the program can assume valid settings
+  theOSystem->logMessage("Validating config options ...\n", 2);
   theOSystem->settings().validate();
 
   // Create the full OSystem after the settings, since settings are
   // probably needed for defaults
-  theOSystem->create();
+  theOSystem->logMessage("Creating the OSystem ...\n", 2);
+  if(!theOSystem->create())
+  {
+    theOSystem->logMessage("ERROR: Couldn't create OSystem\n", 0);
+    return Cleanup();
+  }
 
   // Check to see if the user requested info about a specific ROM,
   // or the list of internal ROMs
   // If so, show the information and immediately exit
   if(theOSystem->settings().getBool("listrominfo"))
   {
+    theOSystem->logMessage("Showing output from 'listrominfo' ...\n", 2);
     theOSystem->propSet().print();
-    Cleanup();
-    return 0;
+    return Cleanup();
   }
   else if(theOSystem->settings().getBool("rominfo"))
   {
-    if(argc > 1 && FilesystemNode::fileExists(romfile))
-      cout << theOSystem->getROMInfo(romfile);
+    theOSystem->logMessage("Showing output from 'rominfo' ...\n", 2);
+    if(argc > 1 && romnode.exists())
+      theOSystem->logMessage(theOSystem->getROMInfo(romfile), 0);
     else
-      cout << "Error: ROM doesn't exist" << endl;
+      theOSystem->logMessage("ERROR: ROM doesn't exist\n", 0);
 
-    Cleanup();
-    return 0;
+    return Cleanup();
   }
   else if(theOSystem->settings().getBool("help"))
   {
+    theOSystem->logMessage("Displaying usage\n", 2);
     theOSystem->settings().usage();
-    Cleanup();
-    return 0;
+    return Cleanup();
   }
 
   // Request that the SDL window be centered, if possible
@@ -153,36 +167,38 @@ int main(int argc, char* argv[])
   //   the ROM actually exists, use it to create a new console.
   // If not, use the built-in ROM launcher.  In this case, we enter 'launcher'
   //   mode and let the main event loop take care of opening a new console/ROM.
-  if(argc == 1 || romfile == "" || !FilesystemNode::fileExists(romfile))
-    theOSystem->createLauncher();
+  if(argc == 1 || romfile == "" || !romnode.exists() || romnode.isDirectory())
+  {
+    theOSystem->logMessage("Attempting to use ROM launcher ...\n", 2);
+    if(theOSystem->settings().getBool("uselauncher"))
+    {
+      if(!theOSystem->createLauncher())
+        return Cleanup();
+    }
+    else
+    {
+      theOSystem->logMessage("Launcher could not be started, showing usage\n", 2);
+      theOSystem->settings().usage();
+      return Cleanup();
+    }
+  }
   else if(theOSystem->createConsole(romfile))
   {
     if(theOSystem->settings().getBool("takesnapshot"))
     {
-      for(int i = 0; i < 60; ++i)  theOSystem->frameBuffer().update();
-      theOSystem->console().system().reset();
+      theOSystem->logMessage("Taking snapshots with 'takesnapshot' ...\n", 2);
+      for(int i = 0; i < 30; ++i)  theOSystem->frameBuffer().update();
       theOSystem->eventHandler().takeSnapshot();
-      Cleanup();
-      return 0;
+      return Cleanup();
     }
 
-    if(theOSystem->settings().getBool("holdreset"))
-      theOSystem->eventHandler().handleEvent(Event::ConsoleReset, 1);
-
-    if(theOSystem->settings().getBool("holdselect"))
-      theOSystem->eventHandler().handleEvent(Event::ConsoleSelect, 1);
-
-    if(theOSystem->settings().getBool("holdbutton0"))
-      theOSystem->eventHandler().handleEvent(Event::JoystickZeroFire1, 1);
-
 #ifdef DEBUGGER_SUPPORT
-    Debugger& dbg = theOSystem->debugger();
-
     // Set up any breakpoint that was on the command line
     // (and remove the key from the settings, so they won't get set again)
     const string& initBreak = theOSystem->settings().getString("break");
     if(initBreak != "")
     {
+      Debugger& dbg = theOSystem->debugger();
       int bp = dbg.stringToValue(initBreak);
       dbg.setBreakPoint(bp, true);
       theOSystem->settings().setString("break", "");
@@ -193,10 +209,7 @@ int main(int argc, char* argv[])
 #endif
   }
   else
-  {
-    Cleanup();
-    return 0;
-  }
+    return Cleanup();
 
   // Swallow any spurious events in the queue
   // These are normally caused by joystick/mouse jitter
@@ -204,9 +217,10 @@ int main(int argc, char* argv[])
   while(SDL_PollEvent(&event)) /* swallow event */ ;
 
   // Start the main loop, and don't exit until the user issues a QUIT command
+  theOSystem->logMessage("Starting main loop ...\n", 2);
   theOSystem->mainLoop();
+  theOSystem->logMessage("Finished main loop ...\n", 2);
 
   // Cleanup time ...
-  Cleanup();
-  return 0;
+  return Cleanup();
 }
