@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Debugger.cxx,v 1.127 2008/05/18 20:04:30 stephena Exp $
+// $Id: Debugger.cxx 2005 2010-04-12 22:26:41Z stephena $
 //============================================================================
 
 #include "bspf.hxx"
@@ -26,6 +27,7 @@
 #include "Version.hxx"
 #include "OSystem.hxx"
 #include "FrameBuffer.hxx"
+#include "FSNode.hxx"
 #include "Settings.hxx"
 #include "DebuggerDialog.hxx"
 #include "DebuggerParser.hxx"
@@ -34,10 +36,10 @@
 #include "Console.hxx"
 #include "System.hxx"
 #include "M6502.hxx"
+#include "Cart.hxx"
 
-#include "EquateList.hxx"
+#include "CartDebug.hxx"
 #include "CpuDebug.hxx"
-#include "RamDebug.hxx"
 #include "RiotDebug.hxx"
 #include "TIADebug.hxx"
 
@@ -56,37 +58,53 @@
 Debugger* Debugger::myStaticDebugger;
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-static const string builtin_functions[][3] = {
-	// { "name", "definition", "help text" }
+static const char* builtin_functions[][3] = {
+  // { "name", "definition", "help text" }
 
-	// left joystick:
-	{ "_joy0left", "!(*SWCHA & $40)", "Left joystick moved left" },
-	{ "_joy0right", "!(*SWCHA & $80)", "Left joystick moved right" },
-	{ "_joy0up", "!(*SWCHA & $10)", "Left joystick moved up" },
-	{ "_joy0down", "!(*SWCHA & $20)", "Left joystick moved down" },
-	{ "_joy0button", "!(*INPT4 & $80)", "Left joystick button pressed" },
+  // left joystick:
+  { "_joy0left", "!(*SWCHA & $40)", "Left joystick moved left" },
+  { "_joy0right", "!(*SWCHA & $80)", "Left joystick moved right" },
+  { "_joy0up", "!(*SWCHA & $10)", "Left joystick moved up" },
+  { "_joy0down", "!(*SWCHA & $20)", "Left joystick moved down" },
+  { "_joy0button", "!(*INPT4 & $80)", "Left joystick button pressed" },
 
-	// right joystick:
-	{ "_joy1left", "!(*SWCHA & $04)", "Right joystick moved left" },
-	{ "_joy1right", "!(*SWCHA & $08)", "Right joystick moved right" },
-	{ "_joy1up", "!(*SWCHA & $01)", "Right joystick moved up" },
-	{ "_joy1down", "!(*SWCHA & $02)", "Right joystick moved down" },
-	{ "_joy1button", "!(*INPT5 & $80)", "Right joystick button pressed" },
+  // right joystick:
+  { "_joy1left", "!(*SWCHA & $04)", "Right joystick moved left" },
+  { "_joy1right", "!(*SWCHA & $08)", "Right joystick moved right" },
+  { "_joy1up", "!(*SWCHA & $01)", "Right joystick moved up" },
+  { "_joy1down", "!(*SWCHA & $02)", "Right joystick moved down" },
+  { "_joy1button", "!(*INPT5 & $80)", "Right joystick button pressed" },
 
-	// console switches:
-	{ "_select", "!(*SWCHB & $02)", "Game Select pressed" },
-	{ "_reset", "!(*SWCHB & $01)", "Game Reset pressed" },
-	{ "_color", "*SWCHB & $08", "Color/BW set to Color" },
-	{ "_bw", "!(*SWCHB & $08)", "Color/BW set to BW" },
-	{ "_diff0a", "!(*SWCHB & $40)", "Right difficulty set to A (easy)" },
-	{ "_diff0b", "*SWCHB & $40", "Right difficulty set to B (hard)" },
-	{ "_diff1a", "!(*SWCHB & $80)", "Right difficulty set to A (easy)" },
-	{ "_diff1b", "*SWCHB & $80", "Right difficulty set to B (hard)" },
+  // console switches:
+  { "_select", "!(*SWCHB & $02)", "Game Select pressed" },
+  { "_reset", "!(*SWCHB & $01)", "Game Reset pressed" },
+  { "_color", "*SWCHB & $08", "Color/BW set to Color" },
+  { "_bw", "!(*SWCHB & $08)", "Color/BW set to BW" },
+  { "_diff0b", "!(*SWCHB & $40)", "Left difficulty set to B (easy)" },
+  { "_diff0a", "*SWCHB & $40", "Left difficulty set to A (hard)" },
+  { "_diff1b", "!(*SWCHB & $80)", "Right difficulty set to B (easy)" },
+  { "_diff1a", "*SWCHB & $80", "Right difficulty set to A (hard)" },
 
-	// empty string marks end of list, do not remove
-	{ "", "", "" }
+  // empty string marks end of list, do not remove
+  { 0, 0, 0 }
 };
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+// Names are defined here, but processed in YaccParser
+static const char* pseudo_registers[][2] = {
+  // { "name", "help text" }
+
+  { "_bank", "Currently selected bank" },
+  { "_rwport", "Address at which a read from a write port occurred" },
+  { "_scan", "Current scanline count" },
+  { "_fcount", "Number of frames since emulation started" },
+  { "_cclocks", "Color clocks on current scanline" },
+  { "_vsync", "Whether vertical sync is enabled (1 or 0)" },
+  { "_vblank", "Whether vertical blank is enabled (1 or 0)" },
+
+  // empty string marks end of list, do not remove
+  { 0, 0 }
+};
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Debugger::Debugger(OSystem* osystem)
@@ -94,28 +112,28 @@ Debugger::Debugger(OSystem* osystem)
     myConsole(NULL),
     mySystem(NULL),
     myParser(NULL),
+    myCartDebug(NULL),
     myCpuDebug(NULL),
-    myRamDebug(NULL),
     myRiotDebug(NULL),
     myTiaDebug(NULL),
     myTiaInfo(NULL),
     myTiaOutput(NULL),
     myTiaZoom(NULL),
     myRom(NULL),
-    myEquateList(NULL),
     myBreakPoints(NULL),
     myReadTraps(NULL),
     myWriteTraps(NULL),
-    myWidth(1030),
-    myHeight(690)
+    myWidth(1050),
+    myHeight(620),
+    myRewindManager(NULL)
 {
   // Get the dialog size
   int w, h;
   myOSystem->settings().getSize("debuggerres", w, h);
   myWidth = BSPF_max(w, 0);
   myHeight = BSPF_max(h, 0);
-  myWidth = BSPF_max(myWidth, 1030u);
-  myHeight = BSPF_max(myHeight, 690u);
+  myWidth = BSPF_max(myWidth, 1050u);
+  myHeight = BSPF_max(myHeight, 620u);
   myOSystem->settings().setSize("debuggerres", myWidth, myHeight);
 
   // Init parser
@@ -136,21 +154,22 @@ Debugger::~Debugger()
 {
   delete myParser;
 
+  delete myCartDebug;
   delete myCpuDebug;
-  delete myRamDebug;
   delete myRiotDebug;
   delete myTiaDebug;
 
-  delete myEquateList;
   delete myBreakPoints;
   delete myReadTraps;
   delete myWriteTraps;
+
+  delete myRewindManager;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Debugger::initialize()
 {
-  GUI::Rect r = getDialogBounds();
+  const GUI::Rect& r = getDialogBounds();
 
   delete myBaseDialog;
   DebuggerDialog *dd = new DebuggerDialog(myOSystem, this,
@@ -163,15 +182,17 @@ void Debugger::initialize()
   myTiaZoom   = dd->tiaZoom();
   myRom       = dd->rom();
   myMessage   = dd->message();
+
+  myRewindManager = new RewindManager(*myOSystem, *dd->rewindButton());
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Debugger::initializeVideo()
+bool Debugger::initializeVideo()
 {
-  GUI::Rect r = getDialogBounds();
+  const GUI::Rect& r = getDialogBounds();
 
   string title = string("Stella ") + STELLA_VERSION + ": Debugger mode";
-  myOSystem->frameBuffer().initialize(title, r.width(), r.height());
+  return myOSystem->frameBuffer().initialize(title, r.width(), r.height());
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -187,8 +208,11 @@ void Debugger::setConsole(Console* console)
   delete myCpuDebug;
   myCpuDebug = new CpuDebug(*this, *myConsole);
 
-  delete myRamDebug;
-  myRamDebug = new RamDebug(*this, *myConsole);
+  delete myCartDebug;
+  // Register any RAM areas in the Cartridge
+  // Zero-page RAM is automatically recognized by CartDebug
+  myCartDebug = new CartDebug(*this, *myConsole, myConsole->cartridge().ramAreas());
+  myCartDebug->loadSymbolFile(myOSystem->romFile());
 
   delete myRiotDebug;
   myRiotDebug = new RiotDebug(*this, *myConsole);
@@ -196,16 +220,9 @@ void Debugger::setConsole(Console* console)
   delete myTiaDebug;
   myTiaDebug = new TIADebug(*this, *myConsole);
 
-  // Initialize equates and breakpoints to known state
-  delete myEquateList;
-  myEquateList = new EquateList();
+  // Initialize breakpoints to known state
   clearAllBreakPoints();
   clearAllTraps();
-
-  autoLoadSymbols(myOSystem->romFile());
-  loadListFile();
-
-  saveOldState();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -232,128 +249,37 @@ void Debugger::quit()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Debugger::autoLoadSymbols(string fileName)
+string Debugger::autoExec()
 {
-  string file = fileName;
+  ostringstream buf;
 
+  // autoexec.stella is always run
+  FilesystemNode autoexec(myOSystem->baseDir() + BSPF_PATH_SEPARATOR +
+                          "autoexec.stella");
+  buf << "autoExec():" << endl
+      << myParser->exec(autoexec) << endl;
+
+  // Also, "romname.stella" if present
+  string file = myOSystem->romFile();
   string::size_type pos;
   if( (pos = file.find_last_of('.')) != string::npos )
-    file.replace(pos, file.size(), ".sym");
+    file.replace(pos, file.size(), ".stella");
   else
-    file += ".sym";
+    file += ".stella";
 
-  string ret = myEquateList->loadFile(file);
-  //  cerr << "loading syms from file " << file << ": " << ret << endl;
-}
+  FilesystemNode romname(file);
+  buf << myParser->exec(romname) << endl;
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-string Debugger::loadListFile(string f)
-{
-  char buffer[255];
-
-  if(f == "")
+  // Init builtins
+  for(int i = 0; builtin_functions[i][0] != 0; i++)
   {
-    f = myOSystem->romFile();
-
-    string::size_type pos;
-    if( (pos = f.find_last_of('.')) != string::npos )
-      f.replace(pos, f.size(), ".lst");
-    else
-      f += ".lst";
+    // TODO - check this for memory leaks
+    int res = YaccParser::parse(builtin_functions[i][1]);
+    if(res != 0) cerr << "ERROR in builtin function!" << endl;
+    Expression* exp = YaccParser::getResult();
+    addFunction(builtin_functions[i][0], builtin_functions[i][1], exp, true);
   }
-
-  ifstream in(f.c_str());
-  if(!in.is_open())
-    return "Unable to read listing from " + f;
-
-  sourceLines.clear();
-  int count = 0;
-  while( !in.eof() )
-  {
-    if(!in.getline(buffer, 255))
-      break;
-
-    if(strlen(buffer) >= 14 &&
-       buffer[0] == ' '     &&
-       buffer[7] == ' '     &&
-       buffer[8] == ' '     &&
-       isxdigit(buffer[9])  &&
-       isxdigit(buffer[12]) &&
-       BSPF_isblank(buffer[13]))
-    {
-      count++;
-      char addr[5];
-      for(int i=0; i<4; i++)
-        addr[i] = buffer[9+i];
-
-      for(char *c = buffer; *c != '\0'; c++)
-        if(*c == '\t') *c = ' ';
-
-      addr[4] = '\0';
-      string a = addr;
-      string b = buffer;
-      sourceLines.insert(make_pair(a, b));
-    }
-  }
-  in.close();
-
-  return valueToString(count) + " lines loaded from " + f;
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-string Debugger::getSourceLines(int addr) const
-{
-  if(sourceLines.size() == 0)
-    return "";
-
-  string ret;
-  string want = to_hex_16(addr);
-
-  bool found = false;
-  pair<ListIter, ListIter> lines = sourceLines.equal_range(want);
-  for(ListIter i = lines.first; i != lines.second; i++)
-  {
-    found = true;
-    ret += i->second;
-    ret += "\n";
-  }
-
-  if(found)
-    return ret;
-  else
-    return "";
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Debugger::autoExec() {
-	// autoexec.stella is always run
-	myPrompt->print("autoExec():\n" +
-			myParser->exec(
-				myOSystem->baseDir() +
-				BSPF_PATH_SEPARATOR +
-				"autoexec.stella") +
-			"\n");
-
-	// also, "romname.stella" if present
-	string file = myOSystem->romFile();
-
-	string::size_type pos;
-	if( (pos = file.find_last_of('.')) != string::npos ) {
-		file.replace(pos, file.size(), ".stella");
-	} else {
-		file += ".stella";
-	}
-	myPrompt->print("autoExec():\n" + myParser->exec(file) + "\n");
-	myPrompt->printPrompt();
-
-	// init builtins
-	for(int i=0; builtin_functions[i][0] != ""; i++) {
-		string f = builtin_functions[i][1];
-		int res = YaccParser::parse(f.c_str());
-		if(res != 0) cerr << "ERROR in builtin function!" << endl;
-		Expression *exp = YaccParser::getResult();
-		addFunction(builtin_functions[i][0], builtin_functions[i][1], exp, true);
-	}
+  return buf.str();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -363,9 +289,9 @@ const string Debugger::run(const string& command)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const string Debugger::valueToString(int value, BaseFormat outputBase)
+string Debugger::valueToString(int value, BaseFormat outputBase)
 {
-  char rendered[32];
+  char buf[32];
 
   if(outputBase == kBASE_DEFAULT)
     outputBase = myParser->base();
@@ -374,32 +300,34 @@ const string Debugger::valueToString(int value, BaseFormat outputBase)
   {
     case kBASE_2:
       if(value < 0x100)
-        sprintf(rendered, Debugger::to_bin_8(value));
+        Debugger::to_bin(value, 8, buf);
       else
-        sprintf(rendered, Debugger::to_bin_16(value));
+        Debugger::to_bin(value, 16, buf);
       break;
 
     case kBASE_10:
       if(value < 0x100)
-        sprintf(rendered, "%3d", value);
+        sprintf(buf, "%3d", value);
       else
-        sprintf(rendered, "%5d", value);
+        sprintf(buf, "%5d", value);
       break;
 
     case kBASE_16_4:
-      sprintf(rendered, Debugger::to_hex_4(value));
+      strcpy(buf, Debugger::to_hex_4(value));
       break;
 
     case kBASE_16:
     default:
       if(value < 0x100)
-        sprintf(rendered, Debugger::to_hex_8(value));
+        sprintf(buf, "%02x", value);
+      else if(value < 0x10000)
+        sprintf(buf, "%04x", value);
       else
-        sprintf(rendered, Debugger::to_hex_16(value));
+        sprintf(buf, "%08x", value);
       break;
   }
 
-  return string(rendered);
+  return string(buf);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -418,8 +346,7 @@ const string Debugger::invIfChanged(int reg, int oldReg)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Debugger::reset()
 {
-  int pc = myCpuDebug->dPeek(0xfffc);
-  myCpuDebug->setPC(pc);
+  myCpuDebug->setPC(dpeek(0xfffc));
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -460,12 +387,13 @@ void Debugger::loadState(int state)
 int Debugger::step()
 {
   saveOldState();
+  mySystem->clearDirtyPages();
 
   int cyc = mySystem->cycles();
 
-  unlockState();
-  myOSystem->console().mediaSource().updateScanlineByStep();
-  lockState();
+  unlockBankswitchState();
+  myOSystem->console().tia().updateScanlineByStep();
+  lockBankswitchState();
 
   return mySystem->cycles() - cyc;
 }
@@ -487,13 +415,14 @@ int Debugger::trace()
   if(mySystem->peek(myCpuDebug->pc()) == 32)
   {
     saveOldState();
+    mySystem->clearDirtyPages();
 
     int cyc = mySystem->cycles();
     int targetPC = myCpuDebug->pc() + 3; // return address
 
-    unlockState();
-    myOSystem->console().mediaSource().updateScanlineByTrace(targetPC);
-    lockState();
+    unlockBankswitchState();
+    myOSystem->console().tia().updateScanlineByTrace(targetPC);
+    lockBankswitchState();
 
     return mySystem->cycles() - cyc;
   }
@@ -566,76 +495,40 @@ int Debugger::cycles()
   return mySystem->cycles();
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const string& Debugger::disassemble(int start, int lines)
-{
-  static string result;
-  ostringstream buffer;
-  string cpubuf;
-
-  do {
-    buffer << myEquateList->getLabel(start, true, 4) << ": ";
-
-    int count = myCpuDebug->disassemble(start, cpubuf, *myEquateList);
-    for(int i = 0; i < count; i++)
-      buffer << hex << setw(2) << setfill('0') << peek(start++) << dec;
-
-    if(count < 3) buffer << "   ";
-    if(count < 2) buffer << "   ";
-
-    buffer << " " << cpubuf << "\n";
-  } while(--lines > 0 && start <= 0xffff);
-
-  result = buffer.str();
-  return result;
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Debugger::disassemble(IntArray& addr, StringList& addrLabel,
-                           StringList& bytes, StringList& data,
-                           int start, int lines)
-{
-  string cpubuf, tmp;
-  char buf[255];
-
-  do
-  {
-    addrLabel.push_back(myEquateList->getLabel(start, true, 4) + ":");
-    addr.push_back(start);
-
-    cpubuf = "";
-    int count = myCpuDebug->disassemble(start, cpubuf, *myEquateList);
-
-    tmp = "";
-    for(int i=0; i<count; i++) {
-      sprintf(buf, "%02x ", peek(start++));
-      tmp += buf;
-    }
-    bytes.push_back(tmp);
-
-    data.push_back(cpubuf);
-  }
-  while(--lines > 0 && start <= 0xffff);
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Debugger::nextScanline(int lines)
 {
   saveOldState();
+  mySystem->clearDirtyPages();
 
-  unlockState();
-  myTiaOutput->advanceScanline(lines);
-  lockState();
+  unlockBankswitchState();
+  while(lines)
+  {
+    myOSystem->console().tia().updateScanline();
+    --lines;
+  }
+  lockBankswitchState();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Debugger::nextFrame(int frames)
 {
   saveOldState();
+  mySystem->clearDirtyPages();
 
-  unlockState();
-  myTiaOutput->advance(frames);
-  lockState();
+  unlockBankswitchState();
+  while(frames)
+  {
+    myOSystem->console().tia().update();
+    --frames;
+  }
+  lockBankswitchState();
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool Debugger::rewindState()
+{
+  return myRewindManager->rewindState();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -656,30 +549,12 @@ void Debugger::clearAllTraps()
   mySystem->m6502().setTraps(NULL, NULL);
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int Debugger::peek(int addr)
-{
-  return mySystem->peek(addr);
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int Debugger::dpeek(int addr)
-{
-  return mySystem->peek(addr) | (mySystem->peek(addr+1) << 8);
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 string Debugger::showWatches()
 {
   return myParser->showWatches();
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Debugger::addLabel(string label, int address)
-{
-  myEquateList->addEquate(label, address);
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Debugger::reloadROM()
 {
@@ -699,24 +574,6 @@ bool Debugger::setBank(int bank)
   return false;
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int Debugger::getBank()
-{
-  return myConsole->cartridge().bank();
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int Debugger::bankCount()
-{
-  return myConsole->cartridge().bankCount();
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-string Debugger::getCartType()
-{
-  return myConsole->cartridge().name();
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Debugger::patchROM(int addr, int value)
 {
@@ -724,26 +581,35 @@ bool Debugger::patchROM(int addr, int value)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Debugger::saveOldState()
+void Debugger::saveOldState(bool addrewind)
 {
+  myCartDebug->saveOldState();
   myCpuDebug->saveOldState();
-  myRamDebug->saveOldState();
   myRiotDebug->saveOldState();
   myTiaDebug->saveOldState();
+
+  // Add another rewind level to the Undo list
+  if(addrewind)  myRewindManager->addState();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Debugger::setStartState()
 {
   // Lock the bus each time the debugger is entered, so we don't disturb anything
-  lockState();
+  lockBankswitchState();
+
+  // Start a new rewind list
+  myRewindManager->clear();
+
+  // Save initial state, but don't add it to the rewind list
+  saveOldState(false);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Debugger::setQuitState()
 {
   // Bus must be unlocked for normal operation when leaving debugger mode
-  unlockState();
+  unlockBankswitchState();
 
   // execute one instruction on quit. If we're
   // sitting at a breakpoint/trap, this will get us past it.
@@ -772,8 +638,8 @@ GUI::Rect Debugger::getTiaBounds() const
 GUI::Rect Debugger::getRomBounds() const
 {
   // The ROM area is the full area to the right of the tabs
-  GUI::Rect dialog = getDialogBounds();
-  GUI::Rect status = getStatusBounds();
+  const GUI::Rect& dialog = getDialogBounds();
+  const GUI::Rect& status = getStatusBounds();
 
   int x1 = status.right + 1;
   int y1 = 0;
@@ -790,8 +656,8 @@ GUI::Rect Debugger::getStatusBounds() const
   // The status area is the full area to the right of the TIA image
   // extending as far as necessary
   // 30% of any space above 1030 pixels will be allocated to this area
-  GUI::Rect dlg = getDialogBounds();
-  GUI::Rect tia = getTiaBounds();
+  const GUI::Rect& dlg = getDialogBounds();
+  const GUI::Rect& tia = getTiaBounds();
 
   int x1 = tia.right + 1;
   int y1 = 0;
@@ -807,9 +673,9 @@ GUI::Rect Debugger::getStatusBounds() const
 GUI::Rect Debugger::getTabBounds() const
 {
   // The tab area is the full area below the TIA image
-  GUI::Rect dialog = getDialogBounds();
-  GUI::Rect tia    = getTiaBounds();
-  GUI::Rect status = getStatusBounds();
+  const GUI::Rect& dialog = getDialogBounds();
+  const GUI::Rect& tia    = getTiaBounds();
+  const GUI::Rect& status = getStatusBounds();
 
   int x1 = 0;
   int y1 = tia.bottom + 1;
@@ -821,35 +687,38 @@ GUI::Rect Debugger::getTabBounds() const
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Debugger::addFunction(const string& name, const string& definition,
+bool Debugger::addFunction(const string& name, const string& definition,
                            Expression* exp, bool builtin)
 {
   functions.insert(make_pair(name, exp));
   if(!builtin)
     functionDefs.insert(make_pair(name, definition));
+
+  return true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Debugger::delFunction(const string& name)
+bool Debugger::delFunction(const string& name)
 {
   FunctionMap::iterator iter = functions.find(name);
   if(iter == functions.end())
-    return;
+    return false;
 
   functions.erase(name);
   delete iter->second;
 
   FunctionDefMap::iterator def_iter = functionDefs.find(name);
   if(def_iter == functionDefs.end())
-    return;
+    return false;
 
   functionDefs.erase(name);
+  return true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-Expression* Debugger::getFunction(const string& name)
+const Expression* Debugger::getFunction(const string& name) const
 {
-  FunctionMap::iterator iter = functions.find(name);
+  FunctionMap::const_iterator iter = functions.find(name);
   if(iter == functions.end())
     return 0;
   else
@@ -857,11 +726,11 @@ Expression* Debugger::getFunction(const string& name)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-string Debugger::getFunctionDef(const string& name)
+const string& Debugger::getFunctionDef(const string& name) const
 {
-  FunctionDefMap::iterator iter = functionDefs.find(name);
+  FunctionDefMap::const_iterator iter = functionDefs.find(name);
   if(iter == functionDefs.end())
-    return "";
+    return EmptyString;
   else
     return iter->second;
 }
@@ -873,43 +742,174 @@ const FunctionDefMap Debugger::getFunctionDefMap() const
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const string Debugger::builtinHelp() const
+string Debugger::builtinHelp() const
 {
-  string result;
+  ostringstream buf;
+  uInt16 len, c_maxlen = 0, i_maxlen = 0;
 
-  for(int i=0; builtin_functions[i][0] != ""; i++)
+  // Get column widths for aligned output (functions)
+  for(int i = 0; builtin_functions[i][0] != 0; ++i)
   {
-    result += builtin_functions[i][0];
-    result += " {";
-    result += builtin_functions[i][1];
-    result += "}\n";
-    result += "     ";
-    result += builtin_functions[i][2];
-    result += "\n";
+    len = strlen(builtin_functions[i][0]);
+    if(len > c_maxlen)  c_maxlen = len;
+    len = strlen(builtin_functions[i][1]);
+    if(len > i_maxlen)  i_maxlen = len;
   }
-  return result;
+
+  buf << endl << "Built-in functions:" << endl;
+  for(int i = 0; builtin_functions[i][0] != 0; ++i)
+  {
+    buf << setw(c_maxlen) << left << builtin_functions[i][0]
+        << setw(2) << right << "{"
+        << setw(i_maxlen) << left << builtin_functions[i][1]
+        << setw(4) << "}"
+        << builtin_functions[i][2]
+        << endl;
+  }
+
+  // Get column widths for aligned output (pseudo-registers)
+  c_maxlen = 0;
+  for(int i = 0; pseudo_registers[i][0] != 0; ++i)
+  {
+    len = strlen(pseudo_registers[i][0]);
+    if(len > c_maxlen)  c_maxlen = len;
+  }
+
+  buf << endl << "Pseudo-registers:" << endl;
+  for(int i = 0; pseudo_registers[i][0] != 0; ++i)
+  {
+    buf << setw(c_maxlen) << left << pseudo_registers[i][0]
+        << setw(2) << " "
+        << setw(i_maxlen) << left << pseudo_registers[i][1]
+        << endl;
+  }
+
+  return buf.str();
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Debugger::getCompletions(const char* in, StringList& list) const
+{
+  FunctionMap::const_iterator iter;
+  for(iter = functions.begin(); iter != functions.end(); ++iter)
+  {
+    const char* l = iter->first.c_str();
+    if(BSPF_strncasecmp(l, in, strlen(in)) == 0)
+      list.push_back(l);
+  }
+
+  for(int i = 0; pseudo_registers[i][0] != 0; ++i)
+    if(BSPF_strncasecmp(pseudo_registers[i][0], in, strlen(in)) == 0)
+      list.push_back(pseudo_registers[i][0]);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Debugger::saveROM(const string& filename) const
 {
-  // TODO: error checking
-  ofstream *out = new ofstream(filename.c_str(), ios::out | ios::binary);
-  bool res = myConsole->cartridge().save(*out);
-  delete out;
-  return res;
+  ofstream out(filename.c_str(), ios::out | ios::binary);
+  if(out.is_open())
+    return myConsole->cartridge().save(out);
+  else
+    return false;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Debugger::lockState()
+void Debugger::lockBankswitchState()
 {
   mySystem->lockDataBus();
   myConsole->cartridge().lockBank();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Debugger::unlockState()
+void Debugger::unlockBankswitchState()
 {
   mySystem->unlockDataBus();
   myConsole->cartridge().unlockBank();
 }
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+Debugger::RewindManager::RewindManager(OSystem& system, ButtonWidget& button)
+  : myOSystem(system),
+    myRewindButton(button),
+    mySize(0),
+    myTop(0)
+{
+  for(int i = 0; i < MAX_SIZE; ++i)
+    myStateList[i] = (Serializer*) NULL;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+Debugger::RewindManager::~RewindManager()
+{
+  for(int i = 0; i < MAX_SIZE; ++i)
+    delete myStateList[i];
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool Debugger::RewindManager::addState()
+{
+  // Create a new Serializer object if we need one
+  if(myStateList[myTop] == NULL)
+    myStateList[myTop] = new Serializer();
+  Serializer& s = *(myStateList[myTop]);
+
+  if(s.isValid())
+  {
+    s.reset();
+    if(myOSystem.state().saveState(s) && myOSystem.console().tia().saveDisplay(s))
+    {
+      // Are we still within the allowable size, or are we overwriting an item?
+      mySize++; if(mySize > MAX_SIZE) mySize = MAX_SIZE;
+
+      myTop = (myTop + 1) % MAX_SIZE;
+      myRewindButton.setEnabled(true);
+      return true;
+    }
+  }
+  return false;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool Debugger::RewindManager::rewindState()
+{
+  if(mySize > 0)
+  {
+    mySize--;
+    myTop = myTop == 0 ? MAX_SIZE - 1 : myTop - 1;
+    Serializer& s = *(myStateList[myTop]);
+
+    s.reset();
+    myOSystem.state().loadState(s);
+    myOSystem.console().tia().loadDisplay(s);
+
+    if(mySize == 0)
+      myRewindButton.setEnabled(false);
+
+    return true;
+  }
+  else
+    return false;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool Debugger::RewindManager::isEmpty()
+{
+  return mySize == 0;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Debugger::RewindManager::clear()
+{
+  for(int i = 0; i < MAX_SIZE; ++i)
+    if(myStateList[i] != NULL)
+      myStateList[i]->reset();
+
+  myTop = mySize = 0;
+
+  // We use Widget::clearFlags here instead of Widget::setEnabled(),
+  // since the latter implies an immediate draw/update, but this method
+  // might be called before any UI exists
+  // TODO - fix this deficiency in the UI core; we shouldn't have to worry
+  //        about such things at this level
+  myRewindButton.clearFlags(WIDGET_ENABLED);
+}
