@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: RiotDebug.cxx,v 1.6 2008/05/15 18:59:56 stephena Exp $
+// $Id: RiotDebug.cxx 2001 2010-04-10 21:37:23Z stephena $
 //============================================================================
 
 #include <sstream>
@@ -53,13 +54,13 @@ const DebuggerState& RiotDebug::getState()
   myState.TIMCLKS  = timClocks();
 
   // Controller port pins
-  Controller& port0 = myConsole.controller(Controller::Left);
+  const Controller& port0 = myConsole.controller(Controller::Left);
   myState.P0_PIN1 = port0.myDigitalPinState[Controller::One];
   myState.P0_PIN2 = port0.myDigitalPinState[Controller::Two];
   myState.P0_PIN3 = port0.myDigitalPinState[Controller::Three];
   myState.P0_PIN4 = port0.myDigitalPinState[Controller::Four];
   myState.P0_PIN6 = port0.myDigitalPinState[Controller::Six];
-  Controller& port1 = myConsole.controller(Controller::Right);
+  const Controller& port1 = myConsole.controller(Controller::Right);
   myState.P1_PIN1 = port1.myDigitalPinState[Controller::One];
   myState.P1_PIN2 = port1.myDigitalPinState[Controller::Two];
   myState.P1_PIN3 = port1.myDigitalPinState[Controller::Three];
@@ -92,13 +93,13 @@ void RiotDebug::saveOldState()
   myOldState.TIMCLKS  = timClocks();
 
   // Controller port pins
-  Controller& port0 = myConsole.controller(Controller::Left);
+  const Controller& port0 = myConsole.controller(Controller::Left);
   myOldState.P0_PIN1 = port0.myDigitalPinState[Controller::One];
   myOldState.P0_PIN2 = port0.myDigitalPinState[Controller::Two];
   myOldState.P0_PIN3 = port0.myDigitalPinState[Controller::Three];
   myOldState.P0_PIN4 = port0.myDigitalPinState[Controller::Four];
   myOldState.P0_PIN6 = port0.myDigitalPinState[Controller::Six];
-  Controller& port1 = myConsole.controller(Controller::Right);
+  const Controller& port1 = myConsole.controller(Controller::Right);
   myOldState.P1_PIN1 = port1.myDigitalPinState[Controller::One];
   myOldState.P1_PIN2 = port1.myDigitalPinState[Controller::Two];
   myOldState.P1_PIN3 = port1.myDigitalPinState[Controller::Three];
@@ -265,7 +266,7 @@ string RiotDebug::dirP0String()
       << (reg & 0x40 ? "" : "left ")
       << (reg & 0x20 ? "" : "left ")
       << (reg & 0x10 ? "" : "left ")
-      << (reg & 0xf0 == 0xf0 ? "(no directions) " : "");
+      << ((reg & 0xf0) == 0xf0 ? "(no directions) " : "");
   return buf.str();
 }
 
@@ -278,7 +279,7 @@ string RiotDebug::dirP1String()
       << (reg & 0x04 ? "" : "left ")
       << (reg & 0x02 ? "" : "left ")
       << (reg & 0x01 ? "" : "left ")
-      << (reg & 0x0f == 0x0f ? "(no directions) " : "");
+      << ((reg & 0x0f) == 0x0f ? "(no directions) " : "");
   return buf.str();
 }
 
@@ -314,7 +315,7 @@ string RiotDebug::toString()
 {
   // TODO: keyboard controllers?
 
-  const RiotState& state = (RiotState&) getState();
+  const RiotState& state    = (RiotState&) getState();
   const RiotState& oldstate = (RiotState&) getOldState();
   string ret;
 
