@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: TIADebug.cxx,v 1.27 2008/04/19 21:11:52 stephena Exp $
+// $Id: TIADebug.cxx 2001 2010-04-10 21:37:23Z stephena $
 //============================================================================
 
 #include "System.hxx"
@@ -24,16 +25,16 @@
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 TIADebug::TIADebug(Debugger& dbg, Console& console)
   : DebuggerSystem(dbg, console),
-    myTIA((TIA&)console.mediaSource())
+    myTIA(console.tia())
 {
-  nusizStrings[0] = "size=8 copy=1";
-  nusizStrings[1] = "size=8 copy=2 spac=8";
-  nusizStrings[2] = "size=8 copy=2 spac=$18";
-  nusizStrings[3] = "size=8 copy=3 spac=8";
-  nusizStrings[4] = "size=8 copy=2 spac=$38";
-  nusizStrings[5] = "size=$10 copy=1";
-  nusizStrings[6] = "size=8 copy=3 spac=$18";
-  nusizStrings[7] = "size=$20 copy=1";
+  nusizStrings[0] = "1 copy";
+  nusizStrings[1] = "2 copies - close (8)";
+  nusizStrings[2] = "2 copies - med (24)";
+  nusizStrings[3] = "3 copies - close (8)";
+  nusizStrings[4] = "2 copies - wide (56)";
+  nusizStrings[5] = "2x (16) sized player";
+  nusizStrings[6] = "3 copies - med (24)";
+  nusizStrings[7] = "4x (32) sized player";
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -157,25 +158,25 @@ void TIADebug::saveOldState()
 }
 
 /* the set methods now use mySystem.poke(). This will save us the
-	trouble of masking the values here, since TIA::poke() will do it
-	for us.
+   trouble of masking the values here, since TIA::poke() will do it
+   for us.
 
-	This means that the GUI should *never* just display the value the
-	user entered: it should always read the return value of the set
-	method and display that.
+   This means that the GUI should *never* just display the value the
+   user entered: it should always read the return value of the set
+   method and display that.
 
-	An Example:
+   An Example:
 
-	User enters "ff" in the AUDV0 field. GUI calls value = tiaDebug->audV0(0xff).
-	The AUDV0 register is only 4 bits wide, so "value" is 0x0f. That's what
-	should be displayed.
+   User enters "ff" in the AUDV0 field. GUI calls value = tiaDebug->audV0(0xff).
+   The AUDV0 register is only 4 bits wide, so "value" is 0x0f. That's what
+   should be displayed.
 
-	In a perfect world, the GUI would only allow one hex digit to be entered...
-	but we allow decimal or binary input in the GUI (with # or \ prefix). The
-	only way to make that work would be to validate the data entry after every
-	keystroke... which would be a pain for both us and the user. Using poke()
-	here is a compromise that allows the TIA to do the range-checking for us,
-	so the GUI and/or TIADebug don't have to duplicate logic from TIA::poke().
+   In a perfect world, the GUI would only allow one hex digit to be entered...
+   but we allow decimal or binary input in the GUI (with # or \ prefix). The
+   only way to make that work would be to validate the data entry after every
+   keystroke... which would be a pain for both us and the user. Using poke()
+   here is a compromise that allows the TIA to do the range-checking for us,
+   so the GUI and/or TIADebug don't have to duplicate logic from TIA::poke().
 */
 
 //	 bool vdelP0(int newVal = -1);
@@ -423,7 +424,7 @@ uInt8 TIADebug::coluP0(int newVal)
   if(newVal > -1)
     mySystem.poke(COLUP0, newVal);
 
-  return myTIA.myCOLUP0 & 0xff;
+  return myTIA.myColor[TIA::_P0] & 0xff;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -432,7 +433,7 @@ uInt8 TIADebug::coluP1(int newVal)
   if(newVal > -1)
     mySystem.poke(COLUP1, newVal);
 
-  return myTIA.myCOLUP1 & 0xff;
+  return myTIA.myColor[TIA::_P1] & 0xff;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -441,7 +442,7 @@ uInt8 TIADebug::coluPF(int newVal)
   if(newVal > -1)
     mySystem.poke(COLUPF, newVal);
 
-  return myTIA.myCOLUPF & 0xff;
+  return myTIA.myColor[TIA::_PF] & 0xff;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -450,7 +451,7 @@ uInt8 TIADebug::coluBK(int newVal)
   if(newVal > -1)
     mySystem.poke(COLUBK, newVal);
 
-  return myTIA.myCOLUBK & 0xff;
+  return myTIA.myColor[TIA::_BK] & 0xff;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -546,6 +547,7 @@ uInt8 TIADebug::posP0(int newVal)
 {
   if(newVal > -1)
     myTIA.myPOSP0 = newVal;
+
   return myTIA.myPOSP0;
 }
 
@@ -554,36 +556,34 @@ uInt8 TIADebug::posP1(int newVal)
 {
   if(newVal > -1)
     myTIA.myPOSP1 = newVal;
+
   return myTIA.myPOSP1;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 TIADebug::posM0(int newVal)
 {
-/* FIXME
   if(newVal > -1)
-    mySystem.poke(???, newVal);
-*/
+    myTIA.myPOSM0 = newVal;
+
   return myTIA.myPOSM0;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 TIADebug::posM1(int newVal)
 {
-/* FIXME
   if(newVal > -1)
-    mySystem.poke(???, newVal);
-*/
+    myTIA.myPOSM1 = newVal;
+
   return myTIA.myPOSM1;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 TIADebug::posBL(int newVal)
 {
-/* FIXME
   if(newVal > -1)
-    mySystem.poke(???, newVal);
-*/
+    myTIA.myPOSBL = newVal;
+
   return myTIA.myPOSBL;
 }
 
@@ -615,7 +615,7 @@ uInt8 TIADebug::hmP0(int newVal)
   if(newVal > -1)
     mySystem.poke(HMP0, newVal << 4);
 
-  return myTIA.myHMP0;
+  return myTIA.myHMP0 >> 4;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -624,7 +624,7 @@ uInt8 TIADebug::hmP1(int newVal)
   if(newVal > -1)
     mySystem.poke(HMP1, newVal << 4);
 
-  return myTIA.myHMP1;
+  return myTIA.myHMP1 >> 4;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -633,7 +633,7 @@ uInt8 TIADebug::hmM0(int newVal)
   if(newVal > -1)
     mySystem.poke(HMM0, newVal << 4);
 
-  return myTIA.myHMM0;
+  return myTIA.myHMM0 >> 4;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -642,7 +642,7 @@ uInt8 TIADebug::hmM1(int newVal)
   if(newVal > -1)
     mySystem.poke(HMM1, newVal << 4);
 
-  return myTIA.myHMM1;
+  return myTIA.myHMM1 >> 4;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -651,7 +651,7 @@ uInt8 TIADebug::hmBL(int newVal)
   if(newVal > -1)
     mySystem.poke(HMBL, newVal << 4);
 
-  return myTIA.myHMBL;
+  return myTIA.myHMBL >> 4;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
