@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: ColorWidget.cxx,v 1.8 2008/02/06 13:45:20 stephena Exp $
+// $Id: ColorWidget.cxx 2001 2010-04-10 21:37:23Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -52,26 +53,17 @@ void ColorWidget::setColor(int color)
   setDirty(); draw();
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void ColorWidget::handleMouseDown(int x, int y, int button, int clickCount)
-{
-// TODO - add ColorDialog, which will show all 256 colors in the
-//         TIA palette
-//  if(isEnabled())
-//    parent()->addDialog(myColorDialog);
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void ColorWidget::drawWidget(bool hilite)
 {
-  FrameBuffer& fb = instance()->frameBuffer();
+  FBSurface& s = dialog().surface();
 
   // Draw a thin frame around us.
-  fb.hLine(_x, _y, _x + _w - 1, kColor);
-  fb.hLine(_x, _y +_h, _x + _w - 1, kShadowColor);
-  fb.vLine(_x, _y, _y+_h, kColor);
-  fb.vLine(_x + _w - 1, _y, _y +_h - 1, kShadowColor);
+  s.hLine(_x, _y, _x + _w - 1, kColor);
+  s.hLine(_x, _y +_h, _x + _w - 1, kShadowColor);
+  s.vLine(_x, _y, _y+_h, kColor);
+  s.vLine(_x + _w - 1, _y, _y +_h - 1, kShadowColor);
 
   // Show the currently selected color
-  fb.fillRect(_x+1, _y+1, _w-2, _h-1, _color);
+  s.fillRect(_x+1, _y+1, _w-2, _h-1, _color);
 }
