@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CpuWidget.cxx,v 1.13 2008/05/15 18:59:56 stephena Exp $
+// $Id: CpuWidget.cxx 2001 2010-04-10 21:37:23Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -24,6 +25,7 @@
 #include "OSystem.hxx"
 #include "GuiObject.hxx"
 #include "Debugger.hxx"
+#include "CartDebug.hxx"
 #include "CpuDebug.hxx"
 #include "Widget.hxx"
 #include "DataGridWidget.hxx"
@@ -32,31 +34,6 @@
 
 #include "CpuWidget.hxx"
 
-// ID's for the various widgets
-// We need ID's, since there are more than one of several types of widgets
-enum {
-  kPCRegID,
-  kCpuRegID
-};
-
-enum {
-  kPCRegAddr,
-  kSPRegAddr,
-  kARegAddr,
-  kXRegAddr,
-  kYRegAddr
-};
-
-enum {
-  kPSRegN = 0,
-  kPSRegV = 1,
-  kPSRegB = 3,
-  kPSRegD = 4,
-  kPSRegI = 5,
-  kPSRegZ = 6,
-  kPSRegC = 7
-};
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 CpuWidget::CpuWidget(GuiObject* boss, const GUI::Font& font, int x, int y)
   : Widget(boss, font, x, y, 16, 16),
@@ -73,24 +50,38 @@ CpuWidget::CpuWidget(GuiObject* boss, const GUI::Font& font, int x, int y)
   xpos = x;  ypos = y;  lwidth = 4 * fontWidth;
   new StaticTextWidget(boss, font, xpos, ypos+2, lwidth-2, fontHeight,
                        "PC:", kTextAlignLeft);
-  myPCGrid = new DataGridWidget(boss, font, xpos + lwidth, ypos, 1, 1, 16, 16);
+  myPCGrid =
+    new DataGridWidget(boss, font, xpos + lwidth, ypos, 1, 1, 4, 16, kBASE_16);
   myPCGrid->setTarget(this);
   myPCGrid->setID(kPCRegID);
   addFocusWidget(myPCGrid);
 
   // Create a read-only textbox containing the current PC label
   xpos += lwidth + myPCGrid->getWidth() + 10;
-  myPCLabel = new EditTextWidget(boss, font, xpos, ypos, fontWidth*15,
+  myPCLabel = new EditTextWidget(boss, font, xpos, ypos, fontWidth*25,
                                  lineHeight, "");
   myPCLabel->setEditable(false);
 
   // Create a 1x4 grid with labels for the other CPU registers
-  xpos = x;  ypos += myPCGrid->getHeight() + 1;
-  myCpuGrid = new DataGridWidget(boss, font, xpos + lwidth, ypos, 1, 4, 8, 8);
+  xpos = x + lwidth;  ypos += myPCGrid->getHeight() + 1;
+  myCpuGrid =
+    new DataGridWidget(boss, font, xpos, ypos, 1, 4, 2, 8, kBASE_16);
   myCpuGrid->setTarget(this);
   myCpuGrid->setID(kCpuRegID);
   addFocusWidget(myCpuGrid);
 
+  // Create a 1x4 grid with decimal and binary values for the other CPU registers
+  xpos = x + lwidth + myPCGrid->getWidth() + 10;
+  myCpuGridDecValue = 
+    new DataGridWidget(boss, font, xpos, ypos, 1, 4, 3, 8, kBASE_10);
+  myCpuGridDecValue->setEditable(false);
+  xpos += myCpuGridDecValue->getWidth() + 5;
+  myCpuGridBinValue = 
+    new DataGridWidget(boss, font, xpos, ypos, 1, 4, 8, 8, kBASE_2);
+  myCpuGridBinValue->setEditable(false);
+
+  // Add labels for other CPU registers
+  xpos = x;
   string labels[4] = { "SP:", "A:", "X:", "Y:" };
   for(int row = 0; row < 4; ++row)
   {
@@ -133,7 +124,7 @@ CpuWidget::~CpuWidget()
 void CpuWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
 {
   int addr = -1, value = -1;
-  CpuDebug& dbg = instance()->debugger().cpuDebug();
+  CpuDebug& dbg = instance().debugger().cpuDebug();
 
   switch(cmd)
   {
@@ -159,24 +150,32 @@ void CpuWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
           // event the rest of the debugger widgets
           ostringstream command;
           command << "pc #" << value;
-          instance()->debugger().run(command.str());
+          instance().debugger().run(command.str());
           break;
         }
 
         case kSPRegAddr:
           dbg.setSP(value);
+          myCpuGridDecValue->setValue(0, value);
+          myCpuGridBinValue->setValue(0, value);
           break;
 
         case kARegAddr:
           dbg.setA(value);
+          myCpuGridDecValue->setValue(1, value);
+          myCpuGridBinValue->setValue(1, value);
           break;
 
         case kXRegAddr:
           dbg.setX(value);
+          myCpuGridDecValue->setValue(2, value);
+          myCpuGridBinValue->setValue(2, value);
           break;
 
         case kYRegAddr:
           dbg.setY(value);
+          myCpuGridDecValue->setValue(3, value);
+          myCpuGridBinValue->setValue(3, value);
           break;
       }
       break;
@@ -241,7 +240,7 @@ void CpuWidget::fillGrid()
 
   // We push the enumerated items as addresses, and deal with the real
   // address in the callback (handleCommand)
-  Debugger& dbg = instance()->debugger();
+  Debugger& dbg = instance().debugger();
   CpuDebug& cpu = dbg.cpuDebug();
   const CpuState& state    = (CpuState&) cpu.getState();
   const CpuState& oldstate = (CpuState&) cpu.getOldState();
@@ -274,6 +273,8 @@ void CpuWidget::fillGrid()
 
   // Finally, update the register list
   myCpuGrid->setList(alist, vlist, changed);
+  myCpuGridDecValue->setList(alist, vlist, changed);
+  myCpuGridBinValue->setList(alist, vlist, changed);
 
   // Update the PS register booleans
   changed.clear();
@@ -281,5 +282,5 @@ void CpuWidget::fillGrid()
     changed.push_back(state.PSbits[i] != oldstate.PSbits[i]);
 
   myPSRegister->setState(state.PSbits, changed);
-  myPCLabel->setEditString(dbg.equates().getLabel(state.PC, true));
+  myPCLabel->setEditString(dbg.cartDebug().getLabel(state.PC, true));
 }
