@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: PromptWidget.cxx,v 1.24 2008/05/01 23:08:24 stephena Exp $
+// $Id: PromptWidget.cxx 2003 2010-04-11 13:44:22Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -21,6 +22,7 @@
 
 #include <iostream>
 #include <fstream>
+#include <algorithm>
 
 #include "ScrollBarWidget.hxx"
 #include "FrameBuffer.hxx"
@@ -29,20 +31,13 @@
 #include "Debugger.hxx"
 #include "DebuggerDialog.hxx"
 #include "DebuggerParser.hxx"
+#include "StringList.hxx"
 
 #include "PromptWidget.hxx"
-#include "EquateList.hxx"
+#include "CartDebug.hxx"
 
 #define PROMPT  "> "
 
-/* TODO:
- * - it is very inefficient to redraw the full thingy when just one char is added/removed.
- *   Instead, we could just copy the GFX of the blank console (i.e. after the transparent
- *   background is drawn, before any text is drawn). Then using that, it becomes trivial
- *   to erase a single character, do scrolling etc.
- * - a *lot* of others things, this code is in no way complete and heavily under progress
- */
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 PromptWidget::PromptWidget(GuiObject* boss, const GUI::Font& font,
                            int x, int y, int w, int h)
@@ -65,14 +60,8 @@ PromptWidget::PromptWidget(GuiObject* boss, const GUI::Font& font,
   // Calculate depending values
   _lineWidth = (_w - kScrollBarWidth - 2) / _kConsoleCharWidth;
   _linesPerPage = (_h - 2) / _kConsoleLineHeight;
-
-  memset(_buffer, 0, kBufferSize * sizeof(int));
   _linesInBuffer = kBufferSize / _lineWidth;
 
-  _currentPos = 0;
-  _scrollLine = _linesPerPage - 1;
-  _firstLineInBuffer = 0;
-
   // Add scrollbar
   _scrollBar = new ScrollBarWidget(boss, font, _x + _w, _y, kScrollBarWidth, _h);
   _scrollBar->setTarget(this);
@@ -80,14 +69,7 @@ PromptWidget::PromptWidget(GuiObject* boss, const GUI::Font& font,
   // Init colors
   _inverse = false;
 
-  // Init History
-  _historyIndex = 0;
-  _historyLine = 0;
-  _historySize = 0;
-  for (int i = 0; i < kHistorySize; i++)
-    _history[i][0] = '\0';
-
-  _promptStartPos = _promptEndPos = -1;
+  clearScreen();
 
   addFocusWidget(this);
 }
@@ -101,9 +83,9 @@ PromptWidget::~PromptWidget()
 void PromptWidget::drawWidget(bool hilite)
 {
 //cerr << "PromptWidget::drawWidget\n";
-  int fgcolor, bgcolor;
+  uInt32 fgcolor, bgcolor;
 
-  FrameBuffer& fb = _boss->instance()->frameBuffer();
+  FBSurface& s = _boss->dialog().surface();
 
   // Draw text
   int start = _scrollLine - _linesPerPage + 1;
@@ -118,11 +100,11 @@ void PromptWidget::drawWidget(bool hilite)
       if(c & (1 << 17)) { // inverse video flag
         fgcolor = _bgcolor;
         bgcolor = (c & 0x1ffff) >> 8;
-        fb.fillRect(x, y, _kConsoleCharWidth, _kConsoleCharHeight, bgcolor);
+        s.fillRect(x, y, _kConsoleCharWidth, _kConsoleCharHeight, bgcolor);
       } else {
         fgcolor = c >> 8;
       }
-      fb.drawChar(&instance()->consoleFont(), c & 0x7f, x, y, fgcolor);
+      s.drawChar(&instance().consoleFont(), c & 0x7f, x, y, fgcolor);
       x += _kConsoleCharWidth;
     }
     y += _kConsoleLineHeight;
@@ -150,7 +132,7 @@ void PromptWidget::handleMouseWheel(int x, int y, int direction)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void PromptWidget::printPrompt()
 {
-  string watches = instance()->debugger().showWatches();
+  string watches = instance().debugger().showWatches();
   if(watches.length() > 0)
     print(watches);
 
@@ -186,7 +168,7 @@ bool PromptWidget::handleKeyDown(int ascii, int keycode, int modifiers)
         addToHistory(command.c_str());
 
         // Pass the command to the debugger, and print the result
-        string result = instance()->debugger().run(command);
+        string result = instance().debugger().run(command);
 
         // This is a bit of a hack
         // Certain commands remove the debugger dialog from underneath us,
@@ -216,11 +198,12 @@ bool PromptWidget::handleKeyDown(int ascii, int keycode, int modifiers)
 
       scrollToCurrent();
       int len = _promptEndPos - _promptStartPos;
+      if(len > 256) len = 256;
 
-      int lastDelimPos = -1;
+	  int lastDelimPos = -1;
       char delimiter = '\0';
 
-      char *str = new char[len + 1];
+      char str[256];
       for (i = 0; i < len; i++)
       {
         str[i] = buffer(_promptStartPos + i) & 0x7f;
@@ -232,46 +215,50 @@ bool PromptWidget::handleKeyDown(int ascii, int keycode, int modifiers)
       }
       str[len] = '\0';
 
-      const char *completionList;
-      const char *prefix;
-      int possibilities;
+      StringList list;
+      string completionList;
+      string prefix;
 
       if(lastDelimPos < 0)
       {
         // no delimiters, do command completion:
-        DebuggerParser& parser = instance()->debugger().parser();
-        possibilities = parser.countCompletions(str);
+        const DebuggerParser& parser = instance().debugger().parser();
+        parser.getCompletions(str, list);
 
-        if(possibilities < 1) {
-          delete[] str;
+        if(list.size() < 1)
           break;
-        }
 
-        completionList = parser.getCompletions();
-        prefix = parser.getCompletionPrefix();
+        sort(list.begin(), list.end());
+        completionList = list[0];
+        for(uInt32 i = 1; i < list.size(); ++i)
+          completionList += " " + list[i];
+        prefix = getCompletionPrefix(list, str);
       }
       else
       {
-        // we got a delimiter, so this must be a label:
-        EquateList& equates = instance()->debugger().equates();
-        possibilities = equates.countCompletions(str + lastDelimPos + 1);
+        // we got a delimiter, so this must be a label or a function
+        const Debugger& dbg = instance().debugger();
 
-        if(possibilities < 1) {
-          delete[] str;
+        dbg.cartDebug().getCompletions(str + lastDelimPos + 1, list);
+        dbg.getCompletions(str + lastDelimPos + 1, list);
+
+        if(list.size() < 1)
           break;
-        }
 
-        // TODO - perhaps use strings instead of char pointers
-        completionList = equates.getCompletions().c_str();
-        prefix = equates.getCompletionPrefix().c_str();
+        sort(list.begin(), list.end());
+        completionList = list[0];
+        for(uInt32 i = 1; i < list.size(); ++i)
+          completionList += " " + list[i];
+        prefix = getCompletionPrefix(list, str + lastDelimPos + 1);
       }
 
-      if(possibilities == 1)
+      if(list.size() == 1)
       {
         // add to buffer as though user typed it (plus a space)
         _currentPos = _promptStartPos + lastDelimPos + 1;
-        while(*completionList != '\0')
-          putcharIntern(*completionList++);
+        const char* clptr = completionList.c_str();
+        while(*clptr != '\0')
+          putcharIntern(*clptr++);
 
         putcharIntern(' ');
         _promptEndPos = _currentPos;
@@ -298,7 +285,6 @@ bool PromptWidget::handleKeyDown(int ascii, int keycode, int modifiers)
         print(prefix);
         _promptEndPos = _currentPos;
       }
-      delete[] str;
       dirty = true;
       break;
     }
@@ -317,7 +303,7 @@ bool PromptWidget::handleKeyDown(int ascii, int keycode, int modifiers)
       break;
 
     case 256 + 24:  // pageup
-      if (instance()->eventHandler().kbdShift(modifiers))
+      if (instance().eventHandler().kbdShift(modifiers))
       {
         // Don't scroll up when at top of buffer
         if(_scrollLine < _linesPerPage)
@@ -333,7 +319,7 @@ bool PromptWidget::handleKeyDown(int ascii, int keycode, int modifiers)
       break;
 
     case 256 + 25:  // pagedown
-      if (instance()->eventHandler().kbdShift(modifiers))
+      if (instance().eventHandler().kbdShift(modifiers))
       {
         // Don't scroll down when at bottom of buffer
         if(_scrollLine >= _promptEndPos / _lineWidth)
@@ -349,7 +335,7 @@ bool PromptWidget::handleKeyDown(int ascii, int keycode, int modifiers)
       break;
 
     case 256 + 22:  // home
-      if (instance()->eventHandler().kbdShift(modifiers))
+      if (instance().eventHandler().kbdShift(modifiers))
       {
         _scrollLine = _firstLineInBuffer + _linesPerPage - 1;
         updateScrollBuffer();
@@ -361,7 +347,7 @@ bool PromptWidget::handleKeyDown(int ascii, int keycode, int modifiers)
       break;
 
     case 256 + 23:  // end
-      if (instance()->eventHandler().kbdShift(modifiers))
+      if (instance().eventHandler().kbdShift(modifiers))
       {
         _scrollLine = _promptEndPos / _lineWidth;
         if (_scrollLine < _linesPerPage - 1)
@@ -375,7 +361,7 @@ bool PromptWidget::handleKeyDown(int ascii, int keycode, int modifiers)
       break;
 
     case 273:  // cursor up
-      if (instance()->eventHandler().kbdShift(modifiers))
+      if (instance().eventHandler().kbdShift(modifiers))
       {
         if(_scrollLine <= _firstLineInBuffer + _linesPerPage - 1)
           break;
@@ -390,7 +376,7 @@ bool PromptWidget::handleKeyDown(int ascii, int keycode, int modifiers)
       break;
 
     case 274:  // cursor down
-      if (instance()->eventHandler().kbdShift(modifiers))
+      if (instance().eventHandler().kbdShift(modifiers))
       {
         // Don't scroll down when at bottom of buffer
         if(_scrollLine >= _promptEndPos / _lineWidth)
@@ -420,11 +406,11 @@ bool PromptWidget::handleKeyDown(int ascii, int keycode, int modifiers)
       break;
 
     default:
-      if (instance()->eventHandler().kbdControl(modifiers))
+      if (instance().eventHandler().kbdControl(modifiers))
       {
         specialKeys(keycode);
       }
-      else if (instance()->eventHandler().kbdAlt(modifiers))
+      else if (instance().eventHandler().kbdAlt(modifiers))
       {
       }
       else if (isprint(ascii))
@@ -495,17 +481,6 @@ void PromptWidget::handleCommand(CommandSender* sender, int cmd,
   }
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-GUI::Rect PromptWidget::getRect() const
-{
-  // Account for attached scrollbar when calculating width
-  int x = getAbsX() - 1,  y = getAbsY() - 1,
-      w = getWidth() + kScrollBarWidth + 2, h = getHeight() + 2;
-
-  GUI::Rect r(x, y, x+w, y+h);
-  return r;
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void PromptWidget::loadConfig()
 {
@@ -518,15 +493,15 @@ void PromptWidget::loadConfig()
     // Display greetings & prompt
     string version = string("Stella ") + STELLA_VERSION + "\n";
     print(version.c_str());
-    print("Debugger is ready\n");
     print(PROMPT);
-    _promptStartPos = _promptEndPos = _currentPos;
-
-    _firstTime = false;
-    _exitedEarly = false;
 
     // Take care of one-time debugger stuff
-    instance()->debugger().autoExec();
+    print(instance().debugger().autoExec().c_str());
+    print(PROMPT);
+
+    _promptStartPos = _promptEndPos = _currentPos;
+    _firstTime = false;
+    _exitedEarly = false;
   }
   else if(_exitedEarly)
   {
@@ -836,7 +811,7 @@ void PromptWidget::print(const string& str)
 void PromptWidget::drawCaret()
 {
 //cerr << "PromptWidget::drawCaret()\n";
-  FrameBuffer& fb = _boss->instance()->frameBuffer();
+  FBSurface& s = _boss->dialog().surface();
 
   int line = _currentPos / _lineWidth;
 
@@ -849,8 +824,8 @@ void PromptWidget::drawCaret()
   int y = _y + displayLine * _kConsoleLineHeight;
 
   char c = buffer(_currentPos);
-  fb.fillRect(x, y, _kConsoleCharWidth, _kConsoleLineHeight, kTextColor);
-  fb.drawChar(&_boss->instance()->consoleFont(), c, x, y + 2, kBGColor);
+  s.fillRect(x, y, _kConsoleCharWidth, _kConsoleLineHeight, kTextColor);
+  s.drawChar(&_boss->instance().consoleFont(), c, x, y + 2, kBGColor);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -860,7 +835,7 @@ void PromptWidget::scrollToCurrent()
 
   if (line + _linesPerPage <= _scrollLine)
   {
-    // TODO - this should only occur for loong edit lines, though
+    // TODO - this should only occur for long edit lines, though
   }
   else if (line > _scrollLine)
   {
@@ -895,3 +870,50 @@ bool PromptWidget::saveBuffer(string& filename)
   out.close();
   return true;
 }
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+string PromptWidget::getCompletionPrefix(const StringList& completions, string prefix)
+{
+  // Search for prefix in every string, progressively growing it
+  // Once a mismatch is found or length is past one of the strings, we're done
+  // We *could* use the longest common string algorithm, but for the lengths
+  // of the strings we're dealing with, it's probably not worth it
+  for(;;)
+  {
+    for(uInt32 i = 0; i < completions.size(); ++i)
+    {
+      const string& s = completions[i];
+      if(s.length() < prefix.length())
+        return prefix;  // current prefix is the best we're going to get
+      else if(s.compare(0, prefix.length(), prefix) != 0)
+      {
+        prefix.erase(prefix.length()-1);
+        return prefix;
+      }
+    }
+    if(completions[0].length() > prefix.length())
+      prefix = completions[0].substr(0, prefix.length() + 1);
+    else
+      return prefix;
+  }
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void PromptWidget::clearScreen()
+{
+  // Initialize start position and history
+  _currentPos = 0;
+  _scrollLine = _linesPerPage - 1;
+  _firstLineInBuffer = 0;
+  _promptStartPos = _promptEndPos = -1;
+  memset(_buffer, 0, kBufferSize * sizeof(int));
+
+  _historyIndex = 0;
+  _historyLine = 0;
+  _historySize = 0;
+  for (int i = 0; i < kHistorySize; i++)
+    _history[i][0] = '\0';
+
+  if(!_firstTime)
+    updateScrollBuffer();
+}
