@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: TiaInfoWidget.cxx,v 1.9 2008/02/06 13:45:20 stephena Exp $
+// $Id: TiaInfoWidget.cxx 2001 2010-04-10 21:37:23Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -121,7 +122,7 @@ void TiaInfoWidget::handleCommand(CommandSender* sender, int cmd, int data, int
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void TiaInfoWidget::loadConfig()
 {
-  Debugger& dbg = instance()->debugger();
+  Debugger& dbg = instance().debugger();
   TIADebug& tia = dbg.tiaDebug();
 
   myFrameCount->setEditString(dbg.valueToString(tia.frameCount(), kBASE_10));
