@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: TiaZoomWidget.cxx,v 1.15 2008/02/06 13:45:20 stephena Exp $
+// $Id: TiaZoomWidget.cxx 2001 2010-04-10 21:37:23Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -56,14 +57,11 @@ TiaZoomWidget::TiaZoomWidget(GuiObject* boss, const GUI::Font& font,
   myYCenter = myNumRows >> 1;
 
   // Create context menu for zoom levels
-  myMenu = new ContextMenu(this, font);
-
-  StringList l;
-  l.push_back("2x zoom");
-  l.push_back("4x zoom");
-  l.push_back("8x zoom");
-
-  myMenu->setList(l);
+  StringMap l;
+  l.push_back("2x zoom", "2");
+  l.push_back("4x zoom", "4");
+  l.push_back("8x zoom", "8");
+  myMenu = new ContextMenu(this, font, l);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -107,8 +105,8 @@ void TiaZoomWidget::zoom(int level)
 void TiaZoomWidget::recalc()
 {
   // Don't go past end of framebuffer
-  const int width  = instance()->console().mediaSource().width(),
-            height = instance()->console().mediaSource().height();
+  const int width  = instance().console().tia().width(),
+            height = instance().console().tia().height();
 
   // Figure out the bounding rectangle for the current center coords
   const int xoff = myNumCols >> 1,
@@ -141,8 +139,8 @@ void TiaZoomWidget::handleMouseDown(int x, int y, int button, int clickCount)
   // Grab right mouse button for zoom context menu
   if(button == 2)
   {
-    myMenu->setPos(x + getAbsX(), y + getAbsY());
-    myMenu->show();
+    // Add menu at current x,y mouse location
+    myMenu->show(x + getAbsX(), y + getAbsY());
   }
 }
 
@@ -203,14 +201,7 @@ void TiaZoomWidget::handleCommand(CommandSender* sender, int cmd, int data, int
   {
     case kCMenuItemSelectedCmd:
     {
-      int item = myMenu->getSelected(), level = 0;
-      if(item == 0)
-        level = 2;
-      else if(item == 1)
-        level = 4;
-      else if(item == 2)
-        level = 8;
-
+      int level = (int) atoi(myMenu->getSelectedTag().c_str());
       if(level > 0)
         zoom(level);
       break;
@@ -222,26 +213,33 @@ void TiaZoomWidget::handleCommand(CommandSender* sender, int cmd, int data, int
 void TiaZoomWidget::drawWidget(bool hilite)
 {
 //cerr << "TiaZoomWidget::drawWidget\n";
-  FrameBuffer& fb = instance()->frameBuffer();
+  FBSurface& s = dialog().surface();
 
-  fb.fillRect(_x+1, _y+1, _w-2, _h-2, kBGColor);
-  fb.box(_x, _y, _w, _h, kColor, kShadowColor);
+  s.fillRect(_x+1, _y+1, _w-2, _h-2, kBGColor);
+  s.box(_x, _y, _w, _h, kColor, kShadowColor);
 
   // Draw the zoomed image
   // This probably isn't as efficient as it can be, but it's a small area
   // and I don't have time to make it faster :)
-  uInt8* currentFrame  = instance()->console().mediaSource().currentFrameBuffer();
-  const int pitch  = instance()->console().mediaSource().width(),
-            width  = myZoomLevel << 1,
-            height = myZoomLevel;
+  const uInt8* currentFrame  = instance().console().tia().currentFrameBuffer();
+  const int width = instance().console().tia().width(),
+            wzoom = myZoomLevel << 1,
+            hzoom = myZoomLevel;
+
+  // Get current scanline position
+  // This determines where the frame greying should start
+  uInt16 scanx, scany, scanoffset;
+  instance().console().tia().scanlinePos(scanx, scany);
+  scanoffset = width * scany + scanx;
 
   int x, y, col, row;
-  for(y = myYoff, row = 0; y < myNumRows+myYoff; ++y, row += height)
+  for(y = myYoff, row = 0; y < myNumRows+myYoff; ++y, row += hzoom)
   {
-    for(x = myXoff, col = 0; x < myNumCols+myXoff; ++x, col += width)
+    for(x = myXoff, col = 0; x < myNumCols+myXoff; ++x, col += wzoom)
     {
-      fb.fillRect(_x + col + 2, _y + row + 2, width, height,
-                  currentFrame[y*pitch + x]);
+      uInt32 idx = y*width + x;
+      uInt32 color = currentFrame[idx] | (idx > scanoffset ? 1 : 0);
+      s.fillRect(_x + col + 2, _y + row + 2, wzoom, hzoom, color);
     }
   }
 }
