@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: TogglePixelWidget.cxx,v 1.7 2008/02/06 13:45:20 stephena Exp $
+// $Id: TogglePixelWidget.cxx 2001 2010-04-10 21:37:23Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -30,7 +31,7 @@
 TogglePixelWidget::TogglePixelWidget(GuiObject* boss, const GUI::Font& font,
                                      int x, int y, int cols, int rows)
   : ToggleWidget(boss, font, x, y, cols, rows),
-    _pixelColor(kBGColor)
+    _pixelColor(0)
 {
   _type = kTogglePixelWidget;
 
@@ -115,16 +116,16 @@ int TogglePixelWidget::getIntState()
 void TogglePixelWidget::drawWidget(bool hilite)
 {
 //cerr << "TogglePixelWidget::drawWidget\n";
-  FrameBuffer& fb = instance()->frameBuffer();
+  FBSurface& s = dialog().surface();
   int row, col;
 
   // Draw the internal grid and labels
   int linewidth = _cols * _colWidth;
   for (row = 0; row <= _rows; row++)
-    fb.hLine(_x, _y + (row * _rowHeight), _x + linewidth, kColor);
+    s.hLine(_x, _y + (row * _rowHeight), _x + linewidth, kColor);
   int lineheight = _rows * _rowHeight;
   for (col = 0; col <= _cols; col++)
-    fb.vLine(_x + (col * _colWidth), _y, _y + lineheight, kColor);
+    s.vLine(_x + (col * _colWidth), _y, _y + lineheight, kColor);
 
   // Draw the pixels
   for (row = 0; row < _rows; row++)
@@ -137,13 +138,11 @@ void TogglePixelWidget::drawWidget(bool hilite)
 
       // Draw the selected item inverted, on a highlighted background.
       if (_currentRow == row && _currentCol == col && _hasFocus)
-        fb.fillRect(x - 4, y - 2, _colWidth+1, _rowHeight+1, kTextColorHi);
+        s.fillRect(x - 4, y - 2, _colWidth+1, _rowHeight+1, kTextColorHi);
 
       // Either draw the pixel in given color, or erase (show background)
-      if(_stateList[pos])
-        fb.fillRect(x - 3, y - 1, _colWidth-1, _rowHeight-1, _pixelColor);
-      else
-        fb.fillRect(x - 3, y - 1, _colWidth-1, _rowHeight-1, kBGColor);
+      s.fillRect(x - 3, y - 1, _colWidth-1, _rowHeight-1,
+                 _stateList[pos] ? _pixelColor : kBGColor);
     }
   }
 }
