@@ -8,18 +8,15 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: AtariVox.cxx,v 1.20 2008/05/19 02:53:57 stephena Exp $
+// $Id: AtariVox.cxx 2001 2010-04-10 21:37:23Z stephena $
 //============================================================================
 
-#ifdef SPEAKJET_EMULATION
-  #include "SpeakJet.hxx"
-#endif
-
 #include "MT24LC256.hxx"
 #include "SerialPort.hxx"
 #include "System.hxx"
@@ -30,21 +27,16 @@ AtariVox::AtariVox(Jack jack, const Event& event, const System& system,
                    const SerialPort& port, const string& portname,
                    const string& eepromfile)
   : Controller(jack, event, system, Controller::AtariVox),
-    mySerialPort((SerialPort*)&port),
+    mySerialPort((SerialPort&)port),
     myEEPROM(NULL),
     myShiftCount(0),
     myShiftRegister(0),
     myLastDataWriteCycle(0)
 {
-#ifndef SPEAKJET_EMULATION
-  if(mySerialPort->openPort(portname))
+  if(mySerialPort.openPort(portname))
     myAboutString = " (using serial port \'" + portname + "\')";
   else
     myAboutString = " (invalid serial port \'" + portname + "\')";
-#else
-  mySpeakJet = new SpeakJet();
-  myAboutString = " (emulating SpeakJet device)";
-#endif
 
   myEEPROM = new MT24LC256(eepromfile, system);
 
@@ -57,11 +49,7 @@ AtariVox::AtariVox(Jack jack, const Event& event, const System& system,
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 AtariVox::~AtariVox()
 {
-#ifndef SPEAKJET_EMULATION
-  mySerialPort->closePort();
-#else
-  delete mySpeakJet;
-#endif
+  mySerialPort.closePort();
   delete myEEPROM;
 }
 
@@ -158,11 +146,7 @@ void AtariVox::clockDataIn(bool value)
       else
       {
         uInt8 data = ((myShiftRegister >> 1) & 0xff);
-    #ifndef SPEAKJET_EMULATION
-        mySerialPort->writeByte(&data);
-    #else
-        mySpeakJet->write(data);
-    #endif
+        mySerialPort.writeByte(&data);
       }
       myShiftRegister = 0;
     }
