@@ -8,15 +8,17 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Cart.cxx,v 1.42 2008/05/16 23:56:21 stephena Exp $
+// $Id: Cart.cxx 2030 2010-05-02 22:52:59Z stephena $
 //============================================================================
 
 #include <cassert>
+#include <cstring>
 #include <sstream>
 
 #include "bspf.hxx"
@@ -29,18 +31,21 @@
 #include "Cart4K.hxx"
 #include "CartAR.hxx"
 #include "CartDPC.hxx"
+#include "CartDPCPlus.hxx"
 #include "CartE0.hxx"
 #include "CartE7.hxx"
+#include "CartEF.hxx"
+#include "CartEFSC.hxx"
+#include "CartF0.hxx"
 #include "CartF4.hxx"
 #include "CartF4SC.hxx"
 #include "CartF6.hxx"
 #include "CartF6SC.hxx"
 #include "CartF8.hxx"
 #include "CartF8SC.hxx"
-#include "CartFASC.hxx"
+#include "CartFA.hxx"
 #include "CartFE.hxx"
 #include "CartMC.hxx"
-#include "CartMB.hxx"
 #include "CartCV.hxx"
 #include "CartUA.hxx"
 #include "CartSB.hxx"
@@ -48,49 +53,103 @@
 #include "MD5.hxx"
 #include "Props.hxx"
 #include "Settings.hxx"
+#ifdef DEBUGGER_SUPPORT
+  #include "Debugger.hxx"
+  #include "CartDebug.hxx"
+#endif
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-Cartridge* Cartridge::create(const uInt8* image, uInt32 size,
-    const Properties& properties, const Settings& settings)
+Cartridge* Cartridge::create(const uInt8* image, uInt32 size, string& md5,
+     string& dtype, string& id, Settings& settings)
 {
   Cartridge* cartridge = 0;
-
-  // Get the type of the cartridge we're creating
-  const string& md5 = properties.get(Cartridge_MD5);
-  string type = properties.get(Cartridge_Type);
+  string type = dtype;
 
   // First consider the ROMs that are special and don't have a properties entry
   // Hopefully this list will be very small
   if(md5 == "bc24440b59092559a1ec26055fd1270e" ||
-     md5 == "75ee371ccfc4f43e7d9b8f24e1266b55")
+     md5 == "75ee371ccfc4f43e7d9b8f24e1266b55" ||
+     md5 == "9905f9f4706223dadee84f6867ede8e3")
   {
     // These two ROMs are normal 8K images, except they must be initialized
     // from the opposite bank compared to normal ones
     type = "F8 swapped";
+    dtype = type;
   }
 
   // Collect some info about the ROM
   ostringstream buf;
-  buf << "  ROM Size:        " << size << endl
-      << "  Bankswitch Type: " << type;
 
   // See if we should try to auto-detect the cartridge type
   // If we ask for extended info, always do an autodetect
+  string autodetect = "";
   if(type == "AUTO-DETECT" || settings.getBool("rominfo"))
   {
     string detected = autodetectType(image, size);
-    buf << " ==> " << detected;
+    autodetect = "*";
     if(type != "AUTO-DETECT" && type != detected)
-      buf << " (auto-detection not consistent)";
+      cerr << "Auto-detection not consistent: " << type << ", " << detected << endl;
 
     type = detected;
   }
-  buf << endl;
-  myAboutString = buf.str();
+  buf << type << autodetect;
+
+  // Check for multicart first; if found, get the correct part of the image
+  if(type == "2IN1")
+  {
+    // Make sure we have a valid sized image
+    if(size == 2*2048 || size == 2*4096 || size == 2*8192 || size == 2*16384)
+    {
+      dtype = type;
+      type = createFromMultiCart(image, size, 2, md5, id, settings);
+      buf << id;
+    }
+
+  }
+  else if(type == "4IN1")
+  {
+    // Make sure we have a valid sized image
+    if(size == 4*2048 || size == 4*4096 || size == 4*8192)
+    {
+      dtype = type;
+      type = createFromMultiCart(image, size, 4, md5, id, settings);
+      buf << id;
+    }
+  }
+  else if(type == "8IN1")
+  {
+    // Make sure we have a valid sized image
+    if(size == 8*2048 || size == 8*4096 || size == 8*8192)
+    {
+      dtype = type;
+      type = createFromMultiCart(image, size, 8, md5, id, settings);
+      buf << id;
+    }
+  }
+  else if(type == "16IN1")
+  {
+    // Make sure we have a valid sized image
+    if(size == 16*2048 || size == 16*4096 || size == 16*8192)
+    {
+      dtype = type;
+      type = createFromMultiCart(image, size, 16, md5, id, settings);
+      buf << id;
+    }
+  }
+  else if(type == "32IN1")
+  {
+    // Make sure we have a valid sized image
+    if(size == 32*2048 || size == 32*4096)
+    {
+      dtype = type;
+      type = createFromMultiCart(image, size, 32, md5, id, settings);
+      buf << id;
+    }
+  }
 
   // We should know the cart's type by now so let's create it
   if(type == "2K")
-    cartridge = new Cartridge2K(image);
+    cartridge = new Cartridge2K(image, size);
   else if(type == "3E")
     cartridge = new Cartridge3E(image, size);
   else if(type == "3F")
@@ -100,13 +159,19 @@ Cartridge* Cartridge::create(const uInt8* image, uInt32 size,
   else if(type == "4K")
     cartridge = new Cartridge4K(image);
   else if(type == "AR")
-    cartridge = new CartridgeAR(image, size, true); //settings.getBool("fastscbios")
+    cartridge = new CartridgeAR(image, size, settings);
   else if(type == "DPC")
     cartridge = new CartridgeDPC(image, size);
+  else if(type == "DPC+")
+    cartridge = new CartridgeDPCPlus(image, size);
   else if(type == "E0")
     cartridge = new CartridgeE0(image);
   else if(type == "E7")
     cartridge = new CartridgeE7(image);
+  else if(type == "EF")
+    cartridge = new CartridgeEF(image);
+  else if(type == "EFSC")
+    cartridge = new CartridgeEFSC(image);
   else if(type == "F4")
     cartridge = new CartridgeF4(image);
   else if(type == "F4SC")
@@ -116,19 +181,19 @@ Cartridge* Cartridge::create(const uInt8* image, uInt32 size,
   else if(type == "F6SC")
     cartridge = new CartridgeF6SC(image);
   else if(type == "F8")
-    cartridge = new CartridgeF8(image, false);
+    cartridge = new CartridgeF8(image);
   else if(type == "F8 swapped")
     cartridge = new CartridgeF8(image, true);
   else if(type == "F8SC")
     cartridge = new CartridgeF8SC(image);
-  else if(type == "FASC")
-    cartridge = new CartridgeFASC(image);
+  else if(type == "FA" || type == "FASC")
+    cartridge = new CartridgeFA(image);
   else if(type == "FE")
     cartridge = new CartridgeFE(image);
   else if(type == "MC")
     cartridge = new CartridgeMC(image, size);
-  else if(type == "MB")
-    cartridge = new CartridgeMB(image);
+  else if(type == "F0" || type == "MB")
+    cartridge = new CartridgeF0(image);
   else if(type == "CV")
     cartridge = new CartridgeCV(image, size);
   else if(type == "UA")
@@ -142,13 +207,45 @@ Cartridge* Cartridge::create(const uInt8* image, uInt32 size,
   else
     cerr << "ERROR: Invalid cartridge type " << type << " ..." << endl;
 
+  if(size < 1024)
+    buf << " (" << size << "B) ";
+  else
+    buf << " (" << (size/1024) << "K) ";
+  myAboutString = buf.str();
+
   return cartridge;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+string Cartridge::createFromMultiCart(const uInt8*& image, uInt32& size,
+    uInt32 numroms, string& md5, string& id, Settings& settings)
+{
+  // Get a piece of the larger image
+  uInt32 i = settings.getInt("romloadcount");
+  size /= numroms;
+  image += i*size;
+
+  // We need a new md5 and name
+  md5 = MD5(image, size);
+  ostringstream buf;
+  buf << " [G" << (i+1) << "]";
+  id = buf.str();
+
+  // Move to the next game the next time this ROM is loaded
+  settings.setInt("romloadcount", (i+1)%numroms);
+
+  if(size <= 2048)       return "2K";
+  else if(size == 4096)  return "4K";
+  else if(size == 8192)  return "F8";
+  else  /* default */    return "4K";
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Cartridge::Cartridge()
+  : myStartBank(0),
+    myBankChanged(true),
+    myBankLocked(false)
 {
-  unlockBank();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -161,7 +258,7 @@ bool Cartridge::save(ofstream& out)
 {
   int size = -1;
 
-  uInt8* image = getImage(size);
+  const uInt8* image = getImage(size);
   if(image == 0 || size <= 0)
   {
     cerr << "save not supported" << endl;
@@ -174,16 +271,56 @@ bool Cartridge::save(ofstream& out)
   return true;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+uInt16 Cartridge::startBank()
+{
+  return myStartBank;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool Cartridge::bankChanged()
+{
+  bool changed = myBankChanged;
+  myBankChanged = false;
+  return changed;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Cartridge::registerRamArea(uInt16 start, uInt16 size,
+                                uInt16 roffset, uInt16 woffset)
+{
+#ifdef DEBUGGER_SUPPORT
+  RamArea area;
+  area.start   = start;
+  area.size    = size;
+  area.roffset = roffset;
+  area.woffset = woffset;
+  myRamAreaList.push_back(area);
+#endif
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Cartridge::triggerReadFromWritePort(uInt16 address)
+{
+#ifdef DEBUGGER_SUPPORT
+  Debugger::debugger().cartDebug().triggerReadFromWritePort(address);
+#endif
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 string Cartridge::autodetectType(const uInt8* image, uInt32 size)
 {
   // Guess type based on size
   const char* type = 0;
 
-  if((size % 8448) == 0)
+  if((size % 8448) == 0 || size == 6144)
   {
     type = "AR";
   }
+  else if(size < 2048)  // Sub2K images
+  {
+    type = "2K";
+  }
   else if((size == 2048) ||
           (size == 4096 && memcmp(image, image + 2048, 2048) == 0))
   {
@@ -199,7 +336,7 @@ string Cartridge::autodetectType(const uInt8* image, uInt32 size)
     else
       type = "4K";
   }
-  else if(size == 8192)  // 8K
+  else if(size == 8*1024)  // 8K
   {
     if(isProbablySC(image, size))
       type = "F8SC";
@@ -220,19 +357,15 @@ string Cartridge::autodetectType(const uInt8* image, uInt32 size)
     else
       type = "F8";
   }
-  else if((size == 10495) || (size == 10496) || (size == 10240))  // 10K - Pitfall2
+  else if(size >= 10240 && size <= 10496)  // ~10K - Pitfall2
   {
     type = "DPC";
   }
-  else if(size == 12288)  // 12K
+  else if(size == 12*1024)  // 12K
   {
-    // TODO - this should really be in a method that checks the first
-    // 512 bytes of ROM and finds if either the lower 256 bytes or
-    // higher 256 bytes are all the same.  For now, we assume that
-    // all carts of 12K are CBS RAM Plus/FASC.
-    type = "FASC";
+    type = "FA";
   }
-  else if(size == 16384)  // 16K
+  else if(size == 16*1024)  // 16K
   {
     if(isProbablySC(image, size))
       type = "F6SC";
@@ -245,7 +378,11 @@ string Cartridge::autodetectType(const uInt8* image, uInt32 size)
     else
       type = "F6";
   }
-  else if(size == 32768)  // 32K
+  else if(size == 29*1024)  // 29K
+  {
+    type = "DPC+";
+  }
+  else if(size == 32*1024)  // 32K
   {
     if(isProbablySC(image, size))
       type = "F4SC";
@@ -253,10 +390,12 @@ string Cartridge::autodetectType(const uInt8* image, uInt32 size)
       type = "3E";
     else if(isProbably3F(image, size))
       type = "3F";
+    else if(isProbablyDPCplus(image, size))
+      type = "DPC+";
     else
       type = "F4";
   }
-  else if(size == 65536)  // 64K
+  else if(size == 64*1024)  // 64K
   {
     if(isProbably3E(image, size))
       type = "3E";
@@ -264,10 +403,18 @@ string Cartridge::autodetectType(const uInt8* image, uInt32 size)
       type = "3F";
     else if(isProbably4A50(image, size))
       type = "4A50";
+    else if(isProbablyEF(image, size))
+    {
+      type = "EF";
+      if(isProbablySC(image, size))
+        type = "EFSC";
+    }
+    else if(isProbablyX07(image, size))
+      type = "X07";
     else
-      type = "MB";
+      type = "F0";
   }
-  else if(size == 131072)  // 128K
+  else if(size == 128*1024)  // 128K
   {
     if(isProbably3E(image, size))
       type = "3E";
@@ -280,6 +427,15 @@ string Cartridge::autodetectType(const uInt8* image, uInt32 size)
     else
       type = "MC";
   }
+  else if(size == 256*1024)  // 256K
+  {
+    if(isProbably3E(image, size))
+      type = "3E";
+    else if(isProbably3F(image, size))
+      type = "3F";
+    else /*if(isProbablySB(image, size))*/
+      type = "SB";
+  }
   else  // what else can we do?
   {
     if(isProbably3E(image, size))
@@ -361,6 +517,14 @@ bool Cartridge::isProbably3E(const uInt8* image, uInt32 size)
   return searchForBytes(image, size, signature, 4, 1);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool Cartridge::isProbablyDPCplus(const uInt8* image, uInt32 size)
+{
+  // DPC+ ARM code has 2 occurrences of the string DPC+
+  uInt8 signature[] = { 0x44, 0x50, 0x43, 0x2B };  // DPC+
+  return searchForBytes(image, size, signature, 4, 2);
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Cartridge::isProbablyE0(const uInt8* image, uInt32 size)
 {
@@ -370,19 +534,20 @@ bool Cartridge::isProbablyE0(const uInt8* image, uInt32 size)
   // search for only certain known signatures
   // Thanks to "stella@casperkitty.com" for this advice
   // These signatures are attributed to the MESS project
-  uInt8 signature[6][3] = {
+  uInt8 signature[8][3] = {
    { 0x8D, 0xE0, 0x1F },  // STA $1FE0
    { 0x8D, 0xE0, 0x5F },  // STA $5FE0
    { 0x8D, 0xE9, 0xFF },  // STA $FFE9
+   { 0x0C, 0xE0, 0x1F },  // NOP $1FE0
+   { 0xAD, 0xE0, 0x1F },  // LDA $1FE0
    { 0xAD, 0xE9, 0xFF },  // LDA $FFE9
    { 0xAD, 0xED, 0xFF },  // LDA $FFED
    { 0xAD, 0xF3, 0xBF }   // LDA $BFF3
   };
-  for(uInt32 i = 0; i < 6; ++i)
-  {
+  for(uInt32 i = 0; i < 8; ++i)
     if(searchForBytes(image, size, signature[i], 3, 1))
       return true;
-  }
+
   return false;
 }
 
@@ -395,25 +560,51 @@ bool Cartridge::isProbablyE7(const uInt8* image, uInt32 size)
   // search for only certain known signatures
   // Thanks to "stella@casperkitty.com" for this advice
   // These signatures are attributed to the MESS project
-  uInt8 signature[2][3] = {
+  uInt8 signature[5][3] = {
+   { 0xAD, 0xE5, 0xFF },  // LDA $FFE5
+   { 0xAD, 0xE5, 0x1F },  // LDA $1FE5
+   { 0x0C, 0xE7, 0x1F },  // NOP $1FE7
    { 0x8D, 0xE7, 0xFF },  // STA $FFE7
-   { 0xAD, 0xE5, 0xFF }   // LDA $FFE5
+   { 0x8D, 0xE7, 0x1F }   // STA $1FE7
   };
-  for(uInt32 i = 0; i < 2; ++i)
-  {
+  for(uInt32 i = 0; i < 5; ++i)
     if(searchForBytes(image, size, signature[i], 3, 1))
       return true;
-  }
+
   return false;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool Cartridge::isProbablyEF(const uInt8* image, uInt32 size)
+{
+  // EF cart bankswitching switches banks by accessing addresses 0xFE0
+  // to 0xFEF, usually with either a NOP or LDA
+  // It's likely that the code will switch to bank 0, so that's what is tested
+  uInt8 signature[2][3] = {
+    { 0x0C, 0xE0, 0xFF },  // NOP $FFE0
+    { 0xAD, 0xE0, 0xFF }   // LDA $FFE0
+  };
+  if(searchForBytes(image, size, signature[0], 3, 1))
+    return true;
+  else
+    return searchForBytes(image, size, signature[1], 3, 1);
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Cartridge::isProbablyUA(const uInt8* image, uInt32 size)
 {
   // UA cart bankswitching switches to bank 1 by accessing address 0x240
-  // using 'STA $240'
-  uInt8 signature[] = { 0x8D, 0x40, 0x02 };  // STA $240
-  return searchForBytes(image, size, signature, 3, 1);
+  // using 'STA $240' or 'LDA $240'
+  uInt8 signature[3][3] = {
+    { 0x8D, 0x40, 0x02 },  // STA $240
+    { 0xAD, 0x40, 0x02 },  // LDA $240
+    { 0xBD, 0x1F, 0x02 }   // LDA $21F,X
+  };
+  for(uInt32 i = 0; i < 3; ++i)
+    if(searchForBytes(image, size, signature[i], 3, 1))
+      return true;
+
+  return false;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -429,14 +620,38 @@ bool Cartridge::isProbably4A50(const uInt8* image, uInt32 size)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Cartridge::isProbablySB(const uInt8* image, uInt32 size)
 {
-  // TODO - add autodetection for this type
-  return false;
+  // SB cart bankswitching switches banks by accessing address 0x0800
+  uInt8 signature[2][3] = {
+    { 0xBD, 0x00, 0x08 },  // LDA $0800,x
+    { 0xAD, 0x00, 0x08 }   // LDA $0800
+  };
+  if(searchForBytes(image, size, signature[0], 3, 1))
+    return true;
+  else
+    return searchForBytes(image, size, signature[1], 3, 1);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Cartridge::isProbably0840(const uInt8* image, uInt32 size)
 {
-  // TODO - add autodetection for this type
+  // 0840 cart bankswitching is triggered by accessing addresses 0x0800
+  // or 0x0840
+  uInt8 signature1[2][3] = {
+    { 0xAD, 0x00, 0x08 },  // LDA $0800
+    { 0xAD, 0x40, 0x08 }   // LDA $0840
+  };
+  for(uInt32 i = 0; i < 2; ++i)
+    if(searchForBytes(image, size, signature1[i], 3, 1))
+      return true;
+
+  uInt8 signature2[2][4] = {
+    { 0x0C, 0x00, 0x08, 0x4C },  // NOP $0800; JMP ...
+    { 0x0C, 0xFF, 0x0F, 0x4C }   // NOP $0FFF; JMP ...
+  };
+  for(uInt32 i = 0; i < 2; ++i)
+    if(searchForBytes(image, size, signature2[i], 4, 1))
+      return true;
+
   return false;
 }
 
@@ -468,10 +683,25 @@ bool Cartridge::isProbablyFE(const uInt8* image, uInt32 size)
     { 0x20, 0x00, 0xF0, 0x84, 0xD6 }   // JSR $F000; STY $D6
   };
   for(uInt32 i = 0; i < 4; ++i)
-  {
     if(searchForBytes(image, size, signature[i], 5, 1))
       return true;
-  }
+
+  return false;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool Cartridge::isProbablyX07(const uInt8* image, uInt32 size)
+{
+  // X07 bankswitching switches to bank 0, 1, 2, etc by accessing address 0x08xd
+  uInt8 signature[3][3] = {
+    { 0xAD, 0x0D, 0x08 },  // LDA $080D
+    { 0xAD, 0x1D, 0x08 },  // LDA $081D
+    { 0xAD, 0x2D, 0x08 }   // LDA $082D
+  };
+  for(uInt32 i = 0; i < 3; ++i)
+    if(searchForBytes(image, size, signature[i], 3, 1))
+      return true;
+
   return false;
 }
 
@@ -489,4 +719,4 @@ Cartridge& Cartridge::operator = (const Cartridge&)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-string Cartridge::myAboutString;
+string Cartridge::myAboutString= "";
