@@ -8,37 +8,60 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Cart2K.cxx,v 1.12 2008/02/06 13:45:20 stephena Exp $
+// $Id: Cart2K.cxx 2012 2010-04-14 22:35:46Z stephena $
 //============================================================================
 
 #include <cassert>
+#include <cstring>
 
 #include "System.hxx"
 #include "Cart2K.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-Cartridge2K::Cartridge2K(const uInt8* image)
+Cartridge2K::Cartridge2K(const uInt8* image, uInt32 size)
 {
+  // Size can be a maximum of 2K
+  if(size > 2048) size = 2048;
+
+  // Set image size to closest power-of-two for the given size
+  mySize = 1;
+  while(mySize < size)
+    mySize <<= 1;
+
+  // The smallest addressable area by Stella is 64 bytes
+  // This should really be read from the System, but for now I'm going
+  // to cheat a little and hard-code it to 64 (aka 2^6)
+  if(mySize < 64)
+    mySize = 64;
+
+  // Initialize ROM with illegal 6502 opcode that causes a real 6502 to jam
+  myImage = new uInt8[mySize];
+  memset(myImage, 0x02, mySize);
+
   // Copy the ROM image into my buffer
-  for(uInt32 addr = 0; addr < 2048; ++addr)
-  {
-    myImage[addr] = image[addr];
-  }
+  memcpy(myImage, image, size);
+
+  // Set mask for accessing the image buffer
+  // This is guaranteed to work, as mySize is a power of two
+  myMask = mySize - 1;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Cartridge2K::~Cartridge2K()
 {
+  delete[] myImage;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Cartridge2K::reset()
 {
+  myBankChanged = true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -51,14 +74,15 @@ void Cartridge2K::install(System& system)
   // Make sure the system we're being installed in has a page size that'll work
   assert((0x1000 & mask) == 0);
 
+  // Map ROM image into the system
   System::PageAccess access;
   access.directPokeBase = 0;
   access.device = this;
+  access.type = System::PA_READ;
 
-  // Map ROM image into the system
   for(uInt32 address = 0x1000; address < 0x2000; address += (1 << shift))
   {
-    access.directPeekBase = &myImage[address & 0x07FF];
+    access.directPeekBase = &myImage[address & myMask];
     mySystem->setPageAccess(address >> shift, access);
   }
 }
@@ -66,13 +90,14 @@ void Cartridge2K::install(System& system)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 Cartridge2K::peek(uInt16 address)
 {
-  return myImage[address & 0x07FF];
+  return myImage[address & myMask];
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Cartridge2K::poke(uInt16, uInt8)
+bool Cartridge2K::poke(uInt16, uInt8)
 {
   // This is ROM so poking has no effect :-)
+  return false;
 } 
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -82,14 +107,14 @@ void Cartridge2K::bank(uInt16 bank)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int Cartridge2K::bank()
+uInt16 Cartridge2K::bank() const
 {
   // Doesn't support bankswitching
   return 0;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int Cartridge2K::bankCount()
+uInt16 Cartridge2K::bankCount() const
 {
   return 1;
 }
@@ -97,21 +122,21 @@ int Cartridge2K::bankCount()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Cartridge2K::patch(uInt16 address, uInt8 value)
 {
-  myImage[address & 0x07FF] = value;
-  return true;
+  myImage[address & myMask] = value;
+  return myBankChanged = true;
 } 
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt8* Cartridge2K::getImage(int& size)
+const uInt8* Cartridge2K::getImage(int& size) const
 {
-  size = 2048;
-  return &myImage[0];
+  size = mySize;
+  return myImage;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Cartridge2K::save(Serializer& out) const
 {
-  string cart = name();
+  const string& cart = name();
 
   try
   {
@@ -119,12 +144,7 @@ bool Cartridge2K::save(Serializer& out) const
   }
   catch(const char* msg)
   {
-    cerr << msg << endl;
-    return false;
-  }
-  catch(...)
-  {
-    cerr << "Unknown error in save state for " << cart << endl;
+    cerr << "ERROR: Cartridge2K::save" << endl << "  " << msg << endl;
     return false;
   }
 
@@ -132,9 +152,9 @@ bool Cartridge2K::save(Serializer& out) const
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool Cartridge2K::load(Deserializer& in)
+bool Cartridge2K::load(Serializer& in)
 {
-  string cart = name();
+  const string& cart = name();
 
   try
   {
@@ -143,12 +163,7 @@ bool Cartridge2K::load(Deserializer& in)
   }
   catch(const char* msg)
   {
-    cerr << msg << endl;
-    return false;
-  }
-  catch(...)
-  {
-    cerr << "Unknown error in load state for " << cart << endl;
+    cerr << "ERROR: Cartridge2K::load" << endl << "  " << msg << endl;
     return false;
   }
 
