@@ -8,17 +8,18 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Cart3E.cxx,v 1.15 2008/03/28 23:29:13 stephena Exp $
+// $Id: Cart3E.cxx 2012 2010-04-14 22:35:46Z stephena $
 //============================================================================
 
 #include <cassert>
+#include <cstring>
 
-#include "Random.hxx"
 #include "System.hxx"
 #include "TIA.hxx"
 #include "Cart3E.hxx"
@@ -31,17 +32,15 @@ Cartridge3E::Cartridge3E(const uInt8* image, uInt32 size)
   myImage = new uInt8[mySize];
 
   // Copy the ROM image into my buffer
-  for(uInt32 addr = 0; addr < mySize; ++addr)
-  {
-    myImage[addr] = image[addr];
-  }
+  memcpy(myImage, image, mySize);
 
-  // Initialize RAM with random values
-  class Random random;
-  for(uInt32 i = 0; i < 32768; ++i)
-  {
-    myRam[i] = random.next();
-  }
+  // This cart can address a 1024 byte bank of RAM @ 0x1000
+  // However, it may not be addressable all the time (it may be swapped out)
+  // so probably most of the time, the area will point to ROM instead
+  registerRamArea(0x1000, 1024, 0x00, 0x400);  // 1024 bytes RAM @ 0x1000
+
+  // Remember startup bank
+  myStartBank = 0;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -53,8 +52,12 @@ Cartridge3E::~Cartridge3E()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Cartridge3E::reset()
 {
-  // We'll map bank 0 into the first segment upon reset
-  bank(0);
+  // Initialize RAM with random values
+  for(uInt32 i = 0; i < 32768; ++i)
+    myRam[i] = mySystem->randGenerator().next();
+
+  // We'll map the startup bank into the first segment upon reset
+  bank(myStartBank);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -67,42 +70,61 @@ void Cartridge3E::install(System& system)
   // Make sure the system we're being installed in has a page size that'll work
   assert((0x1800 & mask) == 0);
 
-  // Set the page accessing methods for the hot spots (for 100% emulation
-  // I would need to chain any accesses below 0x40 to the TIA but for
-  // now I'll just forget about them)
   System::PageAccess access;
+
+  // Set the page accessing methods for the hot spots (for 100% emulation
+  // we need to chain any accesses below 0x40 to the TIA. Our poke() method
+  // does this via mySystem->tiaPoke(...), at least until we come up with a
+  // cleaner way to do it).
+  access.directPeekBase = 0;
+  access.directPokeBase = 0;
+  access.device = this;
+  access.type = System::PA_READWRITE;
   for(uInt32 i = 0x00; i < 0x40; i += (1 << shift))
-  {
-    access.directPeekBase = 0;
-    access.directPokeBase = 0;
-    access.device = this;
     mySystem->setPageAccess(i >> shift, access);
-  }
 
   // Setup the second segment to always point to the last ROM slice
+  access.directPokeBase = 0;
+  access.device = this;
+  access.type = System::PA_READ;
   for(uInt32 j = 0x1800; j < 0x2000; j += (1 << shift))
   {
-    access.device = this;
     access.directPeekBase = &myImage[(mySize - 2048) + (j & 0x07FF)];
-    access.directPokeBase = 0;
     mySystem->setPageAccess(j >> shift, access);
   }
 
-  // Install pages for bank 0 into the first segment
-  bank(0);
+  // Install pages for the startup bank into the first segment
+  bank(myStartBank);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 Cartridge3E::peek(uInt16 address)
 {
-  address = address & 0x0FFF;
+  uInt16 peekAddress = address;
+  address &= 0x0FFF;
 
   if(address < 0x0800)
   {
     if(myCurrentBank < 256)
-      return myImage[(address & 0x07FF) + myCurrentBank * 2048];
+      return myImage[(address & 0x07FF) + (myCurrentBank << 11)];
     else
-      return myRam[(address & 0x03FF) + (myCurrentBank - 256) * 1024];
+    {
+      if(address < 0x0400)
+        return myRam[(address & 0x03FF) + ((myCurrentBank - 256) << 10)];
+      else
+      {
+        // Reading from the write port triggers an unwanted write
+        uInt8 value = mySystem->getDataBusState(0xFF);
+
+        if(bankLocked())
+          return value;
+        else
+        {
+          triggerReadFromWritePort(peekAddress);
+          return myRam[(address & 0x03FF) + ((myCurrentBank - 256) << 10)] = value;
+        }
+      }
+    }
   }
   else
   {
@@ -111,9 +133,9 @@ uInt8 Cartridge3E::peek(uInt16 address)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Cartridge3E::poke(uInt16 address, uInt8 value)
+bool Cartridge3E::poke(uInt16 address, uInt8 value)
 {
-  address = address & 0x0FFF;
+  address &= 0x0FFF;
 
   // Switch banks if necessary. Armin (Kroko) says there are no mirrored
   // hotspots.
@@ -131,17 +153,19 @@ void Cartridge3E::poke(uInt16 address, uInt8 value)
   // 64-byte chunk of address space is "owned" by only one device. If we
   // don't chain the poke to the TIA, then the TIA can't see it...
   mySystem->tia().poke(address, value);
+
+  return false;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Cartridge3E::bank(uInt16 bank)
 { 
-  if(myBankLocked) return;
+  if(bankLocked()) return;
 
   if(bank < 256)
   {
     // Make sure the bank they're asking for is reasonable
-    if((uInt32)bank * 2048 < mySize)
+    if(((uInt32)bank << 11) < uInt32(mySize))
     {
       myCurrentBank = bank;
     }
@@ -149,17 +173,18 @@ void Cartridge3E::bank(uInt16 bank)
     {
       // Oops, the bank they're asking for isn't valid so let's wrap it
       // around to a valid bank number
-      myCurrentBank = bank % (mySize / 2048);
+      myCurrentBank = bank % (mySize >> 11);
     }
   
-    uInt32 offset = myCurrentBank * 2048;
+    uInt32 offset = myCurrentBank << 11;
     uInt16 shift = mySystem->pageShift();
   
     // Setup the page access methods for the current bank
     System::PageAccess access;
-    access.device = this;
     access.directPokeBase = 0;
-  
+    access.device = this;
+    access.type = System::PA_READ;
+
     // Map ROM image into the system
     for(uInt32 address = 0x1000; address < 0x1800; address += (1 << shift))
     {
@@ -173,15 +198,16 @@ void Cartridge3E::bank(uInt16 bank)
     bank %= 32;
     myCurrentBank = bank + 256;
 
-    uInt32 offset = bank * 1024;
+    uInt32 offset = bank << 10;
     uInt16 shift = mySystem->pageShift();
     uInt32 address;
   
     // Setup the page access methods for the current bank
     System::PageAccess access;
-    access.device = this;
     access.directPokeBase = 0;
-  
+    access.device = this;
+    access.type = System::PA_READ;
+
     // Map read-port RAM image into the system
     for(address = 0x1000; address < 0x1400; address += (1 << shift))
     {
@@ -190,6 +216,7 @@ void Cartridge3E::bank(uInt16 bank)
     }
 
     access.directPeekBase = 0;
+    access.type = System::PA_WRITE;
 
     // Map write-port RAM image into the system
     for(address = 0x1400; address < 0x1800; address += (1 << shift))
@@ -198,49 +225,55 @@ void Cartridge3E::bank(uInt16 bank)
       mySystem->setPageAccess(address >> shift, access);
     }
   }
+  myBankChanged = true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int Cartridge3E::bank()
+uInt16 Cartridge3E::bank() const
 {
   return myCurrentBank;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int Cartridge3E::bankCount()
+uInt16 Cartridge3E::bankCount() const
 {
-  return mySize / 2048;
+  // Because the RAM banks always start at 256 and above, we require the
+  // number of ROM banks to be 256
+  // If the RAM banks were simply appended to the number of actual
+  // ROM banks, bank numbers would be ambiguous (ie, would bank 128 be
+  // the last bank of ROM, or one of the banks of RAM?)
+  return 256 + 32;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Cartridge3E::patch(uInt16 address, uInt8 value)
 {
-  address = address & 0x0FFF;
+  address &= 0x0FFF;
+
   if(address < 0x0800)
   {
     if(myCurrentBank < 256)
-      myImage[(address & 0x07FF) + myCurrentBank * 2048] = value;
+      myImage[(address & 0x07FF) + (myCurrentBank << 11)] = value;
     else
-      myRam[(address & 0x03FF) + (myCurrentBank - 256) * 1024] = value;
+      myRam[(address & 0x03FF) + ((myCurrentBank - 256) << 10)] = value;
   }
   else
-  {
     myImage[(address & 0x07FF) + mySize - 2048] = value;
-  }
-  return true;
+
+  return myBankChanged = true;
 } 
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt8* Cartridge3E::getImage(int& size)
+const uInt8* Cartridge3E::getImage(int& size) const
 {
   size = mySize;
-  return &myImage[0];
+  return myImage;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Cartridge3E::save(Serializer& out) const
 {
-  string cart = name();
+  const string& cart = name();
 
   try
   {
@@ -254,12 +287,7 @@ bool Cartridge3E::save(Serializer& out) const
   }
   catch(const char* msg)
   {
-    cerr << msg << endl;
-    return false;
-  }
-  catch(...)
-  {
-    cerr << "Unknown error in save state for " << cart << endl;
+    cerr << "ERROR: Cartridge3E::save" << endl << "  " << msg << endl;
     return false;
   }
 
@@ -267,9 +295,9 @@ bool Cartridge3E::save(Serializer& out) const
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool Cartridge3E::load(Deserializer& in)
+bool Cartridge3E::load(Serializer& in)
 {
-  string cart = name();
+  const string& cart = name();
 
   try
   {
@@ -285,12 +313,7 @@ bool Cartridge3E::load(Deserializer& in)
   }
   catch(const char* msg)
   {
-    cerr << msg << endl;
-    return false;
-  }
-  catch(...)
-  {
-    cerr << "Unknown error in load state for " << cart << endl;
+    cerr << "ERROR: Cartridge3E::load" << endl << "  " << msg << endl;
     return false;
   }
 
