@@ -8,15 +8,17 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Cart3F.cxx,v 1.18 2008/03/28 23:29:13 stephena Exp $
+// $Id: Cart3F.cxx 2012 2010-04-14 22:35:46Z stephena $
 //============================================================================
 
 #include <cassert>
+#include <cstring>
 
 #include "System.hxx"
 #include "TIA.hxx"
@@ -30,10 +32,10 @@ Cartridge3F::Cartridge3F(const uInt8* image, uInt32 size)
   myImage = new uInt8[mySize];
 
   // Copy the ROM image into my buffer
-  for(uInt32 addr = 0; addr < mySize; ++addr)
-  {
-    myImage[addr] = image[addr];
-  }
+  memcpy(myImage, image, mySize);
+
+  // Remember startup bank
+  myStartBank = 0;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -45,8 +47,8 @@ Cartridge3F::~Cartridge3F()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Cartridge3F::reset()
 {
-  // We'll map bank 0 into the first segment upon reset
-  bank(0);
+  // We'll map the startup bank into the first segment upon reset
+  bank(myStartBank);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -62,37 +64,37 @@ void Cartridge3F::install(System& system)
   // Set the page accessing methods for the hot spots (for 100% emulation
   // we need to chain any accesses below 0x40 to the TIA. Our poke() method
   // does this via mySystem->tiaPoke(...), at least until we come up with a
-  // cleaner way to do it.)
+  // cleaner way to do it).
   System::PageAccess access;
+  access.directPeekBase = 0;
+  access.directPokeBase = 0;
+  access.device = this;
+  access.type = System::PA_READWRITE;
   for(uInt32 i = 0x00; i < 0x40; i += (1 << shift))
-  {
-    access.directPeekBase = 0;
-    access.directPokeBase = 0;
-    access.device = this;
     mySystem->setPageAccess(i >> shift, access);
-  }
 
   // Setup the second segment to always point to the last ROM slice
+  access.directPokeBase = 0;
+  access.device = this;
+  access.type = System::PA_READ;
   for(uInt32 j = 0x1800; j < 0x2000; j += (1 << shift))
   {
-    access.device = this;
     access.directPeekBase = &myImage[(mySize - 2048) + (j & 0x07FF)];
-    access.directPokeBase = 0;
     mySystem->setPageAccess(j >> shift, access);
   }
 
-  // Install pages for bank 0 into the first segment
-  bank(0);
+  // Install pages for startup bank into the first segment
+  bank(myStartBank);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 Cartridge3F::peek(uInt16 address)
 {
-  address = address & 0x0FFF;
+  address &= 0x0FFF;
 
   if(address < 0x0800)
   {
-    return myImage[(address & 0x07FF) + myCurrentBank * 2048];
+    return myImage[(address & 0x07FF) + (myCurrentBank << 11)];
   }
   else
   {
@@ -101,9 +103,9 @@ uInt8 Cartridge3F::peek(uInt16 address)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Cartridge3F::poke(uInt16 address, uInt8 value)
+bool Cartridge3F::poke(uInt16 address, uInt8 value)
 {
-  address = address & 0x0FFF;
+  address &= 0x0FFF;
 
   // Switch banks if necessary
   if(address <= 0x003F)
@@ -111,16 +113,22 @@ void Cartridge3F::poke(uInt16 address, uInt8 value)
     bank(value);
   }
 
+  // Pass the poke through to the TIA. In a real Atari, both the cart and the
+  // TIA see the address lines, and both react accordingly. In Stella, each
+  // 64-byte chunk of address space is "owned" by only one device. If we
+  // don't chain the poke to the TIA, then the TIA can't see it...
   mySystem->tia().poke(address, value);
+
+  return false;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Cartridge3F::bank(uInt16 bank)
 { 
-  if(myBankLocked) return;
+  if(bankLocked()) return;
 
   // Make sure the bank they're asking for is reasonable
-  if((uInt32)bank * 2048 < mySize)
+  if(((uInt32)bank << 11) < mySize)
   {
     myCurrentBank = bank;
   }
@@ -128,16 +136,17 @@ void Cartridge3F::bank(uInt16 bank)
   {
     // Oops, the bank they're asking for isn't valid so let's wrap it
     // around to a valid bank number
-    myCurrentBank = bank % (mySize / 2048);
+    myCurrentBank = bank % (mySize >> 11);
   }
 
-  uInt32 offset = myCurrentBank * 2048;
+  uInt32 offset = myCurrentBank << 11;
   uInt16 shift = mySystem->pageShift();
 
   // Setup the page access methods for the current bank
   System::PageAccess access;
-  access.device = this;
   access.directPokeBase = 0;
+  access.device = this;
+  access.type = System::PA_READ;
 
   // Map ROM image into the system
   for(uInt32 address = 0x1000; address < 0x1800; address += (1 << shift))
@@ -145,46 +154,45 @@ void Cartridge3F::bank(uInt16 bank)
     access.directPeekBase = &myImage[offset + (address & 0x07FF)];
     mySystem->setPageAccess(address >> shift, access);
   }
+  myBankChanged = true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int Cartridge3F::bank()
+uInt16 Cartridge3F::bank() const
 {
   return myCurrentBank;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int Cartridge3F::bankCount()
+uInt16 Cartridge3F::bankCount() const
 {
-  return mySize / 2048;
+  return mySize >> 11;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Cartridge3F::patch(uInt16 address, uInt8 value)
 {
-  address = address & 0x0FFF;
+  address &= 0x0FFF;
+
   if(address < 0x0800)
-  {
-    myImage[(address & 0x07FF) + myCurrentBank * 2048] = value;
-  }
+    myImage[(address & 0x07FF) + (myCurrentBank << 11)] = value;
   else
-  {
     myImage[(address & 0x07FF) + mySize - 2048] = value;
-  }
-  return true;
+
+  return myBankChanged = true;
 } 
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt8* Cartridge3F::getImage(int& size)
+const uInt8* Cartridge3F::getImage(int& size) const
 {
   size = mySize;
-  return &myImage[0];
+  return myImage;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Cartridge3F::save(Serializer& out) const
 {
-  string cart = name();
+  const string& cart = name();
 
   try
   {
@@ -193,12 +201,7 @@ bool Cartridge3F::save(Serializer& out) const
   }
   catch(const char* msg)
   {
-    cerr << msg << endl;
-    return false;
-  }
-  catch(...)
-  {
-    cerr << "Unknown error in save state for " << cart << endl;
+    cerr << "ERROR: Cartridge3F::save" << endl << "  " << msg << endl;
     return false;
   }
 
@@ -206,9 +209,9 @@ bool Cartridge3F::save(Serializer& out) const
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool Cartridge3F::load(Deserializer& in)
+bool Cartridge3F::load(Serializer& in)
 {
-  string cart = name();
+  const string& cart = name();
 
   try
   {
@@ -219,12 +222,7 @@ bool Cartridge3F::load(Deserializer& in)
   }
   catch(const char* msg)
   {
-    cerr << msg << endl;
-    return false;
-  }
-  catch(...)
-  {
-    cerr << "Unknown error in load state for " << cart << endl;
+    cerr << "ERROR: Cartridge3F::load" << endl << "  " << msg << endl;
     return false;
   }
 
