@@ -8,18 +8,18 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Cart4A50.cxx,v 1.14 2008/03/28 23:29:13 stephena Exp $
+// $Id: Cart4A50.cxx 2012 2010-04-14 22:35:46Z stephena $
 //============================================================================
 
 #include <cassert>
 #include <cstring>
 
-#include "Random.hxx"
 #include "System.hxx"
 #include "M6532.hxx"
 #include "TIA.hxx"
@@ -35,11 +35,6 @@ Cartridge4A50::Cartridge4A50(const uInt8* image, uInt32 size)
   else                    size = 131072;
   for(uInt32 slice = 0; slice < 131072 / size; ++slice)
     memcpy(myImage + (slice*size), image, size);
-
-  // Initialize RAM with random values
-  class Random random;
-  for(uInt32 i = 0; i < 32768; ++i)
-    myRAM[i] = random.next();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -50,11 +45,17 @@ Cartridge4A50::~Cartridge4A50()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Cartridge4A50::reset()
 {
+  // Initialize RAM with random values
+  for(uInt32 i = 0; i < 32768; ++i)
+    myRAM[i] = mySystem->randGenerator().next();
+
   mySliceLow = mySliceMiddle = mySliceHigh = 0;
   myIsRomLow = myIsRomMiddle = myIsRomHigh = true;
 
   myLastData    = 0xff;
   myLastAddress = 0xffff;
+
+  myBankChanged = true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -69,13 +70,13 @@ void Cartridge4A50::install(System& system)
 
   // Map all of the accesses to call peek and poke
   System::PageAccess access;
+  access.directPeekBase = 0;
+  access.directPokeBase = 0;
+  access.device = this;
+  access.type = System::PA_READ;  // We don't yet indicate RAM areas
+
   for(uInt32 i = 0x1000; i < 0x2000; i += (1 << shift))
-  {
-    access.directPeekBase = 0;
-    access.directPokeBase = 0;
-    access.device = this;
     mySystem->setPageAccess(i >> shift, access);
-  }
 
   // Mirror all access in TIA and RIOT; by doing so we're taking responsibility
   // for that address space in peek and poke below.
@@ -120,7 +121,7 @@ uInt8 Cartridge4A50::peek(uInt16 address)
     else if((address & 0x1f00) == 0x1f00)      // 256B region from 0x1f00 - 0x1fff
     {
       value = myImage[(address & 0xff) + 0x1ff00];
-      if(!myBankLocked && ((myLastData & 0xe0) == 0x60) &&
+      if(!bankLocked() && ((myLastData & 0xe0) == 0x60) &&
          ((myLastAddress >= 0x1000) || (myLastAddress < 0x200)))
         mySliceHigh = (mySliceHigh & 0xf0ff) | ((address & 0x8) << 8) |
                       ((address & 0x70) << 4);
@@ -133,7 +134,7 @@ uInt8 Cartridge4A50::peek(uInt16 address)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Cartridge4A50::poke(uInt16 address, uInt8 value)
+bool Cartridge4A50::poke(uInt16 address, uInt8 value)
 {
   if(!(address & 0x1000))                      // Hotspots below 0x1000
   {
@@ -151,35 +152,49 @@ void Cartridge4A50::poke(uInt16 address, uInt8 value)
     if((address & 0x1800) == 0x1000)           // 2K region at 0x1000 - 0x17ff
     {
       if(!myIsRomLow)
+      {
         myRAM[(address & 0x7ff) + mySliceLow] = value;
+        myBankChanged = true;
+      }
     }
     else if(((address & 0x1fff) >= 0x1800) &&  // 1.5K region at 0x1800 - 0x1dff
             ((address & 0x1fff) <= 0x1dff))
     {
       if(!myIsRomMiddle)
+      {
         myRAM[(address & 0x7ff) + mySliceMiddle] = value;
+        myBankChanged = true;
+      }
     }
     else if((address & 0x1f00) == 0x1e00)      // 256B region at 0x1e00 - 0x1eff
     {
       if(!myIsRomHigh)
+      {
         myRAM[(address & 0xff) + mySliceHigh] = value;
+        myBankChanged = true;
+      }
     }
     else if((address & 0x1f00) == 0x1f00)      // 256B region at 0x1f00 - 0x1fff
     {
-      if(!myBankLocked && ((myLastData & 0xe0) == 0x60) &&
+      if(!bankLocked() && ((myLastData & 0xe0) == 0x60) &&
          ((myLastAddress >= 0x1000) || (myLastAddress < 0x200)))
+      {
         mySliceHigh = (mySliceHigh & 0xf0ff) | ((address & 0x8) << 8) |
                       ((address & 0x70) << 4);
+        myBankChanged = true;
+      }
     }
   }
   myLastData = value;
   myLastAddress = address & 0x1fff;
+
+  return myBankChanged;
 } 
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Cartridge4A50::checkBankSwitch(uInt16 address, uInt8 value)
 {
-  if(myBankLocked) return;
+  if(bankLocked()) return;
 
   // This scheme contains so many hotspots that it's easier to just check
   // all of them
@@ -190,49 +205,59 @@ void Cartridge4A50::checkBankSwitch(uInt16 address, uInt8 value)
     {
       myIsRomHigh = true;
       mySliceHigh = (address & 0xff) << 8;
+      myBankChanged = true;
     }
     else if((address & 0x0f00) == 0x0d00)  // Enable 256B of RAM at 0x1e00 - 0x1eff
     {
       myIsRomHigh = false;
       mySliceHigh = (address & 0x7f) << 8;
+      myBankChanged = true;
     }
     else if((address & 0x0f40) == 0x0e00)  // Enable 2K of ROM at 0x1000 - 0x17ff
     {
       myIsRomLow = true;
       mySliceLow = (address & 0x1f) << 11;
+      myBankChanged = true;
     }
     else if((address & 0x0f40) == 0x0e40)  // Enable 2K of RAM at 0x1000 - 0x17ff
     {
       myIsRomLow = false;
       mySliceLow = (address & 0xf) << 11;
+      myBankChanged = true;
     }
     else if((address & 0x0f40) == 0x0f00)  // Enable 1.5K of ROM at 0x1800 - 0x1dff
     {
       myIsRomMiddle = true;
       mySliceMiddle = (address & 0x1f) << 11;
+      myBankChanged = true;
     }
     else if((address & 0x0f50) == 0x0f40)  // Enable 1.5K of RAM at 0x1800 - 0x1dff
     {
       myIsRomMiddle = false;
       mySliceMiddle = (address & 0xf) << 11;
+      myBankChanged = true;
     }
 
     // Stella helper functions
     else if((address & 0x0f00) == 0x0400)   // Toggle bit A11 of lower block address
     {
       mySliceLow = mySliceLow ^ 0x800;
+      myBankChanged = true;
     }
     else if((address & 0x0f00) == 0x0500)   // Toggle bit A12 of lower block address
     {
       mySliceLow = mySliceLow ^ 0x1000;
+      myBankChanged = true;
     }
     else if((address & 0x0f00) == 0x0800)   // Toggle bit A11 of middle block address
     {
       mySliceMiddle = mySliceMiddle ^ 0x800;
+      myBankChanged = true;
     }
     else if((address & 0x0f00) == 0x0900)   // Toggle bit A12 of middle block address
     {
       mySliceMiddle = mySliceMiddle ^ 0x1000;
+      myBankChanged = true;
     }
   }
 
@@ -244,11 +269,13 @@ void Cartridge4A50::checkBankSwitch(uInt16 address, uInt8 value)
   {
     myIsRomHigh = true;
     mySliceHigh = value << 8;
+    myBankChanged = true;
   }
   else if((address & 0xf75) == 0x75)    // Enable 256B of RAM at 0x1e00 - 0x1eff
   {
     myIsRomHigh = false;
     mySliceHigh = (value & 0x7f) << 8;
+    myBankChanged = true;
   }
 
   // Zero-page hotspots for lower and middle blocks
@@ -260,21 +287,25 @@ void Cartridge4A50::checkBankSwitch(uInt16 address, uInt8 value)
     {
       myIsRomLow = true;
       mySliceLow = (value & 0xf) << 11;
+      myBankChanged = true;
     }
     else if((value & 0xf0) == 0x40)   // Enable 2K of RAM at 0x1000 - 0x17ff
     {
       myIsRomLow = false;
       mySliceLow = (value & 0xf) << 11;
+      myBankChanged = true;
     }
     else if((value & 0xf0) == 0x90)   // Enable 1.5K of ROM at 0x1800 - 0x1dff
     {
       myIsRomMiddle = true;
       mySliceMiddle = ((value & 0xf) | 0x10) << 11;
+      myBankChanged = true;
     }
     else if((value & 0xf0) == 0xc0)   // Enable 1.5K of RAM at 0x1800 - 0x1dff
     {
       myIsRomMiddle = false;
       mySliceMiddle = (value & 0xf) << 11;
+      myBankChanged = true;
     }
   }
 }
@@ -286,36 +317,63 @@ void Cartridge4A50::bank(uInt16)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int Cartridge4A50::bank()
+uInt16 Cartridge4A50::bank() const
 {
   // Doesn't support bankswitching in the normal sense
   return 0;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int Cartridge4A50::bankCount()
+uInt16 Cartridge4A50::bankCount() const
 {
   // Doesn't support bankswitching in the normal sense
+  // There is one 'virtual' bank that can change in many different ways
   return 1;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Cartridge4A50::patch(uInt16 address, uInt8 value)
 {
-  return false;  // TODO
+  if((address & 0x1800) == 0x1000)           // 2K region from 0x1000 - 0x17ff
+  {
+    if(myIsRomLow)
+      myImage[(address & 0x7ff) + mySliceLow] = value;
+    else
+      myRAM[(address & 0x7ff) + mySliceLow] = value;
+  }
+  else if(((address & 0x1fff) >= 0x1800) &&  // 1.5K region from 0x1800 - 0x1dff
+          ((address & 0x1fff) <= 0x1dff))
+  {
+    if(myIsRomMiddle)
+      myImage[(address & 0x7ff) + mySliceMiddle + 0x10000] = value;
+    else
+      myRAM[(address & 0x7ff) + mySliceMiddle] = value;
+  }
+  else if((address & 0x1f00) == 0x1e00)      // 256B region from 0x1e00 - 0x1eff
+  {
+    if(myIsRomHigh)
+      myImage[(address & 0xff) + mySliceHigh + 0x10000] = value;
+    else
+      myRAM[(address & 0xff) + mySliceHigh] = value;
+  }
+  else if((address & 0x1f00) == 0x1f00)      // 256B region from 0x1f00 - 0x1fff
+  {
+    myImage[(address & 0xff) + 0x1ff00] = value;
+  }
+  return myBankChanged = true;
 } 
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt8* Cartridge4A50::getImage(int& size)
+const uInt8* Cartridge4A50::getImage(int& size) const
 {
   size = 131072;
-  return &myImage[0];
+  return myImage;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Cartridge4A50::save(Serializer& out) const
 {
-  string cart = name();
+  const string& cart = name();
 
   try
   {
@@ -342,12 +400,7 @@ bool Cartridge4A50::save(Serializer& out) const
   }
   catch(const char* msg)
   {
-    cerr << msg << endl;
-    return false;
-  }
-  catch(...)
-  {
-    cerr << "Unknown error in save state for " << cart << endl;
+    cerr << "ERROR: Cartridge4A40::save" << endl << "  " << msg << endl;
     return false;
   }
 
@@ -355,9 +408,9 @@ bool Cartridge4A50::save(Serializer& out) const
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool Cartridge4A50::load(Deserializer& in)
+bool Cartridge4A50::load(Serializer& in)
 {
-  string cart = name();
+  const string& cart = name();
 
   try
   {
@@ -384,12 +437,7 @@ bool Cartridge4A50::load(Deserializer& in)
   }
   catch(const char* msg)
   {
-    cerr << msg << endl;
-    return false;
-  }
-  catch(...)
-  {
-    cerr << "Unknown error in load state for " << cart << endl;
+    cerr << "ERROR: Cartridge4A50::load" << endl << "  " << msg << endl;
     return false;
   }
 
