@@ -8,67 +8,70 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CartCV.cxx,v 1.16 2008/02/06 13:45:21 stephena Exp $
+// $Id: CartCV.cxx 2012 2010-04-14 22:35:46Z stephena $
 //============================================================================
 
 #include <cassert>
+#include <cstring>
 
-#include "Random.hxx"
 #include "System.hxx"
 #include "CartCV.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 CartridgeCV::CartridgeCV(const uInt8* image, uInt32 size)
+  : myInitialRAM(0),
+    mySize(size)
 {
-  uInt32 addr;
-  if(size == 2048)
+  if(mySize == 2048)
   {
-    // Copy the ROM image into my buffer
-    for(uInt32 addr = 0; addr < 2048; ++addr)
-    {
-      myImage[addr] = image[addr];
-    }
-
-    // Initialize RAM with random values
-    class Random random;
-    for(uInt32 i = 0; i < 1024; ++i)
-    {
-      myRAM[i] = random.next();
-    }
+    // Copy the ROM data into my buffer
+    memcpy(myImage, image, 2048);
   }
-  else if(size == 4096)
+  else if(mySize == 4096)
   {
     // The game has something saved in the RAM
-    // Usefull for MagiCard program listings
+    // Useful for MagiCard program listings
 
-    // Copy the ROM image into my buffer
-    for(addr = 0; addr < 2048; ++addr)
-    {
-      myImage[addr] = image[addr + 2048];
-    }
-
-    // Copy the RAM image into my buffer
-    for(addr = 0; addr < 1024; ++addr)
-    {
-      myRAM[addr] = image[addr];
-    }
+    // Copy the ROM data into my buffer
+    memcpy(myImage, image + 2048, 2048);
 
+    // Copy the RAM image into a buffer for use in reset()
+    myInitialRAM = new uInt8[1024];
+    memcpy(myInitialRAM, image, 1024);
   }
+
+  // This cart contains 1024 bytes extended RAM @ 0x1000
+  registerRamArea(0x1000, 1024, 0x00, 0x400);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 CartridgeCV::~CartridgeCV()
 {
+  delete[] myInitialRAM;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeCV::reset()
 {
+  if(myInitialRAM)
+  {
+    // Copy the RAM image into my buffer
+    memcpy(myRAM, myInitialRAM, 1024);
+  }
+  else
+  {
+    // Initialize RAM with random values
+    for(uInt32 i = 0; i < 1024; ++i)
+      myRAM[i] = mySystem->randGenerator().next();
+  }
+
+  myBankChanged = true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -82,10 +85,11 @@ void CartridgeCV::install(System& system)
   assert((0x1800 & mask) == 0);
 
   System::PageAccess access;
-  access.directPokeBase = 0;
-  access.device = this;
 
   // Map ROM image into the system
+  access.directPokeBase = 0;
+  access.device = this;
+  access.type = System::PA_READ;
   for(uInt32 address = 0x1800; address < 0x2000; address += (1 << shift))
   {
     access.directPeekBase = &myImage[address & 0x07FF];
@@ -93,20 +97,22 @@ void CartridgeCV::install(System& system)
   }
 
   // Set the page accessing method for the RAM writing pages
+  access.directPeekBase = 0;
+  access.device = this;
+  access.type = System::PA_WRITE;
   for(uInt32 j = 0x1400; j < 0x1800; j += (1 << shift))
   {
-    access.device = this;
-    access.directPeekBase = 0;
     access.directPokeBase = &myRAM[j & 0x03FF];
     mySystem->setPageAccess(j >> shift, access);
   }
 
   // Set the page accessing method for the RAM reading pages
+  access.directPokeBase = 0;
+  access.device = this;
+  access.type = System::PA_READ;
   for(uInt32 k = 0x1000; k < 0x1400; k += (1 << shift))
   {
-    access.device = this;
     access.directPeekBase = &myRAM[k & 0x03FF];
-    access.directPokeBase = 0;
     mySystem->setPageAccess(k >> shift, access);
   }
 }
@@ -114,13 +120,32 @@ void CartridgeCV::install(System& system)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 CartridgeCV::peek(uInt16 address)
 {
-  return myImage[address & 0x07FF];
+  if((address & 0x0FFF) < 0x0800)  // Write port is at 0xF400 - 0xF800 (1024 bytes)
+  {                                // Read port is handled in ::install()
+    // Reading from the write port triggers an unwanted write
+    uInt8 value = mySystem->getDataBusState(0xFF);
+
+    if(bankLocked())
+      return value;
+    else
+    {
+      triggerReadFromWritePort(address);
+      return myRAM[address & 0x03FF] = value;
+    }
+  }  
+  else
+  {
+    return myImage[address & 0x07FF];
+  }  
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void CartridgeCV::poke(uInt16, uInt8)
+bool CartridgeCV::poke(uInt16, uInt8)
 {
-  // This is ROM so poking has no effect :-)
+  // NOTE: This does not handle accessing RAM, however, this function 
+  // should never be called for RAM because of the way page accessing 
+  // has been setup
+  return false;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -130,14 +155,14 @@ void CartridgeCV::bank(uInt16 bank)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int CartridgeCV::bank()
+uInt16 CartridgeCV::bank() const
 {
   // Doesn't support bankswitching
   return 0;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int CartridgeCV::bankCount()
+uInt16 CartridgeCV::bankCount() const
 {
   return 1;
 }
@@ -145,21 +170,33 @@ int CartridgeCV::bankCount()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CartridgeCV::patch(uInt16 address, uInt8 value)
 {
-  myImage[address & 0x07FF] = value;
-  return true;
+  address &= 0x0FFF;
+
+  if(address < 0x0800)
+  {
+    // Normally, a write to the read port won't do anything
+    // However, the patch command is special in that ignores such
+    // cart restrictions
+    // The following will work for both reads and writes
+    myRAM[address & 0x03FF] = value;
+  }
+  else
+    myImage[address & 0x07FF] = value;
+
+  return myBankChanged = true;
 } 
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt8* CartridgeCV::getImage(int& size)
+const uInt8* CartridgeCV::getImage(int& size) const
 {
   size = 2048;
-  return &myImage[0];
+  return myImage;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CartridgeCV::save(Serializer& out) const
 {
-  string cart = name();
+  const string& cart = name();
 
   try
   {
@@ -172,12 +209,7 @@ bool CartridgeCV::save(Serializer& out) const
   }
   catch(const char* msg)
   {
-    cerr << msg << endl;
-    return false;
-  }
-  catch(...)
-  {
-    cerr << "Unknown error in save state for " << cart << endl;
+    cerr << "ERROR: CartridgeCV::save" << endl << "  " << msg << endl;
     return false;
   }
 
@@ -185,9 +217,9 @@ bool CartridgeCV::save(Serializer& out) const
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool CartridgeCV::load(Deserializer& in)
+bool CartridgeCV::load(Serializer& in)
 {
-  string cart = name();
+  const string& cart = name();
 
   try
   {
@@ -201,12 +233,7 @@ bool CartridgeCV::load(Deserializer& in)
   }
   catch(const char* msg)
   {
-    cerr << msg << endl;
-    return false;
-  }
-  catch(...)
-  {
-    cerr << "Unknown error in load state for " << cart << endl;
+    cerr << "ERROR: CartridgeCV::load" << endl << "  " << msg << endl;
     return false;
   }
 
