@@ -8,47 +8,38 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CartDPC.cxx,v 1.21 2008/03/28 23:29:13 stephena Exp $
+// $Id: CartDPC.cxx 2030 2010-05-02 22:52:59Z stephena $
 //============================================================================
 
 #include <cassert>
-#include <iostream>
+#include <cstring>
 
 #include "System.hxx"
 #include "CartDPC.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 CartridgeDPC::CartridgeDPC(const uInt8* image, uInt32 size)
+  : mySystemCycles(0),
+    myFractionalClocks(0.0)
 {
-  uInt32 addr;
+  // Make a copy of the entire image
+  memcpy(myImage, image, BSPF_min(size, 8192u + 2048u + 255u));
 
-  // Make a copy of the entire image as-is, for use by getImage()
-  // (this wastes 12K of RAM, should be controlled by a #ifdef)
-  for(addr = 0; addr < size; ++addr)
-    myImageCopy[addr] = image[addr];
+  // Pointer to the program ROM (8K @ 0 byte offset)
+  myProgramImage = myImage;
 
-  // Copy the program ROM image into my buffer
-  for(addr = 0; addr < 8192; ++addr)
-  {
-    myProgramImage[addr] = image[addr];
-  }
-
-  // Copy the display ROM image into my buffer
-  for(addr = 0; addr < 2048; ++addr)
-  {
-    myDisplayImage[addr] = image[8192 + addr];
-  }
+  // Pointer to the display ROM (2K @ 8K offset)
+  myDisplayImage = myProgramImage + 8192;
 
   // Initialize the DPC data fetcher registers
   for(uInt16 i = 0; i < 8; ++i)
-  {
     myTops[i] = myBottoms[i] = myCounters[i] = myFlags[i] = 0;
-  }
 
   // None of the data fetchers are in music mode
   myMusicMode[0] = myMusicMode[1] = myMusicMode[2] = false;
@@ -56,9 +47,8 @@ CartridgeDPC::CartridgeDPC(const uInt8* image, uInt32 size)
   // Initialize the DPC's random number generator register (must be non-zero)
   myRandomNumber = 1;
 
-  // Initialize the system cycles counter & fractional clock values
-  mySystemCycles = 0;
-  myFractionalClocks = 0.0;
+  // Remember startup bank
+  myStartBank = 1;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -73,8 +63,8 @@ void CartridgeDPC::reset()
   mySystemCycles = mySystem->cycles();
   myFractionalClocks = 0.0;
 
-  // Upon reset we switch to bank 1
-  bank(1);
+  // Upon reset we switch to the startup bank
+  bank(myStartBank);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -97,27 +87,26 @@ void CartridgeDPC::install(System& system)
   // Make sure the system we're being installed in has a page size that'll work
   assert(((0x1080 & mask) == 0) && ((0x1100 & mask) == 0));
 
-  // Set the page accessing methods for the hot spots
   System::PageAccess access;
+
+  // Set the page accessing methods for the hot spots
+  access.directPeekBase = 0;
+  access.directPokeBase = 0;
+  access.device = this;
+  access.type = System::PA_READ;
   for(uInt32 i = (0x1FF8 & ~mask); i < 0x2000; i += (1 << shift))
-  {
-    access.directPeekBase = 0;
-    access.directPokeBase = 0;
-    access.device = this;
     mySystem->setPageAccess(i >> shift, access);
-  }
 
   // Set the page accessing method for the DPC reading & writing pages
+  access.directPeekBase = 0;
+  access.directPokeBase = 0;
+  access.device = this;
+  access.type = System::PA_READWRITE;
   for(uInt32 j = 0x1000; j < 0x1080; j += (1 << shift))
-  {
-    access.directPeekBase = 0;
-    access.directPokeBase = 0;
-    access.device = this;
     mySystem->setPageAccess(j >> shift, access);
-  }
 
-  // Install pages for bank 1
-  bank(1);
+  // Install pages for the startup bank
+  bank(myStartBank);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -146,7 +135,7 @@ inline void CartridgeDPC::updateMusicModeDataFetchers()
   mySystemCycles = mySystem->cycles();
 
   // Calculate the number of DPC OSC clocks since the last update
-  double clocks = ((15750.0 * cycles) / 1193191.66666667) + myFractionalClocks;
+  double clocks = ((20000.0 * cycles) / 1193191.66666667) + myFractionalClocks;
   Int32 wholeClocks = (Int32)clocks;
   myFractionalClocks = clocks - (double)wholeClocks;
 
@@ -195,7 +184,12 @@ inline void CartridgeDPC::updateMusicModeDataFetchers()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 CartridgeDPC::peek(uInt16 address)
 {
-  address = address & 0x0FFF;
+  address &= 0x0FFF;
+
+  // In debugger/bank-locked mode, we ignore all hotspots and in general
+  // anything that can change the internal state of the cart
+  if(bankLocked())
+    return myProgramImage[(myCurrentBank << 12) + address];
 
   // Clock the random number generator.  This should be done for every
   // cartridge access, however, we're only doing it for the DPC and 
@@ -311,14 +305,14 @@ uInt8 CartridgeDPC::peek(uInt16 address)
       default:
         break;
     }
-    return myProgramImage[myCurrentBank * 4096 + address];
+    return myProgramImage[(myCurrentBank << 12) + address];
   }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void CartridgeDPC::poke(uInt16 address, uInt8 value)
+bool CartridgeDPC::poke(uInt16 address, uInt8 value)
 {
-  address = address & 0x0FFF;
+  address &= 0x0FFF;
 
   // Clock the random number generator.  This should be done for every
   // cartridge access, however, we're only doing it for the DPC and 
@@ -353,14 +347,14 @@ void CartridgeDPC::poke(uInt16 address, uInt8 value)
       {
         if((index >= 5) && myMusicMode[index - 5])
         {
-          // Data fecther is in music mode so its low counter value
+          // Data fetcher is in music mode so its low counter value
           // should be loaded from the top register not the poked value
           myCounters[index] = (myCounters[index] & 0x0700) |
               (uInt16)myTops[index];
         }
         else
         {
-          // Data fecther is either not a music mode data fecther or it
+          // Data fetcher is either not a music mode data fetcher or it
           // isn't in music mode so it's low counter value should be loaded
           // with the poked value
           myCounters[index] = (myCounters[index] & 0x0700) | (uInt16)value;
@@ -418,23 +412,25 @@ void CartridgeDPC::poke(uInt16 address, uInt8 value)
         break;
     }
   }
+  return false;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeDPC::bank(uInt16 bank)
 { 
-  if(myBankLocked) return;
+  if(bankLocked()) return;
 
   // Remember what bank we're in
   myCurrentBank = bank;
-  uInt16 offset = myCurrentBank * 4096;
+  uInt16 offset = myCurrentBank << 12;
   uInt16 shift = mySystem->pageShift();
   uInt16 mask = mySystem->pageMask();
 
   // Setup the page access methods for the current bank
   System::PageAccess access;
-  access.device = this;
   access.directPokeBase = 0;
+  access.device = this;
+  access.type = System::PA_READ;
 
   // Map Program ROM image into the system
   for(uInt32 address = 0x1080; address < (0x1FF8U & ~mask);
@@ -443,47 +439,47 @@ void CartridgeDPC::bank(uInt16 bank)
     access.directPeekBase = &myProgramImage[offset + (address & 0x0FFF)];
     mySystem->setPageAccess(address >> shift, access);
   }
+  myBankChanged = true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int CartridgeDPC::bank()
+uInt16 CartridgeDPC::bank() const
 {
   return myCurrentBank;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int CartridgeDPC::bankCount()
+uInt16 CartridgeDPC::bankCount() const
 {
-  return 2; // TODO: support the display ROM somehow
+  return 2;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CartridgeDPC::patch(uInt16 address, uInt8 value)
 {
-  address = address & 0x0FFF;
-  myProgramImage[myCurrentBank * 4096 + address] = value;
-  return true;
+  address &= 0x0FFF;
+
+  // For now, we ignore attempts to patch the DPC address space
+  if(address >= 0x0080)
+  {
+    myProgramImage[(myCurrentBank << 12) + (address & 0x0FFF)] = value;
+    return myBankChanged = true;
+  }
+  else
+    return false;
 } 
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt8* CartridgeDPC::getImage(int& size)
+const uInt8* CartridgeDPC::getImage(int& size) const
 {
   size = 8192 + 2048 + 255;
-
-  int i;
-  for(i = 0; i < 8192; i++)
-    myImageCopy[i] = myProgramImage[i];
-
-  for(i = 0; i < 2048; i++)
-    myImageCopy[i + 8192] = myDisplayImage[i];
-
-  return &myImageCopy[0];
+  return myImage;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CartridgeDPC::save(Serializer& out) const
 {
-  string cart = name();
+  const string& cart = name();
 
   try
   {
@@ -527,12 +523,7 @@ bool CartridgeDPC::save(Serializer& out) const
   }
   catch(const char* msg)
   {
-    cerr << msg << endl;
-    return false;
-  }
-  catch(...)
-  {
-    cerr << "Unknown error in save state for " << cart << endl;
+    cerr << "ERROR: CartridgeDPC::save" << endl << "  " << msg << endl;
     return false;
   }
 
@@ -540,9 +531,9 @@ bool CartridgeDPC::save(Serializer& out) const
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool CartridgeDPC::load(Deserializer& in)
+bool CartridgeDPC::load(Serializer& in)
 {
-  string cart = name();
+  const string& cart = name();
 
   try
   {
@@ -588,12 +579,7 @@ bool CartridgeDPC::load(Deserializer& in)
   }
   catch(const char* msg)
   {
-    cerr << msg << endl;
-    return false;
-  }
-  catch(...)
-  {
-    cerr << "Unknown error in load state for " << cart << endl;
+    cerr << "ERROR: CartridgeDPC::load" << endl << "  " << msg << endl;
     return false;
   }
 
