@@ -8,15 +8,17 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CartE0.cxx,v 1.16 2008/03/28 23:29:13 stephena Exp $
+// $Id: CartE0.cxx 2012 2010-04-14 22:35:46Z stephena $
 //============================================================================
 
 #include <cassert>
+#include <cstring>
 
 #include "System.hxx"
 #include "CartE0.hxx"
@@ -25,10 +27,7 @@
 CartridgeE0::CartridgeE0(const uInt8* image)
 {
   // Copy the ROM image into my buffer
-  for(uInt32 addr = 0; addr < 8192; ++addr)
-  {
-    myImage[addr] = image[addr];
-  }
+  memcpy(myImage, image, 8192);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -43,6 +42,8 @@ void CartridgeE0::reset()
   segmentZero(4);
   segmentOne(5);
   segmentTwo(6);
+
+  myBankChanged = true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -56,10 +57,12 @@ void CartridgeE0::install(System& system)
   assert(((0x1000 & mask) == 0) && ((0x1400 & mask) == 0) &&
       ((0x1800 & mask) == 0) && ((0x1C00 & mask) == 0));
 
-  // Set the page acessing methods for the first part of the last segment
   System::PageAccess access;
+
+  // Set the page acessing methods for the first part of the last segment
   access.directPokeBase = 0;
   access.device = this;
+  access.type = System::PA_READ;
   for(uInt32 i = 0x1C00; i < (0x1FE0U & ~mask); i += (1 << shift))
   {
     access.directPeekBase = &myImage[7168 + (i & 0x03FF)];
@@ -71,10 +74,9 @@ void CartridgeE0::install(System& system)
   access.directPeekBase = 0;
   access.directPokeBase = 0;
   access.device = this;
+  access.type = System::PA_READ;
   for(uInt32 j = (0x1FE0 & ~mask); j < 0x2000; j += (1 << shift))
-  {
     mySystem->setPageAccess(j >> shift, access);
-  }
 
   // Install some default slices for the other segments
   segmentZero(4);
@@ -85,7 +87,7 @@ void CartridgeE0::install(System& system)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 CartridgeE0::peek(uInt16 address)
 {
-  address = address & 0x0FFF;
+  address &= 0x0FFF;
 
   // Switch banks if necessary
   if((address >= 0x0FE0) && (address <= 0x0FE7))
@@ -105,9 +107,9 @@ uInt8 CartridgeE0::peek(uInt16 address)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void CartridgeE0::poke(uInt16 address, uInt8)
+bool CartridgeE0::poke(uInt16 address, uInt8)
 {
-  address = address & 0x0FFF;
+  address &= 0x0FFF;
 
   // Switch banks if necessary
   if((address >= 0x0FE0) && (address <= 0x0FE7))
@@ -122,12 +124,13 @@ void CartridgeE0::poke(uInt16 address, uInt8)
   {
     segmentTwo(address & 0x0007);
   }
+  return false;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeE0::segmentZero(uInt16 slice)
 { 
-  if(myBankLocked) return;
+  if(bankLocked()) return;
 
   // Remember the new slice
   myCurrentSlice[0] = slice;
@@ -136,20 +139,22 @@ void CartridgeE0::segmentZero(uInt16 slice)
 
   // Setup the page access methods for the current bank
   System::PageAccess access;
-  access.device = this;
   access.directPokeBase = 0;
+  access.device = this;
+  access.type = System::PA_READ;
 
   for(uInt32 address = 0x1000; address < 0x1400; address += (1 << shift))
   {
     access.directPeekBase = &myImage[offset + (address & 0x03FF)];
     mySystem->setPageAccess(address >> shift, access);
   }
+  myBankChanged = true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeE0::segmentOne(uInt16 slice)
 { 
-  if(myBankLocked) return;
+  if(bankLocked()) return;
 
   // Remember the new slice
   myCurrentSlice[1] = slice;
@@ -158,20 +163,22 @@ void CartridgeE0::segmentOne(uInt16 slice)
 
   // Setup the page access methods for the current bank
   System::PageAccess access;
-  access.device = this;
   access.directPokeBase = 0;
+  access.device = this;
+  access.type = System::PA_READ;
 
   for(uInt32 address = 0x1400; address < 0x1800; address += (1 << shift))
   {
     access.directPeekBase = &myImage[offset + (address & 0x03FF)];
     mySystem->setPageAccess(address >> shift, access);
   }
+  myBankChanged = true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeE0::segmentTwo(uInt16 slice)
 { 
-  if(myBankLocked) return;
+  if(bankLocked()) return;
 
   // Remember the new slice
   myCurrentSlice[2] = slice;
@@ -180,14 +187,16 @@ void CartridgeE0::segmentTwo(uInt16 slice)
 
   // Setup the page access methods for the current bank
   System::PageAccess access;
-  access.device = this;
   access.directPokeBase = 0;
+  access.device = this;
+  access.type = System::PA_READ;
 
   for(uInt32 address = 0x1800; address < 0x1C00; address += (1 << shift))
   {
     access.directPeekBase = &myImage[offset + (address & 0x03FF)];
     mySystem->setPageAccess(address >> shift, access);
   }
+  myBankChanged = true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -197,38 +206,39 @@ void CartridgeE0::bank(uInt16)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int CartridgeE0::bank()
+uInt16 CartridgeE0::bank() const
 {
   // Doesn't support bankswitching in the normal sense
   return 0;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int CartridgeE0::bankCount()
+uInt16 CartridgeE0::bankCount() const
 {
   // Doesn't support bankswitching in the normal sense
+  // There is one 'virtual' bank that can change in many different ways
   return 1;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CartridgeE0::patch(uInt16 address, uInt8 value)
 {
-  address = address & 0x0FFF;
+  address &= 0x0FFF;
   myImage[(myCurrentSlice[address >> 10] << 10) + (address & 0x03FF)] = value;
   return true;
 } 
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt8* CartridgeE0::getImage(int& size)
+const uInt8* CartridgeE0::getImage(int& size) const
 {
   size = 8192;
-  return &myImage[0];
+  return myImage;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CartridgeE0::save(Serializer& out) const
 {
-  string cart = name();
+  const string& cart = name();
 
   try
   {
@@ -240,12 +250,7 @@ bool CartridgeE0::save(Serializer& out) const
   }
   catch(const char* msg)
   {
-    cerr << msg << endl;
-    return false;
-  }
-  catch(...)
-  {
-    cerr << "Unknown error in save state for " << cart << endl;
+    cerr << "ERROR: CartridgeE0::save" << endl << "  " << msg << endl;
     return false;
   }
 
@@ -253,9 +258,9 @@ bool CartridgeE0::save(Serializer& out) const
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool CartridgeE0::load(Deserializer& in)
+bool CartridgeE0::load(Serializer& in)
 {
-  string cart = name();
+  const string& cart = name();
 
   try
   {
@@ -268,12 +273,7 @@ bool CartridgeE0::load(Deserializer& in)
   }
   catch(const char* msg)
   {
-    cerr << msg << endl;
-    return false;
-  }
-  catch(...)
-  {
-    cerr << "Unknown error in load state for " << cart << endl;
+    cerr << "ERROR: CartridgeE0::load" << endl << "  " << msg << endl;
     return false;
   }
 
