@@ -8,17 +8,18 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CartE7.cxx,v 1.19 2008/03/28 23:29:13 stephena Exp $
+// $Id: CartE7.cxx 2012 2010-04-14 22:35:46Z stephena $
 //============================================================================
 
 #include <cassert>
+#include <cstring>
 
-#include "Random.hxx"
 #include "System.hxx"
 #include "CartE7.hxx"
 
@@ -26,17 +27,17 @@
 CartridgeE7::CartridgeE7(const uInt8* image)
 {
   // Copy the ROM image into my buffer
-  for(uInt32 addr = 0; addr < 16384; ++addr)
-  {
-    myImage[addr] = image[addr];
-  }
+  memcpy(myImage, image, 16384);
 
-  // Initialize RAM with random values
-  class Random random;
-  for(uInt32 i = 0; i < 2048; ++i)
-  {
-    myRAM[i] = random.next();
-  }
+  // This cart can address a 1024 byte bank of RAM @ 0x1000
+  // and 256 bytes @ 0x1800
+  // However, it may not be addressable all the time (it may be swapped out)
+  // so probably most of the time, the area will point to ROM instead
+  registerRamArea(0x1000, 1024, 0x400, 0x00);  // 1024 bytes RAM @ 0x1000
+  registerRamArea(0x1800, 256, 0x100, 0x00);   // 256 bytes RAM @ 0x1800
+
+  // Remember startup bank
+  myStartBank = 0;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -47,9 +48,15 @@ CartridgeE7::~CartridgeE7()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeE7::reset()
 {
+  // Initialize RAM with random values
+  for(uInt32 i = 0; i < 2048; ++i)
+    myRAM[i] = mySystem->randGenerator().next();
+
   // Install some default banks for the RAM and first segment
   bankRAM(0);
-  bank(0);
+  bank(myStartBank);
+
+  myBankChanged = true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -63,35 +70,37 @@ void CartridgeE7::install(System& system)
   assert(((0x1400 & mask) == 0) && ((0x1800 & mask) == 0) &&
       ((0x1900 & mask) == 0) && ((0x1A00 & mask) == 0));
 
-  // Set the page accessing methods for the hot spots
   System::PageAccess access;
+
+  // Set the page accessing methods for the hot spots
+  access.directPeekBase = 0;
+  access.directPokeBase = 0;
+  access.device = this;
+  access.type = System::PA_READ;
   for(uInt32 i = (0x1FE0 & ~mask); i < 0x2000; i += (1 << shift))
-  {
-    access.directPeekBase = 0;
-    access.directPokeBase = 0;
-    access.device = this;
     mySystem->setPageAccess(i >> shift, access);
-  }
 
   // Setup the second segment to always point to the last ROM slice
+  access.directPokeBase = 0;
+  access.device = this;
+  access.type = System::PA_READ;
   for(uInt32 j = 0x1A00; j < (0x1FE0U & ~mask); j += (1 << shift))
   {
-    access.device = this;
     access.directPeekBase = &myImage[7 * 2048 + (j & 0x07FF)];
-    access.directPokeBase = 0;
     mySystem->setPageAccess(j >> shift, access);
   }
   myCurrentSlice[1] = 7;
 
   // Install some default banks for the RAM and first segment
   bankRAM(0);
-  bank(0);
+  bank(myStartBank);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 CartridgeE7::peek(uInt16 address)
 {
-  address = address & 0x0FFF;
+  uInt16 peekAddress = address;
+  address &= 0x0FFF;
 
   // Switch banks if necessary
   if((address >= 0x0FE0) && (address <= 0x0FE7))
@@ -103,16 +112,40 @@ uInt8 CartridgeE7::peek(uInt16 address)
     bankRAM(address & 0x0003);
   }
 
-  // NOTE: The following does not handle reading from RAM, however,
-  // this function should never be called for RAM because of the
-  // way page accessing has been setup
-  return myImage[(myCurrentSlice[address >> 11] << 11) + (address & 0x07FF)];
+  if((myCurrentSlice[0] == 7) && (address < 0x0400))
+  {
+    // Reading from the 1K write port @ $1000 triggers an unwanted write
+    uInt8 value = mySystem->getDataBusState(0xFF);
+
+    if(bankLocked())
+      return value;
+    else
+    {
+      triggerReadFromWritePort(peekAddress);
+      return myRAM[address & 0x03FF] = value;
+    }
+  }
+  else if((address >= 0x0800) && (address <= 0x08FF))
+  {
+    // Reading from the 256B write port @ $1800 triggers an unwanted write
+    uInt8 value = mySystem->getDataBusState(0xFF);
+
+    if(bankLocked())
+      return value;
+    else
+    {
+      triggerReadFromWritePort(peekAddress);
+      return myRAM[1024 + (myCurrentRAM << 8) + (address & 0x00FF)] = value;
+    }
+  }
+  else
+    return myImage[(myCurrentSlice[address >> 11] << 11) + (address & 0x07FF)];
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void CartridgeE7::poke(uInt16 address, uInt8)
+bool CartridgeE7::poke(uInt16 address, uInt8)
 {
-  address = address & 0x0FFF;
+  address &= 0x0FFF;
 
   // Switch banks if necessary
   if((address >= 0x0FE0) && (address <= 0x0FE7))
@@ -127,11 +160,14 @@ void CartridgeE7::poke(uInt16 address, uInt8)
   // NOTE: This does not handle writing to RAM, however, this 
   // function should never be called for RAM because of the
   // way page accessing has been setup
+  return false;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeE7::bankRAM(uInt16 bank)
 { 
+  if(bankLocked()) return;
+
   // Remember what bank we're in
   myCurrentRAM = bank;
   uInt16 offset = bank << 8;
@@ -139,11 +175,11 @@ void CartridgeE7::bankRAM(uInt16 bank)
 
   // Setup the page access methods for the current bank
   System::PageAccess access;
-  access.device = this;
 
   // Set the page accessing method for the 256 bytes of RAM writing pages
   access.directPeekBase = 0;
-  access.directPokeBase = 0;
+  access.device = this;
+  access.type = System::PA_WRITE;
   for(uInt32 j = 0x1800; j < 0x1900; j += (1 << shift))
   {
     access.directPokeBase = &myRAM[1024 + offset + (j & 0x00FF)];
@@ -151,33 +187,36 @@ void CartridgeE7::bankRAM(uInt16 bank)
   }
 
   // Set the page accessing method for the 256 bytes of RAM reading pages
-  access.directPeekBase = 0;
   access.directPokeBase = 0;
+  access.device = this;
+  access.type = System::PA_READ;
   for(uInt32 k = 0x1900; k < 0x1A00; k += (1 << shift))
   {
     access.directPeekBase = &myRAM[1024 + offset + (k & 0x00FF)];
     mySystem->setPageAccess(k >> shift, access);
   }
+  myBankChanged = true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeE7::bank(uInt16 slice)
 { 
-  if(myBankLocked) return;
+  if(bankLocked()) return;
 
   // Remember what bank we're in
   myCurrentSlice[0] = slice;
   uInt16 offset = slice << 11;
   uInt16 shift = mySystem->pageShift();
 
+  System::PageAccess access;
+
   // Setup the page access methods for the current bank
   if(slice != 7)
   {
-    System::PageAccess access;
-    access.device = this;
-    access.directPokeBase = 0;
-
     // Map ROM image into first segment
+    access.directPokeBase = 0;
+    access.device = this;
+    access.type = System::PA_READ;
     for(uInt32 address = 0x1000; address < 0x1800; address += (1 << shift))
     {
       access.directPeekBase = &myImage[offset + (address & 0x07FF)];
@@ -186,12 +225,10 @@ void CartridgeE7::bank(uInt16 slice)
   }
   else
   {
-    System::PageAccess access;
-    access.device = this;
-
     // Set the page accessing method for the 1K slice of RAM writing pages
     access.directPeekBase = 0;
-    access.directPokeBase = 0;
+    access.device = this;
+    access.type = System::PA_WRITE;
     for(uInt32 j = 0x1000; j < 0x1400; j += (1 << shift))
     {
       access.directPokeBase = &myRAM[j & 0x03FF];
@@ -199,24 +236,26 @@ void CartridgeE7::bank(uInt16 slice)
     }
 
     // Set the page accessing method for the 1K slice of RAM reading pages
-    access.directPeekBase = 0;
     access.directPokeBase = 0;
+    access.device = this;
+    access.type = System::PA_READ;
     for(uInt32 k = 0x1400; k < 0x1800; k += (1 << shift))
     {
       access.directPeekBase = &myRAM[k & 0x03FF];
       mySystem->setPageAccess(k >> shift, access);
     }
   }
+  myBankChanged = true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int CartridgeE7::bank()
+uInt16 CartridgeE7::bank() const
 {
   return myCurrentSlice[0];
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int CartridgeE7::bankCount()
+uInt16 CartridgeE7::bankCount() const
 {
   return 8;
 }
@@ -225,22 +264,43 @@ int CartridgeE7::bankCount()
 bool CartridgeE7::patch(uInt16 address, uInt8 value)
 {
   address = address & 0x0FFF;
-  myImage[(myCurrentSlice[address >> 11] << 11) + (address & 0x07FF)] = value;
-  bank(myCurrentSlice[0]);
-  return true;
+
+  if(address < 0x0800)
+  {
+    if(myCurrentSlice[0] == 7)
+    {
+      // Normally, a write to the read port won't do anything
+      // However, the patch command is special in that ignores such
+      // cart restrictions
+      myRAM[address & 0x03FF] = value;
+    }
+    else
+      myImage[(myCurrentSlice[0] << 11) + (address & 0x07FF)] = value;
+  }
+  else if(address < 0x0900)
+  {
+    // Normally, a write to the read port won't do anything
+    // However, the patch command is special in that ignores such
+    // cart restrictions
+    myRAM[1024 + (myCurrentRAM << 8) + (address & 0x00FF)] = value;
+  }
+  else
+    myImage[(myCurrentSlice[address >> 11] << 11) + (address & 0x07FF)] = value;
+
+  return myBankChanged = true;
 } 
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt8* CartridgeE7::getImage(int& size)
+const uInt8* CartridgeE7::getImage(int& size) const
 {
   size = 16384;
-  return &myImage[0];
+  return myImage;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CartridgeE7::save(Serializer& out) const
 {
-  string cart = name();
+  const string& cart = name();
 
   try
   {
@@ -261,12 +321,7 @@ bool CartridgeE7::save(Serializer& out) const
   }
   catch(const char* msg)
   {
-    cerr << msg << endl;
-    return false;
-  }
-  catch(...)
-  {
-    cerr << "Unknown error in save state for " << cart << endl;
+    cerr << "ERROR: CartridgeE7::save" << endl << "  " << msg << endl;
     return false;
   }
 
@@ -274,9 +329,9 @@ bool CartridgeE7::save(Serializer& out) const
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool CartridgeE7::load(Deserializer& in)
+bool CartridgeE7::load(Serializer& in)
 {
-  string cart = name();
+  const string& cart = name();
 
   try
   {
@@ -298,12 +353,7 @@ bool CartridgeE7::load(Deserializer& in)
   }
   catch(const char* msg)
   {
-    cerr << msg << endl;
-    return false;
-  }
-  catch(...)
-  {
-    cerr << "Unknown error in load state for " << cart << endl;
+    cerr << "ERROR: CartridgeE7::load" << endl << "  " << msg << endl;
     return false;
   }
 
