@@ -8,17 +8,18 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CartF4SC.cxx,v 1.17 2008/03/28 23:29:13 stephena Exp $
+// $Id: CartF4SC.cxx 2012 2010-04-14 22:35:46Z stephena $
 //============================================================================
 
 #include <cassert>
+#include <cstring>
 
-#include "Random.hxx"
 #include "System.hxx"
 #include "CartF4SC.hxx"
 
@@ -26,17 +27,13 @@
 CartridgeF4SC::CartridgeF4SC(const uInt8* image)
 {
   // Copy the ROM image into my buffer
-  for(uInt32 addr = 0; addr < 32768; ++addr)
-  {
-    myImage[addr] = image[addr];
-  }
+  memcpy(myImage, image, 32768);
 
-  // Initialize RAM with random values
-  class Random random;
-  for(uInt32 i = 0; i < 128; ++i)
-  {
-    myRAM[i] = random.next();
-  }
+  // This cart contains 128 bytes extended RAM @ 0x1000
+  registerRamArea(0x1000, 128, 0x80, 0x00);
+
+  // Remember startup bank
+  myStartBank = 0;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -47,8 +44,12 @@ CartridgeF4SC::~CartridgeF4SC()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeF4SC::reset()
 {
-  // Upon reset we switch to bank 0
-  bank(0);
+  // Initialize RAM with random values
+  for(uInt32 i = 0; i < 128; ++i)
+    myRAM[i] = mySystem->randGenerator().next();
+
+  // Upon reset we switch to the startup bank
+  bank(myStartBank);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -61,84 +62,101 @@ void CartridgeF4SC::install(System& system)
   // Make sure the system we're being installed in has a page size that'll work
   assert(((0x1080 & mask) == 0) && ((0x1100 & mask) == 0));
 
-  // Set the page accessing methods for the hot spots
   System::PageAccess access;
+
+  // Set the page accessing methods for the hot spots
+  access.directPeekBase = 0;
+  access.directPokeBase = 0;
+  access.device = this;
+  access.type = System::PA_READ;
   for(uInt32 i = (0x1FF4 & ~mask); i < 0x2000; i += (1 << shift))
-  {
-    access.directPeekBase = 0;
-    access.directPokeBase = 0;
-    access.device = this;
     mySystem->setPageAccess(i >> shift, access);
-  }
 
   // Set the page accessing method for the RAM writing pages
+  access.directPeekBase = 0;
+  access.device = this;
+  access.type = System::PA_WRITE;
   for(uInt32 j = 0x1000; j < 0x1080; j += (1 << shift))
   {
-    access.device = this;
-    access.directPeekBase = 0;
     access.directPokeBase = &myRAM[j & 0x007F];
     mySystem->setPageAccess(j >> shift, access);
   }
 
   // Set the page accessing method for the RAM reading pages
+  access.directPokeBase = 0;
+  access.device = this;
+  access.type = System::PA_READ;
   for(uInt32 k = 0x1080; k < 0x1100; k += (1 << shift))
   {
-    access.device = this;
     access.directPeekBase = &myRAM[k & 0x007F];
-    access.directPokeBase = 0;
     mySystem->setPageAccess(k >> shift, access);
   }
 
-  // Install pages for bank 0
-  bank(0);
+  // Install pages for the startup bank
+  bank(myStartBank);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 CartridgeF4SC::peek(uInt16 address)
 {
-  address = address & 0x0FFF;
+  uInt16 peekAddress = address;
+  address &= 0x0FFF;
 
   // Switch banks if necessary
   if((address >= 0x0FF4) && (address <= 0x0FFB))
-  {
     bank(address - 0x0FF4);
-  }
+
+  if(address < 0x0080)  // Write port is at 0xF000 - 0xF080 (128 bytes)
+  {
+    // Reading from the write port triggers an unwanted write
+    uInt8 value = mySystem->getDataBusState(0xFF);
+
+    if(bankLocked())
+      return value;
+    else
+    {
+      triggerReadFromWritePort(peekAddress);
+      return myRAM[address] = value;
+    }
+  }  
 
   // NOTE: This does not handle accessing RAM, however, this function 
   // should never be called for RAM because of the way page accessing 
   // has been setup
-  return myImage[myCurrentBank * 4096 + address];
+  return myImage[(myCurrentBank << 12) + address];
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void CartridgeF4SC::poke(uInt16 address, uInt8)
+bool CartridgeF4SC::poke(uInt16 address, uInt8)
 {
+  address &= 0x0FFF;
+
   // Switch banks if necessary
   if((address >= 0x0FF4) && (address <= 0x0FFB))
-  {
     bank(address - 0x0FF4);
-  }
 
   // NOTE: This does not handle accessing RAM, however, this function 
   // should never be called for RAM because of the way page accessing 
   // has been setup
+  return false;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeF4SC::bank(uInt16 bank)
 { 
-  if(myBankLocked) return;
+  if(bankLocked()) return;
 
   // Remember what bank we're in
   myCurrentBank = bank;
-  uInt16 offset = myCurrentBank * 4096;
+  uInt16 offset = myCurrentBank << 12;
   uInt16 shift = mySystem->pageShift();
   uInt16 mask = mySystem->pageMask();
 
   // Setup the page access methods for the current bank
   System::PageAccess access;
-  access.device = this;
   access.directPokeBase = 0;
+  access.device = this;
+  access.type = System::PA_READ;
 
   // Map ROM image into the system
   for(uInt32 address = 0x1100; address < (0x1FF4U & ~mask);
@@ -147,16 +165,17 @@ void CartridgeF4SC::bank(uInt16 bank)
     access.directPeekBase = &myImage[offset + (address & 0x0FFF)];
     mySystem->setPageAccess(address >> shift, access);
   }
+  myBankChanged = true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int CartridgeF4SC::bank()
+uInt16 CartridgeF4SC::bank() const
 {
   return myCurrentBank;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int CartridgeF4SC::bankCount()
+uInt16 CartridgeF4SC::bankCount() const
 {
   return 8;
 }
@@ -164,22 +183,32 @@ int CartridgeF4SC::bankCount()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CartridgeF4SC::patch(uInt16 address, uInt8 value)
 {
-  address = address & 0x0FFF;
-  myImage[myCurrentBank * 4096 + address] = value;
-  return true;
+  address &= 0x0FFF;
+
+  if(address < 0x0100)
+  {
+    // Normally, a write to the read port won't do anything
+    // However, the patch command is special in that ignores such
+    // cart restrictions
+    myRAM[address & 0x007F] = value;
+  }
+  else
+    myImage[(myCurrentBank << 12) + address] = value;
+
+  return myBankChanged = true;
 } 
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt8* CartridgeF4SC::getImage(int& size)
+const uInt8* CartridgeF4SC::getImage(int& size) const
 {
   size = 32768;
-  return &myImage[0];
+  return myImage;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CartridgeF4SC::save(Serializer& out) const
 {
-  string cart = name();
+  const string& cart = name();
 
   try
   {
@@ -194,12 +223,7 @@ bool CartridgeF4SC::save(Serializer& out) const
   }
   catch(const char* msg)
   {
-    cerr << msg << endl;
-    return false;
-  }
-  catch(...)
-  {
-    cerr << "Unknown error in save state for " << cart << endl;
+    cerr << "ERROR: CartridgeF4SC::save" << endl << "  " << msg << endl;
     return false;
   }
 
@@ -207,9 +231,9 @@ bool CartridgeF4SC::save(Serializer& out) const
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool CartridgeF4SC::load(Deserializer& in)
+bool CartridgeF4SC::load(Serializer& in)
 {
-  string cart = name();
+  const string& cart = name();
 
   try
   {
@@ -224,12 +248,7 @@ bool CartridgeF4SC::load(Deserializer& in)
   }
   catch(const char* msg)
   {
-    cerr << msg << endl;
-    return false;
-  }
-  catch(...)
-  {
-    cerr << "Unknown error in load state for " << cart << endl;
+    cerr << "ERROR: CartridgeF4SC::load" << endl << "  " << msg << endl;
     return false;
   }
 
