@@ -8,27 +8,29 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CartFE.cxx,v 1.10 2008/02/06 13:45:21 stephena Exp $
+// $Id: CartFE.cxx 2012 2010-04-14 22:35:46Z stephena $
 //============================================================================
 
 #include <cassert>
+#include <cstring>
 
 #include "System.hxx"
 #include "CartFE.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 CartridgeFE::CartridgeFE(const uInt8* image)
+  : myLastAddress1(0),
+    myLastAddress2(0),
+    myLastAddressChanged(false)
 {
   // Copy the ROM image into my buffer
-  for(uInt32 addr = 0; addr < 8192; ++addr)
-  {
-    myImage[addr] = image[addr];
-  }
+  memcpy(myImage, image, 8192);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -53,25 +55,31 @@ void CartridgeFE::install(System& system)
 
   // Map all of the accesses to call peek and poke
   System::PageAccess access;
+  access.directPeekBase = 0;
+  access.directPokeBase = 0;
+  access.device = this;
+  access.type = System::PA_READ;
   for(uInt32 i = 0x1000; i < 0x2000; i += (1 << shift))
-  {
-    access.directPeekBase = 0;
-    access.directPokeBase = 0;
-    access.device = this;
     mySystem->setPageAccess(i >> shift, access);
-  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 CartridgeFE::peek(uInt16 address)
 {
   // The bank is determined by A13 of the processor
+  // We keep track of the two most recent accesses to determine which bank
+  // we're in, and when the values actually changed
+  myLastAddress2 = myLastAddress1;
+  myLastAddress1 = address;
+  myLastAddressChanged = true;
+
   return myImage[(address & 0x0FFF) + (((address & 0x2000) == 0) ? 4096 : 0)];
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void CartridgeFE::poke(uInt16, uInt8)
+bool CartridgeFE::poke(uInt16, uInt8)
 {
+  return false;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -81,49 +89,67 @@ void CartridgeFE::bank(uInt16 b)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int CartridgeFE::bank()
+uInt16 CartridgeFE::bank() const
 {
-  // Doesn't support bankswitching in the normal sense
-  return 0;
+  // The current bank depends on the last address accessed
+  return ((myLastAddress1 & 0x2000) == 0) ? 1 : 0;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int CartridgeFE::bankCount()
+uInt16 CartridgeFE::bankCount() const
 {
-  return 1;
+  return 2;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool CartridgeFE::bankChanged()
+{
+  if(myLastAddressChanged)
+  {
+    // A bankswitch occurs when the addresses transition from state to another
+    bool a1 = ((myLastAddress1 & 0x2000) == 0),
+         a2 = ((myLastAddress2 & 0x2000) == 0);
+    myBankChanged = (a1 && !a2) || (a2 && !a1);
+    myLastAddressChanged = false;
+  }
+  else
+  {
+    myBankChanged = false;
+  }
+
+  // In any event, let the base class know about it
+  return Cartridge::bankChanged();
+}
+
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CartridgeFE::patch(uInt16 address, uInt8 value)
 {
   myImage[(address & 0x0FFF) + (((address & 0x2000) == 0) ? 4096 : 0)] = value;
-  return true;
+  return myBankChanged = true;
 } 
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt8* CartridgeFE::getImage(int& size)
+const uInt8* CartridgeFE::getImage(int& size) const
 {
   size = 8192;
-  return &myImage[0];
+  return myImage;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CartridgeFE::save(Serializer& out) const
 {
-  string cart = name();
+  const string& cart = name();
 
   try
   {
     out.putString(cart);
+    out.putInt(myLastAddress1);
+    out.putInt(myLastAddress2);
   }
   catch(const char* msg)
   {
-    cerr << msg << endl;
-    return false;
-  }
-  catch(...)
-  {
-    cerr << "Unknown error in save state for " << cart << endl;
+    cerr << "ERROR: CartridgeFE::save" << endl << "  " << msg << endl;
     return false;
   }
 
@@ -131,23 +157,21 @@ bool CartridgeFE::save(Serializer& out) const
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool CartridgeFE::load(Deserializer& in)
+bool CartridgeFE::load(Serializer& in)
 {
-  string cart = name();
+  const string& cart = name();
 
   try
   {
     if(in.getString() != cart)
       return false;
+
+    myLastAddress1 = (uInt16)in.getInt();
+    myLastAddress2 = (uInt16)in.getInt();
   }
   catch(const char* msg)
   {
-    cerr << msg << endl;
-    return false;
-  }
-  catch(...)
-  {
-    cerr << "Unknown error in load state for " << cart << endl;
+    cerr << "ERROR: CartridgeF8SC::load" << endl << "  " << msg << endl;
     return false;
   }
 
