@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Console.cxx,v 1.147 2008/05/21 21:36:18 stephena Exp $
+// $Id: Console.cxx 2001 2010-04-10 21:37:23Z stephena $
 //============================================================================
 
 #include <cassert>
@@ -31,10 +32,10 @@
 #include "EventHandler.hxx"
 #include "Joystick.hxx"
 #include "Keyboard.hxx"
-#include "M6502Hi.hxx"
-#include "M6502Low.hxx"
+#include "KidVid.hxx"
+#include "Genesis.hxx"
+#include "M6502.hxx"
 #include "M6532.hxx"
-#include "MediaSrc.hxx"
 #include "Paddles.hxx"
 #include "Props.hxx"
 #include "PropsSet.hxx"
@@ -64,14 +65,13 @@
 Console::Console(OSystem* osystem, Cartridge* cart, const Properties& props)
   : myOSystem(osystem),
     myProperties(props),
-    myAVox(0),
     myDisplayFormat("NTSC"),
     myFramerate(60.0),
     myUserPaletteDefined(false)
 {
   myControllers[0] = 0;
   myControllers[1] = 0;
-  myMediaSource = 0;
+  myTIA = 0;
   mySwitches = 0;
   mySystem = 0;
   myEvent = 0;
@@ -98,55 +98,54 @@ Console::Console(OSystem* osystem, Cartridge* cart, const Properties& props)
   myControllers[0] = new Joystick(Controller::Left, *myEvent, *mySystem);
   myControllers[1] = new Joystick(Controller::Right, *myEvent, *mySystem);
 
-  M6502* m6502;
-  if(myOSystem->settings().getString("cpu") == "low")
-    m6502 = new M6502Low(1);
-  else
-    m6502 = new M6502High(1);
+  M6502* m6502 = new M6502(1);
 #ifdef DEBUGGER_SUPPORT
   m6502->attach(myOSystem->debugger());
 #endif
 
-  M6532* m6532 = new M6532(*this);
-  TIA *tia = new TIA(*this, myOSystem->settings());
-  tia->setSound(myOSystem->sound());
+  myCart = cart;
+  myRiot = new M6532(*this);
+  myTIA  = new TIA(*this, myOSystem->sound(), myOSystem->settings());
 
   mySystem->attach(m6502);
-  mySystem->attach(m6532);
-  mySystem->attach(tia);
-  mySystem->attach(cart);
+  mySystem->attach(myRiot);
+  mySystem->attach(myTIA);
+  mySystem->attach(myCart);
 
-  // Remember what my media source is
-  myMediaSource = tia;
-  myCart = cart;
-  myRiot = m6532;
-
-  // Query some info about this console
-  ostringstream about, vidinfo;
+#ifdef DEBUGGER_SUPPORT
+  // The debugger must be added before we run the console for the first time
+  myOSystem->debugger().setConsole(this);
+  myOSystem->debugger().initialize();
+#endif
 
   // Auto-detect NTSC/PAL mode if it's requested
+  string autodetected = "";
   myDisplayFormat = myProperties.get(Display_Format);
-  vidinfo << "  Display Format:  " << myDisplayFormat;
   if(myDisplayFormat == "AUTO-DETECT" ||
      myOSystem->settings().getBool("rominfo"))
   {
     // Run the system for 60 frames, looking for PAL scanline patterns
-    // We assume the first 30 frames are garbage, and only consider
-    // the second 30 (useful to get past SuperCharger BIOS)
-    // Unfortunately, this means we have to always enable 'fastscbios',
-    // since otherwise the BIOS loading will take over 250 frames!
+    // We turn off the SuperCharger progress bars, otherwise the SC BIOS
+    // will take over 250 frames!
+    // The 'fastscbios' option must be changed before the system is reset
+    bool fastscbios = myOSystem->settings().getBool("fastscbios");
+    myOSystem->settings().setBool("fastscbios", true);
     mySystem->reset();
     int palCount = 0;
     for(int i = 0; i < 60; ++i)
     {
-      myMediaSource->update();
-      if(i >= 30 && myMediaSource->scanlines() > 285)
+      myTIA->update();
+      if(myTIA->scanlines() > 285)
         ++palCount;
     }
-    myDisplayFormat = (palCount >= 15) ? "PAL" : "NTSC";
+    myDisplayFormat = (palCount >= 20) ? "PAL" : "NTSC";
     if(myProperties.get(Display_Format) == "AUTO-DETECT")
-      vidinfo << " ==> " << myDisplayFormat;
+      autodetected = "*";
+
+    // Don't forget to reset the SC progress bars again
+    myOSystem->settings().setBool("fastscbios", fastscbios);
   }
+  myConsoleInfo.DisplayFormat = myDisplayFormat + autodetected;
 
   // Set up the correct properties used when toggling format
   // Note that this can be overridden if a format is forced
@@ -157,39 +156,40 @@ Console::Console(OSystem* osystem, Cartridge* cart, const Properties& props)
      myDisplayFormat == "SECAM60")
   {
     // Assume we've got ~262 scanlines (NTSC-like format)
-      myFramerate = 60.0;
+    myFramerate = 60.0;
+    myConsoleInfo.InitialFrameRate = "60";
   }
   else
   {
     // Assume we've got ~312 scanlines (PAL-like format)
     myFramerate = 50.0;
+    myConsoleInfo.InitialFrameRate = "50";
 
     if(myProperties.get(Display_Height) == "210")
       myProperties.set(Display_Height, "250");
   }
 
-  // Add the real controllers for this system
-  setControllers();
-
-  // Bumper Bash requires all 4 directions
   const string& md5 = myProperties.get(Cartridge_MD5);
-  bool allow = (md5 == "aa1c41f86ec44c0a44eb64c332ce08af" ||
-                md5 == "1bf503c724001b09be79c515ecfcbd03");
-  myOSystem->eventHandler().allowAllDirections(allow);
+
+  // Add the real controllers for this system
+  setControllers(md5);
+
+  // Bumper Bash always requires all 4 directions
+  // Other ROMs can use it if the setting is enabled
+  bool joyallow4 = md5 == "aa1c41f86ec44c0a44eb64c332ce08af" ||
+                   md5 == "1bf503c724001b09be79c515ecfcbd03" ||
+                   myOSystem->settings().getBool("joyallow4");
+  myOSystem->eventHandler().allowAllDirections(joyallow4);
 
   // Reset the system to its power-on state
-  // TODO - a reset still isn't completely working with Boulderdash
   mySystem->reset();
 
-  // Finally, show some info about the console
-  about << "  Cart Name:       " << myProperties.get(Cartridge_Name) << endl
-        << "  Cart MD5:        " << myProperties.get(Cartridge_MD5) << endl
-        << "  Controller 0:    " << myControllers[0]->about() << endl
-        << "  Controller 1:    " << myControllers[1]->about() << endl
-        << vidinfo.str() << endl
-        << cart->about();
-
-  myAboutString = about.str();
+  // Finally, add remaining info about the console
+  myConsoleInfo.CartName   = myProperties.get(Cartridge_Name);
+  myConsoleInfo.CartMD5    = myProperties.get(Cartridge_MD5);
+  myConsoleInfo.Control0   = myControllers[0]->about();
+  myConsoleInfo.Control1   = myControllers[1]->about();
+  myConsoleInfo.BankSwitch = cart->about();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -214,14 +214,9 @@ bool Console::save(Serializer& out) const
     if(!mySwitches->save(out))
       return false;
   }
-  catch(char *msg)
+  catch(const char* msg)
   {
-    cerr << msg << endl;
-    return false;
-  }
-  catch(...)
-  {
-    cerr << "Unknown error in save state for \'Console\'" << endl;
+    cerr << "ERROR: Console::save" << endl << "  " << msg << endl;
     return false;
   }
 
@@ -229,7 +224,7 @@ bool Console::save(Serializer& out) const
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool Console::load(Deserializer& in)
+bool Console::load(Serializer& in)
 {
   try
   {
@@ -241,14 +236,9 @@ bool Console::load(Deserializer& in)
     if(!mySwitches->load(in))
       return false;
   }
-  catch(char *msg)
+  catch(const char* msg)
   {
-    cerr << msg << endl;
-    return false;
-  }
-  catch(...)
-  {
-    cerr << "Unknown error in load state for \'Console\'" << endl;
+    cerr << "ERROR: Console::load" << endl << "  " << msg << endl;
     return false;
   }
 
@@ -311,8 +301,7 @@ void Console::toggleColorLoss()
 {
   bool colorloss = !myOSystem->settings().getBool("colorloss");
   myOSystem->settings().setBool("colorloss", colorloss);
-  setColorLossPalette(colorloss);
-  setPalette(myOSystem->settings().getString("palette"));
+  myTIA->enableColorLoss(colorloss);
 
   string message = string("PAL color-loss ") +
                    (colorloss ? "enabled" : "disabled");
@@ -426,24 +415,25 @@ void Console::setProperties(const Properties& props)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Console::initializeVideo(bool full)
+bool Console::initializeVideo(bool full)
 {
   if(full)
   {
     string title = string("Stella ") + STELLA_VERSION +
                    ": \"" + myProperties.get(Cartridge_Name) + "\"";
-    myOSystem->frameBuffer().initialize(title,
-                                        myMediaSource->width() << 1,
-                                        myMediaSource->height());
+    if(!myOSystem->frameBuffer().initialize(title,
+          myTIA->width() << 1, myTIA->height()))
+      return false;
 
     myOSystem->frameBuffer().showFrameStats(
       myOSystem->settings().getBool("stats"));
+
+    setColorLossPalette();
   }
 
   bool enable = myProperties.get(Display_Phosphor) == "YES";
   int blend = atoi(myProperties.get(Display_PPBlend).c_str());
   myOSystem->frameBuffer().enablePhosphor(enable, blend);
-  setColorLossPalette(myOSystem->settings().getBool("colorloss"));
   setPalette(myOSystem->settings().getString("palette"));
 
   // Set the correct framerate based on the format of the ROM
@@ -451,11 +441,13 @@ void Console::initializeVideo(bool full)
   // VideoDialog box or on the commandline, but it can't be saved
   // (ie, framerate is now determined based on number of scanlines).
   int framerate = myOSystem->settings().getInt("framerate");
-  if(framerate > 0) myFramerate = framerate;
+  if(framerate > 0) myFramerate = float(framerate);
   myOSystem->setFramerate(myFramerate);
 
   // Make sure auto-frame calculation is only enabled when necessary
-  myMediaSource->enableAutoFrame(framerate <= 0);
+  myTIA->enableAutoFrame(framerate <= 0);
+
+  return true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -465,17 +457,17 @@ void Console::initializeAudio()
   // The # of channels can be overridden in the AudioDialog box or on
   // the commandline, but it can't be saved.
   int framerate = myOSystem->settings().getInt("framerate");
-  if(framerate > 0) myFramerate = framerate;
+  if(framerate > 0) myFramerate = float(framerate);
   const string& sound = myProperties.get(Cartridge_Sound);
   uInt32 channels = (sound == "STEREO" ? 2 : 1);
 
   myOSystem->sound().close();
   myOSystem->sound().setChannels(channels);
   myOSystem->sound().setFrameRate(myFramerate);
-  myOSystem->sound().initialize();
+  myOSystem->sound().open();
 
   // Make sure auto-frame calculation is only enabled when necessary
-  myMediaSource->enableAutoFrame(framerate <= 0);
+  myTIA->enableAutoFrame(framerate <= 0);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -534,7 +526,7 @@ void Console::changeYStart(int direction)
 
   strval << ystart;
   myProperties.set(Display_YStart, strval.str());
-  ((TIA*)myMediaSource)->frameReset();
+  myTIA->frameReset();
   myOSystem->frameBuffer().refresh();
 
   message = "YStart ";
@@ -572,7 +564,7 @@ void Console::changeHeight(int direction)
 
   strval << height;
   myProperties.set(Display_Height, strval.str());
-  ((TIA*)myMediaSource)->frameReset();
+  myTIA->frameReset();
   initializeVideo();  // takes care of refreshing the screen
 
   message = "Height ";
@@ -581,7 +573,7 @@ void Console::changeHeight(int direction)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Console::setControllers()
+void Console::setControllers(const string& rommd5)
 {
   delete myControllers[0];
   delete myControllers[1];
@@ -637,6 +629,10 @@ void Console::setControllers()
     myControllers[leftPort] = new TrackBall(Controller::Left, *myEvent, *mySystem,
                                             Controller::AmigaMouse);
   }
+  else if(left == "GENESIS")
+  {
+    myControllers[leftPort] = new Genesis(Controller::Left, *myEvent, *mySystem);
+  }
   else
   {
     myControllers[leftPort] = new Joystick(Controller::Left, *myEvent, *mySystem);
@@ -676,41 +672,33 @@ void Console::setControllers()
   }
   else if(right == "ATARIVOX")
   {
-    string eepromfile = myOSystem->baseDir() + BSPF_PATH_SEPARATOR +
-                        "atarivox_eeprom.dat";
-    myControllers[rightPort] = myAVox =
-      new AtariVox(Controller::Right, *myEvent, *mySystem, myOSystem->serialPort(),
+    const string& eepromfile = myOSystem->eepromDir() + BSPF_PATH_SEPARATOR +
+                               "atarivox_eeprom.dat";
+    myControllers[rightPort] = new AtariVox(Controller::Right, *myEvent,
+                   *mySystem, myOSystem->serialPort(),
                    myOSystem->settings().getString("avoxport"), eepromfile);
   }
   else if(right == "SAVEKEY")
   {
-    string eepromfile = myOSystem->baseDir() + BSPF_PATH_SEPARATOR +
+    const string& eepromfile = myOSystem->eepromDir() + BSPF_PATH_SEPARATOR +
                         "savekey_eeprom.dat";
     myControllers[rightPort] = new SaveKey(Controller::Right, *myEvent, *mySystem,
                                            eepromfile);
   }
+  else if(right == "KIDVID")
+  {
+    myControllers[rightPort] = new KidVid(Controller::Right, *myEvent, *mySystem, rommd5);
+  }
+  else if(right == "GENESIS")
+  {
+    myControllers[rightPort] = new Genesis(Controller::Right, *myEvent, *mySystem);
+  }
   else
   {
     myControllers[rightPort] = new Joystick(Controller::Right, *myEvent, *mySystem);
   }
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Console::toggleTIABit(TIA::TIABit bit, const string& bitname, bool show) const
-{
-  bool result = ((TIA*)myMediaSource)->toggleBit(bit);
-  string message = bitname + (result ? " enabled" : " disabled");
-  myOSystem->frameBuffer().showMessage(message);
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Console::enableBits(bool enable) const
-{
-  ((TIA*)myMediaSource)->enableBits(enable);
-  string message = string("TIA bits") + (enable ? " enabled" : " disabled");
-  myOSystem->frameBuffer().showMessage(message);
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Console::loadUserPalette()
 {
@@ -769,7 +757,7 @@ void Console::loadUserPalette()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Console::setColorLossPalette(bool loss)
+void Console::setColorLossPalette()
 {
   // Look at all the palettes, since we don't know which one is
   // currently active
@@ -790,23 +778,18 @@ void Console::setColorLossPalette(bool loss)
     if(palette[i] == 0)
       continue;
 
-    // If color-loss is enabled, fill the odd numbered palette entries
-    // with gray values (calculated using the standard RGB -> grayscale
-    // conversion formula)
+    // Fill the odd numbered palette entries with gray values (calculated
+    // using the standard RGB -> grayscale conversion formula)
     for(int j = 0; j < 128; ++j)
     {
       uInt32 pixel = palette[i][(j<<1)];
-      if(loss)
-      {
-        uInt8 r = (pixel >> 16) & 0xff;
-        uInt8 g = (pixel >> 8)  & 0xff;
-        uInt8 b = (pixel >> 0)  & 0xff;
-        uInt8 sum = (uInt8) (((float)r * 0.2989) +
-                             ((float)g * 0.5870) +
-                             ((float)b * 0.1140));
-        pixel = (sum << 16) + (sum << 8) + sum;
-      }
-      palette[i][(j<<1)+1] = pixel;
+      uInt8 r = (pixel >> 16) & 0xff;
+      uInt8 g = (pixel >> 8)  & 0xff;
+      uInt8 b = (pixel >> 0)  & 0xff;
+      uInt8 sum = (uInt8) (((float)r * 0.2989) +
+                           ((float)g * 0.5870) +
+                           ((float)b * 0.1140));
+      palette[i][(j<<1)+1] = (sum << 16) + (sum << 8) + sum;
     }
   }
 }
@@ -819,6 +802,56 @@ void Console::setFramerate(float framerate)
   myOSystem->sound().setFrameRate(framerate);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Console::toggleTIABit(TIABit bit, const string& bitname, bool show) const
+{
+  bool result = myTIA->toggleBit(bit);
+  string message = bitname + (result ? " enabled" : " disabled");
+  myOSystem->frameBuffer().showMessage(message);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Console::toggleTIACollision(TIABit bit, const string& bitname, bool show) const
+{
+  bool result = myTIA->toggleCollision(bit);
+  string message = bitname + (result ? " collision enabled" : " collision disabled");
+  myOSystem->frameBuffer().showMessage(message);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Console::toggleHMOVE() const
+{
+  if(myTIA->toggleHMOVEBlank())
+    myOSystem->frameBuffer().showMessage("HMOVE blanking enabled");
+  else
+    myOSystem->frameBuffer().showMessage("HMOVE blanking disabled");
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Console::enableBits(bool enable) const
+{
+  myTIA->enableBits(enable);
+  string message = string("TIA bits") + (enable ? " enabled" : " disabled");
+  myOSystem->frameBuffer().showMessage(message);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Console::enableCollisions(bool enable) const
+{
+  myTIA->enableCollisions(enable);
+  string message = string("TIA collisions") + (enable ? " enabled" : " disabled");
+  myOSystem->frameBuffer().showMessage(message);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Console::toggleFixedColors() const
+{
+  if(myTIA->toggleFixedColors())
+    myOSystem->frameBuffer().showMessage("Fixed debug colors enabled");
+  else
+    myOSystem->frameBuffer().showMessage("Fixed debug colors disabled");
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt32 Console::ourNTSCPalette[256] = {
   0x000000, 0, 0x4a4a4a, 0, 0x6f6f6f, 0, 0x8e8e8e, 0,
@@ -1048,14 +1081,12 @@ uInt32 Console::ourUserSECAMPalette[256] = { 0 }; // filled from external file
 Console::Console(const Console& console)
   : myOSystem(console.myOSystem)
 {
-  // TODO: Write this method
   assert(false);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Console& Console::operator = (const Console&)
 {
-  // TODO: Write this method
   assert(false);
 
   return *this;
