@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Control.cxx,v 1.12 2008/05/12 22:40:23 stephena Exp $
+// $Id: Control.cxx 2001 2010-04-10 21:37:23Z stephena $
 //============================================================================
 
 #include <cassert>
@@ -70,6 +71,12 @@ Controller::Controller(Jack jack, const Event& event, const System& system,
     case SaveKey:
       myName = "SaveKey";
       break;
+    case KidVid:
+      myName = "KidVid";
+      break;
+    case Genesis:
+      myName = "Genesis";
+      break;
   }
 }
  
@@ -133,14 +140,14 @@ bool Controller::save(Serializer& out) const
   }
   catch(...)
   {
-    cerr << "Error: Controller::save() exception\n";
+    cerr << "ERROR: Controller::save() exception\n";
     return false;
   }
   return true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool Controller::load(Deserializer& in)
+bool Controller::load(Serializer& in)
 {
   try
   {
@@ -157,7 +164,7 @@ bool Controller::load(Deserializer& in)
   }
   catch(...)
   {
-    cerr << "Error: Controller::load() exception\n";
+    cerr << "ERROR: Controller::load() exception\n";
     return false;
   }
   return true;
