@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: EventHandler.cxx,v 1.224 2008/05/20 13:42:50 stephena Exp $
+// $Id: EventHandler.cxx 2029 2010-04-29 12:46:07Z stephena $
 //============================================================================
 
 #include <sstream>
@@ -33,6 +34,7 @@
 #include "Joystick.hxx"
 #include "Paddles.hxx"
 #include "PropsSet.hxx"
+#include "ListWidget.hxx"
 #include "ScrollBarWidget.hxx"
 #include "Settings.hxx"
 #include "Snapshot.hxx"
@@ -113,6 +115,14 @@ EventHandler::EventHandler(OSystem* osystem)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 EventHandler::~EventHandler()
 {
+  // Free strings created with strdup
+  for(uInt32 i = 0; i < kEmulActionListSize; ++i)
+    if(ourEmulActionList[i].key)
+      free(ourEmulActionList[i].key);
+  for(uInt32 i = 0; i < kMenuActionListSize; ++i)
+    if(ourMenuActionList[i].key)
+      free(ourMenuActionList[i].key);
+
   delete myEvent;
 
 #ifdef JOYSTICK_SUPPORT
@@ -141,9 +151,14 @@ void EventHandler::initialize()
   setActionMappings(kMenuMode);
 
   myGrabMouseFlag = myOSystem->settings().getBool("grabmouse");
+  myUseCtrlKeyFlag = myOSystem->settings().getBool("ctrlcombo");
 
   Joystick::setDeadZone(myOSystem->settings().getInt("joydeadzone"));
   Paddles::setDigitalSpeed(myOSystem->settings().getInt("pspeed"));
+  setPaddleMode(myOSystem->settings().getBool("usemouse") ? 0 : -1, false);
+
+  // Set quick select delay when typing characters in listwidgets
+  ListWidget::setQuickSelectDelay(myOSystem->settings().getInt("listdelay"));
 
   // Set number of lines a mousewheel will scroll
   ScrollBarWidget::setWheelLines(myOSystem->settings().getInt("mwheel"));
@@ -155,43 +170,13 @@ void EventHandler::reset(State state)
   setEventState(state);
   myEvent->clear();
   myOSystem->state().reset();
-}
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void EventHandler::refreshDisplay(bool forceUpdate)
-{
-  switch(myState)
-  {
-    case S_EMULATE:
-    case S_PAUSE:
-      if(&myOSystem->frameBuffer())
-        myOSystem->frameBuffer().refresh();
-      break;
-
-    case S_MENU:     // fall through to next case
-    case S_CMDMENU:
-      if(&myOSystem->frameBuffer())
-        myOSystem->frameBuffer().refresh();
-    case S_LAUNCHER:
-    case S_DEBUGGER:
-      if(myOverlay)
-        myOverlay->refresh();
-      break;
-
-    default:
-      return;
-      break;
-  }
-
-  if(forceUpdate && &myOSystem->frameBuffer())
-    myOSystem->frameBuffer().update();
+  setContinuousSnapshots(0);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void EventHandler::setupJoysticks()
 {
-  bool showinfo = myOSystem->settings().getBool("showinfo");
-
 #ifdef JOYSTICK_SUPPORT
   // Keep track of how many Stelladaptors we've found
   int saCount = 0;
@@ -211,12 +196,10 @@ void EventHandler::setupJoysticks()
   }
 
   // (Re)-Initialize the joystick subsystem
-  if(showinfo)
-    cout << "Joystick devices found:" << endl;
+  myOSystem->logMessage("Joystick devices found:\n", 1);
   if((SDL_InitSubSystem(SDL_INIT_JOYSTICK) == -1) || (SDL_NumJoysticks() <= 0))
   {
-    if(showinfo)
-      cout << "No joysticks present." << endl;
+    myOSystem->logMessage("No joysticks present.\n\n", 1);
     return;
   }
 
@@ -237,6 +220,7 @@ void EventHandler::setupJoysticks()
     }
 
     // Figure out what type of joystick this is
+    ostringstream buf;
     if(name.find("Stelladaptor", 0) != string::npos)
     {
       saCount++;
@@ -247,33 +231,31 @@ void EventHandler::setupJoysticks()
       else if(saCount == 2)
         ourJoysticks[i].name = "Stelladaptor 2";
 
-      if(showinfo)
-        cout << "  " << i << ": " << ourJoysticks[i].name << endl;
+      buf << "  " << i << ": " << ourJoysticks[i].name << endl;
+      myOSystem->logMessage(buf.str(), 1);
     }
     else
     {
       ourJoysticks[i].type = JT_REGULAR;
       ourJoysticks[i].name = SDL_JoystickName(i);
 
-      if(showinfo)
-        cout << "  " << i << ": " << ourJoysticks[i].name << " with "
-             << SDL_JoystickNumAxes(ourJoysticks[i].stick) << " axes, "
-             << SDL_JoystickNumHats(ourJoysticks[i].stick) << " hats, "
-             << SDL_JoystickNumBalls(ourJoysticks[i].stick) << " balls, "
-             << SDL_JoystickNumButtons(ourJoysticks[i].stick) << " buttons"
-             << endl;
+      buf << "  " << i << ": " << ourJoysticks[i].name << " with "
+          << SDL_JoystickNumAxes(ourJoysticks[i].stick) << " axes, "
+          << SDL_JoystickNumHats(ourJoysticks[i].stick) << " hats, "
+          << SDL_JoystickNumBalls(ourJoysticks[i].stick) << " balls, "
+          << SDL_JoystickNumButtons(ourJoysticks[i].stick) << " buttons"
+          << endl;
+      myOSystem->logMessage(buf.str(), 1);
     }
   }
-  if(showinfo)
-    cout << endl;
+  myOSystem->logMessage("\n", 1);
 
   // Map the stelladaptors we've found according to the specified ports
   const string& sa1 = myOSystem->settings().getString("sa1");
   const string& sa2 = myOSystem->settings().getString("sa2");
   mapStelladaptors(sa1, sa2);
 #else
-  if(showinfo)
-    cout << "No joysticks present." << endl;
+  myOSystem->logMessage("No joysticks present.\n", 1);
 #endif
 }
 
@@ -281,8 +263,6 @@ void EventHandler::setupJoysticks()
 void EventHandler::mapStelladaptors(const string& sa1, const string& sa2)
 {
 #ifdef JOYSTICK_SUPPORT
-  bool showinfo = myOSystem->settings().getBool("showinfo");
-
   for(int i = 0; i < kNumJoysticks; i++)
   {
     if(ourJoysticks[i].name == "Stelladaptor 1")
@@ -290,14 +270,12 @@ void EventHandler::mapStelladaptors(const string& sa1, const string& sa2)
       if(sa1 == "left")
       {
         ourJoysticks[i].type = JT_STELLADAPTOR_LEFT;
-        if(showinfo)
-          cout << "  Stelladaptor 1 emulates left joystick port\n";
+        myOSystem->logMessage("  Stelladaptor 1 emulates left joystick port\n", 1);
       }
       else if(sa1 == "right")
       {
         ourJoysticks[i].type = JT_STELLADAPTOR_RIGHT;
-        if(showinfo)
-          cout << "  Stelladaptor 1 emulates right joystick port\n";
+        myOSystem->logMessage("  Stelladaptor 1 emulates right joystick port\n", 1);
       }
     }
     else if(ourJoysticks[i].name == "Stelladaptor 2")
@@ -305,19 +283,16 @@ void EventHandler::mapStelladaptors(const string& sa1, const string& sa2)
       if(sa2 == "left")
       {
         ourJoysticks[i].type = JT_STELLADAPTOR_LEFT;
-        if(showinfo)
-          cout << "  Stelladaptor 2 emulates left joystick port\n";
+        myOSystem->logMessage("  Stelladaptor 2 emulates left joystick port\n", 1);
       }
       else if(sa2 == "right")
       {
         ourJoysticks[i].type = JT_STELLADAPTOR_RIGHT;
-        if(showinfo)
-          cout << "  Stelladaptor 2 emulates right joystick port\n";
+        myOSystem->logMessage("  Stelladaptor 2 emulates right joystick port\n", 1);
       }
     }
   }
-  if(showinfo)
-    cout << endl;
+  myOSystem->logMessage("\n", 1);
 
   myOSystem->settings().setString("sa1", sa1);
   myOSystem->settings().setString("sa2", sa2);
@@ -325,7 +300,7 @@ void EventHandler::mapStelladaptors(const string& sa1, const string& sa2)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void EventHandler::poll(uInt32 time)
+void EventHandler::poll(uInt64 time)
 {
   // Synthesize events for platform-specific hardware
   myOSystem->pollEvent();
@@ -344,38 +319,38 @@ void EventHandler::poll(uInt32 time)
         SDLKey key  = event.key.keysym.sym;
         SDLMod mod  = event.key.keysym.mod;
         uInt8 state = event.key.type == SDL_KEYDOWN ? 1 : 0;
+        bool handled = true;
 
         // An attempt to speed up event processing
         // All SDL-specific event actions are accessed by either
-        // Control/Cmd or Alt/Shift-Cmd keys.  So we quickly check for those.
+        // Control or Alt/Cmd keys.  So we quickly check for those.
         if(kbdAlt(mod) && state)
         {
+      #ifdef MAC_OSX
           // These keys work in all states
-          switch(int(key))
+          if(key == SDLK_q)
           {
-    #ifndef MAC_OSX
-            case SDLK_EQUALS:
-              myOSystem->frameBuffer().changeVidMode(+1);
-              break;
-
-            case SDLK_MINUS:
-              myOSystem->frameBuffer().changeVidMode(-1);
-              break;
-
-            case SDLK_RETURN:
-              myOSystem->frameBuffer().toggleFullscreen();
-              break;
-    #endif
-            case SDLK_g:
-              myOSystem->toggleFrameBuffer();
-              break;
+            handleEvent(Event::Quit, 1);
+          }
+          else
+      #endif
+          if(key == SDLK_RETURN)
+          {
+            myOSystem->frameBuffer().toggleFullscreen();
           }
-
           // These only work when in emulation mode
-          if(myState == S_EMULATE)
+          else if(myState == S_EMULATE)
           {
             switch(int(key))
             {
+              case SDLK_EQUALS:
+                myOSystem->frameBuffer().changeVidMode(+1);
+                break;
+
+              case SDLK_MINUS:
+                myOSystem->frameBuffer().changeVidMode(-1);
+                break;
+
               case SDLK_LEFTBRACKET:
                 myOSystem->sound().adjustVolume(-1);
                 break;
@@ -393,35 +368,67 @@ void EventHandler::poll(uInt32 time)
                 break;
 
               case SDLK_z:
-                myOSystem->console().toggleP0Bit();
+                if(mod & KMOD_SHIFT)
+                  myOSystem->console().toggleP0Collision();
+                else
+                  myOSystem->console().toggleP0Bit();
                 break;
 
               case SDLK_x:
-                myOSystem->console().toggleP1Bit();
+                if(mod & KMOD_SHIFT)
+                  myOSystem->console().toggleP1Collision();
+                else
+                  myOSystem->console().toggleP1Bit();
                 break;
 
               case SDLK_c:
-                myOSystem->console().toggleM0Bit();
+                if(mod & KMOD_SHIFT)
+                  myOSystem->console().toggleM0Collision();
+                else
+                  myOSystem->console().toggleM0Bit();
                 break;
 
               case SDLK_v:
-                myOSystem->console().toggleM1Bit();
+                if(mod & KMOD_SHIFT)
+                  myOSystem->console().toggleM1Collision();
+                else
+                  myOSystem->console().toggleM1Bit();
                 break;
 
               case SDLK_b:
-                myOSystem->console().toggleBLBit();
+                if(mod & KMOD_SHIFT)
+                  myOSystem->console().toggleBLCollision();
+                else
+                  myOSystem->console().toggleBLBit();
                 break;
 
               case SDLK_n:
-                myOSystem->console().togglePFBit();
+                if(mod & KMOD_SHIFT)
+                  myOSystem->console().togglePFCollision();
+                else
+                  myOSystem->console().togglePFBit();
+                break;
+
+              case SDLK_m:
+                myOSystem->console().toggleHMOVE();
+                break;
+
+              case SDLK_COMMA:
+                myOSystem->console().toggleFixedColors();
                 break;
 
               case SDLK_PERIOD:
-                myOSystem->console().enableBits(false);
+                if(mod & KMOD_SHIFT)
+                  myOSystem->console().enableCollisions(false);
+                else
+                  myOSystem->console().enableBits(false);
                 break;
 
               case SDLK_SLASH:
-                myOSystem->console().enableBits(true);
+                if(mod & KMOD_SHIFT)
+                  myOSystem->console().enableCollisions(true);
+                else
+                  myOSystem->console().enableBits(true);
                 break;
 
               case SDLK_p:  // Alt-p toggles phosphor effect
@@ -431,6 +438,26 @@ void EventHandler::poll(uInt32 time)
               case SDLK_l:
                 myOSystem->frameBuffer().toggleFrameStats();
                 break;
+
+              case SDLK_s:  // TODO - make this remappable
+                if(myContSnapshotInterval == 0)
+                {
+                  ostringstream buf;
+                  uInt32 interval = myOSystem->settings().getInt("ssinterval");
+                  buf << "Enabling shotshots in " << interval << " second intervals";
+                  myOSystem->frameBuffer().showMessage(buf.str());
+                  setContinuousSnapshots(interval);
+                }
+                else
+                {
+                  ostringstream buf;
+                  buf << "Disabling snapshots, generated "
+                      << (myContSnapshotCounter / myContSnapshotInterval)
+                      << " files";
+                  myOSystem->frameBuffer().showMessage(buf.str());
+                  setContinuousSnapshots(0);
+                }
+                break;
 #if 0
 // FIXME - these will be removed when a UI is added for event recording
               case SDLK_e:  // Alt-e starts/stops event recording
@@ -457,53 +484,43 @@ void EventHandler::poll(uInt32 time)
 */
 ////////////////////////////////////////////////////////////////////////
 #endif
+              default:
+                handled = false;
+                break;
             }
           }
+          else
+            handled = false;
         }
-        else if(kbdControl(mod) && state)
+        else if(kbdControl(mod) && state && myUseCtrlKeyFlag)
         {
           // These keys work in all states
-          switch(int(key))
+          if(key == SDLK_q)
           {
-            case SDLK_q:
-              handleEvent(Event::Quit, 1);
-              break;
-
-  #ifdef MAC_OSX
-            case SDLK_h:
-            case SDLK_m:
-            case SDLK_SLASH:
-              handleMacOSXKeypress(int(key));
-              break;
-
-            case SDLK_EQUALS:
-              myOSystem->frameBuffer().changeVidMode(+1);
-              break;
-
-            case SDLK_MINUS:
-              myOSystem->frameBuffer().changeVidMode(-1);
-              break;
-
-            case SDLK_RETURN:
-              myOSystem->frameBuffer().toggleFullscreen();
-              break;
-  #endif
-            case SDLK_g:
-              // don't change grabmouse in fullscreen mode
-              if(!myOSystem->frameBuffer().fullScreen())
-              {
-                myGrabMouseFlag = !myGrabMouseFlag;
-                myOSystem->settings().setBool("grabmouse", myGrabMouseFlag);
-                myOSystem->frameBuffer().grabMouse(myGrabMouseFlag);
-              }
-              break;
+            handleEvent(Event::Quit, 1);
+          }
+          else if(key == SDLK_g)
+          {
+            // don't change grabmouse in fullscreen mode
+            if(!myOSystem->frameBuffer().fullScreen())
+            {
+              myGrabMouseFlag = !myGrabMouseFlag;
+              myOSystem->settings().setBool("grabmouse", myGrabMouseFlag);
+              myOSystem->frameBuffer().grabMouse(myGrabMouseFlag);
+            }
           }
-
           // These only work when in emulation mode
-          if(myState == S_EMULATE)
+          else if(myState == S_EMULATE)
           {
             switch(int(key))
             {
+          #ifdef MAC_OSX
+              case SDLK_h:
+              case SDLK_m:
+              case SDLK_SLASH:
+                handleMacOSXKeypress(int(key));
+                break;
+          #endif
               case SDLK_0:  // Ctrl-0 sets the mouse to paddle 0
                 setPaddleMode(0, true);
                 break;
@@ -560,9 +577,20 @@ void EventHandler::poll(uInt32 time)
                   myOSystem->frameBuffer().showMessage("Error saving properties");
                 break;
               }
+
+              default:
+                handled = false;
+                break;
             }
           }
+          else
+            handled = false;
         }
+        else
+          handled = false;
+
+        // Don't pass the key on if we've already taken care of it
+        if(handled) break;
 
         // Handle keys which switch eventhandler state
         // Arrange the logic to take advantage of short-circuit evaluation
@@ -609,7 +637,7 @@ void EventHandler::poll(uInt32 time)
         break;  // SDL_QUIT
 
       case SDL_VIDEOEXPOSE:
-        refreshDisplay();
+        myOSystem->frameBuffer().refresh();
         break;  // SDL_VIDEOEXPOSE
 
 #ifdef JOYSTICK_SUPPORT
@@ -741,6 +769,11 @@ void EventHandler::poll(uInt32 time)
       for(unsigned int i = 0; i < cheats.size(); i++)
         cheats[i]->evaluate();
     #endif
+
+      // Handle continuous snapshots
+      if(myContSnapshotInterval > 0 &&
+        (++myContSnapshotCounter % myContSnapshotInterval == 0))
+        takeSnapshot(myContSnapshotCounter / myContSnapshotInterval);
     }
   }
   else if(myOverlay)
@@ -762,14 +795,16 @@ void EventHandler::handleMouseMotionEvent(SDL_Event& event)
   // Determine which mode we're in, then send the event to the appropriate place
   if(myState == S_EMULATE)
   {
-    int x = event.motion.xrel, y = event.motion.yrel;
-    myEvent->set(Event::MouseAxisXValue, x);
-    myEvent->set(Event::MouseAxisYValue, y);
+    if(myMouseEnabled)
+    {
+      int x = event.motion.xrel, y = event.motion.yrel;
+      myEvent->set(Event::MouseAxisXValue, x);
+      myEvent->set(Event::MouseAxisYValue, y);
+    }
   }
   else if(myOverlay)
   {
     int x = event.motion.x, y = event.motion.y;
-    myOSystem->frameBuffer().translateCoords(x, y);
     myOverlay->handleMouseMotionEvent(x, y, 0);
   }
 }
@@ -779,12 +814,14 @@ void EventHandler::handleMouseButtonEvent(SDL_Event& event, int state)
 {
   // Determine which mode we're in, then send the event to the appropriate place
   if(myState == S_EMULATE)
-    myEvent->set(Event::MouseButtonValue, state);
+  {
+    if(myMouseEnabled)
+      myEvent->set(Event::MouseButtonValue, state);
+  }
   else if(myOverlay)
   {
     // Take window zooming into account
     Int32 x = event.button.x, y = event.button.y;
-    myOSystem->frameBuffer().translateCoords(x, y);
     MouseButton button;
 
     switch(event.button.button)
@@ -892,16 +929,6 @@ void EventHandler::handleJoyHatEvent(int stick, int hat, int value)
 #endif
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void EventHandler::handleResizeEvent()
-{
-  // For now, only the overlay cares about resize events
-  // We don't know which one wants it, so we send it to all of them
-  // These events need to be sent even if the overlay isn't active
-  if(&myOSystem->launcher())
-    myOSystem->launcher().handleResizeEvent();
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void EventHandler::handleEvent(Event::Type event, int state)
 {
@@ -984,8 +1011,15 @@ void EventHandler::handleEvent(Event::Type event, int state)
       if((myState == S_EMULATE || myState == S_CMDMENU) && state)
       {
         myOSystem->settings().saveConfig();
-        myOSystem->deleteConsole();
-        myOSystem->createLauncher();
+
+        // Go back to the launcher, or immediately quit
+        if(myOSystem->settings().getBool("uselauncher"))
+        {
+          myOSystem->deleteConsole();
+          myOSystem->createLauncher();
+        }
+        else
+          myOSystem->quit();
       }
       return;
 
@@ -1092,8 +1126,7 @@ void EventHandler::setActionMappings(EventMode mode)
   for(int i = 0; i < listsize; ++i)
   {
     Event::Type event = list[i].event;
-    if(list[i].key)
-      free(list[i].key);
+    free(list[i].key);  list[i].key = NULL;
     list[i].key = strdup("None");
     string key = "";
     for(int j = 0; j < SDLK_LAST; ++j)   // key mapping
@@ -1199,8 +1232,7 @@ void EventHandler::setActionMappings(EventMode mode)
 
     if(key != "")
     {
-      if(list[i].key)
-        free(list[i].key);
+      free(list[i].key);  list[i].key = NULL;
       list[i].key = strdup(key.c_str());
     }
   }
@@ -1551,6 +1583,8 @@ void EventHandler::setDefaultKeymap(EventMode mode)
 
       myKeyTable[ SDLK_RETURN ][mode]    = Event::UISelect;
       myKeyTable[ SDLK_ESCAPE ][mode]    = Event::UICancel;
+
+      myKeyTable[ SDLK_BACKSPACE ][mode] = Event::UIPrevDir;
       break;
 
     default:
@@ -1661,7 +1695,7 @@ void EventHandler::saveJoyHatMapping()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool EventHandler::isValidList(string& list, IntArray& map, uInt32 length)
+bool EventHandler::isValidList(string& list, IntArray& map, uInt32 length) const
 {
   string key;
   Event::Type event;
@@ -1682,7 +1716,7 @@ bool EventHandler::isValidList(string& list, IntArray& map, uInt32 length)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-inline bool EventHandler::eventIsAnalog(Event::Type event)
+inline bool EventHandler::eventIsAnalog(Event::Type event) const
 {
   switch((int)event)
   {
@@ -1697,7 +1731,7 @@ inline bool EventHandler::eventIsAnalog(Event::Type event)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-StringList EventHandler::getActionList(EventMode mode)
+StringList EventHandler::getActionList(EventMode mode) const
 {
   StringList l;
 
@@ -1719,7 +1753,7 @@ StringList EventHandler::getActionList(EventMode mode)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-Event::Type EventHandler::eventAtIndex(int idx, EventMode mode)
+Event::Type EventHandler::eventAtIndex(int idx, EventMode mode) const
 {
   switch(mode)
   {
@@ -1742,7 +1776,7 @@ Event::Type EventHandler::eventAtIndex(int idx, EventMode mode)
 }  
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-string EventHandler::actionAtIndex(int idx, EventMode mode)
+string EventHandler::actionAtIndex(int idx, EventMode mode) const
 {
   switch(mode)
   {
@@ -1765,7 +1799,7 @@ string EventHandler::actionAtIndex(int idx, EventMode mode)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-string EventHandler::keyAtIndex(int idx, EventMode mode)
+string EventHandler::keyAtIndex(int idx, EventMode mode) const
 {
   switch(mode)
   {
@@ -1788,11 +1822,12 @@ string EventHandler::keyAtIndex(int idx, EventMode mode)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void EventHandler::takeSnapshot()
+void EventHandler::takeSnapshot(uInt32 number)
 {
   // Figure out the correct snapshot name
   string filename;
-  string sspath = myOSystem->settings().getString("ssdir");
+  string sspath = myOSystem->snapshotDir();
+  bool showmessage = number == 0;
 
   if(sspath.length() > 0)
     if(sspath.substr(sspath.length()-1) != BSPF_PATH_SEPARATOR)
@@ -1800,19 +1835,27 @@ void EventHandler::takeSnapshot()
   sspath += myOSystem->console().properties().get(Cartridge_Name);
 
   // Check whether we want multiple snapshots created
-  if(!myOSystem->settings().getBool("sssingle"))
+  if(number > 0)
+  {
+    ostringstream buf;
+    buf << sspath << "_" << number << ".png";
+    filename = buf.str();
+  }
+  else if(!myOSystem->settings().getBool("sssingle"))
   {
     // Determine if the file already exists, checking each successive filename
     // until one doesn't exist
     filename = sspath + ".png";
-    if(FilesystemNode::fileExists(filename))
+    FilesystemNode node(filename);
+    if(node.exists())
     {
       ostringstream buf;
       for(uInt32 i = 1; ;++i)
       {
         buf.str("");
         buf << sspath << "_" << i << ".png";
-        if(!FilesystemNode::fileExists(buf.str()))
+        FilesystemNode next(buf.str());
+        if(!next.exists())
           break;
       }
       filename = buf.str();
@@ -1822,8 +1865,27 @@ void EventHandler::takeSnapshot()
     filename = sspath + ".png";
 
   // Now create a PNG snapshot
-  Snapshot::savePNG(myOSystem->frameBuffer(),
-                    myOSystem->console().properties(), filename);
+  if(myOSystem->settings().getBool("ss1x"))
+  {
+    string msg = Snapshot::savePNG(myOSystem->frameBuffer(),
+                   myOSystem->console().tia(),
+                   myOSystem->console().properties(), filename);
+    if(showmessage)
+      myOSystem->frameBuffer().showMessage(msg);
+  }
+  else
+  {
+    // Make sure we have a 'clean' image, with no onscreen messages
+    myOSystem->frameBuffer().enableMessages(false);
+
+    string msg = Snapshot::savePNG(myOSystem->frameBuffer(),
+                   myOSystem->console().properties(), filename);
+
+    // Re-enable old messages
+    myOSystem->frameBuffer().enableMessages(true);
+    if(showmessage)
+      myOSystem->frameBuffer().showMessage(msg);
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -1831,6 +1893,7 @@ void EventHandler::setPaddleMode(int num, bool showmessage)
 {
   if(num >= 0 && num <= 3)
   {
+    myMouseEnabled = true;
     Paddles::setMouseIsPaddle(num);
     if(showmessage)
     {
@@ -1839,6 +1902,15 @@ void EventHandler::setPaddleMode(int num, bool showmessage)
       myOSystem->frameBuffer().showMessage(buf.str());
     }
   }
+  else
+    myMouseEnabled = false;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void EventHandler::setContinuousSnapshots(uInt32 interval)
+{
+  myContSnapshotInterval = (uInt32) myOSystem->frameRate() * interval;
+  myContSnapshotCounter = 0;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -1876,15 +1948,22 @@ bool EventHandler::enterDebugMode()
   // mode, since it takes care of locking the debugger state, which will
   // probably be modified below
   myOSystem->debugger().setStartState();
-
   setEventState(S_DEBUGGER);
-  myOSystem->createFrameBuffer();
+  if(!myOSystem->createFrameBuffer())
+  {
+    myOSystem->debugger().setQuitState();
+    setEventState(S_EMULATE);
+    myOSystem->frameBuffer().showMessage("Debugger window too large",
+                                         kBottomCenter, true);
+    return false;
+  }
   myOverlay->reStack();
   myOSystem->frameBuffer().setCursorState();
   myOSystem->sound().mute(true);
   myEvent->clear();
 #else
-  myOSystem->frameBuffer().showMessage("Debugger unsupported");
+  myOSystem->frameBuffer().showMessage("Debugger unsupported",
+                                       kBottomCenter, true);
 #endif
 
   return true;
@@ -1913,15 +1992,12 @@ void EventHandler::leaveDebugMode()
 void EventHandler::setEventState(State state)
 {
   myState = state;
+
   switch(myState)
   {
     case S_EMULATE:
       myOverlay = NULL;
       myOSystem->sound().mute(false);
-
-      // Controller types only make sense in Emulate mode
-      myController[0] = myOSystem->console().controller(Controller::Left).type();
-      myController[1] = myOSystem->console().controller(Controller::Right).type();
       break;
 
     case S_PAUSE:
@@ -1955,8 +2031,6 @@ void EventHandler::setEventState(State state)
   myOSystem->stateChanged(myState);
   if(&myOSystem->frameBuffer())
     myOSystem->frameBuffer().stateChanged(myState);
-
-  refreshDisplay();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -2299,7 +2373,9 @@ EventHandler::ActionList EventHandler::ourMenuActionList[kMenuActionListSize] =
   { Event::UISelect,    "Select item",          0 },
 
   { Event::UINavPrev,   "Previous object",      0 },
-  { Event::UINavNext,   "Next object",          0 }
+  { Event::UINavNext,   "Next object",          0 },
+
+  { Event::UIPrevDir,   "Parent directory",     0 }
 };
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
