@@ -8,38 +8,28 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: M6532.cxx,v 1.27 2008/05/22 17:54:54 stephena Exp $
+// $Id: M6532.cxx 2012 2010-04-14 22:35:46Z stephena $
 //============================================================================
 
-#include <assert.h>
+#include <cassert>
+#include <iostream>
+
 #include "Console.hxx"
-#include "M6532.hxx"
-#include "Random.hxx"
 #include "Switches.hxx"
 #include "System.hxx"
-#include "Serializer.hxx"
-#include "Deserializer.hxx"
-#include <iostream>
+
+#include "M6532.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 M6532::M6532(const Console& console)
   : myConsole(console)
 {
-  // Randomize the 128 bytes of memory
-  class Random random;
-
-  for(uInt32 t = 0; t < 128; ++t)
-  {
-    myRAM[t] = random.next();
-  }
-
-  // Initialize other data members
-  reset();
 }
  
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -50,11 +40,13 @@ M6532::~M6532()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void M6532::reset()
 {
-  class Random random;
+  // Randomize the 128 bytes of memory
+  for(uInt32 t = 0; t < 128; ++t)
+    myRAM[t] = mySystem->randGenerator().next();
 
   // The timer absolutely cannot be initialized to zero; some games will
   // loop or hang (notably Solaris and H.E.R.O.)
-  myTimer = (0xff - (random.next() % 0xfe)) << 10;
+  myTimer = (0xff - (mySystem->randGenerator().next() % 0xfe)) << 10;
   myIntervalShift = 10;
   myCyclesWhenTimerSet = 0;
   myInterruptEnabled = false;
@@ -99,18 +91,15 @@ void M6532::install(System& system, Device& device)
   
   // All accesses are to the given device
   System::PageAccess access;
+  access.directPeekBase = 0; 
+  access.directPokeBase = 0;
   access.device = &device;
+  access.type = System::PA_READWRITE;
 
   // We're installing in a 2600 system
   for(int address = 0; address < 8192; address += (1 << shift))
-  {
     if((address & 0x1080) == 0x0080)
-    {
-      access.directPeekBase = 0; 
-      access.directPokeBase = 0;
       mySystem->setPageAccess(address >> shift, access);
-    }
-  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -196,7 +185,7 @@ uInt8 M6532::peek(uInt16 addr)
     case 0x05:    // Interrupt Flag
     case 0x07:
     {
-      if((timerClocks() >= 0) || myInterruptEnabled && myInterruptTriggered)
+      if((timerClocks() >= 0) || (myInterruptEnabled && myInterruptTriggered))
         return 0x00;
       else
         return 0x80;
@@ -213,7 +202,7 @@ uInt8 M6532::peek(uInt16 addr)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void M6532::poke(uInt16 addr, uInt8 value)
+bool M6532::poke(uInt16 addr, uInt8 value)
 {
   // Access RAM directly.  Originally, accesses to RAM could bypass
   // this method and its pages could be installed directly into the
@@ -222,7 +211,7 @@ void M6532::poke(uInt16 addr, uInt8 value)
   if((addr & 0x1080) == 0x0080 && (addr & 0x0200) == 0x0000)
   {
     myRAM[addr & 0x007f] = value;
-    return;
+    return true;
   }
 
   // A2 distinguishes I/O registers from the timer
@@ -253,9 +242,10 @@ void M6532::poke(uInt16 addr, uInt8 value)
       }
 
       default:    // Port B I/O & DDR Registers (Console switches)
-        break;    // hardwired as read-only
+        return false;  // hardwired as read-only
     }
   }
+  return true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -301,7 +291,7 @@ void M6532::setPinState()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool M6532::save(Serializer& out) const
 {
-  string device = name();
+  const string& device = name();
 
   try
   {
@@ -326,14 +316,9 @@ bool M6532::save(Serializer& out) const
     out.putByte((char)myOutTimer[2]);
     out.putByte((char)myOutTimer[3]);
   }
-  catch(char *msg)
+  catch(const char* msg)
   {
-    cerr << msg << endl;
-    return false;
-  }
-  catch(...)
-  {
-    cerr << "Unknown error in save state for " << device << endl;
+    cerr << "ERROR: M6532::save" << endl << "  " << msg << endl;
     return false;
   }
 
@@ -341,9 +326,9 @@ bool M6532::save(Serializer& out) const
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool M6532::load(Deserializer& in)
+bool M6532::load(Serializer& in)
 {
-  string device = name();
+  const string& device = name();
 
   try
   {
@@ -369,14 +354,9 @@ bool M6532::load(Deserializer& in)
     myOutTimer[2] = (uInt8) in.getByte();
     myOutTimer[3] = (uInt8) in.getByte();
   }
-  catch(char *msg)
+  catch(const char* msg)
   {
-    cerr << msg << endl;
-    return false;
-  }
-  catch(...)
-  {
-    cerr << "Unknown error in load state for " << device << endl;
+    cerr << "ERROR: M6532::load" << endl << "  " << msg << endl;
     return false;
   }
 
