@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: MediaFactory.cxx,v 1.12 2008/02/06 13:45:22 stephena Exp $
+// $Id: MediaFactory.cxx 2001 2010-04-10 21:37:23Z stephena $
 //============================================================================
 
 ////////////////////////////////////////////////////////////////////
@@ -60,8 +61,6 @@ FrameBuffer* MediaFactory::createVideo(OSystem* osystem)
     const string& gl_lib = osystem->settings().getString("gl_lib");
     if(FrameBufferGL::loadLibrary(gl_lib))
       fb = new FrameBufferGL(osystem);
-    else
-      cerr << "ERROR: Couldn't dynamically load OpenGL library ...\n";
   }
 #endif
 
@@ -80,16 +79,6 @@ FrameBuffer* MediaFactory::createVideo(OSystem* osystem)
 
   // This should never happen
   assert(fb != NULL);
-  switch(fb->type())
-  {
-    case kSoftBuffer:
-      osystem->settings().setString("video", "soft");
-      break;
-
-    case kGLBuffer:
-      osystem->settings().setString("video", "gl");
-      break;
-  }
 
   return fb;
 }
