@@ -8,27 +8,35 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Random.cxx,v 1.6 2008/05/16 23:56:30 stephena Exp $
+// $Id: Random.cxx 2001 2010-04-10 21:37:23Z stephena $
 //============================================================================
 
 #include <time.h>
+
 #include "OSystem.hxx"
 #include "Random.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Random::Random()
+{
+  initSeed();
+}
+ 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Random::initSeed()
 {
   if(ourSystem)
     myValue = ourSystem->getTicks();
   else
     myValue = (uInt32)time(0);
 }
- 
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt32 Random::next()
 {
