@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Settings.cxx,v 1.147 2008/05/21 16:49:07 stephena Exp $
+// $Id: Settings.cxx 2031 2010-05-03 12:01:43Z stephena $
 //============================================================================
 
 #include <cassert>
@@ -40,22 +41,31 @@ Settings::Settings(OSystem* osystem)
 
   // OpenGL specific options
   setInternal("gl_filter", "nearest");
-  setInternal("gl_aspect", "100");
-  setInternal("gl_fsmax", "never");
+  setInternal("gl_aspectn", "100");
+  setInternal("gl_aspectp", "100");
+  setInternal("gl_fsmax", "false");
   setInternal("gl_lib", "libGL.so");
   setInternal("gl_vsync", "false");
   setInternal("gl_texrect", "false");
+//  setInternal("gl_accel", "true");
 
   // Framebuffer-related options
-  setInternal("zoom_ui", "2");
-  setInternal("zoom_tia", "2");
-  setInternal("fullscreen", "false");
-  setInternal("fullres", "");
+  setInternal("tia_filter", "zoom2x");
+  setInternal("fullscreen", "0");
+  setInternal("fullres", "auto");
   setInternal("center", "true");
   setInternal("grabmouse", "false");
   setInternal("palette", "standard");
   setInternal("colorloss", "false");
   setInternal("timing", "sleep");
+  setInternal("uimessages", "true");
+
+  // TV filter options
+  setInternal("tv_tex", "off");
+  setInternal("tv_bleed", "off");
+  setInternal("tv_noise", "off");
+//  setInternal("tv_curve", "false");  // not yet implemented
+  setInternal("tv_phos", "false");
 
   // Sound options
   setInternal("sound", "true");
@@ -71,38 +81,54 @@ Settings::Settings(OSystem* osystem)
   setInternal("joyaxismap", "");
   setInternal("joyhatmap", "");
   setInternal("joydeadzone", "0");
+  setInternal("joyallow4", "false");
+  setInternal("usemouse", "true");
   setInternal("pspeed", "6");
   setInternal("sa1", "left");
   setInternal("sa2", "right");
+  setInternal("ctrlcombo", "true");
 
   // Snapshot options
   setInternal("ssdir", "");
   setInternal("sssingle", "false");
+  setInternal("ss1x", "false");
+  setInternal("ssinterval", "2");
 
   // Config files and paths
-  setInternal("romdir", "");
+  setInternal("romdir", "~");
   setInternal("statedir", "");
   setInternal("cheatfile", "");
   setInternal("palettefile", "");
   setInternal("propsfile", "");
+  setInternal("eepromdir", "");
 
   // ROM browser options
+  setInternal("uselauncher", "true");
   setInternal("launcherres", "640x480");
-  setInternal("launcherfont", "small");
-  setInternal("romviewer", "false");
+  setInternal("launcherfont", "medium");
+  setInternal("launcherexts", "allfiles");
+  setInternal("romviewer", "0");
   setInternal("lastrom", "");
 
   // UI-related options
   setInternal("debuggerres", "1030x690");
   setInternal("uipalette", "0");
+  setInternal("listdelay", "300");
   setInternal("mwheel", "4");
 
   // Misc options
   setInternal("autoslot", "false");
-  setInternal("showinfo", "false");
-  setInternal("tiafloat", "true");
+  setInternal("showinfo", "1");
+  setInternal("tiadriven", "false");
   setInternal("avoxport", "");
   setInternal("stats", "false");
+  setInternal("audiofirst", "true");
+  setInternal("fastscbios", "false");
+  setExternal("romloadcount", "0");
+  setExternal("maxres", "0x0");
+
+  // Debugger options
+  setInternal("resolvedata", "auto");
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -121,7 +147,7 @@ void Settings::loadConfig()
   ifstream in(myOSystem->configFile().c_str());
   if(!in || !in.is_open())
   {
-    cout << "Error: Couldn't load settings file\n";
+    myOSystem->logMessage("ERROR: Couldn't load settings file\n", 0);
     return;
   }
 
@@ -184,19 +210,32 @@ string Settings::loadCommandLine(int argc, char** argv)
         continue;
       }
 
+      ostringstream buf;
       if(++i >= argc)
       {
-        cerr << "Missing argument for '" << key << "'" << endl;
+        buf << "Missing argument for '" << key << "'" << endl;
+        myOSystem->logMessage(buf.str(), 0);
         return "";
       }
       string value = argv[i];
 
+      buf.str("");
+      buf << "  key = '" << key << "', value = '" << value << "'";
+
       // Settings read from the commandline must not be saved to 
       // the rc-file, unless they were previously set
       if(int idx = getInternalPos(key) != -1)
+      {
         setInternal(key, value, idx);   // don't set initialValue here
+        buf << "(I)\n";
+      }
       else
+      {
         setExternal(key, value);
+        buf << "(E)\n";
+      }
+
+      myOSystem->logMessage(buf.str(), 2);
     }
     else
       return key;
@@ -212,77 +251,80 @@ void Settings::validate()
   int i;
 
   s = getString("video");
-  if(s != "soft" && s != "gl")
-    setInternal("video", "soft");
+  if(s != "soft" && s != "gl")  setInternal("video", "soft");
 
   s = getString("timing");
-  if(s != "sleep" && s != "busy")
-    setInternal("timing", "sleep");
+  if(s != "sleep" && s != "busy")  setInternal("timing", "sleep");
 
 #ifdef DISPLAY_OPENGL
   s = getString("gl_filter");
-  if(s != "linear" && s != "nearest")
-    setInternal("gl_filter", "nearest");
+  if(s != "linear" && s != "nearest")  setInternal("gl_filter", "nearest");
 
-  i = getInt("gl_aspect");
-  if(i < 50 || i > 100)
-    setInternal("gl_aspect", "100");
-
-  s = getString("gl_fsmax");
-  if(s != "never" && s != "ui" && s != "tia" && s != "always")
-    setInternal("gl_fsmax", "never");
+  i = getInt("gl_aspectn");
+  if(i < 80 || i > 120)  setInternal("gl_aspectn", "100");
+  i = getInt("gl_aspectp");
+  if(i < 80 || i > 120)  setInternal("gl_aspectp", "100");
 #endif
 
 #ifdef SOUND_SUPPORT
   i = getInt("volume");
-  if(i < 0 || i > 100)
-    setInternal("volume", "100");
+  if(i < 0 || i > 100)    setInternal("volume", "100");
   i = getInt("freq");
-  if(i < 0 || i > 48000)
-    setInternal("freq", "31400");
+  if(i < 0 || i > 48000)  setInternal("freq", "31400");
   i = getInt("tiafreq");
-  if(i < 0 || i > 48000)
-    setInternal("tiafreq", "31400");
+  if(i < 0 || i > 48000)  setInternal("tiafreq", "31400");
 #endif
 
-  i = getInt("zoom_ui");
-  if(i < 1 || i > 10)
-    setInternal("zoom_ui", "2");
-
-  i = getInt("zoom_tia");
-  if(i < 1 || i > 10)
-    setInternal("zoom_tia", "2");
-
   i = getInt("joydeadzone");
-  if(i < 0)
-    setInternal("joydeadzone", "0");
-  else if(i > 29)
-    setInternal("joydeadzone", "29");
+  if(i < 0)        setInternal("joydeadzone", "0");
+  else if(i > 29)  setInternal("joydeadzone", "29");
 
   i = getInt("pspeed");
-  if(i < 1)
-    setInternal("pspeed", "1");
-  else if(i > 15)
-    setInternal("pspeed", "15");
+  if(i < 1)        setInternal("pspeed", "1");
+  else if(i > 15)  setInternal("pspeed", "15");
+
+  i = getInt("ssinterval");
+  if(i < 1)        setInternal("ssinterval", "2");
+  else if(i > 10)  setInternal("ssinterval", "10");
 
   s = getString("palette");
   if(s != "standard" && s != "z26" && s != "user")
     setInternal("palette", "standard");
 
   s = getString("launcherfont");
-  if(s != "small" && s != "large")
-    setInternal("launcherfont", "small");
+  if(s != "small" && s != "medium" && s != "large")
+    setInternal("launcherfont", "medium");
+
+  i = getInt("romviewer");
+  if(i < 0)       setInternal("romviewer", "0");
+  else if(i > 2)  setInternal("romviewer", "2");
+
+  s = getString("tv_tex");
+  if(s != "normal" && s != "stag")
+    setInternal("tv_tex", "off");
+
+  s = getString("tv_bleed");
+  if(s != "low" && s != "medium" && s != "high")
+    setInternal("tv_bleed", "off");
+
+  s = getString("tv_noise");
+  if(s != "low" && s != "medium" && s != "high")
+    setInternal("tv_noise", "off");
+
+  i = getInt("showinfo");
+  if(i < 0 || i > 2)
+    setInternal("showinfo", "1");
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Settings::usage()
 {
-#ifndef MAC_OSX
   cout << endl
     << "Stella version " << STELLA_VERSION << endl
     << endl
     << "Usage: stella [options ...] romfile" << endl
     << "       Run without any options or romfile to use the ROM launcher" << endl
+    << "       Consult the manual for more in-depth information" << endl
     << endl
     << "Valid options are:" << endl
     << endl
@@ -295,17 +337,29 @@ void Settings::usage()
     << "  -gl_filter    <type>         Type is one of the following:\n"
     << "                 nearest         Normal scaling (GL_NEAREST)\n"
     << "                 linear          Blurred scaling (GL_LINEAR)\n"
-    << "  -gl_aspect    <number>       Scale the width by the given percentage\n"
-    << "  -gl_fsmax     <never|always| Stretch GL image in fullscreen mode\n"
-    << "                 ui|tia\n"
+    << "  -gl_aspectn   <number>       Scale the TIA width by the given percentage in NTSC mode\n"
+    << "  -gl_aspectp   <number>       Scale the TIA width by the given percentage in PAL mode\n"
+    << "  -gl_fsmax     <1|0>          Stretch GL image in fullscreen emulation mode\n"
     << "  -gl_vsync     <1|0>          Enable synchronize to vertical blank interrupt\n"
     << "  -gl_texrect   <1|0>          Enable GL_TEXTURE_RECTANGLE extension\n"
+//    << "  -gl_accel     <1|0>          Enable SDL_GL_ACCELERATED_VISUAL\n"
+    << "  -tv_tex       <off|type>     OpenGL TV texturing, type is one of the following:\n"
+    << "                 normal         Aligned in a grid\n"
+    << "                 stag           Aligned in a staggered grid\n"
+    << "  -tv_bleed     <off|type>     OpenGL TV color bleed, type is one of the following:\n"
+    << "                 low            \n"
+    << "                 medium         \n"
+    << "                 high           \n"
+    << "  -tv_noise     <off|type>     OpenGL TV RF noise emulation, type is one of the following:\n"
+    << "                 low            \n"
+    << "                 medium         \n"
+    << "                 high           \n"
+    << "  -tv_phos      <1|0>          OpenGL TV phosphor burn-off\n"
     << endl
   #endif
-    << "  -zoom_tia     <zoom>         Use the specified zoom level in emulation mode\n"
-    << "  -zoom_ui      <zoom>         Use the specified zoom level in non-emulation mode (ROM browser/debugger)\n"
-    << "  -fullscreen   <1|0>          Play the game in fullscreen mode\n"
-    << "  -fullres      <WxH>          The resolution to use in fullscreen mode\n"
+    << "  -tia_filter   <filter>       Use the specified filter in emulation mode\n"
+    << "  -fullscreen   <1|0|-1>       Use fullscreen mode (1 or 0), or disable switching to fullscreen entirely\n"
+    << "  -fullres      <auto|WxH>     The resolution to use in fullscreen mode\n"
     << "  -center       <1|0>          Centers game window (if possible)\n"
     << "  -grabmouse    <1|0>          Keeps the mouse in the game window\n"
     << "  -palette      <standard|     Use the specified color palette\n"
@@ -314,6 +368,7 @@ void Settings::usage()
     << "  -colorloss    <1|0>          Enable PAL color-loss effect\n"
     << "  -framerate    <number>       Display the given number of frames per second (0 to auto-calculate)\n"
     << "  -timing       <sleep|busy>   Use the given type of wait between frames\n"
+    << "  -uimessages   <1|0>          Show onscreen UI messages for different events\n"
     << endl
   #ifdef SOUND_SUPPORT
     << "  -sound        <1|0>          Enable sound generation\n"
@@ -325,41 +380,58 @@ void Settings::usage()
     << endl
   #endif
     << "  -cheat        <code>         Use the specified cheatcode (see manual for description)\n"
-    << "  -showinfo     <1|0>          Shows some game info\n"
+    << "  -showinfo     <0|1|2>        Shows some application/game info on commandline\n"
     << "  -joydeadzone  <number>       Sets 'deadzone' area for analog joysticks (0-29)\n"
+    << "  -joyallow4    <1|0>          Allow all 4 directions on a joystick to be pressed simultaneously\n"
+    << "  -usemouse     <1|0>          Use mouse for various controllers (paddle, driving, etc)\n"
     << "  -pspeed       <number>       Speed of digital emulated paddle movement (1-15)\n"
     << "  -sa1          <left|right>   Stelladaptor 1 emulates specified joystick port\n"
     << "  -sa2          <left|right>   Stelladaptor 2 emulates specified joystick port\n"
-    << "  -romviewer    <1|0>          Show ROM info viewer in ROM launcher\n"
+    << "  -ctrlcombo    <1|0>          Use key combos involving the Control key (Control-Q for quit may be disabled!)\n"
     << "  -autoslot     <1|0>          Automatically switch to next save slot when state saving\n"
+    << "  -stats        <1|0>          Overlay console info during emulation\n"
+    << "  -audiofirst   <1|0>          Initial audio before video (required for some ATI video cards)\n"
+    << "  -fastscbios   <1|0>          Disable Supercharger BIOS progress loading bars\n"
     << "  -ssdir        <path>         The directory to save snapshot files to\n"
     << "  -sssingle     <1|0>          Generate single snapshot instead of many\n"
-    << "  -stats        <1|0>          Show scanline and framerate info during emulation\n"
+    << "  -ss1x         <1|0>          Generate TIA snapshot in 1x mode (ignore scaling/effects)\n"
+    << "  -ssinterval   <number        Number of seconds between snapshots in continuous snapshot mode\n"
     << endl
-    << "  -listrominfo                 Display contents of stella.pro, one line per ROM entry\n"
     << "  -rominfo      <rom>          Display detailed information for the given ROM\n"
+    << "  -listrominfo                 Display contents of stella.pro, one line per ROM entry\n"
+    << "  -uselauncher  <1|0>          Use the built-in ROM launcher\n"
     << "  -launcherres  <WxH>          The resolution to use in ROM launcher mode\n"
-    << "  -launcherfont <small|large>  Use small or large font in the ROM launcher\n"
+    << "  -launcherfont <small|medium| Use the specified font in the ROM launcher\n"
+    << "                 large>\n"
+    << "  -launcherexts <allfiles|     Show files with the given extensions in ROM launcher\n"
+    << "                 allroms|        (exts is a ':' separated list of extensions)\n"
+    << "                 exts\n"
+    << "  -romviewer    <0|1|2>        Show ROM info viewer at given zoom level in ROM launcher (0 for off)\n"
     << "  -uipalette    <1|2>          Used the specified palette for UI elements\n"
+    << "  -listdelay    <delay>        Time to wait between keypresses in list widgets (300-1000)\n"
     << "  -mwheel       <lines>        Number of lines the mouse wheel will scroll in UI\n"
     << "  -statedir     <dir>          Directory in which to save state files\n"
     << "  -cheatfile    <file>         Full pathname of cheatfile database\n"
     << "  -palettefile  <file>         Full pathname of user-defined palette file\n"
     << "  -propsfile    <file>         Full pathname of ROM properties file\n"
-    << "  -tiafloat     <1|0>          Set unused TIA pins floating on a read/peek\n"
+    << "  -eepromdir    <dir>          Directory in which to save EEPROM files\n"
     << "  -avoxport     <name>         The name of the serial port where an AtariVox is connected\n"
+    << "  -maxres       <WxH>          Used by developers to force the maximum size of the application window\n"
     << "  -help                        Show the text you're now reading\n"
   #ifdef DEBUGGER_SUPPORT
     << endl
     << " The following options are meant for developers\n"
     << " Arguments are more fully explained in the manual\n"
     << endl
+    << "   -resolvedata  <never|       Set automatic code vs. data determination in the disassembler\n"
+    << "                  always|auto>\n"
     << "   -debuggerres  <WxH>         The resolution to use in debugger mode\n"
     << "   -break        <address>     Set a breakpoint at 'address'\n"
     << "   -debug                      Start in debugger mode\n"
     << "   -holdreset                  Start the emulator with the Game Reset switch held down\n"
     << "   -holdselect                 Start the emulator with the Game Select switch held down\n"
     << "   -holdbutton0                Start the emulator with the left joystick button held down\n"
+    << "   -tiadriven    <1|0>         Drive unused TIA pins randomly value on a read/peek\n"
     << endl
     << "   -bs          <arg>          Sets the 'Cartridge.Type' (bankswitch) property\n"
     << "   -type        <arg>          Same as using -bs\n"
@@ -377,10 +449,8 @@ void Settings::usage()
     << "   -height      <arg>          Sets the 'Display.Height' property\n"
     << "   -pp          <arg>          Sets the 'Display.Phosphor' property\n"
     << "   -ppblend     <arg>          Sets the 'Display.PPBlend' property\n"
-    << "   -hmove       <arg>          Sets the 'Emulation.HmoveBlanks' property\n"
   #endif
     << endl;
-#endif
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -404,7 +474,7 @@ void Settings::saveConfig()
   ofstream out(myOSystem->configFile().c_str());
   if(!out || !out.is_open())
   {
-    cout << "Error: Couldn't save settings file\n";
+    myOSystem->logMessage("ERROR: Couldn't save settings file\n", 0);
     return;
   }
 
@@ -481,11 +551,12 @@ void Settings::setString(const string& key, const string& value)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Settings::getSize(const string& key, int& x, int& y) const
 {
+  char c;
   string size = getString(key);
-  replace(size.begin(), size.end(), 'x', ' ');
   istringstream buf(size);
-  buf >> x;
-  buf >> y;
+  buf >> x >> c >> y;
+  if(c != 'x')
+    x = y = -1;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
