@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Switches.cxx,v 1.9 2008/02/06 13:45:22 stephena Exp $
+// $Id: Switches.cxx 2001 2010-04-10 21:37:23Z stephena $
 //============================================================================
 
 #include "Event.hxx"
@@ -116,14 +117,14 @@ bool Switches::save(Serializer& out) const
   }
   catch(...)
   {
-    cerr << "Error: Switches::save() exception\n";
+    cerr << "ERROR: Switches::save() exception\n";
     return false;
   }
   return true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
-bool Switches::load(Deserializer& in)
+bool Switches::load(Serializer& in)
 {
   try
   {
@@ -131,7 +132,7 @@ bool Switches::load(Deserializer& in)
   }
   catch(...)
   {
-    cerr << "Error: Switches::load() exception\n";
+    cerr << "ERROR: Switches::load() exception\n";
     return false;
   }
   return true;
