@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott
+// Copyright (c) 1995-2010 by Bradford W. Mott
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: TIASnd.cxx,v 1.7 2008/02/06 13:45:22 stephena Exp $
+// $Id: TIASnd.cxx 2001 2010-04-10 21:37:23Z stephena $
 //============================================================================
 
 #include "System.hxx"
@@ -107,7 +108,7 @@ void TIASound::set(uInt16 address, uInt8 value)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt8 TIASound::get(uInt16 address)
+uInt8 TIASound::get(uInt16 address) const
 {
   switch(address)
   {
