@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: AudioDialog.cxx,v 1.27 2008/03/23 16:22:46 stephena Exp $
+// $Id: AudioDialog.cxx 2001 2010-04-10 21:37:23Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -29,6 +30,7 @@
 #include "Menu.hxx"
 #include "OSystem.hxx"
 #include "PopUpWidget.hxx"
+#include "StringList.hxx"
 #include "Settings.hxx"
 #include "Sound.hxx"
 #include "Widget.hxx"
@@ -37,8 +39,8 @@
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 AudioDialog::AudioDialog(OSystem* osystem, DialogContainer* parent,
-                         const GUI::Font& font, int x, int y, int w, int h)
-    : Dialog(osystem, parent, x, y, w, h)
+                         const GUI::Font& font)
+  : Dialog(osystem, parent, 0, 0, 0, 0)
 {
   const int lineHeight   = font.getLineHeight(),
             fontWidth    = font.getMaxCharWidth(),
@@ -49,10 +51,11 @@ AudioDialog::AudioDialog(OSystem* osystem, DialogContainer* parent,
   int lwidth = font.getStringWidth("Fragment Size: "),
       pwidth = font.getStringWidth("4096");
   WidgetArray wid;
+  StringMap items;
 
   // Set real dimensions
-//  _w = 35 * fontWidth + 10;
-//  _h = 8 * (lineHeight + 4) + 10;
+  _w = 35 * fontWidth + 10;
+  _h = 8 * (lineHeight + 4) + 10;
 
   // Volume
   xpos = 3 * fontWidth;  ypos = 10;
@@ -70,39 +73,37 @@ AudioDialog::AudioDialog(OSystem* osystem, DialogContainer* parent,
   ypos += lineHeight + 4;
 
   // Fragment size
+  items.clear();
+  items.push_back("128", "128");
+  items.push_back("256", "256");
+  items.push_back("512", "512");
+  items.push_back("1024", "1024");
+  items.push_back("2048", "2048");
+  items.push_back("4096", "4096");
   myFragsizePopup = new PopUpWidget(this, font, xpos, ypos,
                                     pwidth + myVolumeLabel->getWidth() - 4, lineHeight,
-                                    "Fragment size: ", lwidth);
-  myFragsizePopup->appendEntry("128",  1);
-  myFragsizePopup->appendEntry("256",  2);
-  myFragsizePopup->appendEntry("512",  3);
-  myFragsizePopup->appendEntry("1024", 4);
-  myFragsizePopup->appendEntry("2048", 5);
-  myFragsizePopup->appendEntry("4096", 6);
+                                    items, "Fragment size: ", lwidth);
   wid.push_back(myFragsizePopup);
   ypos += lineHeight + 4;
 
   // Output frequency
+  items.clear();
+  items.push_back("11025", "11025");
+  items.push_back("22050", "22050");
+  items.push_back("31400", "31400");
+  items.push_back("44100", "44100");
+  items.push_back("48000", "48000");
   myFreqPopup = new PopUpWidget(this, font, xpos, ypos,
                                 pwidth + myVolumeLabel->getWidth() - 4, lineHeight,
-                                "Output freq: ", lwidth);
-  myFreqPopup->appendEntry("11025", 1);
-  myFreqPopup->appendEntry("22050", 2);
-  myFreqPopup->appendEntry("31400", 3);
-  myFreqPopup->appendEntry("44100", 4);
-  myFreqPopup->appendEntry("48000", 5);
+                                items, "Output freq: ", lwidth);
   wid.push_back(myFreqPopup);
   ypos += lineHeight + 4;
 
   // TIA frequency
+  // ... use same items as above
   myTiaFreqPopup = new PopUpWidget(this, font, xpos, ypos,
                                    pwidth + myVolumeLabel->getWidth() - 4, lineHeight,
-                                   "TIA freq: ", lwidth);
-  myTiaFreqPopup->appendEntry("11025", 1);
-  myTiaFreqPopup->appendEntry("22050", 2);
-  myTiaFreqPopup->appendEntry("31400", 3);
-  myTiaFreqPopup->appendEntry("44100", 4);
-  myTiaFreqPopup->appendEntry("48000", 5);
+                                   items, "TIA freq: ", lwidth);
   wid.push_back(myTiaFreqPopup);
   ypos += lineHeight + 4;
 
@@ -136,49 +137,24 @@ AudioDialog::~AudioDialog()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void AudioDialog::loadConfig()
 {
-  bool b;
-  int i;
-
   // Volume
-  myVolumeSlider->setValue(instance()->settings().getInt("volume"));
-  myVolumeLabel->setLabel(instance()->settings().getString("volume"));
+  myVolumeSlider->setValue(instance().settings().getInt("volume"));
+  myVolumeLabel->setLabel(instance().settings().getString("volume"));
 
   // Fragsize
-  i = instance()->settings().getInt("fragsize");
-  if(i == 128)       i = 1;
-  else if(i == 256)  i = 2;
-  else if(i == 512)  i = 3;
-  else if(i == 1024) i = 4;
-  else if(i == 2048) i = 5;
-  else if(i == 4096) i = 6;
-  myFragsizePopup->setSelectedTag(i);
+  myFragsizePopup->setSelected(instance().settings().getString("fragsize"), "512");
 
   // Output frequency
-  i = instance()->settings().getInt("freq");
-  if(i == 11025)      i = 1;
-  else if(i == 22050) i = 2;
-  else if(i == 31400) i = 3;
-  else if(i == 44100) i = 4;
-  else if(i == 48000) i = 5;
-  else i = 3;  // default to '31400'
-  myFreqPopup->setSelectedTag(i);
+  myFreqPopup->setSelected(instance().settings().getString("freq"), "31400");
 
   // TIA frequency
-  i = instance()->settings().getInt("tiafreq");
-  if(i == 11025)      i = 1;
-  else if(i == 22050) i = 2;
-  else if(i == 31400) i = 3;
-  else if(i == 44100) i = 4;
-  else if(i == 48000) i = 5;
-  else i = 3;  // default to '31400'
-  myTiaFreqPopup->setSelectedTag(i);
+  myTiaFreqPopup->setSelected(instance().settings().getString("tiafreq"), "31400");
 
   // Clip volume
-  b = instance()->settings().getBool("clipvol");
-  myClipVolumeCheckbox->setState(b);
+  myClipVolumeCheckbox->setState(instance().settings().getBool("clipvol"));
 
   // Enable sound
-  b = instance()->settings().getBool("sound");
+  bool b = instance().settings().getBool("sound");
   mySoundEnableCheckbox->setState(b);
 
   // Make sure that mutually-exclusive items are not enabled at the same time
@@ -188,39 +164,31 @@ void AudioDialog::loadConfig()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void AudioDialog::saveConfig()
 {
-  Settings& settings = instance()->settings();
-  string s;
-  bool b;
-  int i;
+  Settings& settings = instance().settings();
 
   // Volume
-  i = myVolumeSlider->getValue();
-  instance()->sound().setVolume(i);
+  settings.setInt("volume", myVolumeSlider->getValue());
+  instance().sound().setVolume(myVolumeSlider->getValue());
 
   // Fragsize
-  s = myFragsizePopup->getSelectedString();
-  settings.setString("fragsize", s);
+  settings.setString("fragsize", myFragsizePopup->getSelectedTag());
 
   // Output frequency
-  s = myFreqPopup->getSelectedString();
-  settings.setString("freq", s);
+  settings.setString("freq", myFreqPopup->getSelectedTag());
 
   // TIA frequency
-  s = myTiaFreqPopup->getSelectedString();
-  settings.setString("tiafreq", s);
+  settings.setString("tiafreq", myTiaFreqPopup->getSelectedTag());
 
   // Enable/disable volume clipping (requires a restart to take effect)
-  b = myClipVolumeCheckbox->getState();
-  settings.setBool("clipvol", b);
+  settings.setBool("clipvol", myClipVolumeCheckbox->getState());
 
   // Enable/disable sound (requires a restart to take effect)
-  b = mySoundEnableCheckbox->getState();
-  instance()->sound().setEnabled(b);
+  instance().sound().setEnabled(mySoundEnableCheckbox->getState());
 
   // Only force a re-initialization when necessary, since it can
   // be a time-consuming operation
-  if(&instance()->console())
-    instance()->console().initializeAudio();
+  if(&instance().console())
+    instance().console().initializeAudio();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -229,13 +197,9 @@ void AudioDialog::setDefaults()
   myVolumeSlider->setValue(100);
   myVolumeLabel->setLabel("100");
 
-#ifdef WIN32
-  myFragsizePopup->setSelectedTag(5);
-#else
-  myFragsizePopup->setSelectedTag(3);
-#endif
-  myFreqPopup->setSelectedTag(3);
-  myTiaFreqPopup->setSelectedTag(3);
+  myFragsizePopup->setSelected("512", "");
+  myFreqPopup->setSelected("31400", "");
+  myTiaFreqPopup->setSelected("31400", "");
 
   myClipVolumeCheckbox->setState(true);
   mySoundEnableCheckbox->setState(true);
