@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: BrowserDialog.cxx,v 1.30 2008/03/23 16:22:46 stephena Exp $
+// $Id: BrowserDialog.cxx 2001 2010-04-10 21:37:23Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -22,6 +23,7 @@
 #include "bspf.hxx"
 
 #include "Dialog.hxx"
+#include "DialogContainer.hxx"
 #include "FSNode.hxx"
 #include "GameList.hxx"
 #include "GuiObject.hxx"
@@ -35,18 +37,17 @@
  * - to select the data dir for a game
  * - to select the place where save games are stored
  * - others???
- * TODO - make this dialog font sensitive
  */
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 BrowserDialog::BrowserDialog(GuiObject* boss, const GUI::Font& font,
-                             int x, int y, int w, int h)
-  : Dialog(boss->instance(), boss->parent(), x, y, w, h),
+                             int max_w, int max_h)
+  : Dialog(&boss->instance(), &boss->parent(), 0, 0, 0, 0),
     CommandSender(boss),
     _fileList(NULL),
     _currentPath(NULL),
     _nodeList(NULL),
-    _mode(AbstractFilesystemNode::kListDirectoriesOnly)
+    _mode(FilesystemNode::kListDirectoriesOnly)
 {
   const int lineHeight   = font.getLineHeight(),
             buttonWidth  = font.getStringWidth("Defaults") + 20,
@@ -56,8 +57,8 @@ BrowserDialog::BrowserDialog(GuiObject* boss, const GUI::Font& font,
 
   // Set real dimensions
   // This is one dialog that can take as much space as is available
-//  _w = _DLG_MIN_SWIDTH - 30;
-//  _h = _DLG_MIN_SHEIGHT - 30;
+  _w = BSPF_min(max_w, 480);
+  _h = BSPF_min(max_h, 380);
 
   xpos = 10;  ypos = 4;
   _title = new StaticTextWidget(this, font, xpos, ypos,
@@ -82,6 +83,11 @@ BrowserDialog::BrowserDialog(GuiObject* boss, const GUI::Font& font,
                                  buttonWidth, buttonHeight, "Go up", kGoUpCmd);
   addFocusWidget(_goUpButton);
 
+  _basedirButton =
+    new ButtonWidget(this, font, 15 + buttonWidth, _h - buttonHeight - 10,
+                     buttonWidth, buttonHeight, "Base Dir", kBaseDirCmd);
+  addFocusWidget(_basedirButton);
+
 #ifndef MAC_OSX
   b = new ButtonWidget(this, font, _w - 2 * (buttonWidth + 7), _h - buttonHeight - 10,
                        buttonWidth, buttonHeight, "Choose", kChooseCmd);
@@ -115,17 +121,30 @@ BrowserDialog::~BrowserDialog()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void BrowserDialog::setStartPath(const string& startpath,
-                                 FilesystemNode::ListMode mode)
+void BrowserDialog::show(const string& title, const string& startpath,
+                         FilesystemNode::ListMode mode, int cmd)
 {
+  // TODO - dialog has to be added before any settings are changed,
+  //        since (for example) changing the title triggers a redraw,
+  //        and the dialog must be added (so it exists) for that to happen
+  // Fixing this requires changes to the underlying widget classes
+  // (ie, changing a widgets contents should signal its dialog that a
+  // redraw is necessary; it shouldn't be responsible for redraw itself)
+  //
+  // Doing it this way has the unfortunate side effect that a previous
+  // title is temporarily visible when re-using the browser for different
+  // purposes
+  parent().addDialog(this);
+
+  _title->setLabel(title);
+  _cmd = cmd;
   _mode = mode;
 
   // If no node has been set, or the last used one is now invalid,
-  // go back to the root/default dir.
+  // go back to the users home dir.
   _node = FilesystemNode(startpath);
-
-  if(!_node.isValid())
-    _node = FilesystemNode();
+  if(!_node.exists())
+    _node = FilesystemNode("~");
 
   // Generally, we always want a directory listing 
   if(!_node.isDirectory() && _node.hasParent())
@@ -145,27 +164,25 @@ void BrowserDialog::updateListing()
   _nodeList->clear();
 
   // Update the path display
-  _currentPath->setLabel(_node.path());
+  _currentPath->setLabel(_node.getRelativePath());
 
   // Read in the data from the file system
-  FSList content = _node.listDir(_mode);
+  FSList content;
+  _node.getChildren(content, _mode);
 
   // Add '[..]' to indicate previous folder
   if(_node.hasParent())
-  {
-    const string& parent = _node.getParent().path();
-    _nodeList->appendGame(" [..]", parent, "", true);
-  }
+    _nodeList->appendGame(" [..]", _node.getParent().getPath(), "", true);
 
   // Now add the directory entries
   for(unsigned int idx = 0; idx < content.size(); idx++)
   {
-    string name = content[idx].displayName();
+    string name = content[idx].getDisplayName();
     bool isDir = content[idx].isDirectory();
     if(isDir)
       name = " [" + name + "]";
 
-    _nodeList->appendGame(name, content[idx].path(), "", isDir);
+    _nodeList->appendGame(name, content[idx].getPath(), "", isDir);
   }
   _nodeList->sortByName();
 
@@ -180,9 +197,6 @@ void BrowserDialog::updateListing()
 
   // Only hilite the 'up' button if there's a parent directory
   _goUpButton->setEnabled(_node.hasParent());
-
-  // Finally, redraw
-  setDirty(); draw();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -199,17 +213,23 @@ void BrowserDialog::handleCommand(CommandSender* sender, int cmd,
       break;
 
     case kGoUpCmd:
+    case kListPrevDirCmd:
       _node = _node.getParent();
       updateListing();
       break;
 
+    case kBaseDirCmd:
+      _node = FilesystemNode(instance().baseDir());
+      updateListing();
+      break;
+
     case kListItemActivatedCmd:
     case kListItemDoubleClickedCmd:
     {
       int item = _fileList->getSelected();
       if(item >= 0)
       {
-        _node = _nodeList->path(item);
+        _node = FilesystemNode(_nodeList->path(item));
         updateListing();
       }
       break;
