@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Dialog.cxx,v 1.60 2008/03/25 13:11:34 stephena Exp $
+// $Id: Dialog.cxx 2001 2010-04-10 21:37:23Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -36,17 +37,19 @@
  */
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Dialog::Dialog(OSystem* instance, DialogContainer* parent,
-               int x, int y, int w, int h)
-  : GuiObject(instance, parent, x, y, w, h),
+               int x, int y, int w, int h, bool isBase)
+  : GuiObject(*instance, *parent, *this, x, y, w, h),
     _mouseWidget(0),
     _focusedWidget(0),
     _dragWidget(0),
     _okWidget(0),
     _cancelWidget(0),
     _visible(true),
-    _center(true),
+    _isBase(isBase),
     _ourTab(NULL),
-    _focusID(0)
+    _surface(NULL),
+    _focusID(0),
+    _surfaceID(-1)
 {
 }
 
@@ -68,6 +71,23 @@ void Dialog::open()
   _result = 0;
   _visible = true;
 
+  // Make sure we have a valid surface to draw into
+  // Technically, this shouldn't be needed until drawDialog(), but some
+  // dialogs cause drawing to occur within loadConfig()
+  // Base surfaces are typically large, and will probably cause slow
+  // performance if we update the whole area each frame
+  // Instead, dirty rectangle updates should be performed
+  // However, this policy is left entirely to the framebuffer
+  // We suggest the hint here, but specific framebuffers are free to
+  // ignore it
+  _surface = instance().frameBuffer().surface(_surfaceID);
+  if(_surface == NULL)
+  {
+    _surfaceID = instance().frameBuffer().allocateSurface(_w, _h, _isBase);
+    _surface   = instance().frameBuffer().surface(_surfaceID);
+  }
+
+  center();
   loadConfig();
 
   // (Re)-build the focus list to use for the widgets which are currently
@@ -85,17 +105,18 @@ void Dialog::close()
   }
 
   releaseFocus();
-  parent()->removeDialog();
+  parent().removeDialog();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Dialog::center()
 {
-  FrameBuffer& fb = instance()->frameBuffer();
-  if(_center && &fb)
+  if(_surface)
   {
-    _x = (fb.baseWidth() - _w)  / 2;
-    _y = (fb.baseHeight() - _h) / 2;
+    const GUI::Rect& screen = instance().frameBuffer().screenRect();
+    uInt32 x = (screen.width() - getWidth()) >> 1;
+    uInt32 y = (screen.height() - getHeight()) >> 1;
+    _surface->setPos(x, y);
   }
 }
 
@@ -138,9 +159,9 @@ void Dialog::addToFocusList(WidgetArray& list, int id)
   // Make sure the array is large enough
   while((int)_ourFocusList.size() <= id)
   {
-	Focus f;
-	f.focusedWidget = NULL;
-	_ourFocusList.push_back(f);
+    Focus f;
+    f.focusedWidget = NULL;
+    _ourFocusList.push_back(f);
   }
 
   _ourFocusList[id].focusList.push_back(list);
@@ -225,13 +246,14 @@ void Dialog::drawDialog()
   if(!isVisible())
     return;
 
+  FBSurface& s = surface();
+
   if(_dirty)
   {
-//    cerr << "Dialog::drawDialog()\n";
-    FrameBuffer& fb = instance()->frameBuffer();
+//    cerr << "Dialog::drawDialog(): w = " << _w << ", h = " << _h << " @ " << &s << endl << endl;
 
-    fb.fillRect(_x+1, _y+1, _w-2, _h-2, kDlgColor);
-    fb.box(_x, _y, _w, _h, kColor, kShadowColor);
+    s.fillRect(_x+1, _y+1, _w-2, _h-2, kDlgColor);
+    s.box(_x, _y, _w, _h, kColor, kShadowColor);
 
     // Make all child widget dirty
     Widget* w = _firstWidget;
@@ -248,12 +270,13 @@ void Dialog::drawDialog()
     // Draw outlines for focused widgets
     redrawFocus();
 
-    // Tell the framebuffer this area is dirty
-    fb.addDirtyRect(_x, _y, _w, _h);
-//cerr << "dirty: x = " << _x << ", y = " << _y << ", w = " << _w << ", h = " << _h << endl;
-
+    // Tell the surface this area is dirty
+    s.addDirtyRect(_x, _y, _w, _h);
     _dirty = false;
   }
+
+  // Commit surface changes to screen
+  s.update();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -318,7 +341,7 @@ void Dialog::handleKeyDown(int ascii, int keycode, int modifiers)
   // Detect selection of previous and next tab headers and objects
   // For some strange reason, 'tab' needs to be interpreted as keycode,
   // not ascii??
-  if(instance()->eventHandler().kbdShift(modifiers))
+  if(instance().eventHandler().kbdShift(modifiers))
   {
     if(ascii == 256+20 && _ourTab)      // left arrow
     {
@@ -339,7 +362,7 @@ void Dialog::handleKeyDown(int ascii, int keycode, int modifiers)
   // Check the keytable now, since we might get one of the above events,
   // which must always be processed before any widget sees it.
   if(e == Event::NoType)
-    e = instance()->eventHandler().eventForKey(keycode, kMenuMode);
+    e = instance().eventHandler().eventForKey(keycode, kMenuMode);
 
   // Unless a widget has claimed all responsibility for data, we assume
   // that if an event exists for the given data, it should have priority.
@@ -414,7 +437,7 @@ void Dialog::handleMouseMoved(int x, int y, int button)
 void Dialog::handleJoyDown(int stick, int button)
 {
   Event::Type e =
-    instance()->eventHandler().eventForJoyButton(stick, button, kMenuMode);
+    instance().eventHandler().eventForJoyButton(stick, button, kMenuMode);
 
   // Unless a widget has claimed all responsibility for data, we assume
   // that if an event exists for the given data, it should have priority.
@@ -439,7 +462,7 @@ void Dialog::handleJoyUp(int stick, int button)
 void Dialog::handleJoyAxis(int stick, int axis, int value)
 {
   Event::Type e =
-    instance()->eventHandler().eventForJoyAxis(stick, axis, value, kMenuMode);
+    instance().eventHandler().eventForJoyAxis(stick, axis, value, kMenuMode);
 
   // Unless a widget has claimed all responsibility for data, we assume
   // that if an event exists for the given data, it should have priority.
@@ -456,7 +479,7 @@ void Dialog::handleJoyAxis(int stick, int axis, int value)
 bool Dialog::handleJoyHat(int stick, int hat, int value)
 {
   Event::Type e =
-    instance()->eventHandler().eventForJoyHat(stick, hat, value, kMenuMode);
+    instance().eventHandler().eventForJoyHat(stick, hat, value, kMenuMode);
 
   // Unless a widget has claimed all responsibility for data, we assume
   // that if an event exists for the given data, it should have priority.
@@ -545,44 +568,33 @@ Widget* Dialog::findWidget(int x, int y)
 }
 
  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Dialog::addOKCancelBGroup(WidgetArray& wid, const GUI::Font& font)
+void Dialog::addOKCancelBGroup(WidgetArray& wid, const GUI::Font& font,
+                               const string& okText, const string& cancelText)
 {
   int buttonWidth  = font.getStringWidth("Cancel") + 15;
   int buttonHeight = font.getLineHeight() + 4;
   ButtonWidget* b;
 #ifndef MAC_OSX
   b = new ButtonWidget(this, font, _w - 2 * (buttonWidth + 7), _h - buttonHeight - 10,
-                       buttonWidth, buttonHeight, "OK", kOKCmd);
+                       buttonWidth, buttonHeight,
+                       okText == "" ? "OK" : okText, kOKCmd);
   wid.push_back(b);
   addOKWidget(b);
   b = new ButtonWidget(this, font, _w - (buttonWidth + 10), _h - buttonHeight - 10,
-                       buttonWidth, buttonHeight, "Cancel", kCloseCmd);
+                       buttonWidth, buttonHeight,
+                       cancelText == "" ? "Cancel" : cancelText, kCloseCmd);
   wid.push_back(b);
   addCancelWidget(b);
 #else
   b = new ButtonWidget(this, font, _w - 2 * (buttonWidth + 7), _h - buttonHeight - 10,
-                       buttonWidth, buttonHeight, "Cancel", kCloseCmd);
+                       buttonWidth, buttonHeight,
+                       cancelText == "" ? "Cancel" : cancelText, kCloseCmd);
   wid.push_back(b);
   addCancelWidget(b);
   b = new ButtonWidget(this, font, _w - (buttonWidth + 10), _h - buttonHeight - 10,
-                       buttonWidth, buttonHeight, "OK", kOKCmd);
+                       buttonWidth, buttonHeight,
+                       okText == "" ? "OK" : okText, kOKCmd);
   wid.push_back(b);
   addOKWidget(b);
 #endif
 }
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-ButtonWidget* Dialog::addButton(const GUI::Font& font, int x, int y,
-                                const string& label, int cmd)
-{
-  // FIXME - this is deprecated, and the UI code should be refactored
-  //         to remove all references to it
-#if 0
-  const int w = 6 * font.getMaxCharWidth(),
-            h = font.getFontHeight() + 6;
-
-  return new ButtonWidget(this, font, x, y, w, h, label, cmd);
-#else
-  return new ButtonWidget(this, font, x, y, kButtonWidth, 16, label, cmd);
-#endif
-}
