@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: DialogContainer.cxx,v 1.44 2008/05/21 14:01:31 stephena Exp $
+// $Id: DialogContainer.cxx 2001 2010-04-10 21:37:23Z stephena $
 //============================================================================
 
 #include "OSystem.hxx"
@@ -28,8 +29,7 @@
 DialogContainer::DialogContainer(OSystem* osystem)
   : myOSystem(osystem),
     myBaseDialog(NULL),
-    myTime(0),
-    myRefreshFlag(false)
+    myTime(0)
 {
   reset();
 }
@@ -42,7 +42,7 @@ DialogContainer::~DialogContainer()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void DialogContainer::updateTime(uInt32 time)
+void DialogContainer::updateTime(uInt64 time)
 {
   // We only need millisecond precision
   myTime = time / 1000;
@@ -87,17 +87,17 @@ void DialogContainer::updateTime(uInt32 time)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void DialogContainer::draw()
+void DialogContainer::draw(bool full)
 {
   // Draw all the dialogs on the stack when we want a full refresh
-  if(myRefreshFlag)
+  if(full)
   {
     for(int i = 0; i < myDialogStack.size(); i++)
     {
+      myDialogStack[i]->center();
       myDialogStack[i]->setDirty();
       myDialogStack[i]->drawDialog();
     }
-    myRefreshFlag = false;
   }
   else if(!myDialogStack.empty())
   {
@@ -108,11 +108,13 @@ void DialogContainer::draw()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DialogContainer::addDialog(Dialog* d)
 {
-  myDialogStack.push(d);
+  const GUI::Rect& image = myOSystem->frameBuffer().imageRect();
+  assert(d->getWidth() <= image.width() && d->getHeight() <= image.height());
 
-  d->center();
+  myDialogStack.push(d);
   d->open();
-  d->setDirty();  // Next update() will take care of drawing
+
+  myOSystem->frameBuffer().refresh();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -124,7 +126,7 @@ void DialogContainer::removeDialog()
 
     // We need to redraw the entire screen contents, since we don't know
     // what was obscured
-    myOSystem->eventHandler().refreshDisplay();
+    myOSystem->frameBuffer().refresh();
   }
 }
 
@@ -175,6 +177,7 @@ void DialogContainer::handleMouseMotionEvent(int x, int y, int button)
 
   // Send the event to the dialog box on the top of the stack
   Dialog* activeDialog = myDialogStack.top();
+  activeDialog->surface().translateCoords(x, y);
   activeDialog->handleMouseMoved(x - activeDialog->_x,
                                  y - activeDialog->_y,
                                  button);
@@ -191,6 +194,7 @@ void DialogContainer::handleMouseButtonEvent(MouseButton b, int x, int y, uInt8
 
   // Send the event to the dialog box on the top of the stack
   Dialog* activeDialog = myDialogStack.top();
+  activeDialog->surface().translateCoords(x, y);
 
   int button = (b == EVENT_LBUTTONDOWN || b == EVENT_LBUTTONUP) ? 1 : 2;
   switch(b)
@@ -323,13 +327,6 @@ void DialogContainer::handleJoyHatEvent(int stick, int hat, int value)
   activeDialog->handleJoyHat(stick, hat, value);
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void DialogContainer::handleResizeEvent()
-{
-  // Send resize event to base dialog; it's up to the dialog to actually listen
-  myBaseDialog->handleCommand(NULL, kResizeCmd, 0, 0);
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DialogContainer::reset()
 {
