@@ -8,17 +8,19 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: EditableWidget.cxx,v 1.27 2008/02/06 13:45:23 stephena Exp $
+// $Id: EditableWidget.cxx 2001 2010-04-10 21:37:23Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
 //============================================================================
 
+#include "Dialog.hxx"
 #include "EditableWidget.hxx"
 
 
@@ -31,7 +33,7 @@ EditableWidget::EditableWidget(GuiObject* boss, const GUI::Font& font,
 {
   _caretVisible = false;
   _caretTime = 0;
-  _caretPos = 0;	// FIXME
+  _caretPos = 0;
 
   _caretInverse = false;
 
@@ -60,6 +62,9 @@ void EditableWidget::setEditString(const string& str)
   if (_editScrollOffset < 0)
     _editScrollOffset = 0;
 
+  if(_editable)
+    startEditMode();
+
   // Make sure the new string is seen onscreen
   setDirty(); draw();
 }
@@ -76,7 +81,7 @@ void EditableWidget::setEditable(bool editable)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool EditableWidget::tryInsertChar(char c, int pos)
 {
-  if (isprint(c))
+  if(isprint(c) && c != '\"')
   {
     _editString.insert(pos, 1, c);
     return true;
@@ -91,13 +96,13 @@ bool EditableWidget::handleKeyDown(int ascii, int keycode, int modifiers)
     return true;
 
   // Ignore all alt-mod keys
-  if(instance()->eventHandler().kbdAlt(modifiers))
+  if(instance().eventHandler().kbdAlt(modifiers))
     return true;
 
   bool handled = true;
   bool dirty = false;
 
-  switch (keycode)
+  switch (ascii)
   {
     case '\n':  // enter/return
     case '\r':
@@ -115,22 +120,24 @@ bool EditableWidget::handleKeyDown(int ascii, int keycode, int modifiers)
 
     case 8:     // backspace
       dirty = killChar(-1);
+      if(dirty)  sendCommand(kEditChangedCmd, ascii, _id);
       break;
 
     case 127:   // delete
       dirty = killChar(+1);
+      if(dirty)  sendCommand(kEditChangedCmd, ascii, _id);
       break;
 
     case 256 + 20:  // left arrow
-      if(instance()->eventHandler().kbdControl(modifiers))
-        dirty = specialKeys(keycode);
+      if(instance().eventHandler().kbdControl(modifiers))
+        dirty = specialKeys(ascii, keycode);
       else if(_caretPos > 0)
         dirty = setCaretPos(_caretPos - 1);
       break;
 
     case 256 + 19:  // right arrow
-      if(instance()->eventHandler().kbdControl(modifiers))
-        dirty = specialKeys(keycode);
+      if(instance().eventHandler().kbdControl(modifiers))
+        dirty = specialKeys(ascii, keycode);
       else if(_caretPos < (int)_editString.size())
         dirty = setCaretPos(_caretPos + 1);
       break;
@@ -144,9 +151,9 @@ bool EditableWidget::handleKeyDown(int ascii, int keycode, int modifiers)
       break;
 
     default:
-      if (instance()->eventHandler().kbdControl(modifiers))
+      if (instance().eventHandler().kbdControl(modifiers))
       {
-        dirty = specialKeys(keycode);
+        dirty = specialKeys(ascii, keycode);
       }
       else if (tryInsertChar((char)ascii, _caretPos))
       {
@@ -186,9 +193,7 @@ void EditableWidget::drawCaret()
   if (!_editable || !isVisible() || !_boss->isVisible() || !_hasFocus)
     return;
 
-  GUI::Rect editRect = getEditRect();
-
-  int color = kTextColorHi;
+  const GUI::Rect& editRect = getEditRect();
   int x = editRect.left;
   int y = editRect.top;
 
@@ -197,8 +202,8 @@ void EditableWidget::drawCaret()
   x += _x;
   y += _y;
 
-  FrameBuffer& fb = _boss->instance()->frameBuffer();
-  fb.vLine(x, y+2, y + editRect.height() - 3, color);
+  FBSurface& s = _boss->dialog().surface();
+  s.vLine(x, y+2, y + editRect.height() - 3, kTextColorHi);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -248,7 +253,7 @@ bool EditableWidget::adjustOffset()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool EditableWidget::specialKeys(int keycode)
+bool EditableWidget::specialKeys(int ascii, int keycode)
 {
   bool handled = true;
 
@@ -260,6 +265,7 @@ bool EditableWidget::specialKeys(int keycode)
 
     case 'c':
       copySelectedText();
+      if(handled) sendCommand(kEditChangedCmd, ascii, _id);
       break;
 
     case 'e':
@@ -268,22 +274,27 @@ bool EditableWidget::specialKeys(int keycode)
 
     case 'd':
       handled = killChar(+1);
+      if(handled) sendCommand(kEditChangedCmd, ascii, _id);
       break;
 
     case 'k':
       handled = killLine(+1);
+      if(handled) sendCommand(kEditChangedCmd, ascii, _id);
       break;
 
     case 'u':
       handled = killLine(-1);
+      if(handled) sendCommand(kEditChangedCmd, ascii, _id);
       break;
 
     case 'v':
       pasteSelectedText();
+      if(handled) sendCommand(kEditChangedCmd, ascii, _id);
       break;
 
     case 'w':
       handled = killLastWord();
+      if(handled) sendCommand(kEditChangedCmd, ascii, _id);
       break;
 
     case 256 + 20:  // left arrow
