@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: EventMappingWidget.cxx,v 1.23 2008/03/23 16:22:46 stephena Exp $
+// $Id: EventMappingWidget.cxx 2001 2010-04-10 21:37:23Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -57,7 +58,7 @@ EventMappingWidget::EventMappingWidget(GuiObject* boss, const GUI::Font& font,
   addFocusWidget(myActionsList);
 
   // Add remap, erase, cancel and default buttons
-  xpos += myActionsList->getWidth() + 15;  ypos += 5;
+  xpos += myActionsList->getWidth() + 5;  ypos += 5;
   myMapButton = new ButtonWidget(boss, font, xpos, ypos,
                                  buttonWidth, buttonHeight,
                                  "Map", kStartMapCmd);
@@ -133,7 +134,7 @@ void EventMappingWidget::startRemapping()
   // And show a message indicating which key is being remapped
   ostringstream buf;
   buf << "Select action for '"
-      << instance()->eventHandler().actionAtIndex(myActionSelected, myEventMode)
+      << instance().eventHandler().actionAtIndex(myActionSelected, myEventMode)
       << "' event";	 	
   myKeyMapping->setTextColor(kTextColorEm);
   myKeyMapping->setLabel(buf.str());
@@ -150,8 +151,8 @@ void EventMappingWidget::eraseRemapping()
     return;
 
   Event::Type event =
-    instance()->eventHandler().eventAtIndex(myActionSelected, myEventMode);
-  instance()->eventHandler().eraseMapping(event, myEventMode);
+    instance().eventHandler().eventAtIndex(myActionSelected, myEventMode);
+  instance().eventHandler().eraseMapping(event, myEventMode);
 
   drawKeyMapping();
 }
@@ -188,7 +189,7 @@ void EventMappingWidget::drawKeyMapping()
   {
     ostringstream buf;
     buf << "Action: "
-        << instance()->eventHandler().keyAtIndex(myActionSelected, myEventMode);
+        << instance().eventHandler().keyAtIndex(myActionSelected, myEventMode);
     myKeyMapping->setTextColor(kTextColor);
     myKeyMapping->setLabel(buf.str());
   }
@@ -201,8 +202,8 @@ bool EventMappingWidget::handleKeyDown(int ascii, int keycode, int modifiers)
   if(myRemapStatus && myActionSelected >= 0)
   {
     Event::Type event =
-      instance()->eventHandler().eventAtIndex(myActionSelected, myEventMode);
-    if(instance()->eventHandler().addKeyMapping(event, myEventMode, keycode))
+      instance().eventHandler().eventAtIndex(myActionSelected, myEventMode);
+    if(instance().eventHandler().addKeyMapping(event, myEventMode, keycode))
       stopRemapping();
   }
   return true;
@@ -215,8 +216,8 @@ void EventMappingWidget::handleJoyDown(int stick, int button)
   if(myRemapStatus && myActionSelected >= 0)
   {
     Event::Type event =
-      instance()->eventHandler().eventAtIndex(myActionSelected, myEventMode);
-    if(instance()->eventHandler().addJoyMapping(event, myEventMode, stick, button))
+      instance().eventHandler().eventAtIndex(myActionSelected, myEventMode);
+    if(instance().eventHandler().addJoyMapping(event, myEventMode, stick, button))
       stopRemapping();
   }
 }
@@ -228,8 +229,8 @@ void EventMappingWidget::handleJoyAxis(int stick, int axis, int value)
   if(myRemapStatus && myActionSelected >= 0)
   {
     Event::Type event =
-      instance()->eventHandler().eventAtIndex(myActionSelected, myEventMode);
-    if(instance()->eventHandler().addJoyAxisMapping(event, myEventMode,
+      instance().eventHandler().eventAtIndex(myActionSelected, myEventMode);
+    if(instance().eventHandler().addJoyAxisMapping(event, myEventMode,
                                                     stick, axis, value))
       stopRemapping();
   }
@@ -244,8 +245,8 @@ bool EventMappingWidget::handleJoyHat(int stick, int hat, int value)
   if(myRemapStatus && myActionSelected >= 0)
   {
     Event::Type event =
-      instance()->eventHandler().eventAtIndex(myActionSelected, myEventMode);
-    if(instance()->eventHandler().addJoyHatMapping(event, myEventMode,
+      instance().eventHandler().eventAtIndex(myActionSelected, myEventMode);
+    if(instance().eventHandler().addJoyHatMapping(event, myEventMode,
                                                    stick, hat, value))
     {
       stopRemapping();
@@ -296,7 +297,7 @@ void EventMappingWidget::handleCommand(CommandSender* sender, int cmd,
       break;
 
     case kDefaultsCmd:
-      instance()->eventHandler().setDefaultMapping(myEventMode);
+      instance().eventHandler().setDefaultMapping(myEventMode);
       drawKeyMapping();
       break;
   }
