@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: GameInfoDialog.cxx,v 1.56 2008/05/16 12:17:23 stephena Exp $
+// $Id: GameInfoDialog.cxx 2001 2010-04-10 21:37:23Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -27,6 +28,7 @@
 #include "PopUpWidget.hxx"
 #include "Props.hxx"
 #include "PropsSet.hxx"
+#include "StringList.hxx"
 #include "TabWidget.hxx"
 #include "Widget.hxx"
 
@@ -35,24 +37,36 @@
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 GameInfoDialog::GameInfoDialog(
       OSystem* osystem, DialogContainer* parent, const GUI::Font& font,
-      GuiObject* boss, int x, int y, int w, int h)
-  : Dialog(osystem, parent, x, y, w, h),
+      GuiObject* boss)
+  : Dialog(osystem, parent, 0, 0, 0, 0),
     CommandSender(boss),
     myPropertiesLoaded(false),
     myDefaultsSelected(false)
 {
   const int lineHeight   = font.getLineHeight(),
+            fontWidth    = font.getMaxCharWidth(),
             fontHeight   = font.getFontHeight(),
             buttonWidth  = font.getStringWidth("Defaults") + 20,
             buttonHeight = font.getLineHeight() + 4;
   const int vBorder = 4;
   int xpos, ypos, lwidth, fwidth, pwidth, tabID;
-  unsigned int i;
   WidgetArray wid;
+  StringMap items, ports, ctrls;
+
+  // Set real dimensions
+  _w = 52 * fontWidth + 8;
+  _h = 12 * (lineHeight + 4) + 10;
+
+  ////////////////////////////////////////////////////////////////////
+  // Some of the following items are also present in GlobalPropsDialog
+  // If any changes are ever made here, GlobalPropsDialog should also
+  // be updated accordingly
+  ////////////////////////////////////////////////////////////////////
 
   // The tab widget
   xpos = 2; ypos = vBorder;
-  myTab = new TabWidget(this, font, xpos, ypos, _w - 2*xpos, _h - 24 - 2*ypos - 15);
+  myTab = new TabWidget(this, font, xpos, ypos, _w - 2*xpos,
+                        _h - buttonHeight - 2*fontHeight - 20);
   addTabWidget(myTab);
   addFocusWidget(myTab);
 
@@ -108,20 +122,53 @@ GameInfoDialog::GameInfoDialog(
   new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
                        "Sound:", kTextAlignLeft);
   pwidth = font.getStringWidth("Stereo");
+  items.clear();
+  items.push_back("Mono", "MONO");
+  items.push_back("Stereo", "STEREO");
   mySound = new PopUpWidget(myTab, font, xpos+lwidth, ypos,
-                            pwidth, lineHeight, "", 0, 0);
-  mySound->appendEntry("Mono", 1);
-  mySound->appendEntry("Stereo", 2);
+                            pwidth, lineHeight, items, "", 0, 0);
   wid.push_back(mySound);
 
   ypos += lineHeight + 3;
   new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
                        "Type:", kTextAlignLeft);
-  pwidth = font.getStringWidth("SB (128-256k SUPERbanking)");
+  pwidth = font.getStringWidth("EFSC (64K H. Runner + ram)");
+  items.clear();
+  items.push_back("Auto-detect",          "AUTO-DETECT");
+  items.push_back("0840 (8K ECONObank)",        "0840" );
+  items.push_back("2IN1 Multicart (4-32K)",     "2IN1" );
+  items.push_back("4IN1 Multicart (8-32K)",     "4IN1" );
+  items.push_back("8IN1 Multicart (16-64K)",    "8IN1" );
+  items.push_back("16IN1 Multicart (32-128K)",  "16IN1");
+  items.push_back("32IN1 Multicart (64-128K)",  "32IN1");
+  items.push_back("2K (64-2048 bytes Atari)",   "2K"   );
+  items.push_back("3E (32K Tigervision)",       "3E"   );
+  items.push_back("3F (512K Tigervision)",      "3F"   );
+  items.push_back("4A50 (64K 4A50 + ram)",      "4A50" );
+  items.push_back("4K (4K Atari)",              "4K"   );
+  items.push_back("AR (Supercharger)",          "AR"   );
+  items.push_back("CV (Commavid extra ram)",    "CV"   );
+  items.push_back("DPC (Pitfall II)",           "DPC"  );
+  items.push_back("DPC+ (Enhanced DPC)",        "DPC+" );
+  items.push_back("E0 (8K Parker Bros)",        "E0"   );
+  items.push_back("E7 (16K M-network)",         "E7"   );
+  items.push_back("EF (64K H. Runner)",         "EF"   );
+  items.push_back("EFSC (64K H. Runner + ram)", "EFSC" );
+  items.push_back("F0 (Dynacom Megaboy)",       "F0"   );
+  items.push_back("F4 (32K Atari)",             "F4"   );
+  items.push_back("F4SC (32K Atari + ram)",     "F4SC" );
+  items.push_back("F6 (16K Atari)",             "F6"   );
+  items.push_back("F6SC (16K Atari + ram)",     "F6SC" );
+  items.push_back("F8 (8K Atari)",              "F8"   );
+  items.push_back("F8SC (8K Atari + ram)",      "F8SC" );
+  items.push_back("FA (CBS RAM Plus)",          "FA"   );
+  items.push_back("FE (8K Decathlon)",          "FE"   );
+  items.push_back("MC (C. Wilkson Megacart)",   "MC"   );
+  items.push_back("SB (128-256K SUPERbank)",    "SB"   );
+  items.push_back("UA (8K UA Ltd.)",            "UA"   );
+  items.push_back("X07 (64K AtariAge)",         "X07"  );
   myType = new PopUpWidget(myTab, font, xpos+lwidth, ypos,
-                           pwidth, lineHeight, "", 0, 0);
-  for(i = 0; i < kNumCartTypes; ++i)
-    myType->appendEntry(ourCartridgeList[i][0], i+1);
+                           pwidth, lineHeight, items, "", 0, 0);
   wid.push_back(myType);
 
   // Add items for tab 0
@@ -137,28 +184,29 @@ GameInfoDialog::GameInfoDialog(
   pwidth = font.getStringWidth("B & W");
   new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
                        "Left Difficulty:", kTextAlignLeft);
+  items.clear();
+  items.push_back("B", "B");
+  items.push_back("A", "A");
   myLeftDiff = new PopUpWidget(myTab, font, xpos+lwidth, ypos,
-                               pwidth, lineHeight, "", 0, 0);
-  myLeftDiff->appendEntry("B", 1);
-  myLeftDiff->appendEntry("A", 2);
+                               pwidth, lineHeight, items, "", 0, 0);
   wid.push_back(myLeftDiff);
 
   ypos += lineHeight + 5;
   new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
                        "Right Difficulty:", kTextAlignLeft);
+  // ... use same items as above
   myRightDiff = new PopUpWidget(myTab, font, xpos+lwidth, ypos,
-                                pwidth, lineHeight, "", 0, 0);
-  myRightDiff->appendEntry("B", 1);
-  myRightDiff->appendEntry("A", 2);
+                                pwidth, lineHeight, items, "", 0, 0);
   wid.push_back(myRightDiff);
 
   ypos += lineHeight + 5;
   new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
                        "TV Type:", kTextAlignLeft);
+  items.clear();
+  items.push_back("Color", "COLOR");
+  items.push_back("B & W", "BLACKANDWHITE");
   myTVType = new PopUpWidget(myTab, font, xpos+lwidth, ypos,
-                             pwidth, lineHeight, "", 0, 0);
-  myTVType->appendEntry("Color", 1);
-  myTVType->appendEntry("B & W", 2);
+                             pwidth, lineHeight, items, "", 0, 0);
   wid.push_back(myTVType);
 
   // Add items for tab 1
@@ -174,49 +222,61 @@ GameInfoDialog::GameInfoDialog(
   pwidth = font.getStringWidth("CX-22 Trakball");
   new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
                        "P0 Controller:", kTextAlignLeft);
+  ctrls.clear();
+  ctrls.push_back("Joystick",       "JOYSTICK"    );
+  ctrls.push_back("Paddles",        "PADDLES"     );
+  ctrls.push_back("BoosterGrip",    "BOOSTERGRIP" );
+  ctrls.push_back("Driving",        "DRIVING"     );
+  ctrls.push_back("Keyboard",       "KEYBOARD"    );
+  ctrls.push_back("CX-22 Trakball", "TRACKBALL22" );
+  ctrls.push_back("CX-80 Mouse",    "TRACKBALL80" );
+  ctrls.push_back("AmigaMouse",     "AMIGAMOUSE"  );
+  ctrls.push_back("AtariVox",       "ATARIVOX"    );
+  ctrls.push_back("SaveKey",        "SAVEKEY"     );
+  ctrls.push_back("Sega Genesis",   "GENESIS"     );
+// TODO  ctrls.push_back("KidVid",         "KIDVID"      );
   myP0Controller = new PopUpWidget(myTab, font, xpos+lwidth, ypos,
-                                   pwidth, lineHeight, "", 0, 0);
-  for(i = 0; i < kNumControllerTypes; ++i)
-    myP0Controller->appendEntry(ourControllerList[i][0], i+1);
+                                   pwidth, lineHeight, ctrls, "", 0, 0);
   wid.push_back(myP0Controller);
 
   xpos += lwidth+myP0Controller->getWidth() + 4;
   new StaticTextWidget(myTab, font, xpos, ypos+1, font.getStringWidth("in "),
                        fontHeight, "in ", kTextAlignLeft);
   xpos += font.getStringWidth("in ");
+  pwidth = font.getStringWidth("right port");
+  ports.clear();
+  ports.push_back("left port", "L");
+  ports.push_back("right port", "R");
   myLeftPort = new PopUpWidget(myTab, font, xpos, ypos, pwidth, lineHeight,
-                               "", 0, kLeftCChanged);
-  myLeftPort->appendEntry("left port", 1);
-  myLeftPort->appendEntry("right port", 2);
+                               ports, "", 0, kLeftCChanged);
   wid.push_back(myLeftPort);
 
   xpos = 10;  ypos += lineHeight + 5;
+  pwidth = font.getStringWidth("CX-22 Trakball");
   new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
                        "P1 Controller:", kTextAlignLeft);
   myP1Controller = new PopUpWidget(myTab, font, xpos+lwidth, ypos,
-                                   pwidth, lineHeight, "", 0, 0);
-  for(i = 0; i < kNumControllerTypes; ++i)
-    myP1Controller->appendEntry(ourControllerList[i][0], i+1);
+                                   pwidth, lineHeight, ctrls, "", 0, 0);
   wid.push_back(myP1Controller);
 
   xpos += lwidth+myP1Controller->getWidth() + 4;
+  pwidth = font.getStringWidth("right port");
   new StaticTextWidget(myTab, font, xpos, ypos+1, font.getStringWidth("in "),
                        fontHeight, "in ", kTextAlignLeft);
   xpos += font.getStringWidth("in ");
   myRightPort = new PopUpWidget(myTab, font, xpos, ypos, pwidth, lineHeight,
-                                "", 0, kRightCChanged);
-  myRightPort->appendEntry("left port", 1);
-  myRightPort->appendEntry("right port", 2);
+                                ports, "", 0, kRightCChanged);
   wid.push_back(myRightPort);
 
   xpos = 10;  ypos += lineHeight + 5;
   pwidth = font.getStringWidth("Yes");
   new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
                        "Swap Paddles:", kTextAlignLeft);
+  items.clear();
+  items.push_back("Yes", "YES");
+  items.push_back("No", "NO");
   mySwapPaddles = new PopUpWidget(myTab, font, xpos+lwidth, ypos,
-                                  pwidth, lineHeight, "", 0, 0);
-  mySwapPaddles->appendEntry("Yes", 1);
-  mySwapPaddles->appendEntry("No", 2);
+                                  pwidth, lineHeight, items, "", 0, 0);
   wid.push_back(mySwapPaddles);
 
 
@@ -233,44 +293,45 @@ GameInfoDialog::GameInfoDialog(
   pwidth = font.getStringWidth("Auto-detect");
   new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
                        "Format:", kTextAlignLeft);
+  items.clear();
+  items.push_back("Auto-detect", "AUTO-DETECT");
+  items.push_back("NTSC",    "NTSC");
+  items.push_back("PAL",     "PAL");
+  items.push_back("SECAM",   "SECAM");
+  items.push_back("NTSC50",  "NTSC50");
+  items.push_back("PAL60",   "PAL60");
+  items.push_back("SECAM60", "SECAM60");
   myFormat = new PopUpWidget(myTab, font, xpos+lwidth, ypos,
-                             pwidth, lineHeight, "", 0, 0);
-  myFormat->appendEntry("Auto-detect", 1);
-  myFormat->appendEntry("NTSC", 2);
-  myFormat->appendEntry("PAL", 3);
-  myFormat->appendEntry("SECAM", 4);
-  myFormat->appendEntry("NTSC50", 5);
-  myFormat->appendEntry("PAL60", 6);
-  myFormat->appendEntry("SECAM60", 7);
-
+                             pwidth, lineHeight, items, "", 0, 0);
   wid.push_back(myFormat);
 
   ypos += lineHeight + 5;
   new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
                        "YStart:", kTextAlignLeft);
   myYStart = new EditTextWidget(myTab, font, xpos+lwidth, ypos,
-                                25, fontHeight, "");
+                                4*fontWidth, fontHeight, "");
   wid.push_back(myYStart);
 
   ypos += lineHeight + 5;
   new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
                        "Height:", kTextAlignLeft);
   myHeight = new EditTextWidget(myTab, font, xpos+lwidth, ypos,
-                                25, fontHeight, "");
+                                4*fontWidth, fontHeight, "");
   wid.push_back(myHeight);
 
   ypos += lineHeight + 5;
   pwidth = font.getStringWidth("Yes");
   new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
                        "Use Phosphor:", kTextAlignLeft);
-  myPhosphor = new PopUpWidget(myTab, font, xpos+lwidth, ypos,
-                               pwidth, lineHeight, "", 0, kPhosphorChanged);
-  myPhosphor->appendEntry("Yes", 1);
-  myPhosphor->appendEntry("No", 2);
+  items.clear();
+  items.push_back("Yes", "YES");
+  items.push_back("No", "NO");
+  myPhosphor = new PopUpWidget(myTab, font, xpos+lwidth, ypos, pwidth,
+                               lineHeight, items, "", 0, kPhosphorChanged);
   wid.push_back(myPhosphor);
 
   myPPBlend = new SliderWidget(myTab, font, xpos + lwidth + myPhosphor->getWidth() + 10,
-                               ypos, 30, lineHeight, "Blend: ",
+                               ypos, 8*fontWidth, lineHeight, "Blend: ",
                                font.getStringWidth("Blend: "),
                                kPPBlendChanged);
   myPPBlend->setMinValue(1); myPPBlend->setMaxValue(100);
@@ -279,18 +340,9 @@ GameInfoDialog::GameInfoDialog(
   myPPBlendLabel = new StaticTextWidget(myTab, font,
                                         xpos + lwidth + myPhosphor->getWidth() + 10 + \
                                         myPPBlend->getWidth() + 4, ypos+1,
-                                        15, fontHeight, "", kTextAlignLeft);
+                                        3*fontWidth, fontHeight, "", kTextAlignLeft);
   myPPBlendLabel->setFlags(WIDGET_CLEARBG);
 
-  ypos += lineHeight + 5;
-  new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
-                       "Use HMBlanks:", kTextAlignLeft);
-  myHmoveBlanks = new PopUpWidget(myTab, font, xpos+lwidth, ypos,
-                                  pwidth, lineHeight, "", 0, 0);
-  myHmoveBlanks->appendEntry("Yes", 1);
-  myHmoveBlanks->appendEntry("No", 2);
-  wid.push_back(myHmoveBlanks);
-
   // Add items for tab 3
   addToFocusList(wid, tabID);
 
@@ -300,7 +352,8 @@ GameInfoDialog::GameInfoDialog(
 
   // Add message concerning usage
   lwidth = font.getStringWidth("(*) Changes to properties require a ROM reload");
-  new StaticTextWidget(this, font, 10, _h - 38, lwidth, fontHeight,
+  new StaticTextWidget(this, font, 10, _h - buttonHeight - fontHeight - 20,
+                       lwidth, fontHeight,
                        "(*) Changes to properties require a ROM reload",
                        kTextAlignLeft);
 
@@ -325,18 +378,18 @@ void GameInfoDialog::loadConfig()
   myPropertiesLoaded = false;
   myDefaultsSelected = false;
 
-  if(&myOSystem->console())
+  if(&instance().console())
   {
-    myGameProperties = myOSystem->console().properties();
+    myGameProperties = instance().console().properties();
     myPropertiesLoaded = true;
     loadView();
   }
-  else if(&myOSystem->launcher())
+  else if(&instance().launcher())
   {
-    const string& md5 = myOSystem->launcher().selectedRomMD5();
+    const string& md5 = instance().launcher().selectedRomMD5();
     if(md5 != "")
     {
-      instance()->propSet().getMD5(md5, myGameProperties);
+      instance().propSet().getMD5(md5, myGameProperties);
       myPropertiesLoaded = true;
       loadView();
     }
@@ -349,151 +402,43 @@ void GameInfoDialog::loadView()
   if(!myPropertiesLoaded)
     return;
 
-  string s;
-  int i;
-
   // Cartridge properties
-  s = myGameProperties.get(Cartridge_Name);
-  myName->setEditString(s);
-
-  s = myGameProperties.get(Cartridge_MD5);
-  myMD5->setLabel(s);
-
-  s = myGameProperties.get(Cartridge_Manufacturer);
-  myManufacturer->setEditString(s);
-
-  s = myGameProperties.get(Cartridge_ModelNo);
-  myModelNo->setEditString(s);
-
-  s = myGameProperties.get(Cartridge_Rarity);
-  myRarity->setEditString(s);
-
-  s = myGameProperties.get(Cartridge_Note);
-  myNote->setEditString(s);
-
-  s = myGameProperties.get(Cartridge_Sound);
-  if(s == "MONO")
-    mySound->setSelectedTag(1);
-  else if(s == "STEREO")
-    mySound->setSelectedTag(2);
-  else
-    mySound->setSelectedTag(0);
-
-  s = myGameProperties.get(Cartridge_Type);
-  for(i = 0; i < kNumCartTypes; ++i)
-  {
-    if(s == ourCartridgeList[i][1])
-      break;
-  }
-  i = (i == kNumCartTypes) ? 0: i + 1;
-  myType->setSelectedTag(i);
+  myName->setEditString(myGameProperties.get(Cartridge_Name));
+  myMD5->setLabel(myGameProperties.get(Cartridge_MD5));
+  myManufacturer->setEditString(myGameProperties.get(Cartridge_Manufacturer));
+  myModelNo->setEditString(myGameProperties.get(Cartridge_ModelNo));
+  myRarity->setEditString(myGameProperties.get(Cartridge_Rarity));
+  myNote->setEditString(myGameProperties.get(Cartridge_Note));
+  mySound->setSelected(myGameProperties.get(Cartridge_Sound), "MONO");
+  myType->setSelected(myGameProperties.get(Cartridge_Type), "AUTO-DETECT");
 
   // Console properties
-  s = myGameProperties.get(Console_LeftDifficulty);
-  if(s == "B")
-    myLeftDiff->setSelectedTag(1);
-  else if(s == "A")
-    myLeftDiff->setSelectedTag(2);
-  else
-    myLeftDiff->setSelectedTag(0);
+  myLeftDiff->setSelected(myGameProperties.get(Console_LeftDifficulty), "B");
+  myRightDiff->setSelected(myGameProperties.get(Console_RightDifficulty), "B");
+  myTVType->setSelected(myGameProperties.get(Console_TelevisionType), "COLOR");
 
-  s = myGameProperties.get(Console_RightDifficulty);
-  if(s == "B")
-    myRightDiff->setSelectedTag(1);
-  else if(s == "A")
-    myRightDiff->setSelectedTag(2);
-  else
-    myRightDiff->setSelectedTag(0);
-
-  s = myGameProperties.get(Console_TelevisionType);
-  if(s == "COLOR")
-    myTVType->setSelectedTag(1);
-  else if(s == "BLACKANDWHITE")
-    myTVType->setSelectedTag(2);
-  else
-    myTVType->setSelectedTag(0);
-
-  s = myGameProperties.get(Console_SwapPorts);
-  myLeftPort->setSelectedTag(s == "NO" ? 1 : 2);
-  myRightPort->setSelectedTag(s == "NO" ? 2 : 1);
+  const string& swap = myGameProperties.get(Console_SwapPorts);
+  myLeftPort->setSelected((swap == "NO" ? "L" : "R"), "L");
+  myRightPort->setSelected((swap == "NO" ? "R" : "L"), "R");
 
   // Controller properties
-  s = myGameProperties.get(Controller_Left);
-  for(i = 0; i < kNumControllerTypes; ++i)
-  {
-    if(s == ourControllerList[i][1])
-      break;
-  }
-  i = (i == kNumControllerTypes) ? 0: i + 1;
-  myP0Controller->setSelectedTag(i);
-
-  s = myGameProperties.get(Controller_Right);
-  for(i = 0; i < kNumControllerTypes; ++i)
-  {
-    if(s == ourControllerList[i][1])
-      break;
-  }
-  i = (i == kNumControllerTypes) ? 0: i + 1;
-  myP1Controller->setSelectedTag(i);
-
-  s = myGameProperties.get(Controller_SwapPaddles);
-  if(s == "YES")
-    mySwapPaddles->setSelectedTag(1);
-  else if(s == "NO")
-    mySwapPaddles->setSelectedTag(2);
-  else
-    mySwapPaddles->setSelectedTag(0);
+  myP0Controller->setSelected(myGameProperties.get(Controller_Left), "JOYSTICK");
+  myP1Controller->setSelected(myGameProperties.get(Controller_Right), "JOYSTICK");
+  mySwapPaddles->setSelected(myGameProperties.get(Controller_SwapPaddles), "NO");
 
   // Display properties
-  s = myGameProperties.get(Display_Format);
-  if(s == "AUTO-DETECT")
-    myFormat->setSelectedTag(1);
-  else if(s == "NTSC")
-    myFormat->setSelectedTag(2);
-  else if(s == "PAL")
-    myFormat->setSelectedTag(3);
-  else if(s == "SECAM")
-    myFormat->setSelectedTag(4);
-  else if(s == "NTSC50")
-    myFormat->setSelectedTag(5);
-  else if(s == "PAL60")
-    myFormat->setSelectedTag(6);
-  else if(s == "SECAM60")
-    myFormat->setSelectedTag(7);
-  else
-    myFormat->setSelectedTag(0);
+  myFormat->setSelected(myGameProperties.get(Display_Format), "AUTO-DETECT");
+  myYStart->setEditString(myGameProperties.get(Display_YStart));
+  myHeight->setEditString(myGameProperties.get(Display_Height));
 
-  s = myGameProperties.get(Display_YStart);
-  myYStart->setEditString(s);
+  const string& phos = myGameProperties.get(Display_Phosphor);
+  myPhosphor->setSelected(phos, "NO");
+  myPPBlend->setEnabled(phos != "NO");
+  myPPBlendLabel->setEnabled(phos != "NO");
 
-  s = myGameProperties.get(Display_Height);
-  myHeight->setEditString(s);
-
-  myPPBlend->setEnabled(false);
-  myPPBlendLabel->setEnabled(false);
-  s = myGameProperties.get(Display_Phosphor);
-  if(s == "YES")
-  {
-    myPhosphor->setSelectedTag(1);
-    myPPBlend->setEnabled(true);
-    myPPBlendLabel->setEnabled(true);
-  }
-  else if(s == "NO")
-    myPhosphor->setSelectedTag(2);
-  else
-    myPhosphor->setSelectedTag(0);
-
-  s = myGameProperties.get(Display_PPBlend);
-  myPPBlend->setValue(atoi(s.c_str()));
-  myPPBlendLabel->setLabel(s);
-
-  s = myGameProperties.get(Emulation_HmoveBlanks);
-  if(s == "YES")
-    myHmoveBlanks->setSelectedTag(1);
-  else if(s == "NO")
-    myHmoveBlanks->setSelectedTag(2);
-  else
-    myHmoveBlanks->setSelectedTag(0);
+  const string& blend = myGameProperties.get(Display_PPBlend);
+  myPPBlend->setValue(atoi(blend.c_str()));
+  myPPBlendLabel->setLabel(blend);
 
   myTab->loadConfig();
 }
@@ -504,115 +449,44 @@ void GameInfoDialog::saveConfig()
   if(!myPropertiesLoaded)
     return;
 
-  string s;
-  int i, tag;
-
   // Cartridge properties
-  s = myName->getEditString();
-  myGameProperties.set(Cartridge_Name, s);
-
-  s = myManufacturer->getEditString();
-  myGameProperties.set(Cartridge_Manufacturer, s);
-
-  s = myModelNo->getEditString();
-  myGameProperties.set(Cartridge_ModelNo, s);
-
-  s = myRarity->getEditString();
-  myGameProperties.set(Cartridge_Rarity, s);
-
-  s = myNote->getEditString();
-  myGameProperties.set(Cartridge_Note, s);
-
-  tag = mySound->getSelectedTag();
-  s = (tag == 1) ? "Mono" : "Stereo";
-  myGameProperties.set(Cartridge_Sound, s);
-
-  tag = myType->getSelectedTag();
-  for(i = 0; i < kNumCartTypes; ++i)
-  {
-    if(i == tag-1)
-    {
-      myGameProperties.set(Cartridge_Type, ourCartridgeList[i][1]);
-      break;
-    }
-  }
+  myGameProperties.set(Cartridge_Name, myName->getEditString());
+  myGameProperties.set(Cartridge_Manufacturer, myManufacturer->getEditString());
+  myGameProperties.set(Cartridge_ModelNo, myModelNo->getEditString());
+  myGameProperties.set(Cartridge_Rarity, myRarity->getEditString());
+  myGameProperties.set(Cartridge_Note, myNote->getEditString());
+  myGameProperties.set(Cartridge_Sound, mySound->getSelectedTag());
+  myGameProperties.set(Cartridge_Type, myType->getSelectedTag());
 
   // Console properties
-  tag = myLeftDiff->getSelectedTag();
-  s = (tag == 1) ? "B" : "A";
-  myGameProperties.set(Console_LeftDifficulty, s);
-
-  tag = myRightDiff->getSelectedTag();
-  s = (tag == 1) ? "B" : "A";
-  myGameProperties.set(Console_RightDifficulty, s);
-
-  tag = myTVType->getSelectedTag();
-  s = (tag == 1) ? "Color" : "BlackAndWhite";
-  myGameProperties.set(Console_TelevisionType, s);
+  myGameProperties.set(Console_LeftDifficulty, myLeftDiff->getSelectedTag());
+  myGameProperties.set(Console_RightDifficulty, myRightDiff->getSelectedTag());
+  myGameProperties.set(Console_TelevisionType, myTVType->getSelectedTag());
 
   // Controller properties
-  tag = myP0Controller->getSelectedTag();
-  for(i = 0; i < kNumControllerTypes; ++i)
-  {
-    if(i == tag-1)
-    {
-      myGameProperties.set(Controller_Left, ourControllerList[i][1]);
-      break;
-    }
-  }
-
-  tag = myP1Controller->getSelectedTag();
-  for(i = 0; i < kNumControllerTypes; ++i)
-  {
-    if(i == tag-1)
-    {
-      myGameProperties.set(Controller_Right, ourControllerList[i][1]);
-      break;
-    }
-  }
-
-  tag = myLeftPort->getSelectedTag();
-  s = (tag == 1) ? "No" : "Yes";
-  myGameProperties.set(Console_SwapPorts, s);
-
-  tag = mySwapPaddles->getSelectedTag();
-  s = (tag == 1) ? "Yes" : "No";
-  myGameProperties.set(Controller_SwapPaddles, s);
+  myGameProperties.set(Controller_Left, myP0Controller->getSelectedTag());
+  myGameProperties.set(Controller_Right, myP1Controller->getSelectedTag());
+  myGameProperties.set(Console_SwapPorts,
+    myLeftPort->getSelectedTag() == "L" ? "NO" : "YES");
+  myGameProperties.set(Controller_SwapPaddles, mySwapPaddles->getSelectedTag());
 
   // Display properties
-  tag = myFormat->getSelectedTag();
-  s = (tag == 7) ? "SECAM60" : (tag == 6) ? "PAL60" : (tag == 5) ? "NTSC50" :
-      (tag == 4) ? "SECAM"   : (tag == 3) ? "PAL"   : (tag == 2) ? "NTSC" :
-      "AUTO-DETECT";
-  myGameProperties.set(Display_Format, s);
-
-  s = myYStart->getEditString();
-  myGameProperties.set(Display_YStart, s);
-
-  s = myHeight->getEditString();
-  myGameProperties.set(Display_Height, s);
-
-  tag = myPhosphor->getSelectedTag();
-  s = (tag == 1) ? "Yes" : "No";
-  myGameProperties.set(Display_Phosphor, s);
-
-  s = myPPBlendLabel->getLabel();
-  myGameProperties.set(Display_PPBlend, s);
-
-  tag = myHmoveBlanks->getSelectedTag();
-  s = (tag == 1) ? "Yes" : "No";
-  myGameProperties.set(Emulation_HmoveBlanks, s);
+  myGameProperties.set(Display_Format, myFormat->getSelectedTag());
+  myGameProperties.set(Display_YStart, myYStart->getEditString());
+  myGameProperties.set(Display_Height, myHeight->getEditString());
+  myGameProperties.set(Display_Phosphor, myPhosphor->getSelectedTag());
+  myGameProperties.set(Display_PPBlend, myPPBlendLabel->getLabel());
 
   // Determine whether to add or remove an entry from the properties set
   if(myDefaultsSelected)
-    instance()->propSet().removeMD5(myGameProperties.get(Cartridge_MD5));
+    instance().propSet().removeMD5(myGameProperties.get(Cartridge_MD5));
   else
-    instance()->propSet().insert(myGameProperties, true);
+    instance().propSet().insert(myGameProperties);
 
   // In any event, inform the Console and save the properties
-  if(&myOSystem->console())
-    instance()->console().setProperties(myGameProperties);
-  instance()->propSet().save(myOSystem->propertiesFile());
+  if(&instance().console())
+    instance().console().setProperties(myGameProperties);
+  instance().propSet().save(instance().propertiesFile());
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -620,7 +494,7 @@ void GameInfoDialog::setDefaults()
 {
   // Load the default properties
   string md5 = myGameProperties.get(Cartridge_MD5);
-  instance()->propSet().getMD5(md5, myGameProperties, true);
+  instance().propSet().getMD5(md5, myGameProperties, true);
 
   // Reload the current dialog
   loadView();
@@ -643,18 +517,18 @@ void GameInfoDialog::handleCommand(CommandSender* sender, int cmd,
       break;
 
     case kLeftCChanged:
-      myRightPort->setSelectedTag(
-        myLeftPort->getSelectedTag() == 2 ? 1 : 2);
+      myRightPort->setSelected(
+        myLeftPort->getSelected() == 1 ? 0 : 1);
       break;
 
     case kRightCChanged:
-      myLeftPort->setSelectedTag(
-        myRightPort->getSelectedTag() == 2 ? 1 : 2);
+      myLeftPort->setSelected(
+        myRightPort->getSelected() == 1 ? 0 : 1);
       break;
 
     case kPhosphorChanged:
     {
-      bool status = myPhosphor->getSelectedTag() == 1 ? true : false;
+      bool status = myPhosphor->getSelectedTag() == "YES";
       myPPBlend->setEnabled(status);
       myPPBlendLabel->setEnabled(status);
       break;
@@ -669,46 +543,3 @@ void GameInfoDialog::handleCommand(CommandSender* sender, int cmd,
       break;
   }
 }
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const char* GameInfoDialog::ourControllerList[kNumControllerTypes][2] = {
-  { "Joystick",       "JOYSTICK"    },
-  { "Paddles",        "PADDLES"     },
-  { "BoosterGrip",    "BOOSTERGRIP" },
-  { "Driving",        "DRIVING"     },
-  { "Keyboard",       "KEYBOARD"    },
-  { "CX-22 Trakball", "TRACKBALL22" },
-  { "CX-80 Mouse",    "TRACKBALL80" },
-  { "AmigaMouse",     "AMIGAMOUSE"  },
-  { "AtariVox",       "ATARIVOX"    },
-  { "SaveKey",        "SAVEKEY"     }
-};
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const char* GameInfoDialog::ourCartridgeList[kNumCartTypes][2] = {
-  { "Auto-detect",       "AUTO-DETECT"   },
-  { "0840 (8K ECONObanking)",     "0840" },
-  { "2K (2K Atari)",              "2K"   },
-  { "3E (32K Tigervision)",       "3E"   },
-  { "3F (512K Tigervision)",      "3F"   },
-  { "4A50 (64K 4A50 + ram)",      "4A50" },
-  { "4K (4K Atari)",              "4K"   },
-  { "AR (Supercharger)",          "AR"   },
-  { "CV (Commavid extra ram)",    "CV"   },
-  { "DPC (Pitfall II)",           "DPC"  },
-  { "E0 (8K Parker Bros)",        "E0"   },
-  { "E7 (16K M-network)",         "E7"   },
-  { "F4 (32K Atari)",             "F4"   },
-  { "F4SC (32K Atari + ram)",     "F4SC" },
-  { "F6 (16K Atari)",             "F6"   },
-  { "F6SC (16K Atari + ram)",     "F6SC" },
-  { "F8 (8K Atari)",              "F8"   },
-  { "F8SC (8K Atari + ram)",      "F8SC" },
-  { "FASC (CBS RAM Plus)",        "FASC" },
-  { "FE (8K Decathlon)",          "FE"   },
-  { "MB (Dynacom Megaboy)",       "MB"   },
-  { "MC (C. Wilkson Megacart)",   "MC"   },
-  { "SB (128-256k SUPERbanking)", "SB"   },
-  { "UA (8K UA Ltd.)",            "UA"   },
-  { "X07 (64K AtariAge)",         "X07"  }
-};
