@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: InputDialog.cxx,v 1.32 2008/05/11 21:18:35 stephena Exp $
+// $Id: InputDialog.cxx 2001 2010-04-10 21:37:23Z stephena $
 //============================================================================
 
 #include "bspf.hxx"
@@ -23,6 +24,7 @@
 #include "Joystick.hxx"
 #include "Paddles.hxx"
 #include "Settings.hxx"
+#include "StringList.hxx"
 #include "EventMappingWidget.hxx"
 #include "EditTextWidget.hxx"
 #include "PopUpWidget.hxx"
@@ -34,17 +36,19 @@
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 InputDialog::InputDialog(OSystem* osystem, DialogContainer* parent,
-                         const GUI::Font& font, int x, int y, int w, int h)
-  : Dialog(osystem, parent, x, y, w, h)
+                         const GUI::Font& font)
+  : Dialog(osystem, parent, 0, 0, 0, 0)
 {
-  const int buttonHeight = font.getLineHeight() + 4;
+  const int lineHeight   = font.getLineHeight(),
+            fontWidth    = font.getMaxCharWidth(),
+            buttonHeight = font.getLineHeight() + 4;
   const int vBorder = 4;
   int xpos, ypos, tabID;
   WidgetArray wid;
 
   // Set real dimensions
-//  _w = 42 * fontWidth + 10;
-//  _h = 12 * (lineHeight + 4) + 10;
+  _w = 42 * fontWidth + 10;
+  _h = 12 * (lineHeight + 4) + 10;
 
   // The tab widget
   xpos = 2; ypos = vBorder;
@@ -55,7 +59,7 @@ InputDialog::InputDialog(OSystem* osystem, DialogContainer* parent,
 
   // 1) Event mapper for emulation actions
   tabID = myTab->addTab("Emul. Events");
-  const StringList& eactions = instance()->eventHandler().getActionList(kEmulationMode);
+  const StringList& eactions = instance().eventHandler().getActionList(kEmulationMode);
   myEmulEventMapper = new EventMappingWidget(myTab, font, 2, 2,
                                              myTab->getWidth(),
                                              myTab->getHeight() - ypos,
@@ -65,7 +69,7 @@ InputDialog::InputDialog(OSystem* osystem, DialogContainer* parent,
 
   // 2) Event mapper for UI actions
   tabID = myTab->addTab("UI Events");
-  const StringList& mactions = instance()->eventHandler().getActionList(kMenuMode);
+  const StringList& mactions = instance().eventHandler().getActionList(kMenuMode);
   myMenuEventMapper = new EventMappingWidget(myTab, font, 2, 2,
                                              myTab->getWidth(),
                                              myTab->getHeight() - ypos,
@@ -98,6 +102,7 @@ void InputDialog::addVDeviceTab(const GUI::Font& font)
             fontHeight = font.getFontHeight();
   int xpos, ypos, lwidth, pwidth, tabID;
   WidgetArray wid;
+  StringMap items;
 
   // Virtual device/ports
   tabID = myTab->addTab("Virtual Devs");
@@ -107,56 +112,19 @@ void InputDialog::addVDeviceTab(const GUI::Font& font)
   lwidth = font.getStringWidth("Stelladaptor 2 is: ");
   pwidth = font.getStringWidth("right virtual port");
 
-  myLeftPort = new PopUpWidget(myTab, font, xpos, ypos, pwidth, lineHeight,
+  items.clear();
+  items.push_back("left virtual port", "left");
+  items.push_back("right virtual port", "right");
+  myLeftPort = new PopUpWidget(myTab, font, xpos, ypos, pwidth, lineHeight, items,
                                "Stelladaptor 1 is: ", lwidth, kLeftChanged);
-  myLeftPort->appendEntry("left virtual port", 1);
-  myLeftPort->appendEntry("right virtual port", 2);
   wid.push_back(myLeftPort);
 
   ypos += lineHeight + 5;
-  myRightPort = new PopUpWidget(myTab, font, xpos, ypos, pwidth, lineHeight,
+  // ... use items from above
+  myRightPort = new PopUpWidget(myTab, font, xpos, ypos, pwidth, lineHeight, items,
                                 "Stelladaptor 2 is: ", lwidth, kRightChanged);
-  myRightPort->appendEntry("left virtual port", 1);
-  myRightPort->appendEntry("right virtual port", 2);
   wid.push_back(myRightPort);
 
-  lwidth = font.getStringWidth("Paddle threshold: ");
-  pwidth = font.getMaxCharWidth() * 5;
-
-  // Add joystick deadzone setting
-  ypos += 2*lineHeight;
-  myDeadzone = new SliderWidget(myTab, font, xpos, ypos, pwidth, lineHeight,
-                                "Joy deadzone: ", lwidth, kDeadzoneChanged);
-  myDeadzone->setMinValue(0); myDeadzone->setMaxValue(29);
-  xpos += myDeadzone->getWidth() + 5;
-  myDeadzoneLabel = new StaticTextWidget(myTab, font, xpos, ypos+1, 24, lineHeight,
-                                         "", kTextAlignLeft);
-  myDeadzoneLabel->setFlags(WIDGET_CLEARBG);
-  wid.push_back(myDeadzone);
-
-  // Add 'mouse to paddle' mapping
-  xpos = 5;  ypos += lineHeight + 3;
-  myPaddleMode = new SliderWidget(myTab, font, xpos, ypos, pwidth, lineHeight,
-                                  "Mouse is paddle: ", lwidth, kPaddleChanged);
-  myPaddleMode->setMinValue(0); myPaddleMode->setMaxValue(3);
-  xpos += myPaddleMode->getWidth() + 5;
-  myPaddleModeLabel = new StaticTextWidget(myTab, font, xpos, ypos+1, 24, lineHeight,
-                                           "", kTextAlignLeft);
-  myPaddleModeLabel->setFlags(WIDGET_CLEARBG);
-  wid.push_back(myPaddleMode);
-
-  // Add paddle speed
-  xpos = 5;  ypos += lineHeight + 3;
-  myPaddleSpeed = new SliderWidget(myTab, font, xpos, ypos, pwidth, lineHeight,
-                                   "Paddle speed: ",
-                                   lwidth, kPSpeedChanged);
-  myPaddleSpeed->setMinValue(1); myPaddleSpeed->setMaxValue(15);
-  xpos += myPaddleSpeed->getWidth() + 5;
-  myPaddleLabel = new StaticTextWidget(myTab, font, xpos, ypos+1, 24, lineHeight,
-                                       "", kTextAlignLeft);
-  myPaddleLabel->setFlags(WIDGET_CLEARBG);
-  wid.push_back(myPaddleSpeed);
-
   // Add AtariVox serial port
   xpos = 5;  ypos += 2*lineHeight;
   int fwidth = _w - xpos - lwidth - 20;
@@ -166,6 +134,44 @@ void InputDialog::addVDeviceTab(const GUI::Font& font)
                                   fwidth, fontHeight, "");
   wid.push_back(myAVoxPort);
 
+  lwidth = font.getStringWidth("Digital paddle speed: ");
+  pwidth = font.getMaxCharWidth() * 8;
+
+  // Add joystick deadzone setting
+  ypos += 2*lineHeight;
+  myDeadzone = new SliderWidget(myTab, font, xpos, ypos, pwidth, lineHeight,
+                                "Joystick deadzone: ", lwidth, kDeadzoneChanged);
+  myDeadzone->setMinValue(0); myDeadzone->setMaxValue(29);
+  xpos += myDeadzone->getWidth() + 5;
+  myDeadzoneLabel = new StaticTextWidget(myTab, font, xpos, ypos+1, 24, lineHeight,
+                                         "", kTextAlignLeft);
+  myDeadzoneLabel->setFlags(WIDGET_CLEARBG);
+  wid.push_back(myDeadzone);
+
+  // Add paddle speed
+  xpos = 5;  ypos += lineHeight + 3;
+  myPaddleSpeed = new SliderWidget(myTab, font, xpos, ypos, pwidth, lineHeight,
+                                   "Digital paddle speed: ",
+                                   lwidth, kPSpeedChanged);
+  myPaddleSpeed->setMinValue(1); myPaddleSpeed->setMaxValue(15);
+  xpos += myPaddleSpeed->getWidth() + 5;
+  myPaddleLabel = new StaticTextWidget(myTab, font, xpos, ypos+1, 24, lineHeight,
+                                       "", kTextAlignLeft);
+  myPaddleLabel->setFlags(WIDGET_CLEARBG);
+  wid.push_back(myPaddleSpeed);
+
+  // Add 'allow all 4 directions' for joystick
+  xpos = 10;  ypos += 2*lineHeight;
+  myAllowAll4 = new CheckboxWidget(myTab, font, xpos, ypos,
+                  "Allow all 4 directions on joystick");
+  wid.push_back(myAllowAll4);
+
+  // Add mouse enable/disable
+  xpos = 10;  ypos += lineHeight + 4;
+  myMouseEnabled = new CheckboxWidget(myTab, font, xpos, ypos,
+                     "Use mouse as a controller");
+  wid.push_back(myMouseEnabled);
+
   // Add items for virtual device ports
   addToFocusList(wid, tabID);
 }
@@ -174,27 +180,30 @@ void InputDialog::addVDeviceTab(const GUI::Font& font)
 void InputDialog::loadConfig()
 {
   // Left & right ports
-  const string& sa1 = instance()->settings().getString("sa1");
-  int lport = sa1 == "right" ? 2 : 1;
-  myLeftPort->setSelectedTag(lport);
-  const string& sa2 = instance()->settings().getString("sa2");
-  int rport = sa2 == "right" ? 2 : 1;
-  myRightPort->setSelectedTag(rport);
+  const string& sa1 = instance().settings().getString("sa1");
+  int lport = sa1 == "right" ? 1 : 0;
+  myLeftPort->setSelected(lport);
+  const string& sa2 = instance().settings().getString("sa2");
+  int rport = sa2 == "right" ? 1 : 0;
+  myRightPort->setSelected(rport);
 
   // Joystick deadzone
-  myDeadzone->setValue(instance()->settings().getInt("joydeadzone"));
-  myDeadzoneLabel->setLabel(instance()->settings().getString("joydeadzone"));
+  myDeadzone->setValue(instance().settings().getInt("joydeadzone"));
+  myDeadzoneLabel->setLabel(instance().settings().getString("joydeadzone"));
 
-  // Paddle mode
-  myPaddleMode->setValue(0);
-  myPaddleModeLabel->setLabel("0");
+  // Mouse/paddle enabled
+  bool usemouse = instance().settings().getBool("usemouse");
+  myMouseEnabled->setState(usemouse);
 
   // Paddle speed
-  myPaddleSpeed->setValue(instance()->settings().getInt("pspeed"));
-  myPaddleLabel->setLabel(instance()->settings().getString("pspeed"));
+  myPaddleSpeed->setValue(instance().settings().getInt("pspeed"));
+  myPaddleLabel->setLabel(instance().settings().getString("pspeed"));
 
   // AtariVox serial port
-  myAVoxPort->setEditString(instance()->settings().getString("avoxport"));
+  myAVoxPort->setEditString(instance().settings().getString("avoxport"));
+
+  // Allow all 4 joystick directions
+  myAllowAll4->setState(instance().settings().getBool("joyallow4"));
 
   myTab->loadConfig();
 }
@@ -203,25 +212,32 @@ void InputDialog::loadConfig()
 void InputDialog::saveConfig()
 {
   // Left & right ports
-  const string& sa1 = myLeftPort->getSelectedTag() == 2 ? "right" : "left";
-  const string& sa2 = myRightPort->getSelectedTag() == 2 ? "right" : "left";
-  instance()->eventHandler().mapStelladaptors(sa1, sa2);
+  const string& sa1 = myLeftPort->getSelectedTag();
+  const string& sa2 = myRightPort->getSelectedTag();
+  instance().eventHandler().mapStelladaptors(sa1, sa2);
 
   // Joystick deadzone
   int deadzone = myDeadzone->getValue();
-  instance()->settings().setInt("joydeadzone", deadzone);
+  instance().settings().setInt("joydeadzone", deadzone);
   Joystick::setDeadZone(deadzone);
 
-  // Paddle mode
-  Paddles::setMouseIsPaddle(myPaddleMode->getValue());
+  // Mouse/paddle enabled
+  bool usemouse = myMouseEnabled->getState();
+  instance().settings().setBool("usemouse", usemouse);
+  instance().eventHandler().setPaddleMode(usemouse ? 0 : -1);
 
   // Paddle speed
   int speed = myPaddleSpeed->getValue();
-  instance()->settings().setInt("pspeed", speed);
+  instance().settings().setInt("pspeed", speed);
   Paddles::setDigitalSpeed(speed);
 
   // AtariVox serial port
-  instance()->settings().setString("avoxport", myAVoxPort->getEditString());
+  instance().settings().setString("avoxport", myAVoxPort->getEditString());
+
+  // Allow all 4 joystick directions
+  bool allowall4 = myAllowAll4->getState();
+  instance().settings().setBool("joyallow4", allowall4);
+  instance().eventHandler().allowAllDirections(allowall4);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -289,23 +305,19 @@ void InputDialog::handleCommand(CommandSender* sender, int cmd,
       break;
 
     case kLeftChanged:
-      myRightPort->setSelectedTag(
-        myLeftPort->getSelectedTag() == 2 ? 1 : 2);
+      myRightPort->setSelected(
+        myLeftPort->getSelected() == 1 ? 0 : 1);
       break;
 
     case kRightChanged:
-      myLeftPort->setSelectedTag(
-        myRightPort->getSelectedTag() == 2 ? 1 : 2);
+      myLeftPort->setSelected(
+        myRightPort->getSelected() == 1 ? 0 : 1);
       break;
 
     case kDeadzoneChanged:
       myDeadzoneLabel->setValue(myDeadzone->getValue());
       break;
 
-    case kPaddleChanged:
-      myPaddleModeLabel->setValue(myPaddleMode->getValue());
-      break;
-
     case kPSpeedChanged:
       myPaddleLabel->setValue(myPaddleSpeed->getValue());
       break;
