@@ -8,42 +8,37 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Launcher.cxx,v 1.22 2008/03/12 22:04:51 stephena Exp $
+// $Id: Launcher.cxx 2001 2010-04-10 21:37:23Z stephena $
 //============================================================================
 
-#include <sstream>
-
-class Properties;
-
 #include "LauncherDialog.hxx"
 #include "Version.hxx"
 #include "OSystem.hxx"
 #include "Settings.hxx"
 #include "FrameBuffer.hxx"
 #include "bspf.hxx"
+
 #include "Launcher.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Launcher::Launcher(OSystem* osystem)
-  : DialogContainer(osystem),
-    myWidth(400),
-    myHeight(300)
+  : DialogContainer(osystem)
 {
-  int w, h;
-  myOSystem->settings().getSize("launcherres", w, h);
-  myWidth = BSPF_max(w, 0);
-  myHeight = BSPF_max(h, 0);
+  myOSystem->settings().getSize("launcherres", (int&)myWidth, (int&)myHeight);
 
-  // Error check the resolution
-  myWidth = BSPF_max(myWidth, 320u);
-  myWidth = BSPF_min(myWidth, osystem->desktopWidth());
-  myHeight = BSPF_max(myHeight, 240u);
+  // The launcher dialog is resizable, within certain bounds
+  // We check those bounds now
+  myWidth  = BSPF_max(myWidth, osystem->desktopWidth() >= 640 ? 640u : 320u);
+  myHeight = BSPF_max(myHeight, osystem->desktopHeight() >= 480 ? 480u : 240u);
+  myWidth  = BSPF_min(myWidth, osystem->desktopWidth());
   myHeight = BSPF_min(myHeight, osystem->desktopHeight());
+
   myOSystem->settings().setSize("launcherres", myWidth, myHeight);
 
   myBaseDialog = new LauncherDialog(myOSystem, this, 0, 0, myWidth, myHeight);
@@ -55,10 +50,10 @@ Launcher::~Launcher()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Launcher::initializeVideo()
+bool Launcher::initializeVideo()
 {
   string title = string("Stella ") + STELLA_VERSION;
-  myOSystem->frameBuffer().initialize(title, myWidth, myHeight);
+  return myOSystem->frameBuffer().initialize(title, myWidth, myHeight);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
