@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: LauncherDialog.cxx,v 1.86 2008/03/30 15:47:10 stephena Exp $
+// $Id: LauncherDialog.cxx 2001 2010-04-10 21:37:23Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -24,17 +25,22 @@
 #include "bspf.hxx"
 
 #include "BrowserDialog.hxx"
+#include "ContextMenu.hxx"
 #include "DialogContainer.hxx"
 #include "Dialog.hxx"
+#include "EditTextWidget.hxx"
 #include "FSNode.hxx"
 #include "GameList.hxx"
 #include "MD5.hxx"
 #include "OptionsDialog.hxx"
+#include "GlobalPropsDialog.hxx"
+#include "LauncherFilterDialog.hxx"
 #include "OSystem.hxx"
 #include "Props.hxx"
 #include "PropsSet.hxx"
 #include "RomInfoWidget.hxx"
 #include "Settings.hxx"
+#include "StringList.hxx"
 #include "StringListWidget.hxx"
 #include "Widget.hxx"
 
@@ -44,63 +50,75 @@
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 LauncherDialog::LauncherDialog(OSystem* osystem, DialogContainer* parent,
                                int x, int y, int w, int h)
-  : Dialog(osystem, parent, x, y, w, h),
+  : Dialog(osystem, parent, x, y, w, h, true),  // use base surface
     myStartButton(NULL),
     myPrevDirButton(NULL),
     myOptionsButton(NULL),
     myQuitButton(NULL),
     myList(NULL),
     myGameList(NULL),
-    myProgressBar(NULL),
     myRomInfoWidget(NULL),
-    mySelectedItem(0),
-    myRomInfoFlag(false)
+    myMenu(NULL),
+    myGlobalProps(NULL),
+    myFilters(NULL),
+    mySelectedItem(0)
 {
-  const GUI::Font& font = instance()->launcherFont();
+  const GUI::Font& font = instance().launcherFont();
 
-  const int fontHeight  = font.getFontHeight();
-  const int bwidth  = (_w - 2 * 10 - 8 * (4 - 1)) / 4;
-  const int bheight = font.getLineHeight() + 4;
-  int xpos = 0, ypos = 0, lwidth = 0;
+  const int fontWidth = font.getMaxCharWidth(),
+            fontHeight = font.getFontHeight(),
+            bwidth  = (_w - 2 * 10 - 8 * (4 - 1)) / 4,
+            bheight = font.getLineHeight() + 4;
+  int xpos = 0, ypos = 0, lwidth = 0, lwidth2 = 0, fwidth = 0;
   WidgetArray wid;
 
-  // Check if we want the ROM info viewer
-  // Make sure it will fit within the current bounds
-  myRomInfoFlag = instance()->settings().getBool("romviewer");
-  if((w < 640 || h < 480) && myRomInfoFlag)
-  {
-    cerr << "Error: ROM launcher too small, deactivating ROM info viewer" << endl;
-    myRomInfoFlag = false;
-  }
-
   // Show game name
   lwidth = font.getStringWidth("Select an item from the list ...");
   xpos += 10;  ypos += 8;
   new StaticTextWidget(this, font, xpos, ypos, lwidth, fontHeight,
                        "Select an item from the list ...", kTextAlignLeft);
 
-  lwidth = font.getStringWidth("XXXX items found");
-  xpos = _w - lwidth - 10;
+  lwidth2 = font.getStringWidth("XXXX items found");
+  xpos = _w - lwidth2 - 10;
   myRomCount = new StaticTextWidget(this, font, xpos, ypos,
-                                    lwidth, fontHeight,
+                                    lwidth2, fontHeight,
                                     "", kTextAlignRight);
 
+  // Add filter that can narrow the results shown in the listing
+  // It has to fit between both labels
+  if(w >= 640)
+  {
+    fwidth = BSPF_min(15 * fontWidth, xpos - 20 - lwidth);
+    xpos -= fwidth + 5;
+    myPattern = new EditTextWidget(this, font, xpos, ypos,
+                                   fwidth, fontHeight, "");
+  }
+
   // Add list with game titles
+  // Before we add the list, we need to know the size of the RomInfoWidget
   xpos = 10;  ypos += fontHeight + 5;
-  int listWidth = myRomInfoFlag ? _w - 350 : _w - 20;
+  int romWidth = 0;
+  int romSize = instance().settings().getInt("romviewer");
+  if(romSize > 1 && w >= 1000 && h >= 760)
+    romWidth = 660;
+  else if(romSize > 0 && w >= 640 && h >= 480)
+    romWidth = 365;
+
+  int listWidth = _w - (romWidth > 0 ? romWidth+5 : 0) - 20;
   myList = new StringListWidget(this, font, xpos, ypos,
                                 listWidth, _h - 28 - bheight - 2*fontHeight);
   myList->setNumberingMode(kListNumberingOff);
   myList->setEditable(false);
   wid.push_back(myList);
+  if(myPattern)  wid.push_back(myPattern);  // Add after the list for tab order
 
   // Add ROM info area (if enabled)
-  if(myRomInfoFlag)
+  if(romWidth > 0)
   {
-    xpos += myList->getWidth() + 15;
-    myRomInfoWidget = new RomInfoWidget(this, instance()->font(), xpos, ypos,
-                                        326, myList->getHeight());
-    wid.push_back(myRomInfoWidget);
+    xpos += myList->getWidth() + 5;
+    myRomInfoWidget =
+      new RomInfoWidget(this, romWidth < 660 ? instance().smallFont() : instance().consoleFont(),
+                        xpos, ypos, romWidth, myList->getHeight());
   }
 
   // Add note textwidget to show any notes for the currently selected ROM
@@ -154,13 +172,31 @@ LauncherDialog::LauncherDialog(OSystem* osystem, DialogContainer* parent,
   mySelectedItem = 0;  // Highlight 'Rom Listing'
 
   // Create an options dialog, similar to the in-game one
-  myOptions = new OptionsDialog(osystem, parent, this, true);  // not in game mode
+  myOptions = new OptionsDialog(osystem, parent, this, w, h, true);  // not in game mode
 
   // Create a game list, which contains all the information about a ROM that
   // the launcher needs
   myGameList = new GameList();
 
   addToFocusList(wid);
+
+  // Create context menu for ROM list options
+  StringMap l;
+  l.push_back("Override properties", "override");
+  l.push_back("Filter listing", "filter");
+  l.push_back("Reload listing", "reload");
+  myMenu = new ContextMenu(this, osystem->font(), l);
+
+  // Create global props dialog, which is used to temporarily overrride
+  // ROM properties
+  myGlobalProps = new GlobalPropsDialog(this, osystem->font());
+
+  // Create dialog whereby the files shown in the ROM listing can be
+  // customized
+  myFilters = new LauncherFilterDialog(this, osystem->font());
+
+  // Figure out which filters are needed for the ROM listing
+  setListFilters();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -168,6 +204,9 @@ LauncherDialog::~LauncherDialog()
 {
   delete myOptions;
   delete myGameList;
+  delete myMenu;
+  delete myGlobalProps;
+  delete myFilters;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -176,13 +215,13 @@ string LauncherDialog::selectedRomMD5()
   string extension;
   int item = myList->getSelected();
   if(item < 0 || myGameList->isDir(item) ||
-     !instance()->isValidRomName(myGameList->name(item), extension))
+     !LauncherFilterDialog::isValidRomName(myGameList->name(item), extension))
     return "";
 
   // Make sure we have a valid md5 for this ROM
   if(myGameList->md5(item) == "")
   {
-    const string& md5 = instance()->MD5FromFile(myGameList->path(item));
+    const string& md5 = instance().MD5FromFile(myGameList->path(item));
     myGameList->setMd5(item, md5);
   }
   return myGameList->md5(item);
@@ -196,13 +235,15 @@ void LauncherDialog::loadConfig()
   if(myList->getList().isEmpty())
   {
     myPrevDirButton->setEnabled(false);
-    myCurrentNode = instance()->settings().getString("romdir");
+    myCurrentNode = FilesystemNode(instance().settings().getString("romdir"));
+    if(!(myCurrentNode.exists() && myCurrentNode.isDirectory()))
+      myCurrentNode = FilesystemNode("~");
 
     updateListing();
   }
   Dialog::setFocus(getFocusList()[mySelectedItem]);
 
-  if(myRomInfoFlag)
+  if(myRomInfoWidget)
     myRomInfoWidget->loadConfig();
 }
 
@@ -216,20 +257,20 @@ void LauncherDialog::enableButtons(bool enable)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void LauncherDialog::updateListing(bool fullReload)
+void LauncherDialog::updateListing(const string& nameToSelect)
 {
   // Start with empty list
   myGameList->clear();
   myDir->setLabel("");
 
-  string romdir = instance()->settings().getString("romdir");
+  string romdir = instance().settings().getString("romdir");
   loadDirListing();
 
   // Only hilite the 'up' button if there's a parent directory
   myPrevDirButton->setEnabled(myCurrentNode.hasParent());
 
   // Show current directory
-  myDir->setLabel(myCurrentNode.path());
+  myDir->setLabel(myCurrentNode.getRelativePath());
 
   // Now fill the list widget with the contents of the GameList
   StringList l;
@@ -247,8 +288,10 @@ void LauncherDialog::updateListing(bool fullReload)
   int selected = -1;
   if(!myList->getList().isEmpty())
   {
-    string lastrom = instance()->settings().getString("lastrom");
-    if(lastrom == "")
+    const string& find =
+      nameToSelect == "" ? instance().settings().getString("lastrom") : nameToSelect;
+
+    if(find == "")
       selected = 0;
     else
     {
@@ -257,7 +300,7 @@ void LauncherDialog::updateListing(bool fullReload)
       for(iter = myList->getList().begin(); iter != myList->getList().end();
           ++iter, ++itemToSelect)	 
       {
-        if(lastrom == *iter)
+        if(find == *iter)
         {
           selected = itemToSelect;
           break;
@@ -276,21 +319,38 @@ void LauncherDialog::loadDirListing()
   if(!myCurrentNode.isDirectory())
     return;
 
-  FSList files = myCurrentNode.listDir(FilesystemNode::kListAll);
+  FSList files;
+  myCurrentNode.getChildren(files, FilesystemNode::kListAll);
 
   // Add '[..]' to indicate previous folder
   if(myCurrentNode.hasParent())
     myGameList->appendGame(" [..]", "", "", true);
 
   // Now add the directory entries
+  bool domatch = myPattern && myPattern->getEditString() != "";
   for(unsigned int idx = 0; idx < files.size(); idx++)
   {
-    string name = files[idx].displayName();
+    string name = files[idx].getDisplayName();
     bool isDir = files[idx].isDirectory();
+
+    // Honour the filtering settings
+    // Showing only certain ROM extensions is determined by the extension
+    // that we want - if there are no extensions, it implies show all files
+    // In this way, showing all files is on the 'fast code path'
     if(isDir)
       name = " [" + name + "]";
+    else if(myRomExts.size() > 0)
+    {
+      // Skip over those names we've filtered out
+      if(!LauncherFilterDialog::isValidRomName(name, myRomExts))
+        continue;
+    }
 
-    myGameList->appendGame(name, files[idx].path(), "", isDir);
+    // Skip over files that don't match the pattern in the 'pattern' textbox
+    if(domatch && !isDir && !matchPattern(name, myPattern->getEditString()))
+      continue;
+
+    myGameList->appendGame(name, files[idx].getPath(), "", isDir);
   }
 
   // Sort the list by rom name (since that's what we see in the listview)
@@ -300,22 +360,21 @@ void LauncherDialog::loadDirListing()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void LauncherDialog::loadRomInfo()
 {
-  if(!(myRomInfoFlag && myRomInfoWidget)) return;
+  if(!myRomInfoWidget) return;
   int item = myList->getSelected();
   if(item < 0) return;
 
   string extension;
   if(!myGameList->isDir(item) &&
-     instance()->isValidRomName(myGameList->name(item), extension))
+     LauncherFilterDialog::isValidRomName(myGameList->name(item), extension))
   {
     // Make sure we have a valid md5 for this ROM
     if(myGameList->md5(item) == "")
-      myGameList->setMd5(item, instance()->MD5FromFile(myGameList->path(item)));
+      myGameList->setMd5(item, instance().MD5FromFile(myGameList->path(item)));
 
     // Get the properties for this entry
     Properties props;
-    const string& md5 = myGameList->md5(item);
-    instance()->propSet().getMD5(md5, props);
+    instance().propSet().getMD5(myGameList->md5(item), props);
 
     myRomInfoWidget->setProperties(props);
   }
@@ -323,6 +382,100 @@ void LauncherDialog::loadRomInfo()
     myRomInfoWidget->clearProperties();
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void LauncherDialog::handleContextMenu()
+{
+  const string& cmd = myMenu->getSelectedTag();
+
+  if(cmd == "override")
+  {
+    parent().addDialog(myGlobalProps);
+  }
+  else if(cmd == "filter")
+  {
+    parent().addDialog(myFilters);
+  }
+  else if(cmd == "reload")
+  {
+    updateListing();
+  }
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void LauncherDialog::setListFilters()
+{
+  const string& exts = instance().settings().getString("launcherexts");
+  myRomExts.clear();
+  LauncherFilterDialog::parseExts(myRomExts, exts);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool LauncherDialog::matchPattern(const string& s, const string& pattern)
+{
+  // This method is modelled after strcasestr, which we don't use
+  // because it isn't guaranteed to be available everywhere
+  // The strcasestr uses the KMP algorithm when the comparisons
+  // reach a certain point, but since we'll be dealing with relatively
+  // short strings, I think the overhead of building a KMP table
+  // each time would be slower than the brute force method used here
+  const char* haystack = s.c_str();
+  const char* needle = pattern.c_str();
+
+  unsigned char b = tolower((unsigned char) *needle);
+
+  needle++;
+  for (;; haystack++)
+  {
+    if (*haystack == '\0')  /* No match */
+      return false;
+
+    /* The first character matches */
+    if (tolower ((unsigned char) *haystack) == b)
+    {
+      const char* rhaystack = haystack + 1;
+      const char* rneedle = needle;
+
+      for (;; rhaystack++, rneedle++)
+      {
+        if (*rneedle == '\0')   /* Found a match */
+          return true;
+        if (*rhaystack == '\0') /* No match */
+          return false;
+
+        /* Nothing in this round */
+        if (tolower ((unsigned char) *rhaystack)
+            != tolower ((unsigned char) *rneedle))
+          break;
+      }
+    }
+  }
+  return false;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void LauncherDialog::handleKeyDown(int ascii, int keycode, int modifiers)
+{
+  // Grab the key before passing it to the actual dialog and check for
+  // Control-R (reload ROM listing)
+  if(instance().eventHandler().kbdControl(modifiers) && keycode == 'r')
+    updateListing();
+  else
+    Dialog::handleKeyDown(ascii, keycode, modifiers);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void LauncherDialog::handleMouseDown(int x, int y, int button, int clickCount)
+{
+  // Grab right mouse button for context menu, send left to base class
+  if(button == 2)
+  {
+    // Add menu at current x,y mouse location
+    myMenu->show(x + getAbsX(), y + getAbsY());
+  }
+  else
+    Dialog::handleMouseDown(x, y, button, clickCount);
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void LauncherDialog::handleCommand(CommandSender* sender, int cmd,
                                    int data, int id)
@@ -343,34 +496,50 @@ void LauncherDialog::handleCommand(CommandSender* sender, int cmd,
         // Directory's should be selected (ie, enter them and redisplay)
         if(myGameList->isDir(item))
         {
+          string dirname = "";
           if(myGameList->name(item) == " [..]")
+          {
             myCurrentNode = myCurrentNode.getParent();
+            if(!myNodeNames.empty())
+              dirname = myNodeNames.pop();
+          }
 		  else
-            myCurrentNode = rom;
-          updateListing();
-        }
-        else if(!instance()->isValidRomName(rom, extension) ||
-                !instance()->createConsole(rom, md5))
-        {
-          instance()->frameBuffer().showMessage("Not a valid ROM file", kMiddleCenter);
+          {
+            myCurrentNode = FilesystemNode(rom);
+            myNodeNames.push(myGameList->name(item));
+          }
+          updateListing(dirname);
         }
         else
         {
-        #if !defined(GP2X)   // Quick GP2X hack to spare flash-card saves
-          instance()->settings().setString("lastrom", myList->getSelectedString());
-        #endif
+          if(LauncherFilterDialog::isValidRomName(rom, extension))
+          {
+            if(instance().createConsole(rom, md5))
+            {
+            #if !defined(GP2X)   // Quick GP2X hack to spare flash-card saves
+              instance().settings().setString("lastrom", myList->getSelectedString());
+            #endif
+            }
+            else
+              instance().frameBuffer().showMessage(
+                  "Error creating console (screen too small)", kMiddleCenter, true);
+          }
+          else
+            instance().frameBuffer().showMessage("Not a valid ROM file",
+                                                 kMiddleCenter, true);
         }
       }
       break;
     }
 
     case kOptionsCmd:
-      parent()->addDialog(myOptions);
+      parent().addDialog(myOptions);
       break;
 
     case kPrevDirCmd:
+    case kListPrevDirCmd:
       myCurrentNode = myCurrentNode.getParent();
-      updateListing(false);
+      updateListing(myNodeNames.empty() ? "" : myNodeNames.pop());
       break;
 
     case kListSelectionChangedCmd:
@@ -379,11 +548,13 @@ void LauncherDialog::handleCommand(CommandSender* sender, int cmd,
 
     case kQuitCmd:
       close();
-      instance()->eventHandler().quit();
+      instance().eventHandler().quit();
       break;
 
     case kRomDirChosenCmd:
-      myCurrentNode = instance()->settings().getString("romdir");
+      myCurrentNode = FilesystemNode(instance().settings().getString("romdir"));
+      if(!(myCurrentNode.exists() && myCurrentNode.isDirectory()))
+        myCurrentNode = FilesystemNode("~");
       updateListing();
       break;
 
@@ -392,17 +563,22 @@ void LauncherDialog::handleCommand(CommandSender* sender, int cmd,
       break;
 
     case kReloadRomDirCmd:
-      updateListing(true);
+      updateListing();
       break;
 
-    case kResizeCmd:
-      // Instead of figuring out how to resize the snapshot image,
-      // we just reload it
-      if(myRomInfoFlag)
-      {
-        myRomInfoWidget->initialize();
-        loadRomInfo();
-      }
+    case kReloadFiltersCmd:
+      setListFilters();
+      updateListing();
+      break;
+
+    case kCMenuItemSelectedCmd:
+      handleContextMenu();
+      break;
+
+    case kEditAcceptCmd:
+    case kEditChangedCmd:
+      // The updateListing() method knows what to do when the text changes
+      updateListing();
       break;
 
     default:
