@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Menu.cxx,v 1.17 2008/02/06 13:45:24 stephena Exp $
+// $Id: Menu.cxx 2001 2010-04-10 21:37:23Z stephena $
 //============================================================================
 
 #include "Dialog.hxx"
@@ -27,7 +28,20 @@ class Properties;
 Menu::Menu(OSystem* osystem)
   : DialogContainer(osystem)
 {
-  myBaseDialog = new OptionsDialog(myOSystem, this, 0, false);  // in game mode
+  // This dialog is overlaid on the main TIA screen; we make sure it will fit
+  // If the TIA can use 1x mode, it implies that the overlay can be no larger
+  // than 320x240
+  // Otherwise we can use 2x mode, in which 640x420 is the minimum TIA size
+  int dw = osystem->desktopWidth(), dh = osystem->desktopHeight();
+  if(dw < 640 || dh < 480)
+  {
+    dw = 320;  dh = 240;
+  }
+  else
+  {
+    dw = 640;  dh = 420;
+  }
+  myBaseDialog = new OptionsDialog(myOSystem, this, 0, dw, dh, false);  // in game mode
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
