@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: ProgressDialog.cxx,v 1.12 2008/03/14 23:52:17 stephena Exp $
+// $Id: ProgressDialog.cxx 2001 2010-04-10 21:37:23Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -30,7 +31,7 @@
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ProgressDialog::ProgressDialog(GuiObject* boss, const GUI::Font& font,
                                const string& message)
-  : Dialog(boss->instance(), boss->parent(), 0, 0, 16, 16),
+  : Dialog(&boss->instance(), &boss->parent(), 0, 0, 16, 16),
     myMessage(NULL),
     mySlider(NULL),
     myStart(0),
@@ -46,8 +47,6 @@ ProgressDialog::ProgressDialog(GuiObject* boss, const GUI::Font& font,
   lwidth = font.getStringWidth(message);
   _w = lwidth + 2 * fontWidth;
   _h = lineHeight * 5;
-  _x = (boss->getWidth() - _w) / 2;
-  _y = (boss->getHeight() - _h) / 2;
 
   xpos = fontWidth; ypos = lineHeight;
   myMessage = new StaticTextWidget(this, font, xpos, ypos, lwidth, fontHeight,
@@ -59,8 +58,7 @@ ProgressDialog::ProgressDialog(GuiObject* boss, const GUI::Font& font,
   mySlider->setMinValue(1);
   mySlider->setMaxValue(100);
 
-  parent()->addDialog(this);
-  instance()->frameBuffer().update();
+  parent().addDialog(this);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -68,12 +66,6 @@ ProgressDialog::~ProgressDialog()
 {
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void ProgressDialog::done()
-{
-  parent()->removeDialog();
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void ProgressDialog::setMessage(const string& message)
 {
@@ -98,6 +90,12 @@ void ProgressDialog::setProgress(int progress)
   if(progress - mySlider->getValue() > myStep)
   {
     mySlider->setValue(progress);
-    instance()->frameBuffer().update();
+
+    // Since this dialog is usually called in a tight loop that doesn't
+    // yield, we need to manually tell the framebuffer that a redraw is
+    // necessary
+    // This isn't really an ideal solution, since all redrawing and
+    // event handling is suspended until the dialog is closed
+    instance().frameBuffer().update();
   }
 }
