@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: RomAuditDialog.cxx,v 1.3 2008/03/14 23:52:17 stephena Exp $
+// $Id: RomAuditDialog.cxx 2001 2010-04-10 21:37:23Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -21,6 +22,7 @@
 
 #include "bspf.hxx"
 
+#include "LauncherFilterDialog.hxx"
 #include "BrowserDialog.hxx"
 #include "DialogContainer.hxx"
 #include "EditTextWidget.hxx"
@@ -29,44 +31,48 @@
 #include "Props.hxx"
 #include "PropsSet.hxx"
 #include "Settings.hxx"
-
 #include "RomAuditDialog.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 RomAuditDialog::RomAuditDialog(OSystem* osystem, DialogContainer* parent,
-                               const GUI::Font& font,
-                               int x, int y, int w, int h)
-  : Dialog(osystem, parent, x, y, w, h),
+                               const GUI::Font& font, int max_w, int max_h)
+  : Dialog(osystem, parent, 0, 0, 0, 0),
     myBrowser(NULL)
 {
   const int vBorder = 8;
-  const int bwidth  = font.getStringWidth("Audit path:") + 20,
-            bheight = font.getLineHeight() + 4,
-            fontHeight = font.getLineHeight(),
+
+  const int lineHeight   = font.getLineHeight(),
+            fontWidth    = font.getMaxCharWidth(),
+            fontHeight   = font.getFontHeight(),
+            buttonWidth  = font.getStringWidth("Audit path:") + 20,
+            buttonHeight = font.getLineHeight() + 4,
             lwidth = font.getStringWidth("ROMs with properties (renamed): ");
   int xpos = vBorder, ypos = vBorder;
   WidgetArray wid;
-  ButtonWidget* b;
+
+  // Set real dimensions
+  _w = 44 * fontWidth + 10;
+  _h = 7 * (lineHeight + 4) + 10;
 
   // Audit path
   ButtonWidget* romButton = 
-    new ButtonWidget(this, font, xpos, ypos, bwidth, bheight, "Audit path:",
-                     kChooseAuditDirCmd);
+    new ButtonWidget(this, font, xpos, ypos, buttonWidth, buttonHeight,
+                     "Audit path:", kChooseAuditDirCmd);
   wid.push_back(romButton);
-  xpos += bwidth + 10;
+  xpos += buttonWidth + 10;
   myRomPath = new EditTextWidget(this, font, xpos, ypos + 2,
-                                 _w - xpos - 10, font.getLineHeight(), "");
+                                 _w - xpos - 10, lineHeight, "");
   wid.push_back(myRomPath);
 
   // Show results of ROM audit
-  xpos = vBorder + 10;  ypos += bheight + 10;
+  xpos = vBorder + 10;  ypos += buttonHeight + 10;
   new StaticTextWidget(this, font, xpos, ypos, lwidth, fontHeight,
                        "ROMs with properties (renamed): ", kTextAlignLeft);
   myResults1 = new StaticTextWidget(this, font, xpos + lwidth, ypos,
                                     _w - lwidth - 20, fontHeight, "",
                                     kTextAlignLeft);
   myResults1->setFlags(WIDGET_CLEARBG);
-  ypos += bheight;
+  ypos += buttonHeight;
   new StaticTextWidget(this, font, xpos, ypos, lwidth, fontHeight,
                        "ROMs without properties: ", kTextAlignLeft);
   myResults2 = new StaticTextWidget(this, font, xpos + lwidth, ypos,
@@ -74,32 +80,18 @@ RomAuditDialog::RomAuditDialog(OSystem* osystem, DialogContainer* parent,
                                     kTextAlignLeft);
   myResults2->setFlags(WIDGET_CLEARBG);
 
-  ypos += bheight + 8;
+  ypos += buttonHeight + 8;
   new StaticTextWidget(this, font, xpos, ypos, _w - 20, fontHeight,
-                       "(*) Warning: this operation cannot be undone",
+                       "(*) WARNING: operation cannot be undone",
                        kTextAlignLeft);
 
-  // Add OK & Cancel buttons
-#ifndef MAC_OSX
-  b = addButton(font, _w - 2 * (kButtonWidth + 7), _h - 24, "Audit", kOKCmd);
-  wid.push_back(b);
-  addOKWidget(b);
-  b = addButton(font, _w - (kButtonWidth + 10), _h - 24, "Close", kCloseCmd);
-  wid.push_back(b);
-  addCancelWidget(b);
-#else
-  b = addButton(font, _w - 2 * (kButtonWidth + 7), _h - 24, "Close", kCloseCmd);
-  wid.push_back(b);
-  addCancelWidget(b);
-  b = addButton(font, _w - (kButtonWidth + 10), _h - 24, "Audit", kOKCmd);
-  wid.push_back(b);
-  addOKWidget(b);
-#endif
-
-  addToFocusList(wid);
+  // Add OK and Cancel buttons
+  wid.clear();
+  addOKCancelBGroup(wid, font, "Audit", "Done");
+  addBGroupToFocusList(wid);
 
   // Create file browser dialog
-  myBrowser = new BrowserDialog(this, font, 60, 20, 200, 200);
+  myBrowser = new BrowserDialog(this, font, max_w, max_h);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -111,7 +103,7 @@ RomAuditDialog::~RomAuditDialog()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void RomAuditDialog::loadConfig()
 {
-  myRomPath->setEditString(instance()->settings().getString("romdir"));
+  myRomPath->setEditString(instance().settings().getString("romdir"));
   myResults1->setLabel("");
   myResults2->setLabel("");
 }
@@ -124,11 +116,12 @@ void RomAuditDialog::auditRoms()
   myResults2->setLabel("");
 
   FilesystemNode node(auditPath);
-  FSList files = node.listDir(FilesystemNode::kListFilesOnly);
+  FSList files;
+  node.getChildren(files, FilesystemNode::kListFilesOnly);
 
   // Create a progress dialog box to show the progress of processing
   // the ROMs, since this is usually a time-consuming operation
-  ProgressDialog progress(this, instance()->launcherFont(),
+  ProgressDialog progress(this, instance().font(),
                           "Auditing ROM files ...");
   progress.setRange(0, files.size() - 1, 5);
 
@@ -139,21 +132,26 @@ void RomAuditDialog::auditRoms()
   {
     string extension;
     if(!files[idx].isDirectory() &&
-       instance()->isValidRomName(files[idx].path(), extension))
+       LauncherFilterDialog::isValidRomName(files[idx].getPath(), extension))
     {
       // Calculate the MD5 so we can get the rest of the info
       // from the PropertiesSet (stella.pro)
-      const string& md5 = instance()->MD5FromFile(files[idx].path());
-      instance()->propSet().getMD5(md5, props);
+      const string& md5 = instance().MD5FromFile(files[idx].getPath());
+      instance().propSet().getMD5(md5, props);
       const string& name = props.get(Cartridge_Name);
 
       // Only rename the file if we found a valid properties entry
-      if(name != "" && name != files[idx].displayName())
+      if(name != "" && name != files[idx].getDisplayName())
       {
-        const string& newfile =
-          auditPath + BSPF_PATH_SEPARATOR + name + "." + extension;
-        if(FilesystemNode::renameFile(files[idx].path(), newfile))
-          renamed++;
+        // Check for terminating separator
+        string newfile = auditPath;
+        if(newfile.find_last_of(BSPF_PATH_SEPARATOR) != newfile.length()-1)
+          newfile += BSPF_PATH_SEPARATOR;
+        newfile += name + "." + extension;
+
+        if(files[idx].getPath() != newfile)
+          if(AbstractFilesystemNode::renameFile(files[idx].getPath(), newfile))
+            renamed++;
       }
       else
         notfound++;
@@ -162,23 +160,12 @@ void RomAuditDialog::auditRoms()
     // Update the progress bar, indicating one more ROM has been processed
     progress.setProgress(idx);
   }
-  progress.done();
+  progress.close();
 
   myResults1->setValue(renamed);
   myResults2->setValue(notfound);
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void RomAuditDialog::openBrowser(const string& title, const string& startpath,
-                                 FilesystemNode::ListMode mode, int cmd)
-{
-  parent()->addDialog(myBrowser);
-
-  myBrowser->setTitle(title);
-  myBrowser->setEmitSignal(cmd);
-  myBrowser->setStartPath(startpath, mode);
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void RomAuditDialog::handleCommand(CommandSender* sender, int cmd,
                                    int data, int id)
@@ -190,14 +177,14 @@ void RomAuditDialog::handleCommand(CommandSender* sender, int cmd,
       break;
 
     case kChooseAuditDirCmd:
-      openBrowser("Select ROM directory to audit:", myRomPath->getEditString(),
-                  FilesystemNode::kListDirectoriesOnly, kAuditDirChosenCmd);
+      myBrowser->show("Select ROM directory to audit:", myRomPath->getEditString(),
+                      FilesystemNode::kListDirectoriesOnly, kAuditDirChosenCmd);
       break;
 
     case kAuditDirChosenCmd:
     {
       FilesystemNode dir(myBrowser->getResult());
-      myRomPath->setEditString(dir.path());
+      myRomPath->setEditString(dir.getPath());
       myResults1->setLabel("");
       myResults2->setLabel("");
       break;
