@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: ScrollBarWidget.cxx,v 1.22 2008/03/02 20:48:51 stephena Exp $
+// $Id: ScrollBarWidget.cxx 2001 2010-04-10 21:37:23Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -32,52 +33,50 @@
  *   and we thus should not highlight the arrows/slider.
  */
 
-#define UP_DOWN_BOX_HEIGHT	10
+#define UP_DOWN_BOX_HEIGHT	12
 
 // Up arrow
 static unsigned int up_arrow[8] = {
+  0x00011000,
+  0x00111100,
+  0x01111110,
+  0x11111111,
   0x00000000,
   0x00000000,
-  0x00001000,
-  0x00001000,
-  0x00011100,
-  0x00011100,
-  0x00110110,
-  0x00100010,
+  0x00000000,
+  0x00000000
 };
 
 // Down arrow
 static unsigned int down_arrow[8] = {
+  0x11111111,
+  0x01111110,
+  0x00111100,
+  0x00011000,
   0x00000000,
   0x00000000,
-  0x00100010,
-  0x00110110,
-  0x00011100,
-  0x00011100,
-  0x00001000,
-  0x00001000,
+  0x00000000,
+  0x00000000
 };
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ScrollBarWidget::ScrollBarWidget(GuiObject* boss, const GUI::Font& font,
                                  int x, int y, int w, int h)
-    : Widget(boss, font, x, y, w, h), CommandSender(boss)
+  : Widget(boss, font, x, y, w, h), CommandSender(boss),
+    _numEntries(0),
+    _entriesPerPage(0),
+    _currentPos(0),
+    _wheel_lines(0),
+    _part(kNoPart),
+    _draggingPart(kNoPart),
+    _sliderHeight(0),
+    _sliderPos(0),
+    _sliderDeltaMouseDownPos(0)
 {
   _flags = WIDGET_ENABLED | WIDGET_TRACK_MOUSE | WIDGET_CLEARBG;
   _type = kScrollBarWidget;
   _bgcolor = kWidColor;
   _bgcolorhi = kWidColor;
-
-  _part = kNoPart;
-  _sliderHeight = 0;
-  _sliderPos = 0;
-
-  _draggingPart = kNoPart;
-  _sliderDeltaMouseDownPos = 0;
-
-  _numEntries = 0;
-  _entriesPerPage = 0;
-  _currentPos = 0;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -136,9 +135,9 @@ void ScrollBarWidget::handleMouseWheel(int x, int y, int direction)
     return;
 
   if(direction < 0)
-    _currentPos -= _WHEEL_LINES;
+    _currentPos -= _wheel_lines ? _wheel_lines : _WHEEL_LINES;
   else
-    _currentPos += _WHEEL_LINES;
+    _currentPos += _wheel_lines ? _wheel_lines : _WHEEL_LINES;
 
   // Make sure that _currentPos is still inside the bounds
   checkBounds(old_pos);
@@ -247,37 +246,35 @@ void ScrollBarWidget::recalc()
 void ScrollBarWidget::drawWidget(bool hilite)
 {
 //cerr << "ScrollBarWidget::drawWidget\n";
-  FrameBuffer& fb = _boss->instance()->frameBuffer();
+  FBSurface& s = _boss->dialog().surface();
   int bottomY = _y + _h;
   bool isSinglePage = (_numEntries <= _entriesPerPage);
 
-  fb.frameRect(_x, _y, _w, _h, kShadowColor);
+  s.frameRect(_x, _y, _w, _h, kShadowColor);
 
   if(_draggingPart != kNoPart)
     _part = _draggingPart;
 
   // Up arrow
-  fb.frameRect(_x, _y, _w, UP_DOWN_BOX_HEIGHT, kColor);
-  fb.drawBitmap(up_arrow, _x, _y,
-                isSinglePage ? kColor :
-                (hilite && _part == kUpArrowPart) ? kScrollColorHi : kScrollColor);
+  s.frameRect(_x, _y, _w, UP_DOWN_BOX_HEIGHT, kColor);
+  s.drawBitmap(up_arrow, _x+2, _y+4, isSinglePage ? kColor :
+               (hilite && _part == kUpArrowPart) ? kScrollColorHi : kScrollColor, 4);
 
   // Down arrow
-  fb.frameRect(_x, bottomY - UP_DOWN_BOX_HEIGHT, _w, UP_DOWN_BOX_HEIGHT, kColor);
-  fb.drawBitmap(down_arrow, _x, bottomY - UP_DOWN_BOX_HEIGHT,
-                isSinglePage ? kColor :
-                (hilite && _part == kDownArrowPart) ? kScrollColorHi : kScrollColor);
+  s.frameRect(_x, bottomY - UP_DOWN_BOX_HEIGHT, _w, UP_DOWN_BOX_HEIGHT, kColor);
+  s.drawBitmap(down_arrow, _x+2, bottomY - UP_DOWN_BOX_HEIGHT + 4, isSinglePage ? kColor :
+               (hilite && _part == kDownArrowPart) ? kScrollColorHi : kScrollColor);
 
   // Slider
   if(!isSinglePage)
   {
-    fb.fillRect(_x, _y + _sliderPos, _w, _sliderHeight,
-               (hilite && _part == kSliderPart) ? kScrollColorHi : kScrollColor);
-    fb.frameRect(_x, _y + _sliderPos, _w, _sliderHeight, kColor);
+    s.fillRect(_x, _y + _sliderPos, _w, _sliderHeight,
+              (hilite && _part == kSliderPart) ? kScrollColorHi : kScrollColor);
+    s.frameRect(_x, _y + _sliderPos, _w, _sliderHeight, kColor);
     int y = _y + _sliderPos + _sliderHeight / 2;
-    fb.hLine(_x + 2, y - 2, _x + _w - 3, kWidColor);
-    fb.hLine(_x + 2, y,     _x + _w - 3, kWidColor);
-    fb.hLine(_x + 2, y + 2, _x + _w - 3, kWidColor);
+    s.hLine(_x + 2, y - 2, _x + _w - 3, kWidColor);
+    s.hLine(_x + 2, y,     _x + _w - 3, kWidColor);
+    s.hLine(_x + 2, y + 2, _x + _w - 3, kWidColor);
   }
 }
 
