@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: TabWidget.cxx,v 1.31 2008/02/06 13:45:24 stephena Exp $
+// $Id: TabWidget.cxx 2001 2010-04-10 21:37:23Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -120,7 +121,7 @@ void TabWidget::disableTab(int tabID)
   assert(0 <= tabID && tabID < (int)_tabs.size());
 
   _tabs[tabID].enabled = false;
-  // TODO - alsa disable all widgets belonging to this tab
+  // TODO - also disable all widgets belonging to this tab
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -252,23 +253,23 @@ void TabWidget::loadConfig()
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void TabWidget::box(int x, int y, int width, int height,
-                    int colorA, int colorB, bool omitBottom)
+                    uInt32 colorA, uInt32 colorB, bool omitBottom)
 {
 //cerr << "TabWidget::box\n";
-  FrameBuffer& fb = _boss->instance()->frameBuffer();
+  FBSurface& s = _boss->dialog().surface();
 
-  fb.hLine(x + 1, y, x + width - 2, colorA);
-  fb.hLine(x, y + 1, x + width - 1, colorA);
-  fb.vLine(x, y + 1, y + height - (omitBottom ? 1 : 2), colorA);
-  fb.vLine(x + 1, y, y + height - (omitBottom ? 2 : 1), colorA);
+  s.hLine(x + 1, y, x + width - 2, colorA);
+  s.hLine(x, y + 1, x + width - 1, colorA);
+  s.vLine(x, y + 1, y + height - (omitBottom ? 1 : 2), colorA);
+  s.vLine(x + 1, y, y + height - (omitBottom ? 2 : 1), colorA);
 
   if (!omitBottom)
   {
-    fb.hLine(x + 1, y + height - 2, x + width - 1, colorB);
-    fb.hLine(x + 1, y + height - 1, x + width - 2, colorB);
+    s.hLine(x + 1, y + height - 2, x + width - 1, colorB);
+    s.hLine(x + 1, y + height - 1, x + width - 2, colorB);
   }
-  fb.vLine(x + width - 1, y + 1, y + height - (omitBottom ? 1 : 2), colorB);
-  fb.vLine(x + width - 2, y + 1, y + height - (omitBottom ? 2 : 1), colorB);
+  s.vLine(x + width - 1, y + 1, y + height - (omitBottom ? 1 : 2), colorB);
+  s.vLine(x + width - 2, y + 1, y + height - (omitBottom ? 2 : 1), colorB);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -279,7 +280,7 @@ void TabWidget::drawWidget(bool hilite)
   // it must assume responsibility for refreshing all its children.
   Widget::setDirtyInChain(_tabs[_activeTab].firstWidget);
 
-  FrameBuffer& fb = instance()->frameBuffer();
+  FBSurface& s = dialog().surface();
 
   const int left1  = _x + 1;
   const int right1 = _x + kTabLeftOffset + _activeTab * (_tabWidth + kTabSpacing);
@@ -287,30 +288,30 @@ void TabWidget::drawWidget(bool hilite)
   const int right2 = _x + _w - 2;
 	
   // Draw horizontal line
-  fb.hLine(left1, _y + _tabHeight - 2, right1, kShadowColor);
-  fb.hLine(left2, _y + _tabHeight - 2, right2, kShadowColor);
+  s.hLine(left1, _y + _tabHeight - 2, right1, kShadowColor);
+  s.hLine(left2, _y + _tabHeight - 2, right2, kShadowColor);
 
   // Iterate over all tabs and draw them
   int i, x = _x + kTabLeftOffset;
   for (i = 0; i < (int)_tabs.size(); ++i)
   {
-    int fontcolor = _tabs[i].enabled ? kTextColor : kColor;
-    int boxcolor = (i == _activeTab) ? kColor : kShadowColor;
+    uInt32 fontcolor = _tabs[i].enabled ? kTextColor : kColor;
+    uInt32 boxcolor = (i == _activeTab) ? kColor : kShadowColor;
     int yOffset = (i == _activeTab) ? 0 : 2;
     box(x, _y + yOffset, _tabWidth, _tabHeight - yOffset, boxcolor, boxcolor, (i == _activeTab));
-    fb.drawString(_font, _tabs[i].title, x + kTabPadding,
-                  _y + yOffset / 2 + (_tabHeight - _fontHeight - 1),
-                  _tabWidth - 2 * kTabPadding, fontcolor, kTextAlignCenter);
+    s.drawString(_font, _tabs[i].title, x + kTabPadding,
+                 _y + yOffset / 2 + (_tabHeight - _fontHeight - 1),
+                 _tabWidth - 2 * kTabPadding, fontcolor, kTextAlignCenter);
     x += _tabWidth + kTabSpacing;
   }
 
   // Draw a frame around the widget area (belows the tabs)
-  fb.hLine(left1, _y + _tabHeight - 1, right1, kColor);
-  fb.hLine(left2, _y + _tabHeight - 1, right2, kColor);
-  fb.hLine(_x+1, _y + _h - 2, _x + _w - 2, kShadowColor);
-  fb.hLine(_x+1, _y + _h - 1, _x + _w - 2, kColor);
-  fb.vLine(_x + _w - 2, _y + _tabHeight - 1, _y + _h - 2, kColor);
-  fb.vLine(_x + _w - 1, _y + _tabHeight - 1, _y + _h - 2, kShadowColor);
+  s.hLine(left1, _y + _tabHeight - 1, right1, kColor);
+  s.hLine(left2, _y + _tabHeight - 1, right2, kColor);
+  s.hLine(_x+1, _y + _h - 2, _x + _w - 2, kShadowColor);
+  s.hLine(_x+1, _y + _h - 1, _x + _w - 2, kColor);
+  s.vLine(_x + _w - 2, _y + _tabHeight - 1, _y + _h - 2, kColor);
+  s.vLine(_x + _w - 1, _y + _tabHeight - 1, _y + _h - 2, kShadowColor);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
