@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: OSystemMACOSX.cxx,v 1.19 2008/05/16 23:56:31 stephena Exp $
+// $Id: OSystemMACOSX.cxx 2001 2010-04-10 21:37:23Z stephena $
 //============================================================================
 
 #include <cstdlib>
@@ -31,15 +32,8 @@
 #include "OSystemMACOSX.hxx"
 #include "MenusEvents.h"
 
-#ifdef HAVE_GETTIMEOFDAY
-  #include <time.h>
-  #include <sys/time.h>
-#endif
-
 extern "C" {
   void macOpenConsole(char *romname);
-  uInt16 macOSXDisplayWidth(void);
-  uInt16 macOSXDisplayHeight(void);
   void setEmulationMenus(void);
   void setLauncherMenus(void);
   void setOptionsMenus(void);
@@ -92,9 +86,10 @@ void macOSXSendMenuEvent(int event)
 OSystemMACOSX::OSystemMACOSX()
   : OSystem()
 {
-  const string& basedir = string(getenv("HOME")) + "/.stella";
-  setBaseDir(basedir);
-  setConfigFile(basedir + "/stellarc");
+  setBaseDir("~/Library/Application Support/Stella");
+
+  // This will be overridden, as OSX uses plist files for settings
+  setConfigFile("~/Library/Application Support/Stella/stellarc");
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -102,26 +97,6 @@ OSystemMACOSX::~OSystemMACOSX()
 {
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt32 OSystemMACOSX::getTicks() const
-{
-#ifdef HAVE_GETTIMEOFDAY
-  timeval now;
-  gettimeofday(&now, 0);
-
-  return (uInt32) (now.tv_sec * 1000000 + now.tv_usec);
-#else
-  return (uInt32) SDL_GetTicks() * 1000;
-#endif
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void OSystemMACOSX::getScreenDimensions(int& width, int& height)
-{
-  width  = (int)macOSXDisplayWidth();
-  height = (int)macOSXDisplayHeight();
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void OSystemMACOSX::stateChanged(EventHandler::State state)
 {
