@@ -8,33 +8,22 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: SettingsMACOSX.cxx,v 1.20 2008/02/06 13:45:24 stephena Exp $
+// $Id: SettingsMACOSX.cxx 2001 2010-04-10 21:37:23Z stephena $
 //============================================================================
 
-#include <cassert>
-#include <sstream>
-#include <fstream>
-
-#include <unistd.h>
-#include <sys/stat.h>
-#include <sys/types.h>
-
 #include "bspf.hxx"
-#include "Console.hxx"
-#include "EventHandler.hxx"
-#include "Version.hxx"
-
 #include "Settings.hxx"
 #include "SettingsMACOSX.hxx"
 
 extern "C" {
-  void prefsSetString(char *key, char *value);
-  void prefsGetString(char *key, char *value);
+  void prefsSetString(const char* key, const char* value);
+  void prefsGetString(const char* key, char* value, int size);
   void prefsSave(void);
 }
 
@@ -56,13 +45,13 @@ SettingsMACOSX::~SettingsMACOSX()
 void SettingsMACOSX::loadConfig()
 {
   string key, value;
-  char cvalue[2048];
+  char cvalue[4096];
   
   // Read key/value pairs from the plist file
   const SettingsArray& settings = getInternalSettings();
   for(unsigned int i = 0; i < settings.size(); ++i)
   {
-    prefsGetString((char *) settings[i].key.c_str(), cvalue);
+    prefsGetString(settings[i].key.c_str(), cvalue, 4090);
     if(cvalue[0] != 0)
       setInternal(settings[i].key, cvalue, i, true);
   }
@@ -74,9 +63,7 @@ void SettingsMACOSX::saveConfig()
   // Write out each of the key and value pairs
   const SettingsArray& settings = getInternalSettings();
   for(unsigned int i = 0; i < settings.size(); ++i)
-  {
-    prefsSetString((char *) settings[i].key.c_str(),
-                   (char *) settings[i].value.c_str());
-  }
+    prefsSetString(settings[i].key.c_str(), settings[i].value.c_str());
+
   prefsSave();
 }
