@@ -8,31 +8,19 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: OSystemUNIX.cxx,v 1.29 2008/05/16 23:56:31 stephena Exp $
+// $Id: OSystemUNIX.cxx 2001 2010-04-10 21:37:23Z stephena $
 //============================================================================
 
-#include <cstdlib>
-#include <sstream>
-#include <fstream>
-
-#include <unistd.h>
-#include <sys/stat.h>
-#include <sys/types.h>
-
 #include "bspf.hxx"
 #include "OSystem.hxx"
 #include "OSystemUNIX.hxx"
 
-#ifdef HAVE_GETTIMEOFDAY
-  #include <time.h>
-  #include <sys/time.h>
-#endif
-
 /**
   Each derived class is responsible for calling the following methods
   in its constructor:
@@ -47,25 +35,11 @@
 OSystemUNIX::OSystemUNIX()
   : OSystem()
 {
-  const string& basedir = string(getenv("HOME")) + "/.stella";
-  setBaseDir(basedir);
-  setConfigFile(basedir + "/stellarc");
+  setBaseDir("~/.stella");
+  setConfigFile("~/.stella/stellarc");
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 OSystemUNIX::~OSystemUNIX()
 {
 }
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt32 OSystemUNIX::getTicks() const
-{
-#ifdef HAVE_GETTIMEOFDAY
-  timeval now;
-  gettimeofday(&now, 0);
-
-  return (uInt32) (now.tv_sec * 1000000 + now.tv_usec);
-#else
-  return (uInt32) SDL_GetTicks() * 1000;
-#endif
-}
