@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: SerialPortUNIX.cxx,v 1.4 2008/04/11 17:56:34 stephena Exp $
+// $Id: SerialPortUNIX.cxx 2001 2010-04-10 21:37:23Z stephena $
 //============================================================================
 
 #include <sys/types.h>
@@ -23,6 +24,7 @@
 #include <sys/termios.h>
 #include <sys/types.h>
 #include <sys/ioctl.h>
+#include <cstring>
 
 #include "SerialPortUNIX.hxx"
 
@@ -36,6 +38,7 @@ SerialPortUNIX::SerialPortUNIX()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 SerialPortUNIX::~SerialPortUNIX()
 {
+  closePort();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -61,7 +64,10 @@ bool SerialPortUNIX::openPort(const string& device)
 void SerialPortUNIX::closePort()
 {
   if(myHandle)
+  {
     close(myHandle);
+    myHandle = 0;
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
