@@ -8,21 +8,20 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
 // Windows CE Port by Kostas Nakos
-// $Id: SoundWinCE.cxx,v 1.7 2008/02/06 13:45:24 stephena Exp $
+// $Id: SoundWinCE.cxx 2001 2010-04-10 21:37:23Z stephena $
 //============================================================================
 
 #ifdef SOUND_SUPPORT
 
 #include "TIASnd.hxx"
 #include "FrameBuffer.hxx"
-#include "Serializer.hxx"
-#include "Deserializer.hxx"
 #include "Settings.hxx"
 #include "System.hxx"
 #include "OSystem.hxx"
@@ -318,7 +317,7 @@ void SoundWinCE::update(void)
 		}
 }
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool SoundWinCE::load(Deserializer& in)
+bool SoundWinCE::load(Serializer& in)
 {
   return true;
 }
